<?php
require_once(dirname(__FILE__).'/class.TableColumn.php');
/**
* Builds a table dynamically from a database query with the 
* option to hide certain fields or add href links to a table row
* built from field data
*/

class DynamicTable 
{
    
    var $query;
    var $sortable;
    var $rowTemplate;
    var $rowStyle;
    var $_sorting;
    
    /**
    * Factory method, returns a dynamic table based on the output class wanted using the given data resource.
    *
    * ex:
    *   $a = DynamicTable::factory( 'adodbQuery', $adodb_result );
    *   $b = DynamicTable::factory( 'pxdbCollection', $pxdbRecords );
    * @param string resource class name (corresponds to class.DynamicTable_ children )
    * @param data resource expected by resource class
    * @return object
    * @access public
    */
    function &factory ( $tableType, $dataResource )
    {
        $className = 'DynamicTable_'.$tableType;
        
        require_once (dirname(__FILE__).'/class.'.$className.'.php');
        
        if (!class_exists($className)) 
        {   
            trigger_error( "Class $className not defined ", E_USER_ERROR);
        }
        
        $obj = new $className( $dataResource );
        $obj->globalInit();
        $obj->localInit();
        return $obj;
    }
    
    /**
    * Initializes some needed default values
    */
    function globalInit()
    {
        // init the following to empty arrays to avoid warnings
        $this->hideFields = array();
        $this->preCols = array();
        $this->extraCols = array();
        $this->aliases = array();
        $this->sortable = FALSE;
        
        $this->typeCallback['string']  = 'stripslashes'; // default text handler
        $this->typeCallback['blob']    = array('stripslashes','nl2br');       
    }
    
    function setFieldCallback( $field, $function )
    {
        $this->fieldCallback[ $field ] = $function;
    }
    
    function setSortable( $bool ) 
    {
        $this->sortable = $bool;
    }
    
    /**
    * By default, a table will show all the fields in data. You can set this to TRUE to reverse that behavior
    */
    function setShowOnlyFields( $bool )
    {
        $this->_showOnlyFields = $bool;
    }
    
    /**
    * Specifies that a field should be shown.
    */
    function showField( $fieldName )
    {
        $this->_showFields[ $fieldName ] = $fieldName;
    }
    
    function showFields()
    {
        if ( 0 < func_num_args() )
        {
            foreach ( func_get_args() as $k=>$fieldName )
            {
                $this->showField( $fieldName );
            }
        }    
    }
    /**
    * Adds a field to the hideFields array. Will not display in table
    *
    * @param string fieldname in data result.
    */
    function hideField( $fieldName ) {
        $this->hideFields[ $fieldName ] = $fieldName;
    }    
    
    /**
    * Add multiple fields to the hideFields array.
    *
    * Call as $table->hideFields( 'field1', 'field2', ..., 'fieldN' );
    *
    * @param multiple field names
    */
    function hideFields()
    {
        if ( 0 < func_num_args() )
        {
            foreach ( func_get_args() as $k=>$fieldName )
            {
                $this->hideField( $fieldName );
            }
        }
    }
    /**
    * Appendsa Column Handler class to the end of the table
    * @param string, name of column handler class ie 'Column_(myname)
    * @param string, heading label to use
    * @param array, parameters to pass to column handler object
    */
    function addColumn( $type, $label, $params )
    {
        $colClass = 'Column_'.$type;
        $this->extraCols[] = new $colClass ( $label, $params );   
    }
    
    /**
    * Appendsa Column Handler class to the beginning of the table
    * @param string, name of column handler class ie 'Column_(myname)
    * @param string, heading label to use
    * @param array, parameters to pass to column handler object
    */
    function addPreColumn( $type, $label, $params )
    {
        $colClass = 'Column_'.$type;
        $this->preCols[] = new $colClass ( $label, $params );   
    }
    /**
    * Aliases a field's column name with another string.
    * 
    * @param string, field name in data set
    * @param string, text to use in heading labels
    */
    function aliasField( $field, $alias) {
        $this->aliases[ "$field" ] = "$alias";
    }
    
    function setFieldSorting ( $field, $bool )
    {
        $this->_sorting[ $field ] = $bool;
    }
    
    function setColWidth ( $field, $width ) {
        $this->colWidth[ $field ] = $width;
    }
    
    function isSortable ( $field )
    {
        //defaults to true unless set to FALSE
        // keep it === since null==false
        if ( FALSE === $this->_sorting[ $field ] )
        {
            return false;
        } else {
            return true;
        }     
    }
    /**
    * Tests if a given field is hidden.
    *
    * @param string, field name
    * @return boolean
    */
    function _isHiddenField ( $field )
    {
        if ( TRUE == $this->_showOnlyFields ) 
        {        
            return !isset( $this->_showFields[ $field ] );         
        } else {
            return isset( $this->hideFields[ $field ] ) ;
        }
    }
    
    /**
    * Gets the label for a field
    *
    * @param string, field name
    * @return string
    */
    function _getFieldLabel( $field )
    {
        if ( isset( $this->aliases[ $field ] ) )
        {
            return $this->aliases[ $field ];
        } else {
            return ucwords( $field );  
        } 
    }
    
    /**
    * Gets the value of the field in the current dataset. Will run it through
    * a typeCalbacl function if one is defined.
    *
    * @param string, the field type
    * @param mixed, the value of the field
    * @return string
    */
    function _getFieldValue ( $fieldType, $data , $fieldName)
    {
        if ( !empty( $this->typeCallback[ $fieldType ] ) )
        { // callback function defined pass data through that array of functions
            settype ( $this->typeCallback[ $fieldType ], 'array');
            foreach ( $this->typeCallback[ $fieldType ] as $i=>$fn )
            {
                $data = call_user_func( $fn, $data);
            }
        } // end if
    
        if ( !empty( $this->fieldCallback[ $fieldName ] ) )
        {
            $data = $this->fieldCallback[ $fieldName ] ( $data );
        }
        return $data;
    
    }
    
    
    function display ($outputClass = null) {
        
        if ( 0 < $this->getRecordCount() )
        {
            
            if (null==$outputClass) 
            {
                $outputClass= 'HTML';
            }
            
            require_once( dirname(__FILE__).'/class.TableOutput.php');
            
            $out = TableOutput::factory( $outputClass, $this );
            
            $out->setSortable( $this->sortable );
            $out->setRowTemplate( $this->rowTemplate );
            
            return $out->display();
        } else {
            return FALSE;
        }
        
        
    }
    
    
    function setRowStyle( $mixed )
    {
        if ( is_array( $mixed ) )
        {
            $this->_rowStyle = $mixed;
        } else {
            $this->_rowStyle = array( $mixed );
        }
    }
    
    function setRowTemplate( $tpl )
    {
        $this->rowTemplate = $tpl;   
    }
    
    /**
    * ABSTRACT METHODS
    */
    function localInit()
    {  /*Initialize child class values here. Called after globalInit(); */ }
    
    function getFieldCount()
    {
       /* returns the number of fields in the dataset */   
    }
    
    function getFieldName( $intCount )
    {
       /* returns the name of the field at $intCount index in the recordset row */   
    }
    
    function getFieldType( $intCount )
    {
       /* returns the PHP type of the field at $intCount index in the recordset row */   
    }
    
    function getRecordCount()
    {
       /* returns the number of records in in the recordset*/   
    }
    
    /**
    * return associative array $fieldName => $value OR false when DONE
    */
    function getNextRow()
    {
       /* returns the next row in the recordset */   
    }

} // end CLASS ****************************************************************************************************

?>
