<?php
// $Id: pxdb_auth.class.php,v 1.13 2006/02/19 23:07:51 sandys1 Exp $

pxdb_import('content.pxdb_data');
pxdb_import('auth.pxdb_user');

/**
* This class handles authentication for Syntax instance.
* @package pxdb.auth
* @author Hans Lellelid <hans@velum.net>
* @access public
*/
class pxdb_auth
{
    /**
    * The username column.
    * @var string
    */
    var $username_col = 'idnum';
    
    /**
    * The password column.
    * @var string
    */
    var $password_col = 'password';
    
    /**
    * @var array
    */
    var $datatype;
    
    /**
    * Stores message displayed on login form.
    * @var string
    */
    var $message = '';
    
    /**
    * The groups that user must be a member of in order to be authorized for this page.
    * @var array
    */
    var $restrict_groups = array();
    
    /**
    * Pages that do not need to pass authentication.
    * @var array
    */
    var $open_pages = array();
    
    /**
    * The unique key/name for this auth session.
    * @var string
    */
    var $auth_key;
    
    /**
    * Template to use for login form.
    * @var string
    */
    var $tpl;
    
    /**
    * For preserving GET vars after authentication
    */
    var $preserved_get_vars;
    
    /**
    * For preserving POST vars after authentication
    */
    var $preserved_post_vars;
    
    
    /**
    * Constructor.
    * @param string $auth_key A unique identifier to use for this authentication / session.
    * @access public
    */
    function pxdb_auth($auth_key = 'auth') 
    {        
        // this uses sessions -- and PHP doesn't do well at auto-starting sessions
        if (!session_id())
        {
            session_start();
        }
        
        $this->debug = PXDB_DEBUG;
        
        $this->auth_key = $auth_key;
        
        $this->db = &pxdb::dbh();
        $this->import();
        
        $prefs = &pxdb_prefs::singleton();
        $this->datatype = $prefs->get_array('dt_user');
        
        if (isset($this->_imported['preserved_get_vars']))
        {
            $this->preserved_get_vars = $this->_imported['preserved_get_vars'];
        }
        
        if (isset($this->_imported['preserved_post_vars']))
        {
            $this->preserved_post_vars = $this->_imported['preserved_post_vars'];
        }
        
        $this->preserve_vars();        
                
    }

    /**
    * Attempts to return a reference to an already-instantiated auth class.
    * Will create a new auth class only if one does not already exist.
    * @param string $auth_key A unique identifier to use for this authentication / session.
    * @return object pxdb_auth
    * @access public
    */    
    function &singleton($auth_key = 'auth')
    {
        static $instance;

        if (!isset($instance))
        {
            $instance = new pxdb_auth($auth_key);
        }

        return $instance;
    }
    
    /**
    * Performs the basic task of loading data into the $_imported var.
    * Sub-classes that override this method should always provide this minimal
    * function signature and should begin by invoke the parent import() method.
    * @param array $source The source values to import.
    * @access public
    */    
    function import($source = null)
    {
        $this->_imported = pxdb::import($source);
    }
    
    /**
    * Allows a page to pass through the auth pipeline w/o authenticating.
    * This is important if you have some pages that should be visible to an un-authenticated user
    * Some examples might include:
    *     - an overview / landing (not login) page
    *   - a "forgot my password" page.
    * @param string $page Relative URL to match for pages that are allowed through.
    * @access public
    */
    function allow($page) 
    {        
        $this->open_pages[] = $page;
    }
    
    /**
    * Determines whether a page is allowed to pass through the authentication pipeline without authenticating.
    * By default (if no page is specified), it will check to see if $_SERVER[PHP_SELF] is in the 
    * $open_pages array.
    * @param string $page The page we are checking for.
    * @return boolean
    * @access public
    */
    function is_allowed($page = null) 
    {
        if ($page == null)
        {
            $page = $_SERVER['PHP_SELF'];
        }
        return in_array($page, $this->open_pages);
    }
    
    /**
    * Set the column to use for username.
    * Looks up the column name corresponding to passed identifer or fname.
    * @param string $field The identifer/fname/id of field to use for username.
    * @access public
    * @return mixed
    */
    function set_username_field($field) 
    {
                
        // instantiate the first datatype in $datatype array
        // if there is more than one datatype, the username field had
        // better exist for both (but we're not going to check that)
        
        $data = new pxdb_data($this->datatype);
        $column = $data->lookup_colname($field);
        
        if (!$column)
        {
            pxdb::trigger_error("Can't find the specified username field '$field' in datatype.", E_USER_WARNING);
            return false;
        }
        else
        {
            $this->username_col = $column;
            return $this->username_col;
        }
    }
    
    /**
    * Sets the records field to use for password.
    * Looks up the column name corresponding to passed identifer or fname.
    * @param string $field The identifer/fname/id of field to use for username.
    * @access public
    * @return mixed
    */
    function set_password_field($field) 
    {        
        $data = new pxdb_data($this->datatype);
        $column = $data->lookup_colname($field);
        
        if (!$column)
        {
            pxdb::trigger_error("Can't find the specified password field '$field' in datatype.", E_USER_WARNING);
            return false;
        }
        else
        {
            $this->password_col = $column;
            return $this->password_col;
        }
    }
    
    /**
    * Authenticate a user.
    * This function uses PHP 4 sessions to store hash based on logged-in user.  This function will
    * try 1) to find an existing session 2) to login based on entered_username and entered_password
    * in the $_imported array.
    * After user is logged-in, this function will ensure that they have privs necessary to view
    * current page, and then it will unpack the preserved GET & POST vars.
    *    
    * @return mixed A new pxdb_user object or false if they are not logged in.
    * @access public
    */
    function &authenticate() 
    {
        // 1) first check if the current page (PHP_SELF) is allowed to pass through the pipeline
        if ($this->is_allowed())
        {
            return true;
        }
        
        if ($this->debug)
        {
            pxdb::print_debug($_SESSION[PXDB_SESSION_VAR], 'green');
        }
        
        // 2) otherwise begin checking for authentication vars
        if (isset($_SESSION[PXDB_SESSION_VAR][$this->auth_key]))
        {
            if ($this->debug)
            {
                pxdb::print_debug('Session found, logging in...', 'bookmark');
            }
                        
            $user = &$this->user($_SESSION[PXDB_SESSION_VAR][$this->auth_key], 'MD5'); // new instance of pxdb_user
    
        }
        elseif (isset($this->_imported['login_attempt']) && $this->_imported['login_attempt'] == md5($_SERVER['REMOTE_ADDR']))
        {
        
            // check login
            if ($this->debug)
            {
                pxdb::print_debug('Session NOT found, processing login attempt ...', 'bookmark'); 
            }
            
            $user = &$this->login();
        
            if (!$user)
            { 
                $var = false;
                return $var; 
            }
        }
        else
        {
            if ($this->debug)
            {
                pxdb::print_debug('No login attempt or session found ...', 'bookmark');
            }
            $return = false;
            return $return;
        }                
        
        // Now check to see if the use has sufficient privileges to proceed.        
        if (sizeof($this->restrict_groups))
        {
            if (is_numeric($this->restrict_groups[0]))
            {
                $matched_groups = array_intersect($this->restrict_groups, $user->ugidarray);        
            }
            else
            {
                $matched_groups = array_intersect($this->restrict_groups, $user->grouparray);
            }
                        
            // if the user is not in any of the required groups            
            if (!sizeof($matched_groups))
            {    
                if ($this->debug)
                {
                    pxdb::print_debug('Access denied -- clearing session & returning.', 'bookmark');
                }
                $this->logout();
                $this->message = pxdb_tools::xlate("Your account is not authorized to view this page.");
                return false;
            }
        
        }

        if ($this->debug)
        {
            pxdb::print_debug("Setting session variable (\$_SESSION[".PXDB_SESSION_VAR."]['{$this->auth_key}']) = " . md5($user->get_user()), 'bookmark');
        }
        $_SESSION[PXDB_SESSION_VAR][$this->auth_key] = md5($user->get_user());
                                
        // de-serialize vars
        if ($this->preserved_post_vars)
        {
            $_POST = array_merge($_POST, unserialize(urldecode($this->preserved_post_vars)));        
            
            if (ini_get('register_globals'))
            {
                $GLOBALS = array_merge($GLOBALS, $_POST);
            }
        }
                
        if ($this->preserved_get_vars)
        {
            $_GET = array_merge($_GET, unserialize(urldecode($this->preserved_get_vars)));                    
            if (ini_get('register_globals'))
            {
                $GLOBALS = array_merge($GLOBALS, $_GET);
            }
        }
        
        //oam: make sure we put preserved post and get vars into the request array too
        $_REQUEST = array_merge($_REQUEST, $_GET, $_POST);
        return $user;
        
    }
    
    /**
    * Restrict access to members of certain groups.
    * Sets the $restrict_groups attribute to the array of group names.
    * @param mixed $groups An array or delimited string of group *names*
    * @access public
    */
    function restrict($groups) 
    {
        if (!is_array($groups))
        {
            $groups = preg_split("/[\s,]/", $groups);
        }
        $this->restrict_groups = $groups;        
    }
    
    /**
    * Gets a new pxdb_user object with the given username or userid.
    * @param mixed $user The username or ID of user to log in.
    * @param string $sql_enc_func The SQL encryption function to use to test the passed value. (e.g. 'MD5')
    * This is used by the authenticate() method, since values are stored in the session
    * encrypted with MD5.
    * @return object A new pxdb_user object.
    * @access public
    */
    function &user($user, $sql_enc_func = null) 
    {        
        if (!is_numeric($user))
        {
            // if username was passed
            
            // check if user datatypes has been overriden or not
            if (empty($this->dt_user))
            {
                $prefs =& pxdb_prefs::singleton();
                $this->dt_user = $prefs->get_pref('dt_user');
            }                
            
            $sql = "
SELECT id                
FROM records 
WHERE
    $sql_enc_func($this->username_col) = ".$this->db->Quote($user)."
    AND datatype IN (". $this->dt_user .")
            ";
            
            $userid = $this->db->GetOne($sql);
            if ($this->debug) 
            {
                pxdb::print_debug("Auth looked up USERID on passed '$user', found: $userid", 'bookmark'); 
            }
        }
        else
        {        // id was passed                
            $userid = $user;         
        }
        
        if ($userid)
        {        
            if ($this->debug) 
            {
                pxdb::print_debug("Getting singleton of ".pxdb::get_class('pxdb_user')." for USERID '$userid'", 'bookmark'); 
            }
            
            eval("\$userObj =& ".pxdb::get_class('pxdb_user')."::singleton($userid);");
            $userObj->set_username_col($this->username_col);
            return $userObj;
        }
        else
        {
            pxdb::trigger_error("No valid USERID found for '$user'", E_USER_WARNING, 'pxdb_auth::user()');
            return false;
        }
    }
    
    /**
    * Overrides the default login datatypes (usualy 1 = user)
    *
    * @param string comma seperated ids
    * @access public
    */
    function set_datatypes($dt)
    {
        $this->dt_user = $dt;
    }
    
    /**
    * Check login information against database.
    * @return mixed The pxdb_user object or false if not able to authenticate user.
    * @access public
    */
    function &login() 
    {
        $prefs = &pxdb_prefs::singleton();
        
        if (empty($this->dt_user))
        {
            $this->dt_user = $prefs->get_list('dt_user');
        }
            
        $entered_username = md5($this->_imported['entered_username']);
        $entered_password = $this->_imported['entered_password'];
        
        if (empty($this->_imported['entered_username']))
        {
            $this->message = pxdb_tools::xlate("You must enter a username.");    
            return false;
        }
        
        if (empty($this->dt_user))
        {
            $this->dt_user = $prefs->get_list('dt_user');
        }
        
        // SFS admins must now be approved
        $sql = "
SELECT    id        
FROM     records 
WHERE    MD5($this->username_col) = ".$this->db->qstr($entered_username)."
    AND password = ENCRYPT(".$this->db->Quote($entered_password).", LEFT(password, 2))
    AND datatype IN (".$this->dt_user.")
    AND approved = 1
        ";
                        
        if ($this->debug)
        {
            pxdb::print_debug($sql, 'db-select');
        }
        
        $userid = $this->db->GetOne($sql);
        
        // make sure it found something
        if ($userid != false)
        {
            return $this->user($userid);
        }
        else
        { 
            $this->message = pxdb_tools::xlate("No matching account was found in the database.");
            $var = false;
            return $var;
        }
    }
    
    /**
    * Clears a user login.
    * Deletes the session variable and destroys (as of 0.0.4) the pxdb_user singleton.
    * @return void 
    * @access public
    */
    function logout()
    {
        // PHP's impl of $_SESSION is still buggy
        unset($_SESSION[PXDB_SESSION_VAR][$this->auth_key]);
        eval(pxdb::get_class('pxdb_user')."::destroy();"); // destroy current pxdb_user singleton
    }
    
    /**
    * Preserve POST and GET variables which will be un-packed after login process is finished.
    * This enables the user to resume exactly where they left off if their session expires.
    * @access private
    */
    function preserve_vars() 
    { 
        $_get_vars = unserialize(urldecode($this->preserved_get_vars));
        if (empty($_get_vars))
        {            
            $this->preserved_get_vars = urlencode(serialize($_GET));    
        }        
        
        $_post_vars = unserialize(urldecode($this->preserved_post_vars));
        if (empty($_post_vars))
        {            
            $this->preserved_post_vars = urlencode(serialize($_POST));    
        }
    }
    
    /**
    * Display login form.
    * This method outputs the processed template to the buffer.
    * @param string $template Template path for login form.
    * @param array $additional_vars Hash of additional vars that should be set for the template to use.
    * @access public
    */    
    function showlogin($template = null, $additional_vars = null)
    {
        $template = ($template ? $template : PXDB_TPLPATH . '/login.tpl');
        
        if (!$this->tpl)
        {
            $this->tpl = &pxdb::tpl();
        }            
        
        $this->tpl->assign('preserved_get_vars', $this->preserved_get_vars);
        $this->tpl->assign('preserved_post_vars', $this->preserved_post_vars);
        
        $this->tpl->assign('entered_username', $this->_imported['entered_username']);
        $this->tpl->assign('message', $this->message);
        $this->tpl->assign('PXDB_VAR', PXDB_GLOBAL_VAR);
        $this->tpl->assign('action', (true === isset($_SERVER['SCRIPT_URI']) ? $_SERVER['SCRIPT_URI'] : $_SERVER['PHP_SELF']));
        $this->tpl->assign('login_attempt', md5($_SERVER['REMOTE_ADDR']));
        
        if (isset($additional_vars['login_ref']))
        {
            $this->tpl->assign('login_ref', $additional_vars['login_ref']);
        }

        if ($additional_vars)
        {
            $this->tpl->assign($additional_vars); // this is an alternative way to assign when using assoc arrays
        }
        
        $this->tpl->display($template);
    }
}
?>