<?php 
/**
* Tagging Class for Syntax CMS Records
*
* Table structure for storing record tags:
*
* @author Oscar Merida <omerida@forumone.com>
 
-- 
-- Table structure for table `stx_tags`
-- 

-- DROP TABLE IF EXISTS `stx_tags`;
CREATE TABLE `stx_tags` (
  `record_id` int(11) NOT NULL default '0',
  `tag` varchar(255) NOT NULL default '',
  UNIQUE KEY `record_id` (`record_id`,`tag`),
  KEY `record_id_2` (`record_id`)
) TYPE=MyISAM;

Usage:

1) Saving or updating tags (space delimited)
$tag_string = 'oscar php coding patterns';
$record_id  = 786; // exisiting record in syntax records table

$record_tags = new StxRecordTags();
$record_tags->setRecordId($record_id);
$record_tags->getTagsFromString($tag_string);
$record_tags->save();

2) Saving or updating tags (comma delimited overrides default 
   delimiter which is whitespace)
$tag_string = 'oscar merida, php, coding patterns';
$record_id  = 786; // exisiting record in syntax records table

$record_tags = new StxRecordTags();
$record_tags->setRecordId($record_id);
$record_tags->getTagsFromString($tag_string, ',');
$record_tags->save();

4) Reading tags for a record:
$record_tags = new StxRecordTags();
$record_tags->setRecordId($record_id);
$record_tags->getTags();
*/
class StxRecordTags 
{
    /**
    * Handle to the syntax database
    */
    var $db = null;
    
    /**
    * Name of the tags table
    */
    var $_tagTable = "stx_tags";
    
    /**
    * Record id these tags apply to
    *
    * @access public
    */
    var $recordId;
    
    /**
    * Array containing Tags for the record
    *    
    * @access public
    */
    var $tags = array();
    
    /**
    * Constructor
    * @access public
    */
    function StxRecordTags()
    {
        $this->db = pxdb::dbh();
    }
    
    /**
    * Sets the record id that this set of tags relates to.
    * @param integer
    * @access public
    */
    function setRecordId($record_id)
    {
        $record_id = (int) $record_id;
        if ( !empty($record_id) )
        {
            $this->recordId = $record_id;
        }
        else
        {
            trigger_error( 'Empty record id: '.$record_id, E_USER_WARNING );
            return false;
        }
    }
    
    /**
    * Parses a string into tags, splitting on the specified delimiter.
    *
    * @param string tags
    * @param string delimiter for preg_split, optional
    * @access public
    * @return boolean
    */
    function getTagsFromString( $input, $delimiter = '\s+' )
    {
        if ( !empty($input) )
        {
            if ( $new_tags = preg_split( "/$delimiter/", $input) )
            {
                // trim up tags, in case delimiter is not spaces
                for ($i = 0; $i < count($new_tags); $i++)
                {
                    $new_tags[$i] = trim($new_tags[$i]);
                }
                $this->tags = $new_tags;
                return true;
            }
            else
            {
                trigger_error( 'Error encountered parsing tags string', E_USER_WARNING );
                return false;
            }
            // todo write oop api for working with tags
        }
    }
    
    /**
    * Saves a single tag to record in db table
    * @param string tag
    * @access private
    */
    function saveTag($tag)
    {
        $sql = 'INSERT INTO `' . $this->_tagTable . '`'
              ." VALUES ('" . $this->recordId . "', " . $this->db->qstr(trim($tag)) .")";
              
        if ( $this->db->execute($sql) )
        {
            return true;
        }
        else
        {
            trigger_error( 'Query encountered saving tag: ' .$tag, E_USER_WARNING );
            return false;
        }
        
    }
    
    /**
    * Saves or Updates the tags-to-records relationship.
    * @return boolean
    * @access public
    */
    function save()
    {
        // this is a two step process, first delete all tags for the record then save the new set
        if ( !empty($this->recordId))
        {
            $this->deleteRecordTags();
            if ( !empty($this->tags) )
            {
                $tag_count = count($this->tags);
                $success = true;
                for ( $i=0; $i < $tag_count; $i++)
                {
                    if ( false == $this->saveTag($this->tags[$i]) )
                    {
                        $success = false;
                    }
                }
                
                if ( !$success )
                {
                    trigger_error( 'Error writing one or more tags', E_USER_WARNING );
                    return false;
                }
                else
                {
                    return true;
                }
            }
        }
        else
        {
            trigger_error( 'save() called but record_id is empty', E_USER_WARNING );
            return false;
        }
    }
    
    /**
    * Deletes all tags for a record
    * @return boolean
    * @access private
    */
    function deleteRecordTags()
    {
        if ( !empty($this->recordId))
        {
            $sql = 'DELETE FROM `' . $this->_tagTable . '`'
                  ." WHERE record_id='" . $this->recordId . "'"
            ;
            
            if ( $this->db->execute($sql) )
            {
                return true;
            }
            else
            {
                trigger_error( 'Query error', E_USER_WARNING );
                return false;
            }
            
        }
        else
        {
            trigger_error( 'deleteRecordTags() called but record_id is empty', E_USER_WARNING );
            return false;
        }
    }
    
    /**
    * Returns all the tags related to a record 
    *
    * @access public
    * @return array
    */    
    function getTags()
    {
        if ( !empty($this->recordId))
        {
            
            if ( empty($this->tags ) )
            {
                // try to load them from the database
                $sql = 'SELECT tag FROM `' . $this->_tagTable . '`'
                      ." WHERE record_id='" . $this->recordId . "'"
                ;
                
                if ( $query = $this->db->execute($sql) )
                {                    
                    while( $row = $query->fetchRow() )
                    {
                        $this->tags[] = stripslashes($row['tag']);
                    }
                }
                else
                {
                    trigger_error( 'Query error', E_USER_WARNING );
                    return false;
                }
            }
            
            return $this->tags;
            
        }
        else
        {
            trigger_error( 'fetchAll() called but record_id is empty', E_USER_WARNING );
            return false;
        }
    }
    
    /**
    * Returns all the record id's associated with a tag
    * TODO: would it be useful to return a collection of records?
    * @param string tag
    * @return array of record id's
    * @access public
    */
    function getTagRecords($tag)
    {
        $tag = trim($tag);
        if ( !empty($tag) )
        {
            $sql = 'SELECT DISTINCT record_id FROM `' . $this->_tagTable . '`'
                          ." WHERE tag=" . $this->db->qstr($tag);
                    ;            
            $records = array();
            if ( $query = $this->db->execute($sql) )
            {
                while( $row = $query->fetchRow() )
                {
                    $records[] = $row['record_id'];
                }
                return $records;
            }
            else
            {
                return false;
            }
        }
    }
}

?>
