<?php
// $Id: pxdb_validator.class.php,v 1.6 2005/10/17 05:40:30 sandys1 Exp $

pxdb_import('content.input.pxdb_confront');
/**
* This class provides a basic server-side Syntax form validation framework.
*
* Form validation in this case means both the post-processing of data -- e.g. adding the http:// to the beginning
* of a URL if it was ommitted -- and the checking of the post-processed data against the requirements established
* in the database -- or in any custom functions you define.  On the code level, there is a parallel between methods
* for processing and for validating data.  There is also a parallel for the ways that this class can be customized.
*
* Validation happens *before* any records are written to the database.  This means that if you want to make sure
* that the 'name' column is unique before allowing a record to be written, you may write a validation function
* that does a lookup on records and then returns an error or false (i.e. the data passed the validation test).
*
* Important:  although this class is instantiated by pxdb_input, it is only instantiated to retrieve the 
* errors from the global namespace.  Therefore it is not important to register validation functions or set
* other validator properties to the validation class instantiated by (or bound to) pxdb_input.  The real validation
* is performed by the save() method of pxdb_record class.
*
* @package pxdb.content.input
* @author Hans Lellelid <hans@velum.net>
* @version $Revision: 1.6 $
*/
class pxdb_validator extends pxdb_confront  
{
    /**
    * @var array
    * @access private
    */            
    var $class_methods;    
    
    /**
    * Array of validator functions for a field.
    * @var array
    * @access private
    */
    var $type_validators;

    /**
    * Array of validator functions for a field type.
    * @var array
    * @access private
    */
    var $field_validators;

    /**
    * Array of post-processor functions for a field type.
    * @var array
    * @access private
    */
    var $type_processors;
    
    /**
    * Array of post-processor functions for a field.
    * @var array
    * @access private
    */    
    var $field_processors;
    
    /**
    * Initialize vars and load the validator variables if present.
    * @access public
    */
    function pxdb_validator() 
    {
        parent::pxdb_confront();
        
        $this->class_methods = get_class_methods(get_class($this));
                                    
        $this->type_validators = array();
        $this->field_validators = array();
        
        // check if this class is being instantiated after validation has occurred
        if (isset($this->_imported['validator'])) 
        {
            $this->errors = $this->_imported['validator'];
        }
        elseif (PXDB_USE_SESSION) 
        {
            $this->errors = $_SESSION[PXDB_GLOBAL_VAR]['validator'];
        }
        else
        {
            $this->errors = array();
        }
    }
    
    /**
    * Attempts to return a reference to an already-instantiated validator class.
    * Will create a new validator class only if one does not already exist.
    * @return object pxdb_validator or false on error.
    * @access public
    */    
    function &singleton()
    {
        static $instance;
        if (!isset($instance)) 
        {
            $instance = new pxdb_validator();
        }
        return $instance;
    }
    
    /**
    * Set path to use for including validator scripts.
    * @param string $path The path to use for including validator scripts.
    * @access public
    */
    function set_validators_path($path) 
    {
        $this->validators_path = $path;
    }
    
    /**
    * Register a function for validating a particular type of input.
    * @param string $type The type of input to validate (e.g. 'url', 'ltxt' or 'primaryrel')
    * @param mixed $function the name of the function. (or array of object reference + function)
    * @access public
    */
    function register_type_validator($type, $function) 
    {
        $this->type_validators[$type] = $function;    
    }

    /**
    * Register a function for validating a particular field.
    * @param string $field The name of input field to validate (e.g. 'name', 'idnum')
    * @param mixed $function the name of the function. (or array of object reference + function)
    * @param mixed $datatype The datatype this field belongs to -- in case more than one dt has defined that field.
    * @access public
    * @return boolean
    */    
    function register_field_validator($field, $function, $datatype = null) 
    {
        $tfid = $this->lookup_tfid($field, $datatype);
        if (!$tfid) 
        {
            trigger_error("[pxdb_validator] Unable to register validator for non-existant field: $field.", E_USER_WARNING);
            return false;
        }
        $this->field_validators[$tfid] = $function;
        return true;
    }
    

    /**
    * Register a function for post-processing a particular type of input.
    * @param string $type The type of input to process (e.g. 'url', 'ltxt' or 'primaryrel')
    * @param mixed $function the name of the function. (or array of object reference + function)
    * @access public
    */
    function register_type_processor($type, $function) 
    {
        $this->type_processors[$type] = $function;    
    }

    /**
    * Register a function for post-processing a particular field.
    * @param string $field The name of input field to process (e.g. 'name', 'idnum')
    * @param mixed $function the name of the function. (or array of object reference + function)
    * @param mixed $datatype The datatype this field belongs to -- in case more than one dt has defined that field.
    * @access public
    * @return boolean
    */    
    function register_field_processor($field, $function, $datatype = null) 
    {
        $tfid = $this->lookup_tfid($field, $datatype);
        if (!$tfid) 
        {
            trigger_error("[pxdb_validator] Unable to register processor for non-existant field: $field.", E_USER_WARNING);
            return false;
        }
        $this->field_processors[$tfid] = $function;
        return true;
    }


    /**
    * Create an error message for missing required field.
    * @param array $data The data hash that is passed to validator functions.
    * @see #validate
    * @return string
    * @access private
    */
    function _create_required_msg($data) 
    {
        return pxdb_tools::xlate("You must enter a value for {label}.", $data['label']);
    }

    /**
    * Get any errors for a form element.
    * Given the typesfield ID and fieldname, this function checks to see whether entered data
    * meets up to specified expectations.
    * @param string $fname The array key to lookup for the error.
    * @return mixed A hash with error information -- or false if there is no error.
    */
    function get_error($fname) 
    {
        if (isset($this->errors[$fname]))
        {
            return $this->errors[$fname];
        }
        else
        {
            return false;
        }
    }

    /**
    * Create an error message for non-unique "unique" field.
    * @param array $data The data hash that is passed to validator functions.
    * @return string The [translated] error message.
    * @see #validate
    * @access private
    */
    function _create_unique_msg($data) 
    {    
        return pxdb_tools::xlate("You must enter a unique value for {label}.", $data['label']);        
    }
    
    /**
    * Checks to see whether the passed field is unique within the datatype.
    * @param array $data The data hash that is passed to validator functions.
    * @return string The error message.
    * @see #validate
    * @access private
    */
    function _check_unique($data) 
    {
        // Remember, when checking uniqueness make sure to exclude
        // the current record (if _params['id'] exists)
        if ($this->_params['id']) 
        {
            $id_constraint_sql = "AND id <> ".$this->_params['id'];
        }
        
        // 1) instantiate typesfield.\
        $objf =& new pxdb_typesfield($data['id']);
        
            
        // 2) is it a simple field ? or relationship
        switch($objf->get_field_type()) 
        {
            case "simple":
            
                $dbf = new pxdb_dbfield($objf->get_fieldid());
                
                switch($dbf->get_field_type()) 
                {
                    case "pickid":
                    
                        if ($objf->is_multiple()) 
                        {
                            // more complex
                            pxdb::trigger_error("Currently using unique with multi-value pick lists is not supported.", E_USER_WARNING);
                            
                        }
                        else
                        {
                            // simple check to see if value exists
                            $sql = "
SELECT
    id 
FROM
    records 
WHERE
    ".$dbf->get_fname()." = ".$this->db->Quote($data['value'])."
    AND datatype = ".$objf->get_typeid()." 
    $id_constraint_sql";
                            $rs = $this->db->Execute($sql);
                            if ($this->debug) 
                            {
                                pxdb::print_debug($sql, 'db-access');
                            }
                            if (!$rs->EOF) 
                            {
                                $error = $this->_create_unique_msg($data);
                            }
                        }
                    break;
                
                    default:
                    
                        // simple check to see if value exists
                        $sql = "
SELECT     id 
FROM     records 
WHERE    ".$dbf->get_fname()." = ".$this->db->Quote($data['value'])."
    AND datatype = ".$objf->get_typeid()."
    $id_constraint_sql";
                        $rs = $this->db->Execute($sql);
                        if ($this->debug) 
                        {
                            pxdb::print_debug($sql, 'db-access');
                        }
                        if (!$rs->EOF) 
                        {
                            $error = $this->_create_unique_msg($data);
                        }
                        
                    break;
                }
                
            break;
            
            case "relationship":
            
                // what does this mean?  no other record in same datatype 
                // can have exactly the same matches from primaryrel table
                // so a) get the value (possibly array) and compare it against matching relid / related values in primaryrel
                // FOR NOW THIS IS UNSUPPORTED
                pxdb::trigger_error("Currently using unique with relationships is not supported.", E_USER_WARNING);
                
            break;
        }
        
        return $error;
    }
    
    /**
    * Generic validation function.
    * @param array $data
    * @return string
    * @access private
    */
    function _validate($data) 
    {
        return ''; // return empty string
    }
    
    /**
    * Generic (default) post-processor function.    
    * @param array $data
    * @return mixed Post-processed value.
    * @access private
    */
    function _process($data) 
    {
        if (is_string($data['value'])) 
        {
            return trim($data['value']);
        }
        else
        {
            return $data['value'];
        }
    }
    
    /**
    * Validates an email address.
    * @param array $data Hash of field properties
    * @see pxdb_widgets
    * @return string Error found. 
    * @access private
    */
    function _validate_email($data) 
    {        
        $error = "";        
        if (!preg_match("/[A-Za-z0-9]([A-Za-z0-9._]*[A-Za-z0-9]|())@[A-Za-z0-9]([A-Za-z0-9.\-]*[A-Za-z0-9]|())\.[A-Za-z]+/", $data['value'])) 
        {
            $error = "The Email you entered for field \"$data[label]\" is not valid.";
        }                
        return $error;
    }

    /**
    * Validates a number.
    * @param array $data Hash of field properties
    * @return string Error found;
    * @access private
    */
    function _validate_num($data) 
    {
        $error = "";
        if (false === is_numeric($data['value'])) 
        {
            $error = "You must enter a number for \"$data[label]\".";
        }        
        return $error;
    }
    
    /**
    * Validates a password.
    * @param array $data Hash of field properties
    * @return string Error found;
    * @access private
    */
    function _validate_pass2($data) 
    {
        // there were 2 passwords entered: fieldname and fieldname_verify
        $verifypass = $data['name'] . '_verify';
        $error = '';
        if ($this->_imported['data'][$verifypass] != $data['value']) 
        {
            $error = "The passwords you entered do not match.";
        }        
        return $error;
    }
            
    /**
    * Include a post-processor script from a (php) file.
    * @param string $plugin_data The name of the processor plugin to include AND parameters.
    * @param array $data Hash of data to pass to validator function.
    * @return mixed
    * @access private
    */
    function _include_processor($plugin_data, &$data) 
    {
        $plugin_info = pxdb::parse_plugin($plugin_data);            
        
        $plugin = $plugin_info['script'];
        $params = $plugin_info['params'];
        
        // build the path -- based on type of widget                        
        $ppath = PXDB_PLUGPATH . "/processors/$plugin.php";
        
        if (file_exists($ppath)) 
        {
            extract($data); // extract $id, $type, $name into function namespace
            // simplifies testing of processor components if we don't have to build processors with object referneces
            $db = &$this->db; 
            
            include($ppath);
            
            return $value; // this value must be set by the included block of code. (or else the included block must call "return" itself)
        }
        else
        {
            trigger_error("Post-processor not found at specified path: $ppath", E_USER_WARNING);
            return $this->_invoke_class_processor($data);
        }
    }

    /**
    * Include a validator script from a (php) file.
    * @param string $plugin_data The name of the validator plugin to include and parameters.
    * @param array $data Hash of data to pass to validator function.
    * @return mixed
    * @access private
    */
    function _include_validator($plugin_data, &$data) 
    {
        $plugin_info = pxdb::parse_plugin($plugin_data);            
            
        $plugin = $plugin_info['script'];
        $params = $plugin_info['params'];
        
        // build the path -- based on type of widget                        
        $vpath = PXDB_PLUGPATH . "/validators/$plugin.php";
        
        if (file_exists($vpath)) 
        {
            extract($data); // extract $id, $type, $name into function namespace
            
            // simplifies testing of validator components if we don't have to build validators with object referneces
            $db = &$this->db; 
            
            $error = false;
            
            include($vpath);
            
            return $error; // this value must be set by the included block of code. (or else the included block must call "return" itself)
        }
        else
        {
            pxdb::trigger_error("Validator not found at specified path: $vpath", E_USER_WARNING);
            //$this->warnings[] = "Validator not found at specified path.";
            
            return $this->_invoke_class_validator($data);
        }
    }
    
    /**
    * Invokes the class method for a specific validator.
    * @param $data Hash of field properties.
    * @return mixed The error returned by class method or false.
    * @access private
    */
    function _invoke_class_validator($data) 
    {
        $method = '_validate_'.$data['type'];
        
        if (in_array($method, $this->class_methods)) 
        {                                   
            return $this->$method($data);                    
        }
        else
        {
            // generic
            return $this->_validate($data);
        }
    }

    /**
    * Invokes the class method for a specific post-processor.
    * @param $data Hash of field properties.
    * @return mixed The new value returned by post-processor.
    * @access private
    */
    function _invoke_class_processor($data) 
    {
        $method = '_process_'.$data['type'];
        
        if (in_array($method, $this->class_methods)) 
        {
            return $this->$method($data);                    
        }
        else
        {
            // generic
            return $this->_process($data);
        }
    }


    /**
    * Performs post-process function on data for a particular typesfield.
    * 
    * @param int $tfid The pxdb_typesfield id.
    * @param string $fname The fieldname.
    * @param mixed $value The value to be processed.
    * @return mixed The error [hash array] that was produced or false if no error.
    * @access public
    */
    function process($tfid, $fname, $value) 
    {
        // 1) instantiate the typesfield
        $objf = new pxdb_typesfield($tfid);
        
        // 2) compile a $data hash that will be passed to specific validators
        $data = array(
            'id' => $objf->get_id(),
            'label' => $objf->get_displayname(),
            'type' => $objf->get_field_metatype(),
            'name' => $fname,
            'value' => $value
       );
        
        // 3) Perform post-processing on value
        if (isset($this->field_processors[$objf->get_id()])) 
        {
            // a) check if there is a registered *field* post-processor function
            return $this->_invoke_user_method($this->field_processors[$objf->get_id()], $data, $this);
        }
        elseif ($objf->get_processor() && pxdb::plugin_exists('processors', $objf->get_processor())) 
        {
            // b) check to see if a processor is defined in typesfields table
            return $this->_include_processor($objf->get_processor(), $data);
        }
        elseif (isset($this->type_processors[$objf->get_field_metatype()])) 
        {
            // c) check if there is a registered *type* post-processor function
            return  $this->_invoke_user_method($this->type_processors[$objf->get_field_metatype()], $data, $this);
        }
        else
        {
            // d) check if there is a class function for this
            return $this->_invoke_class_processor($data);
        }
    } // process()
    
    /**
    * Validates data for a particular typesfield.
    * Sets a global array $pxdb[validation] that will be read in by a subsequent instantiation of this
    * class.
    * @param integer $tfid The pxdb_typesfield id.
    * @param string $fname The fieldname.
    * @param mixed $value The value to be validated.
    * @return mixed The error [hash array] that was produced or false if no error.
    * @access public
    */
    function validate($tfid, $fname, $value) 
    {
        // $global_var = PXDB_GLOBAL_VAR;
        //-- using $GLOBALS global ${$global_var};
        // we need to set the [validator] component of this global array.
        // so that subsequent instantiations of classes are aware that we have identified the errors.
                     
        // Initialize
        $error = array();
        if (true === empty($this->errors[$fname]))
        {
            $this->errors[$fname] = false;
        }
        
        // 1) instantiate the typesfield
        $objf = new pxdb_typesfield($tfid);
        
        // 2) compile a $data hash that will be passed to specific validators
        $data = array(
            'id' => $objf->get_id(),
            'label' => $objf->get_displayname(),
            'type' => $objf->get_field_metatype(),
            'name' => $fname,
            'value' => $value,
            'record_id' => $this->_params['id'] // may be set if we are editing
       ); 
                    
        // 3) Perform validation that is specified in DB
        //  a) required
        if ($objf->is_required() && ($value === "" || (is_array($value) && sizeof($value) == 1 && $value[0] === ""))) 
        {
            $this->errors[$fname] = $this->_create_required_msg($data);
        }                    
        
        // b) unique
        if ($objf->is_unique() && !$this->errors[$fname] && !($value === "" || (is_array($value) && sizeof($value) == 1 && $value[0] === ""))) 
        {
            // check to see if it's unique
            $this->errors[$fname] = $this->_check_unique($data);
        }
        
        // 4) Perform individual validation -- if value is not empty
        if ($value !== "" && !$this->errors[$fname]) 
        {
            if (isset($this->field_validators[$objf->get_id()])) 
            {
                // a) check if there is a registered *field* validator function            
                $this->errors[$fname] =  $this->_invoke_user_method($this->field_validators[$objf->get_id()], $data, $this);
            }
            elseif ($objf->get_validator() && pxdb::plugin_exists('validators', $objf->get_validator())) 
            {
                // b) check to see if a validator is defined in typesfields table
                $this->errors[$fname] =  $this->_include_validator($objf->get_validator(), $data);
            }
            elseif (isset($this->type_validators[$objf->get_field_metatype()])) 
            {
                // c) check if there is a registered *type* validator function
                $this->errors[$fname] =  $this->_invoke_user_method($this->type_validators[$objf->get_field_metatype()], $data, $this);
            }
            else
            {
                // d) check if there is a class function for this
                $this->errors[$fname] = $this->_invoke_class_validator($data);
            }
        
        } // if
        
        // Add the error we discovered to the GLOBALS array
        $GLOBALS[PXDB_GLOBAL_VAR]['validator'][$fname] =  $this->errors[$fname];
        
        return $this->errors[$fname];
    } // validate()
} // class
?>