<?php
// $Id: pxdb_widgets.class.php,v 1.19 2006/03/14 23:46:59 sandys1 Exp $

pxdb_import('content.input.pxdb_confront');

/**
* This is the class that is responsible for creating widgets for use in input forms.
*
* This class is used by the pxdb_input class.  This widgets provided in this class
* can be customized in three ways (note: analogous to pxdb_validator class). Also note:
* updateDatetime() is a JavaScript function, not a PHP function, despite what the
* documentation implies.
*
* @package pxdb.content.input
* @author Hans Lellelid <hans@velum.net>
* @version $Revision: 1.19 $
* @access public
*/
class pxdb_widgets extends pxdb_confront
{

    /**
    * Holds CSS types (e.g. style/class) and styles for various conditions.
    * @var array
    * @access public
    */
    var $css = array();


    /**
    * Widgets registered for a particular field.
    * @var array
    * @access public
    */
    var $field_widgets = array();


    /**
    * Read-only widgets registered for a particular field.
    * @var array
    * @access public
    */
    var $field_ro_widgets = array();

    /**
    * Widgets registered for a type of field.
    * @var array
    * @access public
    */
    var $type_widgets = array();

    /**
    * Read-only widgets registered for a type of field.
    * @var array
    * @access public
    */
    var $type_ro_widgets = array();

    /**
    * Hash of elementname / JavaScript code values that will be set onSubmit.
    * @var array
    * @access public
    */
    var $jscript_value_calculators = array();

    /**
    * The methods in this class (for speed only set in constructor).
    * @var array
    * @access public
    */
    var $class_methods;

    /**
    * The input class that is requesting the widget.
    * @var object pxdb_input
    * @access public
    */
    var $input;

    /**
    * Whether a popup calendar has already been included (needs to be generalized)
    * @var boolean
    * @access private
    */
    var $popcalendar_included;
    
    /**
    * Current time, using MySQL database for timing consistency
    * @var string
    * @access private
    */
    var $_current_time = null;

    /**
    * Calls parent constructor & loads parameters array.
    * @access public
    */
    function pxdb_widgets()
    {
        parent::pxdb_confront();
        $this->class_methods = get_class_methods(get_class($this));
    }

    /**
    * Attempts to return a reference to an already-instantiated widget class.
    * Will create a new widget class only if one does not already exist.
    * @return object pxdb_widget
    * @access public
    */
    function &singleton()
    {
        static $instance;

        if (!isset($instance))
        {
            $instance = new pxdb_widgets();
        }

        return $instance;
    }

    /**
    * Register a function to display a widget for a field.
    * @param string $field The field name that will use this widget.
    * @param mixed $function The function name or array of object reference and method name.
    * @param mixed $datatype The datatype this field belongs to -- in case more than one dt has defined that field.
    * @access public
    */
    function register_field_widget($field, $function, $datatype = null)
    {
        $tfid = $this->lookup_tfid($field, $datatype);
        if (!$tfid)
        {
            pxdb::trigger_error("[pxdb_widgets] Unable to register widget for non-existant field: $field.");
        }
        $this->field_widgets[$tfid] = $function;
    }

    /**
    * Register a function to display a read-only (display) widget for a field.
    * @param string $field The field name that will use this widget.
    * @param mixed $function The function name or array of object reference and method name.
    * @param mixed $datatype The datatype this field belongs to -- in case more than one dt has defined that field.
    * @access public
    */
    function register_field_ro_widget($field, $function, $datatype = null)
    {
        $tfid = $this->lookup_tfid($field, $datatype);
        if (!$tfid) { pxdb::trigger_error("[pxdb_widgets] Unable to register widget for non-existant field: $field."); }
        $this->field_ro_widgets[$tfid] = $function;
    }

    /**
    * Register a function to display a widget for a field type.
    * @param string $type The field type (e.g. 'url', 'primaryrel') that will use this widget.
    * @param mixed $function The function name or array of object reference and method name.
    * @access public
    */
    function register_type_widget($type, $function)
    {
        $this->type_widgets[$type] = $function;
    }

    /**
    * Register a function to display a widget for a field.
    * @param string $type The field type (e.g. 'url', 'primaryrel') that will use this widget.
    * @param mixed $function The function name or array of object reference and method name.
    * @access public
    */
    function register_type_ro_widget($type, $function)
    {
        $this->type_ro_widgets[$type] = $function;
    }


    /**
    * Adds a JavaScript value calculator for the specified field.
    * A widget may require that onSubmit a value for a field is calculated
    * from several different (non-pxdb?) elements, etc.  This method allows
    * a widget to specify what the value should be set to when the form is submitted.
    * @param string $field_element The name of the field that we are setting, e.g. pxdb[data][date2]
    * @param string $value The value we will set this to -- could be a javascript function, series of statements, etc.
    * @access public
    */
    function add_onsubmit_value_calculator($field_element, $value)
    {
        $this->jscript_value_calculators[$field_element] = $value;
    }

    /**
    * Removes a JavaScript value calculator for the specified field.
    * @param string $field_element The field for which we want to remove any calculator.
    * @access public
    */
    function del_onsubmit_value_calculator($field_element)
    {
        unset($this->jscript_value_calculators[$field_element]);
    }

    /**
    * Gets the array of onSubmit value calculators.
    * @return array The hash of fields / javascript code (values).
    * @access public
    */
    function get_onsubmit_value_calculators()
    {
        return $this->jscript_value_calculators;
    }

    /**
    * Gets the JavaScript for onSubmit.
    * @param string $form_name
    * @return string The javascript that can be inserted in the onSubmit = "" paramter of the form.
    * @access public
    */
    function get_onsubmit_value_calculators_jscript($form_name)
    {
        $jscript = '';
        reset($this->jscript_value_calculators);
        while (list($param, $value) = each($this->jscript_value_calculators))
        {
            $el = "document.forms['{$form_name}'].elements['$param']";
            $jscript .= "if ($el) { $el.value = $value; }  ";
        }
        reset($this->jscript_value_calculators);
        return $jscript;
    }

    /**
    * Sets the CSS to use in widgets.
    * This method can set any number of different class CSS tag variables;
    * by default only default_css, error_css, and required_css are implemented
    * by the widgets.  However, if you register your own widget functions you may
    * want to implement new CSS options in your widgets class.
    * @param string $css The style text or CSS class name.
    * @param string $type The type of tag this is -- e.g. "default" or "error" or "required".
    * @access public
    */
    function set_css($css, $type = 'default')
    {
        // set the CSS tag type
        if (preg_match('/[\s]|[:]/', $css))
        {
            $this->css[$type]['type'] = 'style';
        }
        else
        {
            $this->css[$type]['type'] = 'class';
        }

        // set the CSS style
        $this->css[$type]['style'] = $css;

        // set the CSS tag -- for speed later.
        $this->css[$type]['attribute'] = ' ' . $this->css[$type]['type'] . '="' . $this->css[$type]['style'] . '"';

    }

    /**
    * Gets the type of CSS we need for the current region.
    * Examines the data passed and returns an appropriate CSS category --
    * e.g. 'default' or 'error'.  You will want to override this method if you
    * want to create widgets that use new CSS categories.
    * @param array $data Hash of widget properties.
    * @return string The category of CSS (e.g. 'error', 'default').
    * @access private
    */
    function _get_meta_css($data)
    {
        $types = array_keys($this->css);
        if ($data['error'] && in_array('error', $types))
        {
            return 'error';
        }
        elseif ($data['required'] && in_array('required', $types))
        {
            return 'required';
        }
        else
        {
            return 'default';
        }
    }

    /**
    * Gets the CSS attribute to use for a widget.
    * Examines the data passed and returns an appropriate CSS attribute --
    * by default $default_css_attribute; if there's an error, $error_css_attribute, etc.
    * @param array $data The hash of widget properties.
    * @return string The CSS tag to use.
    * @access private
    */
    function _get_css_attribute($data)
    {
        if (isset($this->css[$this->_get_meta_css($data)]['attribute']))
        {
            return $this->css[$this->_get_meta_css($data)]['attribute'];
        }
    }

    /**
    * Gets the CSS style text or class name to use for a widget.
    * Examines the data passed and returns an appropriate CSS tag --
    * by default $default_css; if there's an error, $error_css, etc.
    * @param array $data The hash of widget properties.
    * @return string The CSS to use.
    * @access private
    */
    function _get_css_style($data)
    {
        if (isset($this->css[$this->_get_meta_css($data)]['style']))
        {
            return $this->css[$this->_get_meta_css($data)]['style'];
        }
    }

    /**
    * Gets the CSS attribute type for a widget.
    * @param array $data The hash of widget properties.
    * @return string The CSS tag type to use.
    * @access private
    */
    function _get_css_type($data)
    {
        if (isset($this->css[$this->_get_meta_css($data)]['type']))
        {
            return $this->css[$this->_get_meta_css($data)]['type'];
        }
    }

    /**
    * Generates an id attribute for a widget in a standard way
    *
    * @access private
    * @return string id attribute prepended with space
    * @param array $data Associative array of widget properties
    */
    function _get_id_attribute($data)
    {
        return ' id="' . $data['form_name'] . '_' . $data['name'] . '"';
    }

    /**
    * Cleans output for display.
    * This function strips slashes from values in a smart way and returns html entities.
    * @param string $value The value to clean.
    * @access private
    * @return string The cleaned value.
    */
    function _clean_output($value)
    {
        // 1) strip slashes if applicable
        if (get_magic_quotes_gpc())
        {
            $value = stripslashes($value);
        }

        // 2) convert special chars
        $value = htmlspecialchars($value);
        return $value;
    }

    /**
    * For use when there is no defined widget function.
    * @param array $data The hash of widget properties.
    * @access private
    * @return string input widget
    */
    function _widget($data)
    {
        return '<input type="text" name="' . $data['element_name'] . '" value="' . $this->_clean_output($data['value']) . '"' . $this->_get_id_attribute($data) . ' size="40"' . $this->_get_css_attribute($data) . ' />';
    }

    /**
    * Read-only counterpart of the generic widget function.
    * @param array $data The hash of widget properties.
    * @access private
    * @return string read only widget
    */
    function _ro_widget($data)
    {
        return '<span' . $this->_get_id_attribute($data) . '>' . $data['value'] . '</span>';
    }

    /**
    * Number widget
    * @param array $data The hash of widget properties.
    * @access private
    * @return string input widget
    */
    function _widget_num($data)
    {
        return '<input type="text" name="' . $data['element_name'] . '" value="' . $this->_clean_output($data['value']) . '"' . $this->_get_id_attribute($data) . ' size="40"' . $this->_get_css_attribute($data) . ' />';
    }

    /**
    * Read only number widget
    * @param array $data The hash of widget properties.
    * @access private
    * @return string read only widget
    */
    function _ro_widget_num($data)
    {
        return '<span' . $this->_get_id_attribute($data) . '>' . $data['value'] . '</span>';
    }

    /**
    * Text widget
    * @param array $data The hash of widget properties.
    * @access private
    * @return string input widget
    */
    function _widget_txt($data)
    {
        return '<input type="text" name="' . $data['element_name'] . '" value="' . $this->_clean_output($data['value']) . '"' . $this->_get_id_attribute($data) . ' size="40"' . $this->_get_css_attribute($data) . ' />';
    }

    /**
    * Read-only text widget
    * @param array $data The hash of widget properties.
    * @access private
    * @return string read only widget
    */
    function _ro_widget_txt($data)
    {
        return '<span' . $this->_get_id_attribute($data) . '>' . $data['value'] . '</span>';
    }

    /**
    * Amount widget
    * @param array $data The hash of widget properties.
    * @access private
    * @return string input widget
    */
    function _widget_amount($data)
    {
        return '<input type="text" name="' . $data['element_name'] . '" value="' . $this->_clean_output($data['value']) . '"' . $this->_get_id_attribute($data) . ' size="40"' . $this->_get_css_attribute($data) . ' />';
    }

    /**
    * Read-only amount widget
    * @param array $data The hash of widget properties.
    * @access private
    * @return string read only widget
    */
    function _ro_widget_amount($data)
    {
        return '<span' . $this->_get_id_attribute($data) . '>' . number_format($data['value']) . '</span>';
    }

    /**
    * Long text widget
    * @param array $data The hash of widget properties.
    * @access private
    * @return string input widget
    */
    function _widget_ltxt($data)
    {
        return '<textarea name="' . $data['element_name'] . '" wrap="soft" cols="40" rows="10"' . $this->_get_id_attribute($data) . $this->_get_css_attribute($data) . '>' . $this->_clean_output($data['value']) . '</textarea>';
    }

    /**
    * Duplicates _text() behavior from functions.lib.php in syntaxcms library
    * @param array $data The hash of widget properties.
    * @access private
    * @return string read only widget
    */
    function _ro_widget_ltxt($data)
    {
        if (!preg_match('/<br[\/]*>|<p>/i', $data['value']))
        {
            return '<div'. $this->_get_id_attribute($data) . '>' . nl2br($data['value']) . '</div>';
        }
        else
        {
            return '<span' . $this->_get_id_attribute($data) . '>' . $data['value']. '</span>';
        }
    }

    /**
    * Email widget
    * @param array $data The hash of widget properties.
    * @access private
    * @return string input widget
    */
    function _widget_email($data)
    {
        return '<input type="text" name="' . $data['element_name'] . '" value="' . $this->_clean_output($data['value']) . '"' . $this->_get_id_attribute($data) . ' size="40"' . $this->_get_css_attribute($data) . ' />';
    }

    /**
    * Read-only email widget
    * @param array $data The hash of widget properties.
    * @access private
    * @return string read only widget
    */
    function _ro_widget_email($data)
    {
        return '<span' . $this->_get_id_attribute($data) . '>' . $data['value'] . '</span>';
    }

    /**
    * Password widget
    * @param array $data The hash of widget properties.
    * @access private
    * @return string input widget
    */
    function _widget_password($data)
    {
        return '<input type="password" name="' . $data['element_name'] . '" value="' . $this->_clean_output($data['value']) . '" size="20" ' . $this->_get_id_attribute($data) . $this->_get_css_attribute($data) . ' />';
    }

    /**
    * Read-only password widget
    * @param array $data The hash of widget properties.
    * @access private
    * @return string read only widget
    */
    function _ro_widget_password($data)
    {
        return '<span' . $this->_get_id_attribute($data) . '>****</span>';
    }

    /**
    * Password confirmation widget
    * @param array $data The hash of widget properties.
    * @access private
    * @return string input widget
    */
    function _widget_pass2($data)
    {
        $verifypass = $data['name'] . '_verify';
        $verifypass_el = $this->_get_form_element_name('data', $verifypass);

        if (!$this->_imported['data'][$verifypass])
        {
            $this->_imported['data'][$verifypass] = $data['value'];
        }

        $widget = '
        <input type="password" name="' . $data['element_name'] . '" value="' . $data['value'] . '" size="20"' . $this->_get_id_attribute($data) . $this->_get_css_attribute($data) . ' /><br />
        <input type="password" name="' . $verifypass_el . '" id="' . $data['form_name'] . '_' . $verifypass . '" value="' . $this->_imported['data'][$verifypass] . '" size="20"' . $this->_get_css_attribute($data) . ' /> <label for="' . $data['form_name'] . '_' . $verifypass . '">(verify)</label>';
        return $widget;
    }

    /**
    * Read-only password confirmation widget
    * @param array $data The hash of widget properties.
    * @access private
    * @return string read only widget
    */
    function _ro_widget_pass2($data)
    {
        return $this->_ro_widget_password($data);
    }

    /**
    * Phone number widget
    * @param array $data The hash of widget properties.
    * @access private
    * @return string input widget
    */
    function _widget_phone($data)
    {
        return '<input type="text" name="' . $data['element_name'] . '" value="' . $this->_clean_output($data['value']) . '" size="20"' . $this->_get_id_attribute($data) . $this->_get_css_attribute($data) . ' />';
    }

    /**
    * Read-only phone number widget
    * @param array $data The hash of widget properties.
    * @access private
    * @return string read only widget
    */
    function _ro_widget_phone($data)
    {
        return '<span' . $this->_get_id_attribute($data) . '>' . $data['value'] . '</span>';
    }

    /**
    * Currency widget
    * @param array $data The hash of widget properties.
    * @access private
    * @return string input widget
    */
    function _widget_currency($data)
    {
        return '<input type="text" name="' . $data['element_name'] . '" value="' . $this->_clean_output($data['value']) . '" size="20"' . $this->_get_id_attribute($data) . $this->_get_css_attribute($data) . ' />';
    }

    /**
    * Read-only currency widget
    * @param array $data The hash of widget properties.
    * @access private
    * @return string read only widget
    */
    function _ro_widget_currency($data)
    {
        return '<span' . $this->_get_id_attribute($data) . '>' . $data['value'] . '</span>';
    }

    /**
    * Build the date widget
    * @param array $data The hash of widget properties.
    * @access private
    * @return string input widget
    */
    function _widget_date($data)
    {
        $timevar = 'time_'.$data['id'];
        $datevar = 'date_'.$data['id'];
        $popupvar = 'popupcalbutton_'.$data['id'];

        if (isset($this->_widget_params[$timevar]))
        {
            $timeval = $this->_widget_params[$timevar];
        }
        else
        {
            $timeval = null;
        }

        if (isset($this->_widget_params[$datevar]))
        {
            $dateval = $this->_widget_params[$datevar];
        }
        else
        {
            $dateval = null;
        }
        $time_element = $this->_get_form_element_name('widget_param', $timevar);
        $date_element = $this->_get_form_element_name('widget_param', $datevar);
        $popup_element = $this->_get_form_element_name('widget_param', $popupvar);

        // get time options
        $timeA = array();
        $todaySQL = date('Y-m-d');
        // SFS -- ...or, we could just loop from 0 to 23 and inner loop
        // 0 45 in increments of 15
        for ($h = 0; $h < 24; $h++)
        {
            for ($m = 0; $m < 60; $m += 15)
            {
                $time_string = sprintf('%02d', $h) . ':'.sprintf('%02d', $m) . ':00';
                $timeA[] = array(
                    'value' => $time_string,
                    'label' => date('g:i A', strtotime($todaySQL . ' ' . $time_string))
               );
            }
        }

        if (!$timeval)
        {
            if ($data['value'])
            {
                $timeval = date('H:i:s', strtotime($data['value']));
            }
            else
            {
                // get most recent time (15 minutes increments)
                /*
                $curr_min = date('i');
                $most_recent_min = sprintf('%02d', (int) (floor($curr_min / 15) * 15));
                $curr_hour = date('H');
                $timeval = "$curr_hour:$most_recent_min:00";
                */
                $timeval = $this->get_time_quarters();
            }
        }

        if (!$dateval)
        {
            if ($data['value'])
            {
                $dateval = date('Y-m-d', strtotime($data['value']));
            }
            else
            {
                $dateval = date('Y-m-d');
            }
        }

        // SFS give logical default value
        if (null === $data['value'])
        {
            $data['value'] = $dateval.' '.$timeval.':00';
        }

        $timeL = new ListWidget($timeA, $time_element, $timeval);

        // Get the relative URL for the Syntax site
        $pub_rel = pxdb::get_pxdb_www();

        $widget = "
<script type=\"text/javascript\">
    function updateDatetime(inForm,inDate,inTime,targField)
    {
        targField.value = inDate.value + ' ' + inTime.options[inTime.selectedIndex].value;
        return targField.value;
    }
</script>

<input type=\"hidden\" name=\"$data[element_name]\" value=\"$data[value]\" />

<input name=\"$date_element\" size=\"11\"  value=\"$dateval\" onBlur=\"updateDatetime(this.form,this.form['$date_element'], this.form['$time_element'], this.form['$data[element_name]'])\" />

" . $timeL->selectList(1, "class=\"time\" onChange=\"updateDatetime(this.form, this.form['$date_element'], this.form['$time_element'], this.form['$data[element_name]'])\"" . $this->_get_id_attribute($data))

// added by AVC, 1/14/05 - prevents duplicate popcalendar.js inclusions
. ($this->popcalendar_included ? '' : "

<script type=\"text/javascript\" src=\"$pub_rel/widgets/calendar/popcalendar.js\"></script>

")

. "<script type=\"text/javascript\">
    if (!document.layers)
    {
        document.write(\" <input type=\\\"button\\\" onclick=\\\"popUpCalendar(document.forms['$data[form_name]'].elements['$popup_element'], document.forms['$data[form_name]'].elements['$date_element'], 'yyyy-mm-dd')\\\" name=\\\"$popup_element\\\" value=\\\"select\\\" />\");
    }
//-->
</script>

<input type=\"button\" value=\"clear\" onclick=\"this.form['$data[element_name]'].value = ''; this.form['$date_element'].value = ''; this.form['$time_element'].selectedIndex = 0;\" />
        ";

        // added by AVC, 1/14/05 - prevents duplicate popcalendar.js inclusions
        $this->popcalendar_included = true;

        $this->add_onsubmit_value_calculator($data['element_name'], "this['$date_element'].value + ' ' + this['$time_element'].options[this['$time_element'].selectedIndex].value");

        return $widget;
    }

    /**
    * Read-only date widget
    * @param array $data The hash of widget properties.
    * @access private
    * @return string read only widget
    */
    function _ro_widget_date($data)
    {

        if (preg_match("/[\d]+:[\d]+:[\d]+/", $data['value']))
        {
            $format = 'M j, Y - h:i A';
        }
        else
        {
            $format = 'M j, Y';
        }

        return '<span' . $this->_get_id_attribute($data) . '>' . date($format, strtotime($data['value'])) . '</span>';

    }

    /**
    * URL widget
    * @param array $data The hash of widget properties.
    * @access private
    * @return string input widget
    */
    function _widget_url($data)
    {
        return '<input type="text" name="' . $data['element_name'] . '" value="' . $this->_clean_output($data['value']) . '" size="40"' . $this->_get_id_attribute($data) . $this->_get_css_attribute($data) . ' />';
    }

    /**
    * Read-only URL widget
    * @param array $data The hash of widget properties.
    * @access private
    * @return string read only widget
    */
    function _ro_widget_url($data)
    {
        return '<span' . $this->_get_id_attribute($data) . '>' . $data['value'] . '</span>';
    }

    /**
    * Boolean radio widget
    * @param array $data The hash of widget properties.
    * @access private
    * @return string input widget
    */
    function _widget_boolean_radio($data)
    {
        if (!$data['options'])
        {
            $data['options'] = array(array('value' => 1, 'label' => 'Yes'), array('value' => 0, 'label' => 'No'));
        }
        $widgL = new ListWidget($data['options'], $data['element_name'], $data['value']);

        $widgL->setCSS($this->_get_css_style($data));

        return $widgL->radioButtonsTable(1, '', $this->_get_id_attribute($data));
    }

    /**
    * Read-only (View) widget for boolean field
    * @param array $data The hash of widget properties.
    * @access private
    * @return string read only widget
    */
    function _ro_widget_boolean_radio($data)
    {
        if (!$options)
        {
            $options = array(array('value' => 1, 'label' => 'Yes'), array('value' => 0, 'label' => 'No'));
        }
        return '<span' . $this->_get_id_attribute($data) . '>' . $options[$data['value']] . '</span>';
    }

    /**
    * Checkbox boolean widget
    * @param array $data The hash of widget properties.
    * @access private
    * @return string input widget
    */
    function _widget_boolean_check($data)
    {
        // Important, because checkbox elements are not sent to the server when blank,
        // we have to manually specify a hidden element of the same name with the value 0.
        $widget = '
<input type="hidden" name="' . $data['element_name'] .'" value="0" />
<input type="checkbox" name="' . $data['element_name'] . '" value="1"' . ($data['value'] ? ' checked="checked"' : '') . $this->_get_id_attribute($data) . $this->_get_css_attribute($data) . ' />';
        return $widget;
    }

    /**
    * Read-only (View) widget for boolean checkbox field
    * @param array $data The hash of widget properties.
    * @access private
    * @return string read only widget
    */
    function _ro_widget_boolean_check($data)
    {
        return '<span' . $this->_get_id_attribute($data) . '>' . ($data['value'] ? 'Y' : 'N') . '</span>';
    }

    /**
    * Generic boolean widget
    * @param array $data The hash of widget properties.
    * @access private
    * @return string input widget
    */
    function _widget_boolean($data)
    {
        // defaults to a checkbox
        return $this->_widget_boolean_check($data);
    }

    /**
    * Read-only (View) widget for boolean field
    * @param array $data The hash of widget properties.
    * @access private
    * @return string read only widget
    */
    function _ro_widget_boolean($data)
    {
        // defaults to a checkbox
        return $this->_ro_widget_boolean_check($data);
    }

    /**
    * Primaryrel widget
    * @param array $data The hash of widget properties.
    * @access private
    * @return string input widget
    */
    function _widget_primaryrel($data)
    {
        if (!$data['multiple'])
        {
            $widgL = new ListWidget($data['options'], $data['element_name'], $data['value']);
            $widgL->setDefaultElement(array('value' => '', 'label' => '[choose]'));
            $widgL->setCSS($this->_get_css_style($data));
            return $widgL->selectList(0, $this->_get_id_attribute($data));
        }
        else
        {
            // multiple

            // complex widget similar to old showform.cgi widget
            $id = $data['id']; // current id -- for use in varnames

            // program logic must be emboded in this widget
            switch($this->_widget_params["mode_$id"])
            {
                case 'add':

                    if (!$data['value']) $data['value'] = array();
                    if (!in_array($this->_widget_params["pick_$id"], $data['value']))
                    {
                        $data['value'][] = $this->_widget_params["pick_$id"];
                    }

                break;

                case 'del':

                    $data['value'] = array_diff($data['value'], array($this->_widget_params["delid_$id"]));

                break;
            }

            $widgL = new ListWidget($data['options'], $this->_get_form_element_name('widget_param',"pick_$id"), $this->_widget_params["pick_$id"]);
            $widgL->setDefaultElement(array('value' => '', 'label' => '[choose]'));
            $widgL->setCSS($this->_get_css_style($data));

            $formtxt = '
<input type="hidden" name="' . $this->_get_form_element_name('widget_param',"mode_$id") . '" value="" />
<input type="hidden" name="' . $this->_get_form_element_name('widget_param',"delid_$id") . '" value="" />
<table>
    <tr>
        <td>
            ' . $widgL->selectList(0, $this->_get_id_attribute($data)) . '
        </td>
        <td><input type="button" name="' . $this->_get_form_element_name('widget_param',"addbtn_$id") . "\" value=\"Add\" onclick=\"this.form['" . $this->_get_form_element_name('widget_param',"mode_$id") . "'].value = 'add'; this.form.submit()\"" . $this->_get_css_attribute($data) . ' /></td>
    </tr>
            ';

            if (!$data['value'])
            {
                $formtxt .= '
    <tr>
        <td colspan="2" align="center">
            <input type="hidden" name="' .$data['element_name'] . '" value="" />
            (Choose items to relate from list above.)
        </td>
    </tr>
                ';
            }
            else
            {
                foreach ($data['value'] as $val)
                {
                    if ($val)
                    {
                        $sql = "SELECT name FROM records WHERE id = $val";
                        $rs =& $this->db->Execute($sql);
                        $formtxt .= '
    <tr>
        <td>
            ' . $rs->fields['name'] . '
            <input type="hidden" name="' . $data['element_name'] . '[]" value="' . $val . "\" />
        </td>
        <td>
            <input type=\"button\" name=\"delbtn\" value=\"del\" onclick=\"this.form['" . $this->_get_form_element_name('widget_param', "mode_$id") . "'].value = 'del'; this.form['" . $this->_get_form_element_name('widget_param', "delid_$id") . "'].value = '$val'; this.form.submit();\" />
        </td>
    </tr>
                        ";
                    }
                }
            }

            $formtxt .= '
</table>
            ';

            return $formtxt;

        }    // if multiple
    }

    /**
    * Read-only (View) widget for primaryrel field
    * @param array $data The hash of widget properties.
    * @access private
    * @return string read only widget
    */
    function _ro_widget_primaryrel($data)
    {
        // lookup all values for 'name' in records based on
        // ids in 'value'

        if (!is_array($data['value']))
        {
            if (! $data['value']) return; // SFS: this prevents a fatal error
            $data['value'] = array($data['value']);
        }

        foreach ($data['value'] as $id)
        {
            // lookup the name for that id
            $sql = "SELECT name FROM records WHERE id = $id";
            $prel_rs = $this->db->Execute($sql);
            while (!$prel_rs->EOF)
            {
                $labels[] = $prel_rs->fields['name'];
                $prel_rs->MoveNext();
            }
        }

        return '<span' . $this->_get_id_attribute($data) . '>' . implode('<br />', $labels) . '</span>';

    }

    /**
    * Pick ID widget
    * @param array $data The hash of widget properties.
    * @access private
    * @return string input widget
    */
    function _widget_pickid($data)
    {
        $widgL = new ListWidget($data['options'], $data['element_name'], $data['value']);
        $widgL->setCSS($this->_get_css_style($data));

        if ($data['multiple'])
        {
            return $widgL->selectList(5);
        }
        else
        {
            $widgL->setDefaultElement(array('value' => '', 'label' => '[choose]'));
            return $widgL->selectList(0, $this->_get_id_attribute($data));
        }
    }

    /**
    * Read-only (View) widget for pick ID field
    * @param array $data The hash of widget properties.
    * @access private
    * @return string read only widget
    */
    function _ro_widget_pickid($data)
    {
        // lookup the labels for the values in $data
        if (!is_array($data['value']))
        {
            $data['value'] = array($data['value']);
        }

        $labels = array();

        // get the pick table name
        $objf = new pxdb_typesfield($data['id']);
        $dbf = new pxdb_dbfield($objf->get_fieldid());
        $pick_idfield = ($dbf->get_pick_column() ? $dbf->get_pick_column() : "id");
        $pick_table = $objf->replace_tags($dbf->get_pick_table());

        foreach ($data['value'] as $id)
        {
            // lookup the name for that id
            $sql = "SELECT name FROM $pick_table pick WHERE pick.$pick_idfield = $id";
            $pick_rs = $this->db->Execute($sql);

            while (!$pick_rs->EOF)
            {
                $labels[] = $pick_rs->fields['name'];
            }
        }

        return '<span' . $this->_get_id_attribute($data) . '>' . implode('<br />', $labels) . '</span>';
    }

    /**
    * Pick widget
    * @param array $data The hash of widget properties.
    * @access private
    * @return string input widget
    */
    function _widget_pick($data)
    {
        $widgL = new ListWidget($data['options'], $data['element_name'], $data['value']);
        $widgL->setDefaultElement(array('value' => '', 'label' => '[choose]'));
        $widgL->setCSS($this->_get_css_style($data));
        return $widgL->selectList(0, $this->_get_id_attribute($data));
    }

    /**
    * Read-only (View) widget for pick field
    * @param array $data The hash of widget properties.
    * @access private
    * @return string read only widget
    */
    function _ro_widget_pick($data)
    {
        // get the pick table name
        $objf = new pxdb_typesfield($data['id']);
        $dbf = new pxdb_dbfield($objf->get_fieldid());
        $pick_column = ($dbf->get_pick_column() ? $dbf->get_pick_column() : "name");
        $pick_table = $objf->replace_tags($dbf->get_pick_table());
        $sql = "SELECT name FROM $pick_table pick WHERE pick.$pick_column = ".$this->db->qstr($data['value']);
        $pick_rs = $this->db->Execute($sql);
        $label = $pick_rs->fields['name'];
        return '<span' . $this->_get_id_attribute($data) . '>' . $label . '</span>';
    }

    /**
    * Upload widget
    * @param array $data The hash of widget properties.
    * @access private
    * @return string input widget
    */
    function _widget_upload($data)
    {
        // conceivably value of a widget object could also be
        // a URL -- in which case we don't want to pass the hidden
        // fields.
        $_var_remove = 'remove_'.$data['id'];

        $widget = '<span ' . $this->_get_css_attribute($data) . '>';
        $widget .= '<input type="hidden" name="' . $this->_get_form_element_name('widget_param',$_var_remove) . "\" value=\"0\" />\n";

        if (!empty($this->_imported['widget'][$_var_remove]) && empty($this->_widget_params[$_var_remove]))
        {
            // OAM: convoluted import function failed for some reason
            $this->_widget_params = $this->_imported['widget'];
        }

        if (is_array($data['value']) && (!isset($this->_widget_params[$_var_remove]) || !$this->_widget_params[$_var_remove]))
        {
            // we only pass fields if we are uploading a new file -- which means that tmp is set.
            if (isset($data['value']['tmp']))
            {
                $widget .= '<input type="hidden" name="' . $data['element_name'] . '[type]" value="' . $data['value']['type'] . "\" />\n";
                $widget .= '<input type="hidden" name="' . $data['element_name'] . '[name]" value="' . $data['value']['name'] . "\" />\n";
                $widget .= '<input type="hidden" name="' . $data['element_name'] . "[size]\" value=\"" . $data['value']['size'] . "\" />";
                $widget .= '<input type="hidden" name="' . $data['element_name'] . "[tmp]\" value=\"" . $data['value']['tmp'] . "\" />\n";
            }

            // that means that we are mid-upload
            $widget .= 'Current file: <strong>' . $data['value']['name'] . '</strong> [<a href="javascript: document.' . $data['form_name'] . "['" . $this->_get_form_element_name('widget_param', $_var_remove) . "'].value = 1; document." . $data['form_name'] . '.submit();">remove</a>]<br />';
            $widget .= 'Choose a new file to upload.<br />';

        }
        else
        {
            $widget .= 'Choose a file to upload.<br />';
        }

        if (isset($this->_widget_params[$_var_remove]) && $this->_widget_params[$_var_remove])
        {
            $widget .= '<input type="hidden" name="' . $data['element_name']."\" value=\"\" />\n";
        }

        $widget .= "<input type=\"file\" name=\"" . $this->_get_form_element_name('data', 'new_' . $data['name']) . '" value="" size="40"' . $this->_get_id_attribute($data) . $this->_get_css_attribute($data) . ' />';
        $widget .= '</span>';

        // oam: need to tell pxdb_commit to expects a file upload
        if (true == $data['required'] && empty($data['value']['name']))
        {
            $widget .= "\n" . '<input type="hidden" name="pxdb[reqd_files][new_'.$data['name'].']" value="1" />';
        }

        return $widget;
    }

    /**
    * Read-only (View) widget for upload field
    * @param array $data The hash of widget properties.
    * @access private
    * @return string read only widget
    */
    function _ro_widget_upload($data)
    {
        return '<span' . $this->_get_id_attribute($data) . '>' . $data['value'] . '</span>';
    }

    /**
    * Determine whether a plugin exists for widget.
    * Upload widget
    * @param array $data The hash of widget properties.
    * @access private
    * @return boolean
    */
    function _widget_plugin_exists($data)
    {
        $fname = $data['type'] . ($data['priv']['write'] == false ? '_ro' : '') . '.php';
        return (pxdb::plugin_exists('widgets', $fname));
    }

    /**
    * Includes a widget from a (php) file.
    * If no widget is found at the specified path, the appropriate class method will be called instead.
    * You PHP script will be included inside a function & therefore will have only local scope.  Of course
    * you can use references to methods of this object with $this->methodname() calls, however, some variables
    * are copied so that you should not need to do this.  In particular your PHP script may use the following variables
    *
    *    $data        -- the Hash of widget properties (e.g. 'name', 'label', 'error', etc.)
    *     $params        -- params passed to widgets, from pxdb[widgets] array
    *    $db            -- the ADOdb connection
    *    $css_tag     -- the CSS tag (i.e. "style = 'color: red'" or "class = 'widgetclass'") for this widget
    *    $css_style    -- the CSS style text or class name for this widget
    *    $css_type    -- the CSS tag type (i.e. "style" or "class") for this widget
    *
    * In addition it is expected that your code will set a variable named $widget with the actual HTML
    * that is to be displayed.
    * @param string $plugin_data The name of widget plugin to include AND params.
    * @param array $data Hash of widget properties.
    * @return string The widget returned by included file.
    * @access private
    */
    function _include_widget($plugin_data, &$data)
    {
        $plugin_info = pxdb::parse_plugin($plugin_data);
        $plugin = $plugin_info['script'];
        $params = $plugin_info['params'];

        // build the path -- based on type of widget
        $wpath = PXDB_PLUGPATH . '/widgets/' . $plugin . ($data['priv']['write'] == false ? '_ro' : '') . '.php';

        if (file_exists($wpath))
        {
            extract($data);

            // simplifies testing of widget components if we don't have to build widgets with object referneces
            $db =& $this->db;
            $css_tag = $this->_get_css_attribute($data);
            $css_style = $this->_get_css_style($data);
            $css_type = $this->_get_css_type($data);

            include($wpath);

            return $widget; // this value must be set by the included block of code. (or else the included block must call "return" itself)
        }
        else
        {
            trigger_error("Widget not found at specified path: $wpath", E_USER_WARNING);
            return $this->_invoke_class_method($data);
        }
    }

    /**
    * Invokes the class method for specified widget.
    * @param array $data Hash of widget properties.
    * @return string The widget returned by class method.
    * @access private
    */
    function _invoke_class_method($data)
    {
        if ($data['priv']['write'] == false)
        {
            $pfx = '_ro';
        }
        else
        {
            $pfx = '';
        }

        $method = $pfx.'_widget_'.$data['type'];

        if (!in_array($method, $this->class_methods))
        {
            trigger_error("pxdb_widgets is missing method: $method()", E_USER_WARNING);
            $method = $pfx.'_widget';
        }

        return $this->$method($data);
    }

    /**
    * The broker function that fetches a widget.
    * This function first checks for a registered *field* function, then a *type* function, then
    * it looks for a widget to include, and if all else fails then the class method is invoked.
    * @param $data Hash of widget properties.
    * @param object pxdb_input $input The input object (useful for getting the form name, etc.)
    * @return string The built widget (i.e. HTML)
    * @access public
    */
    function get_widget(&$data, &$input)
    {
        // alias the widget we just grabbed
        $this->input =& $input;

        // perform any modifications to $data

        // build element name once
        $data['element_name'] = $this->_get_form_element_name('data', $data['name']);

        // get the form name
        $data['form_name'] = $this->input->get_form_name();

        if (false == $data['priv']['write'])
        {
            $pfx = '_ro';
        }
        else
        {
            $pfx = '';
        }

        $fieldarr = 'field' . $pfx . '_widgets';
        $typearr  = 'type' . $pfx . '_widgets';

        // must alias the arrays -- PHP won't recognize the syntax otherwise
        $field_methods =& $this->$fieldarr;
        $type_methods =& $this->$typearr;

        if (isset($field_methods[$data['id']]))
        {
            // a) check if there is a field widget registered
            return $this->_invoke_user_method($field_methods[$data['id']], $data, $this);
        }
        elseif ($data['widget'] && pxdb::plugin_exists('widgets', $data['widget']))
        {
            // b) check to see if a widget is defined in typesfields table
            return $this->_include_widget($data['widget'], $data);
        }
        elseif (isset($type_methods[$data['type']]))
        {
            // c) check if there is a type widget registered
            return $this->_invoke_user_method($type_methods[$data['type']], $data, $this);
        }
        else
        {
            // d) use local class method
            return $this->_invoke_class_method($data);
        }
    } // get_widget()
    
    /**
    * Gets the time in 15 minute increments, to take advantage of MySQL's query caching
    *
    * @access public				
    * @return string ISO-formatted hour and minute to the previous 15 minute increment
    */
    function get_time_quarters()
    {
        if (null === $this->_current_time)
        {     
            $dbh = &pxdb::dbh();
            $sql = "SELECT DATE_FORMAT(NOW(), '%H!%i') as time_now;";
            $q = $dbh->execute( $sql );
            $time = $q->getAll();
            
            $time_parts = explode('!', $time[0]['time_now']);
            $nowToHours = $time_parts[0].':';
            $mins = $time_parts[1];
            
            if ( $mins <15 )
            {
                $nowToHours .= '00:00';
            }
            else if ( $mins < 30 )
            {
                $nowToHours .= '15:00';
            }
            else if ( $mins < 45 )
            {
                $nowToHours .= '30:00';
            }
            else
            {
                $nowToHours .= '45:00';
            } // end if
            $this->_current_time = $nowToHours;
        }
        return $this->_current_time;
    }
}
?>