<?php
// $Id: pxdb_collection.class.php,v 1.14 2006/01/05 17:17:47 omerida Exp $

pxdb_import('content.pxdb_data');
pxdb_import('content.output.pxdb_record');
pxdb_import('metadata.pxdb_view');
pxdb_import('metadata.pxdb_typesfield');
pxdb_import('metadata.pxdb_dbfield');

/**
* A top-level class for accessing a collection of content in a Syntax instance.
* This class includes some methods that are just wrappers for ADOdb methods -- e.g. execute_page() is essentially
* a wrapper for PageExecute().  This will, however, provide needed flexibility for extending the functionality
* of these ADOdb methods and for exchanging ADOdb with another solution (PEAR) should it become necessary.
* @package pxdb.content.output
* @version $Revision: 1.14 $
* @author Hans Lellelid <hans@velum.net>
* @access public
*/
class pxdb_collection extends pxdb_data
{

    /**
    * The record set of current fetch.
    * @var object ADORecordSet
    * @access public
    */
    var $rs=null;

    /**
    * The SQL that generated the current record set.
    * @var string
    * @access public
    */
    var $sql=null;

    /**
    * Stores order by for SQL.
    * @var array
    */
    var $orderby;

    /**
    * Used for order by on pickid columns.
    * @var array
    */
    var $pick_joins;

    /**
    * Used for order by on weight table.
    * @var array
    */
    var $weight_joins;

    /**
    * Whether data has been filtered by user privileges.
    * @var boolean
    * @access public
    */
    var $priv_filtered = false;

    /**
    * Boolean used to locally override global priv filtering
    *
    */
    var $_public_privs;

    /**
    * Can set the group/user ids to priv filter for here
    */
    var $_priv_ugids = null;

    /**
    * Tracks if find method has been called before execute
    *
    * @var boolean
    * @access private
    */
    var $_find_called = false;
    
    /**
    * @param mixed $dt Current datatype(s): Array, list, integer.
    * @access public
    */
    function pxdb_collection($dt=null)
    {
        parent::pxdb_data();

        $this->pick_joins   = array();
        $this->weight_joins = array();
        $this->orderby = array();

        $this->set_datatype($dt);

        // generate default SQL for returning all with no filtering
        if (null !== $dt && '' !== $dt) 
        {
            $this->sql = $this->_object_sql();
        }
    }

    /**
    * Get the current class SQL statement.
    * This is useful if you want to you your own DB accessing tools to fetch content
    * or if you want to further filter the SQL produced by a collection. (this allows you
    * to get around issues of needing to combine search-type and filter-type pxdb_search classes.
    *
    * @return string The class SQL statement.
    * @access public
    */
    function get_sql()
    {
        return $this->sql;
    }

    /**
    * Checks to see if bound auth object is authorized to access (default = 'read') this record.
    *
    * @param integer $record_id the ID of the record we are checking.
    * @param string $priv The priv we are looking for.
    * @return boolean
    * @access public
    */
    function is_authorized($record_id, $priv='read')
    {
        if ($this->auth) 
        {
            if ($this->auth->is_admin()) 
            {
                return true;
            }
            else
            {
                return pxdb_privs::is_readable('record', $record_id, $this->auth->get_sql_ugids());
            }
        }
        else
        {
            return false;         // default to FALSE
        }
    }

    /**
    * Gets the next record in result set ($rs).
    * @param string $classname The name of the class to return as a record. (default is pxdb_record)
    * @return object The next record in result set.
    * @access public
    */
    function &fetch_record($classname=null)
    {
        $record = null;

        $keep_going = true;

        if ($this->rs == null) 
        {
            $this->execute();
        }

        while(!$this->rs->EOF && $keep_going) 
        {
            // if authentication object is bound... && query has not already been filtered
            // for privs (e.g. if the query was built using a pxdb_search object, then privs
            // have already been taken care of).

            if (true == $this->_public_privs && $this->auth && !$this->priv_filtered) 
            {
                // ...and authorized to view this record?
                if ($this->is_authorized($this->rs->fields['id'])) 
                {
                    $authorized = true;
                }
                else
                {
                    $authorized = false;
                }
            }
            else
            {
                $authorized = true;
            }

            if ($authorized) 
            {
                if (!$classname) 
                {
                    $classname = pxdb::get_class('pxdb_record', $this->rs->fields['datatype']);
                }
                $record = new $classname($this->rs->fields);
                $record->show_unapproved = $this->show_unapproved; //for methods like get_related, which perform queries
                $keep_going = false; // success: don't iterate again
            }
            else
            {
                trigger_error("You do not have permissions to read record ".$this->rs->fields['id'], E_USER_WARNING);
                $keep_going = true; // failed: iterate again
            }
            $this->rs->MoveNext();
        }
        return $record;
    }

    /**
    * Executes passed SQL or SQL derived from passed pxdb_search object.
    * If SQL is passed just sets the class var so that calls to find_record() will
    * be able to execute the query.
    * @param mixed $data The SQL statement or object search on.
    * @return string The SQL that will be used in the query.
    * @access public
    */
    function &find($data=null)
    {
        // reset priv_filtered var
        $this->priv_filtered = false;

        if (is_object($data)) 
        {
            // assume it's a pxdb_search object
            // OAM: added test for pxdb_search class
            if ('pxdb_search' == strtolower(get_class($data)) || 'pxdb_filter' == strtolower(get_class($data)))
            {
                $this->sql = $this->build_sql($data);
            }
            else
            {
                trigger_error('pxdb_collection::find, object passed but is not pxdb_search or pxdb_filter object', E_USER_ERROR);
            }
        }
        elseif ($data != null) 
        {
            // assume they passed a query
            // we just need to make sure that results are viewable
            $this->sql = $this->priv_filter_sql($data);
        }
        else
        {
            // no filter or SQL has been provided.  That means we should build SQL
            // for current datatype -- if only this collection is for only one datatype
            // OR -- build a minimal Syntax SQL statement if none or more than one datatype
            // is being used.

            if (sizeof($this->get_datatype_array()) == 1) 
            {
                // get sql for the object
                
                $this->sql = $this->_object_sql();
            }
            else
            {
                
                $this->sql = $this->_minimal_sql();
            }
        }
        
        $this->_find_called = TRUE;
        return $this->sql;
    }

    /**
    * Wrapper for ADOdb Execute
    * @return boolean Status of query execution
    * @access public
    */
    function &execute()
    {
        if ( false == $this->_find_called )
        {
            $this->find();
        }
        
        if (!$this->sql)
        {
            $this->find();
        }
        $this->rs = $this->db->Execute($this->sql); // not by reference!
        return $this->rs;
    }

    /**
    * Wrapper for ADOdb SelectLimit.
    * Sets the class $rs attribute.
    * @param integer $limit
    * @param integer $offset
    * @return mixed ADOdb result set or false on error.
    * @access public
    */
    function &execute_limit($limit=-1,$offset=-1)
    {
        if ( false == $this->_find_called )
        {
            $this->find();
        }
        
        if (empty($this->sql))
        {
            trigger_error('No SQL defined for execute_limit()', E_USER_WARNING); 
            return false; 
        }
        $this->rs = $this->db->SelectLimit($this->sql, $limit, $offset); // not by reference!
        return $this->rs;
    }

    /**
    * Wrapper for ADOdb ExecutePage
    * @param integer $nrows The number of rows per page.
    * @param integer $page The page to show.
    * @return mixed ADOdb result set or false on error.
    * @access public
    */
    function &raw_execute_page($nrows, $page)
    {
        if ( false == $this->_find_called )
        {
            $this->find();
        }
        
        if (empty($this->sql))
        {
            trigger_error('No SQL defined for execute_page()', E_USER_WARNING);
            return false; 
        }
        
        // oam added check to see if page=0
        if (0 == $page)
        {
            $page = 1;
        }
        
        $this->rs = $this->db->PageExecute($this->sql, $nrows, $page, $cache_ttl); // not by reference!
        
        return $this->rs;
    }
    
    /**
    * Rewrite ExecutePage to use a limit and avoid the performance penalty of 
    * using the same query twice.
    *
    * @param integer $nrows The number of rows per page.
    * @param integer $page The page to show.
    * @return mixed ADOdb result set or false on error.
    * @access public
    */
    function &execute_page($nrows, $page)
    {

        if ( false == $this->_find_called )
        {
            $this->find();
        }
        
        if (empty($this->sql))
        {
            trigger_error('No SQL defined for execute_page()', E_USER_WARNING);
            return false; 
        }

        // oam added check to see if page=0 or empty or null somehow
        if (0 == $page)
        {
            $page = 1;
        }

        $total_records = $this->fetchCount();
        $offset = ($page - 1) * $nrows;
        
        if ($this->rs = $this->db->SelectLimit($this->sql, $nrows, $offset))
        {
            $isFirstPage = (1 == $page);
            $lastPage = ceil($total_records / $nrows);
            $isLastPage = ($lastPage == $page);            

            // oam set some adodb resultset parameters that paginate and other
            // code may expect
            $this->rs->_maxRecordCount = $total_records;
            $this->rs->rowsPerPage = $nrows;
            $this->rs->AbsolutePage($page);
            $this->rs->AtFirstPage($isFirstPage);
            $this->rs->AtLastPage($isLastPage);
        }

        //$this->rs = $this->db->PageExecute($this->sql, $nrows, $page, $cache_ttl); // not by reference!
        
        return $this->rs;
    }

    /**
    * Wrapper for ADOdb recordset RecordCount().
    * Note that if you are doing a paged-query (execute_page()) this will only return
    * the number of records on current page (current recordset).  To find the total number
    * of records use the total_num_records() method.
    * @since 0.0.2
    * @return integer Number of records in current recordset.
    * @access public
    */
    function num_records()
    {
        if (!$this->rs) 
        {
            $this->execute();
        }
        return $this->rs->RecordCount();
    }

    /**
    * Get total number of records (if using paged queries).
    * Currently ADOdb does not have good support for getting total number of records when using PageExecute().
    * @since 0.0.2
    * @return integer Total number of records found.
    * @access public
    */
    function total_num_records()
    {
        if (!$this->rs) 
        {
            $this->execute();
        }
        // oam, added the following to get something back.  the way adodb pagination works (see adodb.inc.php)
        // it looks like the _maxRecordCount is actually returned as a property of the recordset now.
        if (!empty ($this->rs->_maxRecordCount))
        {
            return $this->rs->_maxRecordCount;
        }
        else if ( isset( $this->rs->connection->_maxRecordCount ) )
        {
            return $this->rs->connection->_maxRecordCount;
        }
        else if ( !empty($this->rs->_numOfRows) )
        {
            return $this->rs->_numOfRows;
        }
        else
        {
            // must be empty
            return 0;
        }
    }

    /**
    * EXPERIMENTAL, UNIMPLEMENTED Specifies that the result set is a weighted child-datatype list.
    * Child datatypes are datatypes that have specified parent relids (a relationship relating child
    * datatype to the parent datatype(s)).  It is assumed that if you specify that this result set
    * is a weighted_childtype, that you have also applied a filter restricting the results to the children
    * of one particular record.  If you don't restrict your results, then the order/placement of results
    * will be incorrect. EXPERIMENTAL, UNIMPLIMENTED
    * @param integer $relid The RELID relating the children
    * @access public
    */
    function weighted_childtype($relid)
    {
        // array because in future may need to support multiple weight tables / joins,
        // though weight tables would then need unique aliases.
        $this->weight_joins[] = "LEFT JOIN weights ON weights.relid = $relid AND weights.recid = 
        {records_alias}.id";
        $this->orderby = array_merge(array('weights.weight'), $this->orderby);
    }

    /**
    * Sets the order without doing any checking on column type, etc.
    * @param mixed $orderby ORDER BY string or array of strings
    * @access public
    */
    function set_order_strict($orderby)
    {
        // split into different components

        if (!is_array($orderby))
        {
            $orderby = explode(',' , $orderby);
        }
        $this->orderby = $orderby;
    }

    /**
    * Sets the order for the "ORDER BY" clause.
    * Designed to be used if this class is building your whole SQL statement for you. Setting order
    * can actually be a complex task, since any lookups on pickids are performed -- and tables
    * joined (LEFT) if necessary.
    * @param mixed $orderby ORDER BY string or array of strings.
    * @param array $join_cols Hash of [field names] => [related col to orderby]
    * @access public
    */
    function set_order($orderby, $join_cols=null)
    {
        // split into different components
        if (false === is_array($orderby))
        {        
            $orderby = explode(',' , $orderby);
        }

        if (!$join_cols) 
        {
            $join_cols = array(); 
        }

        for ($i = 0, $z = sizeof($orderby); $i < $z; $i++) 
        {
            // orderby pieces
            $oby = $orderby[$i];
            $oby = preg_replace("/[^\.]+\./", "", $oby);
            $oby = trim($oby);
            $oby_pieces = explode(" ", $oby);

            // if it's a pickid then we need to left join in order for the orderby to work
            $tfid = $this->lookup_tfid($oby_pieces[0]);
            $objf = &new pxdb_typesfield($tfid);

            if ($objf->get_field_type() == 'relationship') 
            {
                trigger_error("Cannot set ORDER BY to a relationship field.", E_USER_WARNING);
                return false;
            }
            else
            {
                $dbf = &new pxdb_dbfield($objf->get_fieldid());
                if ($dbf->get_field_type() == 'pickid') 
                {
                    $pick_alias = uniqid('oby_pick_');
                    $pick_table = $objf->replace_tags($dbf->get_pick_table());

                    if (!isset($join_cols[$oby])) 
                    {
                        $pick_orderby = pxdb::get_pick_orderby($pick_table);
                    }
                    else
                    {
                        $pick_orderby = $join_cols[$oby];
                    }

                    $oby = $pick_alias . '.' . $pick_orderby;
                    // SFS pick up ordering
                    if ($oby_pieces[1])
                    {
                        $oby .= ' ' . $oby_pieces[1];
                    }

                    if ($objf->is_multiple())
                    {
                        // join via pickrel table
                        $pick_xref_alias = $pick_alias . '_xref';
                        $pick_sql = "
LEFT JOIN
    pickrel $pick_xref_alias
        ON $pick_xref_alias.recid = {records_alias}.id
        AND $pick_xref_alias.f_id = " . $dbf->get_id() . "
LEFT JOIN
    $pick_table $pick_alias
        ON $pick_alias." . ($dbf->get_pick_column() ? $dbf->get_pick_column() : 'id') . " = $pick_xref_alias.p_id\n";
                    }
                    else
                    {
                        // join directly from records to pick table
                        $pick_sql = "
LEFT JOIN
    $pick_table $pick_alias
        ON $pick_alias.id = {records_alias}." . $dbf->get_fname() . "\n";
                    }
                    $this->pick_joins[] = $pick_sql;
                }
            }
            $orderby[$i] = $oby;
        }
        $this->orderby = $orderby;
        
        // OAM reset to make sure find is called before execute
        $this->_find_called = FALSE;
    }

    /**
    * Returns the current order set for the collection
    *
    * @access public
    * @return mixed, returns false if no order is set
    * @author Oscar Merida
    */
    function get_order()
    {
        if ( isset( $this->orderby ) && !empty( $this->orderby ) )
        {
            return $this->orderby;
        }
        else
        {
            return false;
        }
    }
    
    /**
    * Sets show_unapproved, whether results that haven't been approved should be returned.
    *
    * @access public
    * @return boolean True or false on error
    * @param boolean $set
    */
    function set_show_unapproved($set)
    {
        if (false === is_bool($set))
        {
           trigger_error('You must pass set_show_unapproved a boolean', E_USER_WARNING);
           return false;
        }
        $this->show_unapproved = $set;        
        
        //OAM: make sure to update the sql to reflect this change
        $this->find();
        return true;
    }
    

    /**
    * Filters a SQL statement to only return entries that current user has read privs for.
    * Works by inserting an INNER JOIN in the FROM clause (actually, before the WHERE clause)
    * of the passed SQL.
    * @param string $sql The SQL statement to filter.
    * @return string $sql The filtered SQL statement.
    * @access public
    */
    function priv_filter_sql($sql)
    {
        if (!preg_match("/inner join priv /i", $sql))
        {
            $records_alias = pxdb_sql::get_table_alias('records', $sql);
            $priv_sql = $this->_get_priv_sql($records_alias);
            $sql = preg_replace("/([\s]+where[\s]+)/i", " $priv_sql $1", $sql);
        }
        $this->priv_filtered = true;
        return $sql;
    }
    
    /**
    * Returns the clause needed in collection's sql to restrict by datatype.  For
    * single datatype uses =, multiple uses IN()
    *
    * @access private
    * @return string
    */
    function _get_datatype_where()
    {
        $a = $this->get_datatype_array();

        if (1 < count($a))
        {
            // use IN
            return ' IN ('.implode(',' , $a).') ';
        }
        else if (1 == count($a)) 
        {
        
            return '='.$a[0].' ';
        }
        else
        {
            trigger_error ('No datatype list returned', E_USER_WARNING);
            return false;
        }
    }
    /**
    * Builds the default SQL for getting records for this datatype.
    * Duplicates some functionality of build_sql() but is designed to be.
    * @return string The SQL for this datatype
    * @access private
    */
    function _object_sql()
    {
        // OAM: optimization, if there is only one item in the datatype list, use = instead of IN to
        // make sure that the right indexes are used        
        
        $records_alias = 'r';

        $orderby_sql  = $this->_get_oby_sql($records_alias);
        $oby_join_sql = $this->_get_oby_join_sql($records_alias);
        $priv_sql     = $this->_get_priv_sql($records_alias);

        $sql = "
SELECT DISTINCT
    " . $this->sql_aliased_columns($records_alias) . "
FROM
    records $records_alias
    $priv_sql
    $oby_join_sql
WHERE
    $records_alias.datatype" . $this->_get_datatype_where();
        
    if ( !isset($this->show_unapproved) || false == $this->show_unapproved )
    {
        $sql.=" AND $records_alias.approved = 1";
    }
    
    $sql .= "
ORDER BY
    $orderby_sql";

    ;    
        return $sql;
    }

    /**
    * Build minimal SQL.
    * This is for cases when pxdb_collection is being used for multiple (or no) datatypes.  In these
    * cases it doesn't make sense to fetch the aliased columns list, etc. -- so a very minimal SQL statement
    * is built.
    * @return string The SQL
    * @access private
    * @since 0.0.3
    */
    function _minimal_sql()
    {
        $records_alias = 'r';

        $select_sql   = $this->sql_minimal_columns($records_alias);
        $orderby_sql  = $this->_get_oby_sql($records_alias);
        $oby_join_sql = $this->_get_oby_join_sql($records_alias);
        $priv_sql     = $this->_get_priv_sql($records_alias);

        $sql = "
SELECT DISTINCT
    $select_sql
FROM
    records $records_alias
    $priv_sql
    $oby_join_sql
WHERE
    0 < 1 /* ensures valid statement to go with following AND(s) */
    " . ($this->get_datatype() ? "AND $records_alias.datatype" . $this->_get_datatype_where() : "") . "
    " . (!isset($this->show_unapproved) || $this->show_unapproved == false ? " AND $records_alias.approved = 1" : "") . "
ORDER BY
    $orderby_sql";

        return $sql;
    }

    /**
    * Builds SQL for JOINS that support the ORDER BY clause
    * @param string $records_alias The alias to the records table
    * @return string SQL for JOINS related to ORDER BY clause
    * @access private
    */
    function _get_oby_join_sql($records_alias)
    {
        $join_sql = '';

        if ($this->pick_joins)
        {
            $join_sql = pxdb_tools::populate(implode(" ", $this->pick_joins), array('records_alias' => $records_alias));
        }

        if ($this->weight_joins)
        {
            $join_sql .= pxdb_tools::populate(implode(" ", $this->weight_joins), array('records_alias' => $records_alias));
        }

        return $join_sql;

    }

    /**
    * Get SQL for ORDER BY statement.
    * @param string $records_alias The alias to the records table
    * @return string SQL for ORDER BY statemetn
    * @access private
    */
    function _get_oby_sql($records_alias)
    {
        $orderby = ($this->orderby ? $this->orderby : array("$records_alias.name"));
        $z = sizeof($orderby);
        //echo $z;
        for ($i = 0; $i < $z; $i++) 
        {
            // orderby has already been trimmed by set_order() $orderby[$i] = trim($orderby[$i]);
            //if (!preg_match("/^[^\.]+\./", $orderby[$i]))

            if (FALSE === strpos($orderby[$i], '.'))
            {
                $orderby[$i] =  $records_alias . '.' . $orderby[$i];
            }
        }
        return implode(', ', $orderby);
    }

    /**
    * Sets the collection to enforce public privs if no user is bound to it
    *
    * Defaults to false, use this method to turn it on.
    *
    * @param boolean
    */
    function use_public_privs($set=true)
    {
        $this->_public_privs = $set;
    }

    /**
    * Explicitly sets the user and group ids to use when filtering privs
    *
    * By default, if a logged in user is detected it will use all their groups, otherwise
    * the collection should restrict to the Everyone(0) group
    *
    * @param array or string comma-seperated list of user/group ids
    * @access public
    */
    function set_priv_ugids( $ugids )
    {
        if( is_array( $ugids ) )
        {
            $this->_priv_ugids = join( ',', $ugids);
        }
        else
        {
            $this->_priv_ugids = $ugids;
        }
    }
    /**
    * Gets the JOIN SQL to restrict by authenticated privs.
    * @param string $records_alias The alias to the records table
    * @return string SQL
    * @access private
    */
    function _get_priv_sql($records_alias)
    {
        $priv_sql = '';

        // OAM === because public privs should be explicitly set to false to override it, otherwise
        // a zero or null value would result true below.
        if (FALSE === $this->_public_privs) 
        {
            // override priv filtering
            return;
        }

        if( null !== $this->_priv_ugids )
        {
            $priv_alias = $records_alias . '_priv';
            $priv_sql = "INNER JOIN priv $priv_alias ON $priv_alias.recid = $records_alias.id AND $priv_alias.".PXSQL_L . pxdb_privs::_priv_col('read') . PXSQL_R." = 1 AND $priv_alias.ugid IN (".$this->_priv_ugids.")";
            $this->priv_filtered = true;
        }
        else if ($this->auth)
        {
            $priv_alias = $records_alias . '_priv';
            $priv_sql = "INNER JOIN priv $priv_alias ON $priv_alias.recid = $records_alias.id AND $priv_alias.".PXSQL_L . pxdb_privs::_priv_col('read') . PXSQL_R." = 1 AND $priv_alias.ugid IN (".$this->auth->get_sql_ugids().")";
            $this->priv_filtered = true;
        }
        else if (true === $this->_public_privs)
        {
            // oam - added a way for force a collection to check for public privs (ugid=0, 'read')
            $priv_alias = $records_alias . '_priv';
            $priv_sql = "INNER JOIN priv $priv_alias ON $priv_alias.recid = $records_alias.id AND $priv_alias.".PXSQL_L . pxdb_privs::_priv_col('read') . PXSQL_R." = 1 AND $priv_alias.ugid IN (0)";
            $this->priv_filtered = true;
        }
        // oam - alternatively you can set the following define to true and
        // not worry about doing this per collection on a script
        else if (defined('ENFORCE_PUBLIC_PRIVS') && true==ENFORCE_PUBLIC_PRIVS)
        {
            // oam - added a way for force a collection to check for public privs (ugid=0, 'read')
            // added this global constant so that you can avoid having to set this on all public collections
            // and instead have this set in a global config file such as statiuc.conf.php
            $priv_alias = $records_alias . '_priv';
            $priv_sql = "INNER JOIN priv $priv_alias ON $priv_alias.recid = $records_alias.id AND $priv_alias.".PXSQL_L . pxdb_privs::_priv_col('read') . PXSQL_R." = 1 AND $priv_alias.ugid IN (0)";
            $this->priv_filtered = true;
        }
        return $priv_sql;
    }

    /**
    * Builds a SQL statement based on a filter.
    * This method builds a complete SQL statement -- including privilege filtering and approval filtering
    * based on values in the passed filter.
    * @param object $search A pxdb_search object for which to build SQL.
    * @return string The SQL statement.
    * @access public
    */
    function build_sql($search)
    {
        // what's the alias for the records table?
        $records_alias = ($search->records_alias) ? $search->records_alias : $search->default_records_alias;

        // which columns should we select -- depends if this is for one datatype or multiple datatypes
        if (sizeof($this->get_datatype_array()) == 1) 
        {
            $select_sql = $this->sql_aliased_columns($records_alias);
        }
        else
        {
            $select_sql = $this->sql_minimal_columns($records_alias);
        }

        // get any special orderby sql
        $orderby_sql = $this->_get_oby_sql($records_alias);

        // get priv filtering SQL
        $priv_sql = $this->_get_priv_sql($records_alias);

        // JOIN sql from search class
        // oam: changed get_join_sql to return 2 parts: the join sql and any orderby clauses for it
        // to implement sorting based on noderel.weight
        $join_parts = $search->get_join_sql();
        $join_sql   = $join_parts['sql'];

        if ( isset( $join_parts['order'] ) )
        {        
            $join_order = $join_parts['order'];
        
            if (1 < count($join_order))
            {            
                $join_order = implode(',' , $join_order);
            }                 
            $join_order.= ', ';
        }
        else
        {
            $join_order = '';
        }
        
        // WHERE sql from search class
        $where_sql = $search->get_where_sql();        
        
        // special join SQL that is depended on by oby_sql above
        $oby_join_sql = $this->_get_oby_join_sql($records_alias);

        $sql = "
SELECT DISTINCT
    $select_sql
FROM
    records $records_alias
    $join_sql
    $priv_sql
    $oby_join_sql
WHERE
    $where_sql
    ".($where_sql ? "AND " : " ")."$records_alias.datatype".$this->_get_datatype_where();
    
    if ( !isset($this->show_unapproved) || true != $this->show_unapproved )
    {
        $sql.=" AND $records_alias.approved = 1";
    }
    
    $sql .= "
ORDER BY $join_order $orderby_sql";

        return $sql;
    }

    /**
    * Faster function for getting the total number of records in a collection (see also total_num_records)
    * Instead of getting the resultset and then counting the number of records, we ask mysql to do the heavy lifting.
    *
    * @private
    * Added by oscar merida
    */
    function _count_sql($search)
    {
        // what's the alias for the records table?
        $records_alias = ($search->records_alias) ? $search->records_alias : $search->default_records_alias;

        // which columns should we select -- depends if this is for one datatype or multiple datatypes
        /*
        if (sizeof($this->get_datatype_array()) == 1) 
        {
        
            $select_sql = $this->sql_aliased_columns($records_alias);
        }
        else
        {
        
            $select_sql = $this->sql_minimal_columns($records_alias);
        }
        */
        // get any special orderby sql
        //$orderby_sql = $this->_get_oby_sql($records_alias);
               
        // get priv filtering SQL
        $priv_sql = $this->_get_priv_sql($records_alias);
        
        // JOIN sql from search class
        $join_sql = $search->get_join_sql();
        // WHERE sql from search class
        $where_sql = $search->get_where_sql();

        // special join SQL that is depended on by oby_sql above
        //$oby_join_sql = $this->_get_oby_join_sql($records_alias);

        $sql = "
SELECT DISTINCT count(DISTINCT $records_alias.id) as total
FROM
    records $records_alias
    $join_sql
    $priv_sql

WHERE
    $where_sql
    " . ($where_sql ? "AND " : " ") . "$records_alias.datatype IN (" . $this->get_datatype_list() . ")
    " . (!isset($this->show_unapproved) || $this->show_unapproved == false ? " AND $records_alias.approved = 1" : "") . "
";

        return $sql;
    }

    /**
    * Prepares sql for getting a count of matching rows for a collection.
    *
    * Should really use fetchCount method.
    *
    * @access public
    */
    function findCount($search)
    {
        $countSql = $this->_count_sql($search);
        $this->find($countSql);
    }

    /**
    * Conveniece method - returns the total number of records in the collection using mysql count() function for speed.
    * instead of total_num_records
    *
    * @param  filter or sql or nothing 
    * @return integer
    * @access public
    */
    function fetchCount($search = null)
    {        
        if (null == $search) 
        {
            // if we call with no parameter, assumes that find has already been called to build the sql statement
            $sql = $this->get_sql();
            
            if (empty($sql))
            {
                trigger_error('No SQL defined yet for collection.', E_USER_ERROR);
                return false;
            }
            else
            {                
                $sql_parts1 = explode('FROM', $sql);
                // account for orderby potentially being in here
                $sql_parts2 = explode('ORDER', $sql_parts1[1]);                
                $sql = 'SELECT DISTINCT count(DISTINCT r.id) as total FROM '.$sql_parts2[0];
                $dbh =& pxdb::dbh();
                $result = $dbh->getAll($sql);
                return $result[0]['total'];                
            }
        }
        else
        {
            $this->findCount($search);
            $result = $this->execute();
            $r = $result->fetchRow();
            return $r['total'];
        }
    }
}

?>