<?php
// $Id: pxdb_search.class.php,v 1.15 2006/02/17 21:12:03 omerida Exp $

pxdb_import('content.pxdb_data');
pxdb_import('metadata.pxdb_typesfield');
pxdb_import('metadata.pxdb_dbfield');

/**
* Flexible framework for filtering SQL results based on specified criteria.
* 
* Read the documentation for instructions and examples on searching & filtering result sets. Filters
* may be used in conjunction with the pxdb_collection class.  By themselves instances of this class will
* not filter on privileges or record approval status (they will only filter on values specified through
* the add_value() or add_var() methods).
*
* @package pxdb.content.output
* @author Hans Lellelid <hans@velum.net>
* @version $Revision: 1.15 $
* @access public
*/
class pxdb_search extends pxdb_data 
{

    /**
    * Hash of fields we are looking for here.
    * @var array
    * @access public
    */
    var $fields;
    
    /**
    * The array of related records.
    * @var array
    * @access public
    */

    var $related;

    /**
    * The default top-level records table alias.
    * @var string
    * @access public
    */
    var $default_records_alias = 'r';

    /**
    * The alias for the current record table.
    * @var string
    * @access public
    */
    var $records_alias = '';
    
    /**
    * The conjunction / logical operator to use in the WHERE clause.
    * @var string
    * @access private
    */
    var $_lop = 'OR';

    /**
    * The conjunction / logical operator to use in cases where many values are passed.
    * @var string
    * @access private
    */
    var $_value_lop = 'OR';    
    
    /**
    * Initializes the filter system. Use the factory class methods instead.
    * @param mixed $dt The datatype(s) that this filter affects. (important for knowing how to look up field names to correponsing typesfields rows)
    * @access public
    */
    function pxdb_search($dt = null) 
    {
        parent::pxdb_data();

        // initialize
        $this->fields  = array();
        $this->related = array();

        // set datatype if passed
        if (null !== $dt) 
        {
            $this->set_datatype($dt); 
        }
    }
    
    /**
    * Factory method for returning a 'search' version of this class.
    * Currently identical to constructor.
	* @param mixed $dt The datatype(s) that this filter affects. (important for knowing how to look up field names to correponsing typesfields rows)
    * @return Object Instance of this class in search mode.
    * @access public
    */
    function &search($dt = null) 
    {
        $new =& new pxdb_search($dt);
        $new->set_type('search');
        return $new;
    }
    
    /**
    * Factory method for returning a 'filter' version of this class.
    * @param mixed $dt The datatype(s) that this filter affects. (important for knowing how to look up field names to correponsing typesfields rows)
    * @return object Reference to an instance of this class in filter mode.
    * @access public
    */
    function &filter($dt=null) 
    {
        $new =& new pxdb_search($dt);
        $new->set_type('filter');
        return $new;        
    }
    
    /**
    * Set the conjunction to use between multiple values specified for a given field.
    * @param string $conj The conjunction / logical operator to use. (e.g. 'AND' or 'OR')
    * @since 0.0.2
    * @access public
    */
    function set_value_conj($conj)
    {
        $this->_value_lop = $conj;
    }
    
    /**
    * Sets the conjunction to use.
    * Currently there is no difference between this function and set_type().  You can also
    * pass 'filter' or 'search' to set_type() which is what happens when this class is
    * instantiated using the pxdb_search::filter() or pxdb_search::search() methods.
    * @param string $conj The conjunction to set.
    * @return string The operator set.
    * @since 0.0.2
    * @access public
    */
    function set_conj($conj)
    {
        return $this->set_type($conj);
    }
    /**
    * Sets the type for this filter:  'search' or 'filter'.
    * This method sets the logical operator (or conjunction) that is used in the WHERE SQL.
    * @param string $type The type of filter this is.
    * @return boolean On success of setting filter.
    * @access public
    */
    function set_type($type) 
    {
        switch ($type) 
        {
            case 'search':
                $this->_lop = 'OR';
            break;
            
            case 'filter':
                $this->_lop = 'AND';
            break;
            default:
                $this->_lop = $type;
        }
        
        return $this->_lop;
    }
    
    /**
    * Based on filter values, gets the SQL that will be inserted into the *WHERE* clause of the SQL statement.
    * Uses current field values and field values of any records related to the current record.
    * @param object $obj The current pxdb_filter object -- used for recursion.
    * @param string $records_alias The alias of the current records table -- used for recursion.
    * @return string The text that should be inserted in the WHERE clause of SQL statement to activate these filters.
    * @access public
    */
    function get_where_sql($obj = null, $records_alias = null) 
    {
        // OAM HACK
        // use a global sequential number to try to make record aliases more
        // consistent from one page load to another and make the sql statement trigger the query cache
        global $table_alias_counter;

        if (empty($table_alias_counter))
        {
            $table_alias_counter = 1;
        } // end if
        
        if ($obj === null)
        {
            $obj =& $this;
        }
        
        $sql = '(';
        $a = 0;
                
        // set records alias -- defaults to $default_records_alias        
        if (null === $records_alias)
        {
            $records_alias = $obj->records_alias ? $obj->records_alias : $obj->default_records_alias; 
        }
        
        // if there are any related records, descend into and get WHERE SQL from those:
        if ($obj->related)
        {                                
            // for each related record:  (note: placement of sizeof() for speed optimzation)
            for ($i = 0, $z = sizeof($obj->related); $i < $z; $i++) 
            {                               
                $rel_obj =& $obj->related[$i]['filter'];

                if (!$rel_obj->records_alias)
                {
//                    $rel_obj->records_alias = uniqid('records_');
                    $rel_obj->records_alias = 'records_'.$table_alias_counter;
                    $table_alias_counter++;
                }
                
                // unique id
                $rel_records = $rel_obj->records_alias;                
                $rel_where_sql = $this->get_where_sql($rel_obj, $rel_records);
                
                if ($rel_where_sql) 
                {
                    $sql .= ($a++ ? "\n $this->_lop " : "") . $rel_where_sql;
                }                            
            }
        } // if        
        
        // at this point there are no more related records, so we should handle the current object's field values
        
        // Now  handle the fields
        // ----------------------
        // SFS while is faster        
        while ( list(, $field) = each($obj->fields))
        {
            $values = isset($field['value']) ? $field['value'] : pxdb::import_var($field['var']);
            
            // will be null if var is not set -- and we only filter on set vars
            if (null !== $values) 
            {
                if (!is_array($values))
                {
                    $values = array($values);
                } // normalize
                            
                $sql .= ($a++ ? "\n $this->_lop " : "");
                                
                $o = 0;
                
                if ('pickrel' == $field['type'])
                {
                    $table_alias = $field['table_alias'];
                }
                else 
                {
                    $table_alias = $records_alias;
                }
                
                $sql .= '(';                
                
                // OAM fix, if its a pickrel and the value has a comma then
                // we should allow comma-seperated lists here
                
                if ('pickrel'==$field['type'] && is_array($field['value']) && 'id'==$field['column'])
                {
                    
                    
                    $replace_map = array(
                        'table_alias' => $table_alias,
                        'column' => $field['column'],
                        'value' => join(',' , $field['value']),
                    );
                    
                    echo '<pre>'; print_r( $replace_map ); echo '</pre>';
                    $criteria = pxdb_tools::populate($field['sql'] , $replace_map);
                    $sql .= ($o++ ? ' '.$this->_value_lop.' ' : '') . $criteria;
                }
                else 
                {
                    // SFS while is faster
                    while (list (, $value) = each ($values))
                    {               
                        $replace_map['table_alias'] = $table_alias;
                        $replace_map['column'] = $field['column'];                        
                        $replace_map['value'] = $field['quote'] ? $this->db->Quote($value) : $value;
                                                                        
                        $criteria = pxdb_tools::populate($field['sql'] , $replace_map);
                        
                        $sql .= ($o++ ? ' '.$this->_value_lop.' ' : '') . $criteria;
                    } // end while
                } //end if
                
                $sql .= ')';
            }
        }        
        // SFS While is faster, but you must reset persistant arrays like object properties
        reset($obj->fields);
                
        $sql .= ')';
    
        if (!preg_match("/[^\(\)]/", $sql))
        {
            $sql = '';
        }
        
        return $sql;                
    }    
    
    /**
    * Based on filter values, gets the SQL that will be inserted into the *FROM* clause of the SQL statement.
    * This includes pickrel values (JOINing to pickrel table) and also any related recors also need to be INNER JOINed
    * to current records table.
    * @param object $obj The current pxdb_filter object -- used for recursion.
    * @param string $records_alias The alias of the current records table -- used for recursion.
    * @return array The text that should be inserted in the FROM clause of SQL statement to activate these filters as well as the related order by fields.
    * @access public
    */    
    function get_join_sql($obj = null, $records_alias = null) 
    {
        // OAM HACK
        // use a global sequential number to try to make record aliases more
        // consistent from one page load to another and make the sql statement trigger the query cache
        global $table_alias_counter;

        if (empty($table_alias_counter))
        {
            $table_alias_counter = 1;
        } // end if
                        
        if (null === $obj)
        {
            $obj =& $this;
        }

        if (null === $records_alias) 
        {
            $records_alias = $this->default_records_alias;
        }
        
        $sql = '';

        // if there are any related filters, we need to descend into them and get SQL first
        if (!empty($obj->related))
        {            
            for ($i = 0, $z = sizeof($obj->related); $i < $z; $i++) 
            {
                $rel_obj =& $obj->related[$i]['filter']; // by reference!
                                
                if (empty($rel_obj->records_alias)) 
                {
                    // $rel_obj->records_alias = uniqid('records_');
                    $rel_obj->records_alias = 'records_'.$table_alias_counter;
                    $table_alias_counter++;
                }
                
                // need a unique string of text for the table aliase
                $xref = $obj->related[$i]['reltype'] . '_' . $rel_obj->records_alias . '_xref';
                $rel_records =  $rel_obj->records_alias 
                                ? $rel_obj->records_alias 
                                : $seed . '_records';
                
                // OAM 11/8/2004: don't use shorthand for determining if the relid is an array
                // or not.  It was failing when it was an array with only one element
                if (isset($obj->related[$i]['relid']) && is_array($obj->related[$i]['relid']))
                {
                    $relid = join(',' , $obj->related[$i]['relid']);
                } 
                else if (isset($obj->related[$i]['relid']))
                {                    
                    $relid = $obj->related[$i]['relid'];
                }
                
                // OAM 4/29/2005: relid can be empty when we're dealing a noderel (parent or child type
                // relationship.
                
                if (isset( $relid ))
                {
                    $replace_map = array('records_alias' => $records_alias,
                                          'xref_alias' => $xref,
                                          'relid' => $relid,
                                          'related_alias' => $rel_records
                    );
                }
                else
                {
                    $replace_map = array('records_alias' => $records_alias,
                                          'xref_alias' => $xref,                                          
                                          'related_alias' => $rel_records
                    );
                }
                
                
                $sql .= pxdb_tools::populate($obj->related[$i]['sql'], $replace_map);
                
                if (isset($obj->related[$i]['weight_sort']))
                {
                    $orderby = pxdb_tools::populate($obj->related[$i]['weight_sort'], $replace_map);
                }
                
                // recurse to get join sql for the related filter
                $related = $obj->get_join_sql($rel_obj, $rel_records);
                
                if (!empty($related))
                {
                    $r_sql = $related ['sql'];
                    $r_order = $related ['order'];
                }
                else 
                {
                    $r_sql = null;
                    $r_order = null;
                }
                
                if (!empty($r_sql))
                {
                    $sql .= $r_sql;
                    $orderby = array_merge($orderby, $r_order);
                }          
                
            } // end for

        } // end if related
        
        // at this point there are no more related fields -- so build the SQL for joining pickrel tables
        // SFS while is faster
        while (list(, $prel) = each($obj->fields))
        {
            // we only need to add join SQL for pickrel fields
            if ('pickrel' == $prel['type']) 
            {
                
                
                $xref = $prel['table_alias'] . '_xref';
                $pick = $prel['table_alias'];                

                /*
                if ($prel['multiple']) 
                {
                    $sql .= "\n    LEFT JOIN pickrel $xref ON $xref.recid = $records_alias.id AND $xref.f_id = $prel[fid]\n";
                    $sql .= "\n    LEFT JOIN $prel[table] $pick ON $pick.$prel[id_col] = $xref.p_id\n";
                } 
                else 
                {
                    $sql .= "\n    LEFT JOIN $prel[table] $pick ON $pick.$prel[id_col] = $records_alias.$prel[fname]\n";
                }
                */
                
                // OAM - starting in syntax something other, all pick values are stored 
                // in pickrel table, regardless if the field is multiple or not
                $sql .= "\n    LEFT JOIN pickrel $xref ON $xref.recid = $records_alias.id AND $xref.f_id = $prel[fid]\n";
                $sql .= "\n    LEFT JOIN $prel[table] $pick ON $pick.$prel[id_col] = $xref.p_id\n";
                    
            } // if type = 'pickrel'                
        
        } // while $obj->fields            
        // SFS while is faster, but remember to reset persistant arrays
        reset($obj->fields);
        // oam: changed to handle sorting by weight of a field in the related table (in this case noderel.weight)   
        
        if (empty($sql) && empty($orderby))
        {
            return null;
        } 
        else 
        {
            if (isset($orderby) && is_array($orderby))
            {
                $orderby = join(', ', $orderby);
            }
            
            if (isset( $orderby))
            {
                return array('sql'=>$sql, 'order'=>$orderby);
            }
            else
            {
                return array('sql'=>$sql );
            }
        }

    }
    
    /**
    * Merges an pxdb_filter object with the current pxdb_filter object.
    * Fields, related records, pickrel items, datatypes are all merged, with values of current
    * 'name ASC' for $orderby, then new value will be 'date 1 DESC, name ASC'
    * @param object $filter The other pxdb_filter object to merge.
    * @access public
    */
    function merge(&$filter) 
    {
        // merging means adding values from $filter object into this object

        // that means adding together:
        //  -elements of related array
        //  -fields
        //  -pickid
        //  -datatype

        // Copy Related Records
        // not sure if array_merge will preserve memory location array_merge($this->related, $filter->related)

        $s = sizeof($this->related);
        $z = sizeof($filter->related) + $s;

        for ($i = $s; $i < $z; $i++)
        {
            $this->related[$i]['reltype'] =& $filter->related[$i-$s]['reltype'];
            $this->related[$i]['filter'] =& $filter->related[$i-$s]['filter'];
            $this->related[$i]['relid'] =& $filter->related[$i-$s]['relid'];
            $this->related[$i]['sql'] =& $filter->related[$i-$s]['sql'];
        }

        // Copy Fields
        $this->fields = array_merge($this->fields, $filter->fields);

        // Copy Datatypes
        $this->datatype = array_merge($this->datatype, $filter->datatype);
    }

    /**
    * Relate an pxdb_filter to the current filter via primaryrel table.
    * @param object $filter The pxdb_filter object to be related.
    * @param mixed $relid The relid(s) by which this object can be related.
    * @return boolean Returns true on success
    * @access public
    */    
    function add_related(&$filter, $relid=null) 
    {        
        if (!is_array($relid))
        {             
            // $relid = $relid !== null ? preg_split("/[,\s]/", $relid) : array();            
            if (null !== $relid)
            {
                $relid  = explode(',' , $relid);
                array_walk($relid,  'trim');                
            }
            else 
            {
                $relid = array();   
            }
                        
        }            
        
        $rel = array();
        $rel['reltype'] = 'primary';
        $rel['filter'] =& $filter;
        $rel['relid' ] = $relid;
        
        $rel['sql'] = "LEFT JOIN primaryrel {xref_alias} ON {xref_alias}.id1 = {records_alias}.id ".($relid ? "AND {xref_alias}.relid IN ({relid})" : "")."\n";
        $rel['sql'] .= "LEFT JOIN records {related_alias} ON {related_alias}.id = {xref_alias}.id2\n";
        
        $this->related[] =& $rel;
            
        return true;
    }

    /**
    * Relate an pxdb_filter to the current filter via secondaryrel table.
    * @param object $filter The pxdb_filter object to related.
    * @access public
    */        
    function add_secondaryrel_related(&$filter) 
    {
        $rel = array();
        $rel['reltype'] = 'secondary';
        $rel['filter'] =&  $filter;

        $rel['sql'] = "LEFT JOIN secondaryrel {xref_alias} ON {xref_alias}.id1 = {records_alias}.id\n";
        $rel['sql'] .= "LEFT JOIN records {related_alias} ON {related_alias}.id = {xref_alias}.id2\n";
        
        $this->related[] =& $rel;    
    
    }

    /**
    * Relate an pxdb_filter as the parent of the current object (noderel).
    *
    * OAM: extended add_parent, the first parameter can now be a filter, or a section object, section id, or a section idnum.
    * @param object $filter The pxdb_filter to related.
    * @param int $noderel_id The noderel ID to be related.
    * @param string $weight_sort String to indicate if the subsequent collection will be sorted by this noderel.weight, if empty no sorting, otherwise specify ASC or DESC.
    * @access public
    */        
    function add_parent($filter, $noderel_id = null, $weight_sort = NULL) 
    {
        $rel = array();
        $rel['reltype'] = 'parent';
        
        if (is_object($filter) && 'pxdb_search' == get_class($filter))
        {
            $rel['filter'] =& $filter;
        }
        else
        {
            // now we create an internal filter class based on this object                    
            $parent_filter =& pxdb_search::filter();    
            if (is_object($filter))
            {
                // $filter is pxdb_record or childthereof and we add ID
                $ancestors = pxdb_tools::get_ancestors($filter);                
                if (isset($ancestors['pxdb_record']))
                {
                    $parent_filter->add_value('id', $filter->get_id());
                }
                else 
                {
                    trigger_error ('Object passed to add_parent is not a pxdb_record or descendant of pxdb_record', E_USER_WARNING);
                    return false;
                }
            }
            else if (is_numeric($filter)) 
            {
                // $filter is a numeric ID
                $parent_filter->add_value('id', (int) $filter);
            }
            else if (is_string($filter)) 
            {
                // $filter is the idnum string
                $parent_filter->add_value('idnum', $filter);
            }
            else 
            {
                trigger_error('Unrecognized parameter passed as first parameter to add_parent.  Should specify one of: pxdb_search, pxdb_record, pxdb_record child, record id value, or idnum field value', E_USER_WARNING);
                return false;
            }
            $rel['filter'] = $parent_filter;
        }
        
        $rel['sql'] = " LEFT JOIN noderel {xref_alias} ON {xref_alias}.child = {records_alias}.id\n";
        
        if (is_numeric($noderel_id))
        {
            $rel['sql'] .= "    AND {xref_alias}.noderelid = '$noderel_id'";
        }
        
        $rel['sql'] .= " LEFT JOIN records {related_alias} ON {related_alias}.id = {xref_alias}.parent\n";
        
        if (null !== $weight_sort)
        {
            $weight_sort = strtoupper($weight_sort);            
            if ('ASC' !== $weight_sort && 'DESC' !== $weight_sort)
            {
                trigger_error('Add_parent sort parameter expects string "ASC" or "DESC", assuming ASC', E_USER_WARNING);
                $weight_sort = 'ASC';
            }
            $rel['weight_sort'] = " {xref_alias}.weight $weight_sort";
        }
        $this->related[] =& $rel;
    }

    /**
    * Relate an pxdb_filter as the child of the current object (noderel).
    * @param object $filter The pxdb_filter to related.
    * @param int $noderel_id The noderel ID to be related.
    * @access public
    */    
    function add_child(&$filter, $noderel_id = null) 
    {
        $rel = array();
        $rel['reltype'] = 'child';
        $rel['filter'] =& $filter;

        $rel['sql'] = "LEFT JOIN noderel {xref_alias} ON {xref_alias}.parent = {records_alias}.id\n";
        if (is_numeric($noderel_id)) $rel['sql'] .= "    AND {xref_alias}.noderelid = '$noderel_id'";
        $rel['sql'] .= "LEFT JOIN records {related_alias} ON {related_alias}.id = {xref_alias}.child\n";
        
        $this->related[] =& $rel;    
        
    }    
    
    /**
    * Adds a filter value.
    * Note: filter values are *not* able to be passed on.  Also, if you add a filter
    * value using this method, is_filtered() will still report FALSE if no vars are defined.
    * @param string $fname The field name to filter on.  (May also be a typesfields displayname)
    * @param string $value The value that the given field should match.
    * @param string $relop The way that the field and value are related. (i.e. field='value' or field LIKE '%value%', etc.)
    * @param string $fieldopt An option field name to lookup up on if getting a pickid value.
    * @param boolean $quote whether or not to quote value
    * @return boolean (documented in _add_field)
    * @access public
    */
    function add_value($fname, $value, $relop='=', $fieldopt=null, $quote=true)
    {
        // oam enhancements: if you pass add_var an array of values,
        // automatically use IN
        if ('=' == $relop && is_array($value))
        {            
            return $this->_add_field($fname, join(',' , $value), 'IN', 'value', 'id', false);
        } 
        else if ('!=' == $relop && is_array($value)) 
        {            
           
            return $this->_add_field($fname, join(',' , $value), 'NOT IN', 'value', 'id', false);        
        } 
        else 
        {            
            $quote = (TRUE ==$quote) ? 1 : 0; 
            return $this->_add_field($fname, $value, $relop, 'value', $fieldopt, $quote);
        }   
    }
    
    /**
    * Adds a field based on a varname.
    * This is generally the preferred method for adding filters.  Why?  Because when you 
    * add a filter based on a varname (instead of a value) you can use the pass() method to 
    * pass the filters (the variables) to another page.  Also, the is_filtered() will check
    * to see if any of the filter vars are set. (is_filtered() will return false when only
    * value filters are set.)
    * @param string $fname The field name to filter on.  (May also be a typesfields displayname)
    * @param string $varname The variable name that holds the value for the given field.
    * @param string $relop The way that the field and value are related. (i.e. field='value' or field LIKE '%value%', etc.)
    * @param string $fieldopt An option field name to lookup up on if getting a pickid value.
    * @param boolean $quote whether or not to quote value
    * @return boolean (documented in _add_field)
    * @access public
    */
    function add_var($fname, $varname, $relop='=', $fieldopt=null, $quote=true) 
    {
        return $this->_add_field($fname, $varname, $relop, 'var', $fieldopt, $quote);
    }
    
    /**
    * Get the SQL snippet for a given rel operator.
    * This may need to be moved into a RDBMS-specific library.
    * @param string $relop The relop for which to return SQL.
    * @return string The SQL snippet.
    * @access private
    */
    function _get_relop_sql($relop) 
    {
        switch ($relop) 
        {            
            case 'like':
                $sql = '{table_alias}.{column} LIKE {value}';
            break;
                        
            case 'rlike':
            case 'regexp':
                $sql = '{table_alias}.{column} REGEXP {value}';
            break;

            case 'fulltext':
                $sql = 'MATCH ({table_alias}.{column}) AGAINST ({value})';
            break;
            
            case 'in':
                $sql = '{table_alias}.{column} IN ({value}) ';
            break;
            
            case 'not in':
                $sql = '{table_alias}.{column} NOT IN ({value}) ';
            break;
                        
            default:
                // $relop is the operator -- this means '>' '<' '>=' '!=' etc are all valid.
                $sql = "{table_alias}.{column} $relop {value}";
            break;
        }
        return $sql;
    }
    
    /**
    * Adds a field to be filtered on.
    * This field may be a regular value or a pickid-type field.
    * @param string $fname The field name to filter on.  (May also be a typesfields displayname)
    * @param string $var The name of the variable that holds the filter value for this field.
    * @param string $relop The way that the field and value are related. (i.e. field='value' or field LIKE '%value%', etc.)
    * @param string $source Whether the source is a variable or a value
    * @param string $fieldopt An option field name to lookup up on if getting a pickid value.
    * @param boolean $quote whether or not to quote value
    * @return boolean
    * @access protected
    * @access private
    */    
    function _add_field($fname, $var, $relop='=', $source='var', $fieldopt=null, $quote=true) 
    {     
        // normalize input
        $relop = strtolower($relop);
                        
        // default is to assume that we are dealing with a normal field
        // and that column name will be $fname param
        $field['type'] = 'simple';
            
        // whether or not to quote value -- default is true
        $field['quote'] = $quote;
            
        // we do not set values when setting up the fields currently,
        // *but* it is still used in this function to guess whether the 
        // value is a pickid or is actually the value we want to use
        if ('value' != $source) 
        {
            $value = pxdb::import_var($var);
        } 
        else 
        {
            $value = $var;
        }
        
        if (in_array($fname, pxdb::get_pxdb_reserved_fields()))  
        {
            // if it's a reserved var -- e.g. 'datatype'
            $field['column'] = $fname;
            
            $field['sql'] = $this->_get_relop_sql($relop);
            $field[$source] = $var;
                                
            $this->fields[] = $field;            
            
            return true;            
        }
        elseif ($tfid = $this->lookup_tfid($fname)) 
        {            
            // determine whether specified $fname corresponds to a pickid column            
            // if a matching typesfield row was found ...
            $objf =& new pxdb_typesfield($tfid);            
            if ('relationship' == $objf->get_field_type()) 
            {                  
                // ..  it's a relationship
                if ($fieldopt) 
                {
                    $rel_field = $fieldopt;
                }
                else
                {
                    $rel_field = 'id';
                }
                                
                // create new filter object                
                $filter =& pxdb_search::filter($objf->get_otype());
                                
                $filter->_add_field($rel_field, $var, $relop, $source, null, $field['quote']);
                
                // relate to current filter.                
                $this->add_related($filter, $objf->get_relid());                
            } 
            else 
            {
                $dbf =& new pxdb_dbfield($objf->get_fieldid());

                // ... (or) that maching row seems to be of the 'pickid' type                
                if ('pickid' == $dbf->get_field_type())
                {
                    $field['type'] = 'pickrel';
                    
                    // oam, the following has to be true so that we always
                    // get the pick values out of pickrel 
                    $field['multiple'] = $objf->is_multiple();
                    
                    // note: if more than one datatype is defined we're may have trouble here.
                    // If the table in question is a variable-named table -- e.g. $(objecttype)type it
                    // will use the datatype to replace $(objecttype) with the current name of the datatype.
                    //
                    // if you *must* get around this, I would suggest calling set_datatype() directly before
                    // adding a field that uses a variable table name -- then you can change it back afterwards.
                    
                    $field['table'] = $objf->replace_tags($dbf->get_pick_table()); 
                                        
                    if ('multiple'==$field['type'] && $field['multiple'])
                    {
                        // SFS -- Um, what goes here?
                    }
                    
                    $field['id_col'] = ( null != $dbf->get_pick_column() ? $dbf->get_pick_column() : 'id');
                    $field['table_alias'] = $dbf->get_id() . '_' . $field['table'] . '_pick';
                    $field['fname'] = $dbf->get_fname();
                    $field['fid'] = $dbf->get_id();
                    
                    if ($fieldopt) 
                    { // field was specified
                        $field['column'] = $fieldopt;                        
                    } 
                    else 
                    {
                        // oam - if the pick column is id and the value has commas, make it an array
                        if ('id' == $field['id_col'] && false !== strpos($value, ','))
                        {
                            $value = explode(',', $value);
                            for ( $j = 0; $j < count($value); $j++)
                            {
                                $value[$j] = trim($value[$j]);
                            }
                            $field['column'] = $field['id_col'];
                        }
                        // build support for filtering on numeric id -OR- the text value
                        else if (!is_numeric($value))
                        {
                            $field['column'] = 'name';
                        }
                        else 
                        {
                            // default is to assume that for pickids we are being passed the id
                            $field['column'] = $field['id_col'];                    
                        }
                    }
                }
                else
                {
                    // typesfield row was found, but we're not dealing with a pickid
    
                    // set column to the dbfname that corresponds to this typesfield (may be
                    // different if they used passed a displayname instead of fname)
                    if ( $column = $dbf->get_fname() )
                    {
                        $field['column'] = $dbf->get_fname();
                    }
                    else
                    {
                        trigger_error( "Could not lookup fname in dbfields/relationships for field '$fname'", E_USER_WARNING );
                        $field['column'] = null; // prevents further notices
                    }
                }
            
                // Build the SQL necessary
                $field['sql'] = $this->_get_relop_sql($relop);
                $field[$source] = $var;
                                
                $this->fields[] = $field;
            } // if (field type = relationship)
            
            return true;                
        }
        else
        {
            // if tfid or member of reserved fields            
            trigger_error("No field found that matches: $fname. (datatype ".$this->get_datatype_list().")", E_USER_WARNING);
            return false;        
        } 
        
    }
    
    /**
    * Recursive function to get all variable names that are used by this record & any related records
    * @param Object $obj The object we are working with (used by recursion; will be set to $this on first invocation).
    * @return array Array of all variables used by this filter.
    * @access public
    */
    function get_vars($obj = null) 
    {
        $current_vars = array(); //init
        if (null === $obj) 
        {
            $obj =& $this; 
        } 
        
        while (list(,$field) = each($this->fields)) 
        {
            if (isset($field['var']))
            { 
                // there may be some fields that are not var-fields, but value fields
                $current_vars[] = $field['var'];
            }            
        }
                
        if ($obj->related) 
        {                                
            for($i=0, $z = sizeof($obj->related); $i < $z; $i++)
            {                                                                        
                $related_vars = $this->get_vars($obj->related[$i]['filter']);                
                $current_vars = array_merge($current_vars, $related_vars);
            }                    
        }
                
        return $current_vars;
    }
    
    /** 
    * Cleans out any PHP global variable arrays from the passed variable name.
    * E.g.     $_REQUEST['myfilter'] -> 'myfiter'
    *         $_REQUEST[pxdb][testvar] -> pxdb[testvar]
    * @param string $varname Variable name to clean.
    * @return string the cleaned variable name.
    * @access private
    */
    function _clean_varname($varname) 
    {
        $matches = array();
        if (preg_match("/(_REQUEST|_POST|_GET|HTTP_[^_]+_VARS)/", $varname, $matches))
        {                
            // fix varname
            $regex = preg_quote($matches[1])."\[([^\]]+)\]";
            $varname = preg_replace("/$regex/", "$1", $varname);                                
        } 
        
        return $varname;
    }
    
    /**
    * Pass currently set filters via URL or HTML form elements.
    * @param string $method The way to pass vars -- e.g. 'form' or 'url'.
    * @access public
    */    
    function pass($method = 'form') 
    {
        $vars = array_unique($this->get_vars());
        $i = 0;
        while (list(,$varname) = each ($vars))
        {
            // if the $var is of type _REQUEST _POST _GET then we need to strip
            // out the first part...                        
            $element_name = $this->_clean_varname($varname);
            
            // get the actual value (from global scope)
            $value = pxdb::import_var($varname);
            
            if ($method == 'form')
            {
                $formstr .= "<input type=\"hidden\" name=\"$element_name\" value=\"$value\" />\n";
            }
            else
            {
                $formstr .= (($i++ ? '&' : '') . $element_name . '=' . urlencode($value));
            }
        } 
        
        return $formstr;
    }
    
    /**
    * Whether or not any of the filter values are currently set.
    * @return boolean
    * @access public
    */
    function is_filtered() 
    {
        $found = false;
        
        $vars = array_unique($this->get_vars());        
        
        for ($found = false, $z = sizeof($vars), $i=0; $i < $z && !$found; $i++)
        {
            if (pxdb::import_var($vars[$i]))
            {
                $found = true;
            }
        }
        
        return $found;    
    }


    /**
    * Clears the values that are set for the filters in the query string.
    * @return string the current query string ($QUERY_STRING) without any filter information in it.
    * @access public
    */
    function clean_querystring() 
    {
        // strip out any of the filters from the query string
        $query = $_SERVER['QUERY_STRING'];
        $vars = array_unique($this->get_vars());    
                                
        while (list(,$varname) = each($vars))
        {
            $varname = $this->_clean_varname($varname); // remove anything like $_REQUEST[] from the variable name
            // problem: this isn't finding the end of line correctly %%%
            $query = preg_replace("/$varname=[\[\]]*[^&]*[&]/", "", $query);
            $query = preg_replace("/$varname=[\[\]]*[^&$]*/", "", $query);
        }
        
        return $query;
    
    }
    
    /**
    * Get an array of values for all set filters.
    * 
    * @return array Values for currently defined filters.
    * @access public
    */
    function get_values_array() 
    {               
        $vars = $this->get_vars();
        $values = array();
        if ($vars)
        {
            while (list(,$varname) = each($vars))
            {
                $val = pxdb::import_var($varname); // will return NULL if ${$varname} is not set.
                if ($val !== NULL)
                {
                    $values[] = $val;
                }
            }
        }
        
        $values = array_unique($values);
        
        return $values;
    }
    
    /**
    * Get a formatted list of the current filter values.
    * 
    * This method is for convenience only -- but it has a serious limitation: if you are using multi-select filters
    * (i.e. arrays) you won't be able to use this method -- and will have to build your own display using get_values_array()
    * method.
    *
    * @param string $delim The separator for the items of the array
    * @param string $smart_end The text to add to last element -- e.g. if $smart_element = "and": Boston, Bermuda, and Baltimore.
    * @return string Formatted text list of the values for currently specified filters.    
    * @access public
    */
    function get_values_list($delim=', ', $smart_end='') 
    {
        // fetch the array
        $valuesA = $this->get_values_array();
        
        /*
        OAM: how bout we avoid this stupid preg and just prepend $smart_end to the last element before imploding?
        if($smart_end) {
            // the first .* is greedy and *should* grab everything up to the *last* occurrence of $delim
            $values_list = preg_replace("/(.*)(".preg_quote($delim).")(.*?)$/", "\\1$smart_end\\2", $values_list);
        }
        */

        // format the results
        if ($smart_end)
        {
            $values_list[count($values_list)-1] = $smart_end.' '.$values_list[count($values_list)-1];         
        }
        $values_list = implode($delim, $valuesA);
        
        return $values_list;
    }
    
    /**
    * Filters a SQL statement by adding in statements to the JOIN and WHERE clauses.
    * This method provides additional flexibility in working with filters and SQL statements.
    * You could probably build your entire SQL statement simply using combinations of filter &
    * searches, but it may be easier to just build a basic SQL statement and then add the filters.
    * NOTE: This will *not* filter on priv table.
    * @param string $sql The SQL statement to filter.
    * @return string The filtered SQL.
    * @access public
    */    
    function filter_sql($sql) 
    {
        // because filtering adds LEFT joins, we need to make sure 
        // the DISTINCT keyword is included (of course this won't get all cases)
        if (!preg_match("/DISTINCT/i", $sql))
        {
            $sql = preg_replace("/(select[\s]+)/i", "$1 DISTINCT ", $sql);
        }
                
        $this->records_alias = pxdb_sql::get_table_alias('records', $sql);
                
        $join_sql = $this->get_join_sql();        
        
        // SFS get_join_sql now returns associative array
        $sql = preg_replace("/([\s]+where[\s]+)/i", " \n$join_sql[sql] \n $1", $sql);
        
        // we need to add an "AND" or $this->_lop after our SQL and before theirs         
        $where_sql = $this->get_where_sql();
        
        if($where_sql)
        {
            $sql = preg_replace("/([\s]+where[\s]+)/i", "$1 $where_sql $this->_lop ", $sql);
        }
                
        return $sql;
    }
}
?>