<?php
// $Id: pxdb_search_new.class.php,v 1.1.1.1 2004/09/10 19:27:10 omerida Exp $

pxdb_import('content.pxdb_data');
pxdb_import('metadata.pxdb_typesfield');
pxdb_import('metadata.pxdb_dbfield');

/**
* Flexible framework for filtering SQL results based on specified criteria.
* 
* Read the documentation for instructions and examples on searching & filtering result sets. Filters
* may be used in conjunction with the pxdb_collection class.  By themselves instances of this class will
* not filter on privileges or record approval status (they will only filter on values specified through
* the add_value() or add_var() methods).
*
* @package pxdb.content.output
* @author Hans Lellelid <hans@velum.net>
* @version $Revision: 1.1.1.1 $
* @access public
*/
class pxdb_filter extends pxdb_data 
{

    /**
	* Hash of fields we are looking for here.
	* @var array
	* @access public
	*/
    var $fields;


	/**
	* The array of related records.
	* @var array
	* @access public
	*/

	var $related;

	/**
	* The default top-level records table alias.
	* @var string
	* @access public
	*/
	var $default_records_alias = 'r';

	/**
	* The alias for the current record table.
	* @var string
	* @access public
	*/
	var $records_alias = '';
	
	/**
	* The conjunction / logical operator to use in the WHERE clause.
	* @var string
	* @access private
	*/
	var $_lop = 'OR';

	/**
	* The conjunction / logical operator to use in cases where many values are passed.
	* @var string
	* @access private
	*/
	var $_value_lop = 'OR';	
	
	/**
	* Initializes the filter system. Use the factory class methods instead.
	* @param mixed $dt The datatype(s) that this filter affects. (important for knowing how to look up field names to correponsing typesfields rows)
    * @access public
	*/
    function pxdb_search($dt=null) 
	{

	    parent::pxdb_data();

		// initialize

		$this->fields = array();
		$this->related = array();

		// set datatype if passed
		
		if($dt) { $this->set_datatype($dt); }
		
	}
	
	/**
	* Factory method for returning a 'search' version of this class.
	* Currently identical to constructor.
	* @param mixed $dt The datatype(s) that this filter affects. (important for knowing how to look up field names to correponsing typesfields rows)
	* @return Object Instance of this class in search mode.
	* @access public
	*/
	function &search($dt=null) 
	{
		$new = &new pxdb_search($dt);
		$new->set_type('search');
		return $new;
	}
	
	/**
	* Factory method for returning a 'filter' version of this class.
	* @param mixed $dt The datatype(s) that this filter affects. (important for knowing how to look up field names to correponsing typesfields rows)
	* @return object Reference to an instance of this class in filter mode.
	* @access public
	*/
	function &filter($dt=null) 
	{
		$new = &new pxdb_search($dt);
		$new->set_type('filter');
		return $new;		
	}
	
	/**
	* Set the conjunction to use between multiple values specified for a given field.
	* @param string $conj The conjunction / logical operator to use. (e.g. 'AND' or 'OR')
	* @since 0.0.2
	* @access public
	*/
	function set_value_conj($conj)
	{
		$this->_value_lop = $conj;
	}
	
	/**
	* Sets the conjunction to use.
	* Currently there is no difference between this function and set_type().  You can also
	* pass 'filter' or 'search' to set_type() which is what happens when this class is
	* instantiated using the pxdb_search::filter() or pxdb_search::search() methods.
	* @param string $conj The conjunction to set.
	* @return string The operator set.
	* @since 0.0.2
	* @access public
	*/
	function set_conj($conj)
	{
		return $this->set_type($conj);
	}
	/**
	* Sets the type for this filter:  'search' or 'filter'.
	* This method sets the logical operator (or conjunction) that is used in the WHERE SQL.
	* @param string $type The type of filter this is.
	* @return boolean On success of setting filter.
	* @access public
	*/
	function set_type($type) 
	{
		switch($type) {
			case 'search':
				$this->_lop = 'OR';
			break;
			
			case 'filter':
				$this->_lop = 'AND';
			break;
			default:
				$this->_lop = $type;
		}
		
		return $this->_lop;
	}
	
	/**
	* Based on filter values, gets the SQL that will be inserted into the *WHERE* clause of the SQL statement.
	* Uses current field values and field values of any records related to the current record.
	* @param object $obj The current pxdb_filter object -- used for recursion.
	* @param string $records_alias The alias of the current records table -- used for recursion.
	* @return string The text that should be inserted in the WHERE clause of SQL statement to activate these filters.
	* @access public
	*/
	function get_where_sql($obj=null, $records_alias=null) 
	{
	
	    if($obj === null) { $obj = &$this; } 
		
		$sql = '(';
		$a = 0;
				
		// set records alias -- defaults to $default_records_alias		
		if($records_alias === null) { $records_alias = ($obj->records_alias ? $obj->records_alias : $obj->default_records_alias); }

		// if there are any related records, descend into and get WHERE SQL from those:
		if($obj->related) {					
			
			// for each related record:  (note: placement of sizeof() for speed optimzation)
		    for($i=0, $z=sizeof($obj->related); $i < $z; $i++) {								
				$rel_obj = &$obj->related[$i]['filter'];
				
				if(!$rel_obj->records_alias ) {
					$rel_obj->records_alias = uniqid('records_');
				}
				// unique id
				$rel_records = $rel_obj->records_alias;
				$rel_where_sql = $this->get_where_sql($rel_obj, $rel_records);
				if($rel_where_sql) {
					$sql .= ($a++ ? "\n $this->_lop " : "") . $rel_where_sql;
				}							
			}
		
		} // if		
		
		// at this point there are no more related records, so we should handle the current object's field values
		
		// Now  handle the fields
		// ----------------------
				
		foreach($obj->fields as $field) {
			
			$values = (isset($field['value']) ? $field['value'] : pxdb::import_var($field['var']));
			
			if($values !== null) { // will be null if var is not set -- and we only filter on set vars
				
				if(!is_array($values)) { $values = array($values); } // normalize
							
				$sql .= ($a++ ? "\n $this->_lop " : "");
								
				$o=0;
				
				if($field['type'] == 'pickrel') {				
					$table_alias = $field['table_alias'];						
				} else {
					$table_alias = $records_alias;			
				}
				
				$sql .= '(';				
				foreach($values as $value) {
					$replace_map = array('table_alias' => $table_alias, 'column' => $field['column'], 'value' => ($field['quote'] ? $this->db->Quote($value) : $value) );				
					$criteria = pxdb_tools::populate($field['sql'] , $replace_map);
					$sql .= ($o++ ? ' '.$this->_value_lop.' ' : '') . $criteria;
				}	
				$sql .= ')';						
				
			}
		}		
				
		$sql .= ')';
	
		if(!preg_match("/[^\(\)]/", $sql)) { $sql = ''; }
		
		return $sql;				
	}	
	
	/**
	* Based on filter values, gets the SQL that will be inserted into the *FROM* clause of the SQL statement.
	* This includes pickrel values (JOINing to pickrel table) and also any related recors also need to be INNER JOINed
	* to current records table.
	* @param object $obj The current pxdb_filter object -- used for recursion.
	* @param string $records_alias The alias of the current records table -- used for recursion.
	* @return string The text that should be inserted in the FROM clause of SQL statement to activate these filters.
	* @access public
	*/	
	function get_join_sql($obj=null, $records_alias=null) 
	{
				
	    if($obj === null) { $obj = &$this; }

		if($records_alias === null) { $records_alias = ($this->records_alias ? $this->records_alias : $this->default_records_alias); }
		
		$sql = '';

		// if there are any related filters, we need to descend into them and get SQL first
		if($obj->related) {
		
		    for($i=0, $z=sizeof($obj->related); $i < $z; $i++) {
							
			    $rel_obj = &$obj->related[$i]['filter']; // by reference!
				
				if(!$rel_obj->records_alias ) {
					$rel_obj->records_alias = uniqid('records_');
				}
				// need a unique string of text for the table aliase
			    $xref = $obj->related[$i]['reltype'] . '_' . $rel_obj->records_alias . '_xref';
				$rel_records =  ($rel_obj->records_alias ? $rel_obj->records_alias : $seed . '_records' );
				
				$replace_map = array( 'records_alias' => $records_alias, 'xref_alias' => $xref, 'relid' => (is_array($obj->related[$i]['relid']) ? implode(',', $obj->related[$i]['relid']) : $obj->related[$i]['relid']), 'related_alias' => $rel_records);
												
		        $sql .= pxdb_tools::populate($obj->related[$i]['sql'], $replace_map);
				
				// recurse to get join sql for the related filter
				$sql .= $obj->get_join_sql($rel_obj, $rel_records);
				
			}

		}
		
		// at this point there are no more related fields -- so build the SQL for joining pickrel tables

			foreach($obj->fields as $prel) {
				
				// we only need to add join SQL for pickrel fields
				if($prel['type'] == 'pickrel') {
			
					$xref = $prel['table_alias'] . '_xref';
					$pick = $prel['table_alias'];				

					if($prel['multiple']) {
						$sql .= " LEFT JOIN pickrel $xref ON $xref.recid = $records_alias.id AND $xref.f_id = $prel[fid]\n";
						$sql .= "   LEFT JOIN $prel[table] $pick ON $pick.$prel[id_col] = $xref.p_id";
					} else {
						$sql .= "   LEFT JOIN $prel[table] $pick ON $pick.$prel[id_col] = $records_alias.$prel[fname]";
					}
						
				} // if type = 'pickrel'				
			
			} // foreach $obj->fields			

		return $sql;

	}
	
	/**
	* Merges an pxdb_filter object with the current pxdb_filter object.
	* Fields, related records, pickrel items, datatypes are all merged, with values of current
	* object taking priority -- e.g. if current object has 'date1 DESC' for $orderby and object being merged has 
	* 'name ASC' for $orderby, then new value will be 'date 1 DESC, name ASC'
	* @param object $filter The other pxdb_filter object to merge.
	* @access public
	*/
	function merge(&$filter) 
	{

	    // merging means adding values from $filter object into this object

		// that means adding together:
		//  -elements of related array
		//  -fields
		//  -pickid
		//  -datatype


		// Copy Related Records
		// not sure if array_merge will preserve memory location array_merge($this->related, $filter->related)

        $s = sizeof($this->related);
		$z = sizeof($filter->related) + $s;

		for($i = $s; $i < $z; $i++) {
			$this->related[$i]['reltype'] = &$filter->related[$i-$s]['reltype'];
		    $this->related[$i]['filter'] = &$filter->related[$i-$s]['filter'];
			$this->related[$i]['relid'] = &$filter->related[$i-$s]['relid'];
			$this->related[$i]['sql'] = &$filter->related[$i-$s]['sql'];
		}

		// Copy Fields

		$this->fields = array_merge($this->fields, $filter->fields);

		// Copy Datatypes
			
		$this->datatype = array_merge($this->datatype, $filter->datatype);

	}

	/**
	* Relate an pxdb_filter to the current filter via primaryrel table.
	* @param object $filter The pxdb_filter object to be related.
	* @param mixed $relid The relid(s) by which this object can be related.
	* @return boolean Returns true on success
	* @access public
	*/	
	function add_related(&$filter, $relid=null) 
	{


		if(!is_array($relid)) { $relid = ($relid !== null ? preg_split("/[,\s]/", $relid) : array()); }			
		
		$rel = array();
		$rel['reltype'] = 'primary';
		$rel['filter'] = &$filter;
		$rel['relid'] = $relid;
		
		$rel['sql'] = "LEFT JOIN primaryrel {xref_alias} ON {xref_alias}.id1 = {records_alias}.id ".($relid ? "AND {xref_alias}.relid IN ({relid})" : "")."\n";
		$rel['sql'] .= "LEFT JOIN records {related_alias} ON {related_alias}.id = {xref_alias}.id2\n";
		
		$this->related[] = &$rel;
	
		return true;
	}

	/**
	* Relate an pxdb_filter to the current filter via secondaryrel table.
	* @param object $filter The pxdb_filter object to related.
	* @access public
	*/		
	function add_secondaryrel_related(&$filter) 
	{	
		
		$rel = array();
		$rel['reltype'] = 'secondary';
		$rel['filter'] = & $filter;

		$rel['sql'] = "LEFT JOIN secondaryrel {xref_alias} ON {xref_alias}.id1 = {records_alias}.id\n";
		$rel['sql'] .= "LEFT JOIN records {related_alias} ON {related_alias}.id = {xref_alias}.id2\n";
		
		$this->related[] = &$rel;	
	
	}

	/**
	* Relate an pxdb_filter as the parent of the current object (noderel).
	* @param object $filter The pxdb_filter to related.
	* @access public
	*/		
	function add_parent(&$filter) 
	{
	
	
		$rel = array();
		$rel['reltype'] = 'parent';
		$rel['filter'] =&$filter;

		$rel['sql'] = "LEFT JOIN noderel {xref_alias} ON {xref_alias}.child = {records_alias}.id\n";
		$rel['sql'] .= "LEFT JOIN records {related_alias} ON {related_alias}.id = {xref_alias}.parent\n";
		
		$this->related[] = &$rel;	
		
	}

	/**
	* Relate an pxdb_filter as the child of the current object (noderel).
	* @param object $filter The pxdb_filter to related.
	* @access public
	*/	
	function add_child(&$filter) 
	{
	
	
		$rel = array();
		$rel['reltype'] = 'child';
		$rel['filter'] = &$filter;

		$rel['sql'] = "LEFT JOIN noderel {xref_alias} ON {xref_alias}.parent = {records_alias}.id\n";
		$rel['sql'] .= "LEFT JOIN records {related_alias} ON {related_alias}.id = {xref_alias}.child\n";
		
		$this->related[] = &$rel;	
		
	}	
	
	/**
	* Adds a filter value.
	* Note: filter values are *not* able to be passed on.  Also, if you add a filter
	* value using this method, is_filtered() will still report FALSE if no vars are defined.
	* @param string $fname The field name to filter on.  (May also be a typesfields displayname)
	* @param string $value The value that the given field should match.
	* @param string $relop The way that the field and value are related. (i.e. field='value' or field LIKE '%value%', etc.)
	* @param string $fieldopt An option field name to lookup up on if getting a pickid value.
	* @param boolean $quote whether or not to quote value
	* @return boolean (documented in _add_field)
	* @access public
	*/
	function add_value($fname, $value, $relop='=', $fieldopt=null, $quote=true)
	{
		return $this->_add_field($fname, $value, $relop, 'value', $fieldopt, $quote);
	}
	
	/**
	* Adds a field based on a varname.
	* This is generally the preferred method for adding filters.  Why?  Because when you 
	* add a filter based on a varname (instead of a value) you can use the pass() method to 
	* pass the filters (the variables) to another page.  Also, the is_filtered() will check
	* to see if any of the filter vars are set. (is_filtered() will return false when only
	* value filters are set.)
	* @param string $fname The field name to filter on.  (May also be a typesfields displayname)
	* @param string $varname The variable name that holds the value for the given field.
	* @param string $relop The way that the field and value are related. (i.e. field='value' or field LIKE '%value%', etc.)
	* @param string $fieldopt An option field name to lookup up on if getting a pickid value.
	* @param boolean $quote whether or not to quote value
	* @return boolean (documented in _add_field)
	* @access public
	*/
	function add_var($fname, $varname, $relop='=', $fieldopt=null, $quote=true) 
	{
		return $this->_add_field($fname, $varname, $relop, 'var', $fieldopt, $quote);
	}
	
	/**
	* Get the SQL snippet for a given rel operator.
	* This may need to be moved into a RDBMS-specific library.
	* @param string $relop The relop for which to return SQL.
	* @return string The SQL snippet.
	* @access private
	*/
	function _get_relop_sql($relop) 
	{
		
		switch($relop) {
			case 'like':
				$sql = '{table_alias}.{column} LIKE {value}';
			break;

			case 'rlike':
			case 'regexp':
				$sql = '{table_alias}.{column} REGEXP {value}';
			break;

			case 'fulltext':
				$sql = 'MATCH ({table_alias}.{column}) AGAINST ({value})';
			break;
			
			default:
				// $relop is the operator -- this means '>' '<' '>=' '!=' etc are all valid.
				$sql = "{table_alias}.{column} $relop {value}";
		}
		return $sql;
	}
	
	/**
	* Adds a field to be filtered on.
	* This field may be a regular value or a pickid-type field.
	* @param string $fname The field name to filter on.  (May also be a typesfields displayname)
	* @param string $var The name of the variable that holds the filter value for this field.
	* @param string $relop The way that the field and value are related. (i.e. field='value' or field LIKE '%value%', etc.)
	* @param string $source Whether the source is a variable or a value
	* @param string $fieldopt An option field name to lookup up on if getting a pickid value.
	* @param boolean $quote whether or not to quote value
	* @return boolean
	* @access protected
	* @access private
	*/	
	function _add_field($fname, $var, $relop='=', $source='var', $fieldopt=null, $quote=true) 
	{

	    // normalize input
			$relop = strtolower($relop);
						
		// default is to assume that we are dealing with a normal field
		// and that column name will be $fname param
			
			$field['type'] = 'simple';
			
		// whether or not to quote value -- default is true
		
			$field['quote'] = $quote;
			
		// we do not set values when setting up the fields currently,
		// *but* it is still used in this function to guess whether the 
		// value is a pickid or is actually the value we want to use
		
			if($source != 'value') {
				$value = pxdb::import_var($var);
			} else {
				$value = $var;
			}
		
		
		if ( in_array($fname, pxdb::get_pxdb_reserved_fields()) )  {
			
			// if it's a reserved var -- e.g. 'datatype'
			
			$field['column'] = $fname;
			
			$field['sql'] = $this->_get_relop_sql($relop);
			$field[$source] = $var;
					
			$this->fields[] = $field;			
			
			return true;
			
		} elseif($tfid = $this->lookup_tfid($fname)) {
		
			// determine whether specified $fname corresponds to a pickid column			
			
			// if a matching typesfield row was found ...
			
			
			  $objf = &new pxdb_typesfield($tfid);
			  
			  
			  if($objf->get_field_type() == 'relationship') {
			  	
				// ..  it's a relationship
				
				if($fieldopt) $rel_field = $fieldopt; else $rel_field = 'id';
				
				// create new filter object	
				$filter = &pxdb_search::filter($objf->get_otype());
				$filter->_add_field($rel_field, $var, $relop, $source);
				
				// relate to current filter.
				$this->add_related($filter, $objf->get_relid());
				
			  } else {
			  
				  $dbf = &new pxdb_dbfield($objf->get_fieldid());
	
				// ... (or) that maching row seems to be of the 'pickid' type
				
				  if($dbf->get_field_type() == 'pickid') {			  				
					
					$field['type'] = 'pickrel';
					
					$field['multiple'] = $objf->is_multiple();
					
					// note: if more than one datatype is defined we're may have trouble here.
					// If the table in question is a variable-named table -- e.g. $(objecttype)type it
					// will use the datatype to replace $(objecttype) with the current name of the datatype.
					//
					// if you *must* get around this, I would suggest calling set_datatype() directly before
					// adding a field that uses a variable table name -- then you can change it back afterwards.
					
					$field['table'] = $objf->replace_tags($dbf->get_pick_table()); 
														
					$field['id_col'] = ($dbf->get_pick_column() ? $dbf->get_pick_column() : 'id');
					$field['table_alias'] = $dbf->get_id() . '_' . $field['table'] . '_pick';
					$field['fname'] = $dbf->get_fname();
					$field['fid'] = $dbf->get_id();
					
					if($fieldopt) { // field was specified
						$field['column'] = $fieldopt;						
					} else {
						// build support for filtering on numeric id -OR- the text value
						if(!is_numeric($value)) {
							$field['column'] = 'name';
						} else {
							// default is to assume that for pickids we are being passed the id
							$field['column'] = $field['id_col'];					
						}
					}
					
				} else {
				
					// typesfield row was found, but we're not dealing with a pickid
	
					// set column to the dbfname that corresponds to this typesfield (may be
					// different if they used passed a displayname instead of fname)
					$field['column'] = $dbf->get_fname();
					
				}
				
					// Build the SQL necessary
				
					$field['sql'] = $this->_get_relop_sql($relop);
					$field[$source] = $var;
					
					$this->fields[] = $field;	
					
					
		
			} // if (field type = relationship)
			
			return true;	
			
		} else { // if tfid or member of reserved fields
			
			pxdb::trigger_error("No field found that matches: $fname. (datatype ".$this->get_datatype_list().")", E_USER_WARNING);
			return false;		
		} 
		
	}
		
		
	/**
	* Recursive function to get all variable names that are used by this record & any related records
	* @param Object $obj The object we are working with (used by recursion; will be set to $this on first invocation).
	* @return array Array of all variables used by this filter.
	* @access public
	*/
	function get_vars($obj=null) 
	{
	
	    if($obj === null) { $obj = &$this; } 
		
		foreach($this->fields as $field) {
			if(isset($field['var'])) { // there may be some fields that are not var-fields, but value fields
				$current_vars[] = $field['var'];
			}			
		}
				
		if($obj->related) {								
		    for($i=0, $z=sizeof($obj->related); $i < $z; $i++) {																		
				$related_vars = $this->get_vars($obj->related[$i]['filter']);				
				$current_vars = array_merge($current_vars, $related_vars);
			}					
		}
		
		return $current_vars;
		
	}
	
	/** 
	* Cleans out any PHP global variable arrays from the passed variable name.
	* E.g. 	$_REQUEST['myfilter'] -> 'myfiter'
	* 		$_REQUEST[pxdb][testvar] -> pxdb[testvar]
	* @param string $varname Variable name to clean.
	* @return string the cleaned variable name.
	* @access private
	*/
	function _clean_varname($varname) 
	{
	
			if(preg_match("/(_REQUEST|_POST|_GET|HTTP_[^_]+_VARS)/", $varname, $matches)) {				
				// fix varname
				$regex = preg_quote($matches[1])."\[([^\]]+)\]";
				$varname = preg_replace("/$regex/", "$1", $varname);								
			} 
			
		return $varname;
	}
	
	/**
	* Pass currently set filters via URL or HTML form elements.
	* @param string $method The way to pass vars -- e.g. 'form' or 'url'.
	* @access public
	*/	
	function pass($method='form') 
	{

		$vars = array_unique($this->get_vars());
		
		foreach($vars as $varname) {
				
			
			// if the $var is of type _REQUEST _POST _GET then we need to strip
			// out the first part...						
			
				$element_name = $this->_clean_varname($varname);
			
			// get the actual value (from global scope)
			
				$value = pxdb::import_var($varname);
			
			if($method == 'form') {
				$formstr .= "<input type=\"hidden\" name=\"$element_name\" value=\"$value\">\n";
			} else {
				$formstr .= (($i++ ? '&' : '') . $element_name . '=' . urlencode($value));
			}
		
		} 
		
		return $formstr;			
	
	}
	
	/**
	* Whether or not any of the filter values are currently set.
	* @return boolean
	* @access public
	*/
	function is_filtered() 
	{
		
		$found = false;
		
		$vars = array_unique($this->get_vars());		
		
		for($found=false, $z=sizeof($vars), $i=0; $i < $z && !$found; $i++) {
		
			if(pxdb::import_var($vars[$i])) { $found = true; }

		}
		
		return $found;	
	}


	/**
	* Clears the values that are set for the filters in the query string.
	* @return string the current query string ($QUERY_STRING) without any filter information in it.
	* @access public
	*/
	function clean_querystring() 
	{					
		
		// strip out any of the filters from the query string
		
		$query = $_SERVER['QUERY_STRING'];
		$vars = array_unique($this->get_vars());	
								
		foreach ($vars as $varname) {
			$varname = $this->_clean_varname($varname); // remove anything like $_REQUEST[] from the variable name
			// problem: this isn't finding the end of line correctly %%%
			$query = preg_replace("/$varname=[\[\]]*[^&]*[&]/", "", $query);
			$query = preg_replace("/$varname=[\[\]]*[^&$]*/", "", $query);
		}
		
		return $query;
	
	}
	
	/**
	* Get an array of values for all set filters.
	* 
	* @return array Values for currently defined filters.
	* @access public
	*/
	function get_values_array() 
	{			
						
		$vars = $this->get_vars();
		$values = array();
		
		foreach($vars as $varname) {
			$val = pxdb::import_var($varname); // will return NULL if ${$varname} is not set.
			if($val !== NULL) {
				$values[] = $val;
			}			
		}
		
		$values = array_unique($values);
		
		return $values;
		
	}
	
	/**
	* Get a formatted list of the current filter values.
	* 
	* This method is for convenience only -- but it has a serious limitation: if you are using multi-select filters
	* (i.e. arrays) you won't be able to use this method -- and will have to build your own display using get_values_array()
	* method.
	*
	* @param string $delim The separator for the items of the array
	* @param string $smart_end The text to add to last element -- e.g. if $smart_element = "and": Boston, Bermuda, and Baltimore.
	* @return string Formatted text list of the values for currently specified filters.	
	* @access public
	*/
	function get_values_list($delim=', ', $smart_end='') 
	{
	
		// fetch the array
		$valuesA = $this->get_values_array();
		
		// format the results
		$values_list = implode($delim, $valuesA);
		
		if($smart_end) {
			// the first .* is greedy and *should* grab everything up to the *last* occurrence of $delim
			$values_list = preg_replace("/(.*)(".preg_quote($delim).")(.*?)$/", "\\1$smart_end\\2", $values_list);
		}
	
		return $values_list;
	}
	
	/**
	* Filters a SQL statement by adding in statements to the JOIN and WHERE clauses.
	* This method provides additional flexibility in working with filters and SQL statements.
	* You could probably build your entire SQL statement simply using combinations of filter &
	* searches, but it may be easier to just build a basic SQL statement and then add the filters.
	* NOTE: This will *not* filter on priv table.
	* @param string $sql The SQL statement to filter.
	* @return string The filtered SQL.
	* @access public
	*/	
	function filter_sql($sql) 
	{
	
		// because filtering adds LEFT joins, we need to make sure 
		// the DISTINCT keyword is included (of course this won't get all cases)
		
		if(!preg_match("/DISTINCT/i", $sql)) {
			$sql = preg_replace("/(select[\s]+)/i", "$1 DISTINCT ", $sql);
		}
				
		$this->records_alias = pxdb_sql::get_table_alias('records', $sql);
				
		$join_sql = $this->get_join_sql();		
		$sql = preg_replace("/([\s]+where[\s]+)/i", " \n$join_sql \n $1", $sql);
		
		// we need to add an "AND" or $this->_lop after our SQL and before theirs
		 
		$where_sql = $this->get_where_sql();
		if($where_sql) {
			$sql = preg_replace("/([\s]+where[\s]+)/i", "$1 $where_sql $this->_lop ", $sql);
		}
				
		return $sql;

	}

}

?>