<?php
/**
* Creates a pager to navigate a Syntax Collection. Expects that you've used execute_page on your
* collection and uses that info to draw the pages
*
* Usage:
*   $pager = new Stx_Collection_Pager( $collection, $Request );
*
*/
class Stx_Collection_Pager
{
    var $collection;
    var $_links;
    var $_curr_page;
    var $_max_links;
    var $total_pages;
    var $request;
    
    /**
    * Constructor
    *
    * @param a syntax collection, expects execute_page was called on it    
    * @param request object, for building the url
    * @param integer number of links to show at once (defaults to 5)
    * @return object
    */
    function Stx_Collection_Pager( $collection, $request )
    {
        $this->collection = $collection;        
        $this->request = $request;
        $this->init();
    }
   
    /**
    * Initializes internal variables
    *
    * @access private;
    */
    function init()
    {
        // from adodb
        $this->_curr_page = $this->collection->rs->_currentPage;
        
        // total pages is the ceiling of the number of records divided by number of records per page
        if ( isset($this->collection->rs->rowsPerPage) && 0 < $this->collection->rs->rowsPerPage )
        {
            $pages = (float) $this->collection->rs->_maxRecordCount / $this->collection->rs->rowsPerPage;    
            $pages = (int) ceil($pages);
        }
        else
        {
            //recordset is not paginated? 
            $pages = 1;
        }
        
        $this->total_pages = $pages;                
    }
    
    /**
    * Returns the total number of pages in set
    *
    * @access public
    */
    function get_total_pages()
    {
        return $this->total_pages;
    }
    
    /**
    * Returns number of rows per page
    *
    * @access public
    */
    function get_rows_per_page()
    {
        return $this->collection->rs->rowsPerPage;
    }
    
    /**
    * Sets an upper limit on the number of paging links to display
    *
    * @param integer
    * @access public
    */
    function set_max_links( $max )
    {
        $this->_max_links = $max;
    }
    
    /**
    * Returns the array of page link numbers centered about current page.
    *
    * @param integer number of links to return.
    * @return array
    * @access private
    */
    function get_page_links( $max = 5 )
    {
        $this->set_max_links($max);
        
        //echo '<pre>'; print_r ( $this ); echo '</pre>';
        // start by assuming we are centering the links around  the current page
        // div by two to "center" links around current page
        $lo = ($this->_curr_page - 1) - ( floor($this->_max_links/2) );
        $hi = $lo + $this->_max_links;		
    	//echo "$lo // $hi<br>";
        
        // correct hi and lo so that they aren't negative or exceed the max number of pages
        // if we are near pages on the ends 
        if ($lo < 1) 
        {
            $hi = $hi - $lo; // if lo is less than zero this actually adds it to high
            $lo = 1;
        }
        
        // we are near the final pages
        if ($hi > $this->get_total_pages() )  
        {
            if ($lo != 0) 
            {
                $lo = $lo - ($hi-$this->get_total_pages());
                if ( $lo < 1 )
                {
                    // back to the start
                    $lo=1;
                }
            } // end if	
            $hi = $this->get_total_pages();	
        } 
    
        if ($lo==1 && ($hi-$lo >= $this->_max_links) )
        {
            $lo += 1;	
        } // end if
    
        if ( ($hi<=$this->get_total_pages()) && $lo>1 && ($hi-$lo >= $this->_max_links) )
        {
            $lo +=1;
        } // end if
    
        //echo "$lo // $hi<br>";
        for ($i=$lo; $i<=$hi; $i++) 
        {
            $this->_links[$i] = $i;
        }
        return $this->_links;
    
    } // end func

    /**
    * Returns the current page's URI for building links
    *
    * @access public
    */
    function get_page_uri( $page )
    {
        $qs = $this->request->getVarsAsQueryString( array('page' ) );
        
        if ( empty( $qs ) )
        {
            return $_SERVER['SCRIPT_URI'].'?page='.$page;
        }
        else
        {
            return $_SERVER['SCRIPT_URI'].'?'.$qs.'&page='.$page;
        }
    }
    
    /**
    * Tests if we are viewing the first page
    *
    * @return boolean
    * @access public
    */
    function at_first_page()
    {
        // reuse adodb code
        return $this->collection->rs->AtFirstPage();
    }
    
    /**
    * Tests if we are viewing the last page
    *
    * @return boolean
    * @access public
    */
    function at_last_page()
    {
        // reuse adodb code
        return $this->collection->rs->AtLastPage();
    }
    
    /**
    * Returns current page number
    *
    * @return integer
    * @access public
    */
    function get_current_page()
    {
        return $this->_curr_page;
    }
    
}
?>