<?php 
/**
* Dispatcher to execute custom behaviors when an arbitrary event occurs
*
* Event listeners are classes defined in PXDB_EVENTPATH that extend the base
* pxdb_event_listener class to implement some custom response.
*
* Some standard event labels are defined at the end of this file as well:
*   SYNTAX_EVENT_INSERT - a new record has been added
*   SYNTAX_EVENT_UPDATE - an existing record is updated
*   SYNTAX_EVENT_DELETE - an existing record was deleted
*   SYNTAX_EVENT_APPROVE - a record is approved
*
* @author Oscar Merida <oscar@oscarm.org>
* @package pxdb.content.events
* @version $Revision: 1.4 $ 
* @access public
*/
class pxdb_event_dispatcher
{
    /**
    * Internal queue of events to notify when a trigger is activated.
    */    
    var $_notify_on = array();
    
    /**
    * Constructor
    */
    function pxdb_event_dispatcher()
    {
        
    }
    
    /**
    * Loads and hooks in all the event listeners in the specified directory.
    *
    * Expects all files in the directory to be names <name>.class.php and the 
    * class name should same as <name>
    *
    * @access private
    */
    function loadListeners( $path )
    {        
        pxdb_import( 'events.pxdb_event_listener');
        
        // go through $path. Load and register the event_listeners defined therein
        if ($handle = opendir( $path )) 
        {
           while (false !== ($file = readdir($handle))) 
           {
               
               if ( 0!==strpos( $file, '.' ) // ignore ones stat start with a period
                   && is_file(  $path.'/'.$file ) 
               ) 
               {
                   // only load files with class.php in the filename
                   if ( FALSE !== strpos( $file, '.class.php' ) )
                   {
                       require_once( $path.'/'.$file );
                       
                       // get the expected classname;
                       $fname = explode( '.', $file );
                       $classname = $fname[0];
                       
                       $listener = new $classname;
                       
                       if ( is_subclass_of( $listener, 'pxdb_event_listener' ) )
                       {
                           $listener->init();
                           
                           // now register
                           $this->registerListener( $listener );
                       }
                   }
               }
           }
           closedir($handle);
        } // end if
        
    }
    /**
    * Singleton - returns same instance of this object on each call.
    *
    * @return object
    * @access public
    */
    function &singleton()
    {
        static $dispatcher;
        
        if ( !isset( $dispatcher ) )
        {
            $dispatcher = new pxdb_event_dispatcher();
            
            if ( defined( 'PXDB_EVENTPATH' ) )
            {
                $dispatcher->loadListeners( PXDB_EVENTPATH );                
            } 
            else 
            {
                trigger_error( 'pxdb_event_dispatcher: PXDB_EVENTPATH is not defined', E_USER_WARNING );
                return false;
            }
        }
        
        return $dispatcher;
    }
    
    /**
    * Adds a single listener to the notification list for a trigger.
    *
    * @param object listener object
    * @param string trigger name
    * @access public
    */
    function mapTrigger( $listener, $trigger )
    {
        if ( empty( $this->_notify_on[ $trigger ] ) )
        {
            $this->_notify_on[ $trigger ] = array();
        }
        
        return array_push( $this->_notify_on[ $trigger ], $listener );
    }
    
    /**
    * Maps all listener triggers internally
    *
    * @access private
    */
    function registerListener( $listener )
    {
        if ( is_subclass_of( $listener, 'pxdb_event_listener' ) )
        {
            $triggers = $listener->getTriggers();
            
            //$listener_name = get_class( $listener );
            
            if ( empty( $triggers ) )
            {
                trigger_error( "Listener $listener_name has no triggers defined", E_USER_WARNING );
                return false;
            }
            else
            {            
                while( list(, $trigger ) = each( $triggers ) )
                {
                    $this->mapTrigger( $listener, $trigger );
                }
            }
            
            return true;
        }
        else
        {
            trigger_error( 'Passed object is not a pxdb_event_listener descendant.', E_USER_WARNING );
            return false;
        }
    }
    
    /**
    * Used by scripts to initiate notification of a listeners
    *
    * @param string trigger name, expected by listeners
    * @param array parameters to pass to listeners on notification    
    * @access public
    */
    function notify( $trigger, $parameters)
    {
        // Don't pass the notification if notices for the source object for this trigger
        // has been suppressed
        if (true == $this->isNotificationSuppressed($trigger, $parameters['id'])) 
	{
            return false;
        }
        
        if ( isset( $this->_notify_on[ $trigger ] ) && !empty( $this->_notify_on[ $trigger ] ) )
        {

            while( list( ,$listener_object ) = each( $this->_notify_on[ $trigger ] ) )
            {                
                $listener_object->execute( $trigger, $parameters );                
            }                        
            // since its an object property
            
            reset( $this->_notify_on[ $trigger ] );

        }
        else
        {
            return false;
        }
    }
    
    /**
    * Used by scripts to suppress notifications for certain triggers on
    * particular records/objects. This is user session specific and allows
    * developers of event listeners to work on Syntax objects and suppress
    * triggers on those objects. This is one way to prevent infinite loops
    * but also allows more control of when notifications are processed.
    *
    * @param string trigger - name of trigger to suppress
    * @param int id - record id for object to suppress triggers
    * @access public
    */
    function suppressNotification($trigger, $id)
    {
	    $_SESSION['notify']['suppress'][$trigger][$id] = true;
    }
    
    /**
    * Used after suppressNotification() to turn notifications
    * back on. You must call this after working with your object or you
    * could suppress notifications on that record for the rest of the user
    * session.
    *
    * @param string trigger - name of trigger to unsuppress
    * @param int id - record id for object to unsuppress triggers
    * @access public
    */
    function unsuppressNotification($trigger, $id)
    {
	    $_SESSION['notify']['suppress'][$trigger][$id] = false;
    }

    /**
    * Similar to unsuppressNotification() except this completely resets the
    * suppress (stored in the PHP session) return the system back to it's
    * previous state. On the other hand unsuppressNotification() leaves evidence
    * that the trigger has been suppressed which can be detected by calling the
	* wasNotificationSuppressed() method.
    *
    * @param string trigger - name of trigger to unsuppress and reset
    * @param int id - record id for object to unsuppress and reset triggers
    * @access public
    */
    function resetSuppressNotification($trigger, $id)
    {
	    $_SESSION['notify']['suppress'][$trigger][$id] = NULL;
	    unset($_SESSION['notify']['suppress'][$trigger][$id]);
	    session_unregister($_SESSION['notify']['suppress'][$trigger][$id]);
    }
	
    /**
    * Tests to see if a notification trigger for a particular record/object has
    * been suppressed.
    *
    * @param string trigger - name of trigger to test
    * @param int id - record id for object to test
    * @access public
    */
    function isNotificationSuppressed($trigger, $id)
    {
	if (isset($_SESSION['notify']['suppress'][$trigger][$id]))
	{
		return $_SESSION['notify']['suppress'][$trigger][$id];
	}
	else
	{
		return false;
	}
    }
    
    /**
    * Tests to see if a notification trigger for a particular record/object that
    * is currently active was suppressed at some point during the user session.
    *
    * @param string trigger - name of trigger to test
    * @param int id - record id for object to test
    * @access public
    */
    function wasNotificationSuppressed($trigger, $id)
    {
	if ( isset($_SESSION['notify']['suppress'][$trigger][$id]) &&
		false == $_SESSION['notify']['suppress'][$trigger][$id] )
	{
		return true;
	}
    }
} // end class

/**   SYNTAX_EVENT_INSERT - a new record has been added */
define( 'SYNTAX_EVENT_INSERT' , 'syntax_event_insert' );

/**   SYNTAX_EVENT_UPDATE - an existing record is updated */
define( 'SYNTAX_EVENT_UPDATE' , 'syntax_event_update' );

/**   SYNTAX_EVENT_DELETE - an existing record was deleted */
define( 'SYNTAX_EVENT_DELETE' , 'syntax_event_delete' );

/**   SYNTAX_EVENT_APPROVE - a record is approved */
define( 'SYNTAX_EVENT_APPROVE' , 'syntax_event_approve' );

/**   SYNTAX_EVENT_UNAPPROVE - a record is unapproved */
define( 'SYNTAX_EVENT_UNAPPROVE' , 'syntax_event_unapprove' );

/**   SYNTAX_EVENT_PUBLISH - a record is publishd */
define( 'SYNTAX_EVENT_PUBLISH' , 'syntax_event_publish' );

/**   SYNTAX_EVENT_UNPUBLISH - a record is unpublishd */
define( 'SYNTAX_EVENT_UNPUBLISH' , 'syntax_event_unpublish' );

/**   SYNTAX_EVENT_PREDELETE - a record is predeleted */
define( 'SYNTAX_EVENT_PREDELETE' , 'syntax_event_predelete' );

/**   SYNTAX_EVENT_PREUPDATE - a record is about to be updated */
define( 'SYNTAX_EVENT_PREUPDATE' , 'syntax_event_preupdate' );
?>
