<?php
/**
* Handles datatype level privileges
*
* Datatype level privs include :
*  View: read the datatype, similar to expose currently 
*  Add: add new records
*  Edit: Edit exisiting records
*  Approve: Approve existing records
*  Delete: Delete existing records
*
* Usage to check datatype_privs:
*   $datatype_privs = new pxdb_datatyp_privs( $ugid );
*
* @author Oscar Merida <oscar@oscarm.org>
* @version $Revision: 1.3 $
* @package pxdb.metadata
* @access public
*/

class pxdb_datatype_privs 
{
        
    /**
    * User group id, can be one or multiple arrays
    */
        var $ugid;
    /**
    * Enforces datatypes table's expose column.  Default to on
    * @see set_exposed;
    */
        var $exposed;
        
    /**
    * restrict to a specific datatype subset
    * @see set_restrict_datatypes
    */
        var $restrict_datatypes;
    /**
    * Caches looked up values for this ugid
    *
    * @access private
    */
        var $_cache;
    
    /**
    * Constructor
    *
    * @param mixed integer or array of group ids
    * @return object
    * @access public
    */
    function pxdb_datatype_privs( $ugid )
    {
        $this->ugid = $ugid;
        $this->exposed = TRUE;
        $this->restrict_datatypes = null;        
    }
    
    /**
    * Sets the exposed property.that enforces datatypes table's expose column
    *
    * @param boolean
    */
    function set_exposed( $bool )
    {
        $tyis->exposed = $bool;
    }
    
    /**
    * Use to restrict datatypes to a specific subset of ids.
    *
    * @param array of datatype ids
    * @access public
    */
    function set_restrict_datatypes( $dt_array )
    {
        if( !is_array( $dt_array ) )
        {
            $dt_array = (array) $dt_array;
        }
        
        $this->restrict_datatypes = $dt_array;
        
    }
    /**
    * Returns an array of objects that the user has the designeated priv on
    *
    * For example, to get all the objects a group can view:
    * usage: $obj = $privVar->get_object_array( DATATYPE_PRIV_VIEW );
    * Also, the class will setup constants to use instead of strings based
    * on the priv_ columns in your database table.    
    *
    * @param string      
    * @return array
    */
    function get_object_array( $priv )
    {        
        $priv_col = 'priv_'.$priv;
        
        $sql = 'SELECT d.id, d.type, d.label, dt.'.$priv_col.' as `'.$priv.'`'
              .' FROM datatypes d'
              .' INNER JOIN datatype_privs dt on (d.id=dt.datatype_id and dt.'.$priv_col.'=1 '     
        ;
        
        if ( is_array( $this->ugid ) )
        {
            $sql.=' and dt.ugid IN ('.join(',' , $this->ugid) .') )';
        } else {
            $sql.=' and dt.ugid='.$this->ugid.')';
        }
        
        $sql .= ( TRUE==$this->exposed ) ? " WHERE d.expose='Y'" : '';
                
        if ( !empty( $this->restrict_datatypes ) )
        {
            $sql.=' AND d.id IN ('.join(',', $this->restrict_datatypes).')';
        }
        
        $sql .= ' ORDER BY d.weight, d.label';        
        
        $dbh =& pxdb::dbh();
        $query = $dbh->execute( $sql );
        while ( $row = $query->FetchRow() )
        {
            $dt[ $row['id'] ] = $row;
                                    
            // cache            
            $this->_cache_save( $row['id'], $priv, $row[ $priv ] ); 
        }
        
        return $dt;
    }
    /**
    * Return all privileges for a datatype in an array where key is the ugid field.
    *
    * @param integer datatype id
    * @return array
    * @access public
    * @static
    */    
    function getAllByDatatype( $dt )
    {
        $dbh = &pxdb::dbh();
        
        $sql = 'SELECT dp.*, r.name, r.datatype FROM datatype_privs dp '
              .' INNER JOIN records r on ( r.id=dp.ugid )'
              .' WHERE datatype_id='.$dt              
              .' ORDER BY r.datatype, r.name';
              
        $query = $dbh->execute( $sql );
        
        $privs = array();
        
        if ( 0 < $query->RowCount() )
        {
            while ( $row = $query->FetchRow() )
            {
                $privs[ $row['ugid'] ] = $row;
            }
        
            return $privs;
        } else {
            return false;
        }
    }
    
    /**
    * Checks if the priv is allowed 
    *
    * @param integer datatype id
    * @param string priv name
    * @return boolean
    */
    function is_allowed( $dt, $priv_name )
    {        
        $_cache = $this->_cache_check( $dt, $priv_name );
        if ( null!== $_cache )
        {
            return $_cache;
        } else {
            return $this->_lookup( $dt, $priv_name );
        }
    }
        
    /**
    * Looks up the priv in the database
    *
    * @param integer datatype_id
    * @param string priv name
    * @return boolean
    */
    function _lookup( $dt, $priv_name )
    {
        $priv_col = 'priv_'.$priv_name;
        
        $sql = 'SELECT '.$priv_col.' as `priv`'
              .' FROM datatype_privs dt'
              .' WHERE dt.datatype_id='.$dt
        ;
        
        if ( is_array( $this->ugid ) )
        {
            $sql.=' and dt.ugid IN ('.join(',' , $this->ugid) .')';
        } else {
            $sql.=' and dt.ugid='.$this->ugid;
        }
        
        // this gets trues by default
        $sql.= " ORDER BY $priv_col DESC LIMIT 1"; 
        
        $dbh = &pxdb::dbh();
        
        $query = $dbh->execute( $sql );
        
        if ( 0 < $query->RowCount() )
        {
            $row = $dbh->GetAll( $sql );        
            
            // cache
            $this->_cache_save( $dt, $priv_name, $row[0]['priv'] );        
            return $row[0]['priv'];
        } else {
            $this->_cache_save( $dt, $priv_name, 0 );
            return false;
        }
        
    }
    
    /**
    * Saves looked up values in memory
    *
    * @access private
    */    
    function _cache_save ( $dt, $priv, $val )
    {        
        $this->_cache[$dt][ $priv ] = $val; 
    }
    
    /**
    * Check if a priv, datatype combo is cached. Returns NULL if its not cached
    *
    * @param integer datatype id
    * @param string priv name
    * @retuarn integer or null
    */
    function _cache_check( $dt, $priv_name )
    {                                
        if ( isset( $this->_cache[$dt][$priv_name] ) )
        {                        
            return $this->_cache[$dt][$priv_name];
        } else {
            return NULL;
        }
    }
    
    /**
    * Returns all groups that do not have a privilege to the datatype
    *
    * @param integer datatype id
    * @param integer datatype id of the groups datatype
    */
    
    function getUnassignedGroups( $dt, $DATATYPE_GROUP )
    {             
        $dbh = &pxdb::dbh();        
        $sql = 'SELECT r.id as rec_id, r.name, dp.* '
              .' FROM records r'
              .' LEFT JOIN datatype_privs dp on ( r.id=dp.ugid and datatype_id='.$dt.')'
              .' WHERE r.datatype='.$DATATYPE_GROUP.' and dp.id IS NULL'
        ;
                
        $query = $dbh->execute( $sql );
        
        $groups = array();
        while ( $row = $query->FetchRow() )
        {
            $groups[ $row['rec_id'] ] = $row['name'];
        }
        
        return $groups;
        
        
    }
    
    /**
    * Grants the privileges in the privArray to the group on the specified datatype 
    *
    * @param integer datatype
    * @param integer group id
    * @param array key is the priv name (read|add|edit|delete|approve), value is 1|0
    * @access public    
    */
    function grant( $dt, $ugid, $privArray ) 
    {
    
        if ( pxdb_datatype_privs::_has_privs_defined( $dt, $ugid ) )
        {
            //update
            pxdb_datatype_privs::_update_row( $dt, $ugid, $privArray );            
        }
        else 
        {
            // insert
            pxdb_datatype_privs::_add_row( $dt, $ugid, $privArray );
        }
    }
    
    /**
    * Deletes all privileges to a dataype for a group
    *
    * @param integer datatype
    * @param integer group id    
    * @access public
    */
    function revoke_all( $dt, $ugid )
    {
        if ( empty( $dt ) || empty( $ugid ) )
        {
            trigger_error( 'datatype_privs::revoke_all, nothing to do EMPTY datatype_id and/or group id specified', E_USER_WARNING );
            return;
        }
        
        if ( pxdb_datatype_privs::_has_privs_defined( $dt, $ugid ) ) 
        {
            $sql = 'DELETE from datatype_privs'
                  ." WHERE datatype_id='$dt' and ugid='$ugid'"
            ;
            
            $dbh = &pxdb::dbh();
            return $dbh->execute( $sql );
        }
    }
    
    /**
    * Adds a new priv row to the database.
    * 
    * @param integer datatype
    * @param integer group id
    * @param array key is the priv name (read|add|edit|delete|approve), value is 1|0
    * @access private
    */
    function _add_row( $dt, $ugid, $privArray ) 
    {
        
        
        $privCols = pxdb_datatype_privs::get_priv_columns(); 
                
        $sql = "INSERT INTO datatype_privs VALUES ( '', '$dt', '$ugid' ";
     
        while( list(,$priv) = each( $privCols ) )
        {            
            $sql.= !empty( $privArray[ $priv ] ) ? ", 1" : ", 0"; 
        }
                
        $sql.=' )';

        $dbh = &pxdb::dbh();        
        return $dbh->execute( $sql );
    }
    
    /**
    * Adds a new priv row to the database.
    * 
    * @param integer datatype
    * @param integer group id
    * @param array key is the priv name (read|add|edit|delete|approve), value is 1|0
    * @access private
    */
    function _update_row( $dt, $ugid, $privArray ) {
        $dbh = &pxdb::dbh();
        
        $privCols = pxdb_datatype_privs::get_priv_columns(); 
                
        $sql = "UPDATE datatype_privs SET ";
     
        while( list(,$priv) = each( $privCols ) )
        {            
            $update = 'priv_'.$priv.'=';
            $update.= !empty( $privArray[ $priv ] ) ? "1" : "0";
            $updates[] = $update;
        }
        $sql.=' '.implode(', ' , $updates );
        $sql.=" WHERE datatype_id='$dt' and ugid='$ugid'";
        
        
        return $dbh->execute( $sql );
    }
    
    /**
    * Tests if the dt has a row in the database for the group specified
    *
    * @param integer datatype
    * @param integer group id
    * @return boolean
    * @access private
    */
    function _has_privs_defined( $dt, $ugid ) {
        $dbh = &pxdb::dbh();
                                 
        $sql = "SELECT * FROM datatype_privs WHERE datatype_id='$dt' and ugid='$ugid'";
        
        $query = $dbh->execute( $sql );
        
        if ( 0 < $query->RowCount() ) 
        {
            return true;
        }
        else 
        {
            return false;
        }
        
    }
    
    /**
    * Returns an array of the defined priv colums
    *
    * @return array 
    */
    function get_priv_columns() {
    
        $dbh = &pxdb::dbh();
        $priv_fields = $dbh->MetaColumnNames( 'datatype_privs' );
        while( list(,$field_name) = each( $priv_fields ) )
        {
            
            if ( 0 === strpos( $field_name, 'priv_' ) )
            {
                $defined_privs[] = str_replace ( 'priv_', '' ,$field_name );    
            }
        }
    
        return $defined_privs;                
    }
    
} // 

// OAM: this is here to automatically generate php constants based on the db fields
$defined_privs = pxdb_datatype_privs::get_priv_columns();
while ( list( ,$privc )  = each( $defined_privs ) )
{
    $namec = 'DATATYPE_PRIV_'.strtoupper( $privc );
    
    if ( !defined( "$namec" ) )
    {
        define( "$namec", $privc );
    }         
}

unset( $defined_privs );
unset ( $namec );
?>