<?php
// $Id: pxdb_prefs.class.php,v 1.3 2005/10/17 05:40:29 sandys1 Exp $

/**
* Class for managing preferences for an Syntax instance.
* This class is instantiated automatically by the pxdb class, but it may also be instantiated
* separately.  To maximize speed of instantiation, preferences are looked up and set on demand.
* (directly from/to the database).   Ideally there would be a system for allowing the class to read in
* a local copy of all preferences, if desired.
*
* This class expects to find a table in current-selected db. Table should
* be named pxdb_prefs and have these text (varchar / tinytext / whatever) columns:
*     pref_name
*     pref_key
*     pref_value
*
* @package pxdb
* @author Hans Lellelid <hans@velum.net>
* @version $Revision: 1.3 $
* @access public
*/
class pxdb_prefs
{

    /**
    * Whether preferences are case sensitive.
    * @var boolean
    * @access public
    */
    var $case_sensitive = false;

    /**
    * Whether or not environment (table existence) has been verified.
    * @var boolean
     */
    var $checked = false;

    /**
    * ADOdb Connection.
    * @var object
    * @access public
    */
    var $db;

    /**
    * Cache of preferences we lookup.
    * @var array
    * @access public
    */
    var $cache;

    /**
    * Whether to disable caching.
    * This may be necessary if you are writing pages that are modifying and
    * retrieving [modified] preferences in the same page.
    */
    var $disable_cache = false;

    /**
    * Constructor does nothing but set the database.
    * @access public
    */
    function pxdb_prefs()
    {
        $this->db = &pxdb::dbh();
    }

    /**
    * Attempts to return a reference to an already-instantiated prefs class.
    * Will create a new prefs class only if one does not already exist.
    * @return object pxdb_prefs or false on error.
    * @access public
    */
    function &singleton()
    {
        static $instance;

        if (!isset($instance))
        {
            $instance = new pxdb_prefs();
        }

        return $instance;
    }

    /**
    * Looks up a preference in the db.
    * Will look for a lowercase version of pref_name and pref_key unless $case_sensitive is true.  (note:
    * pref_name and pref_key will also have been made lowercase by set_pref() method)
    * @param string $pname The name of the preference to retrieve
    * @param mixed $pkey The [optional] key of the preference to retrieve
    * @return mixed Contents of the pref
    * @access public
    */
    function get_pref($pname, $pkey = null)
    {
        if (false !== ($pref = $this->_cached($pname, $pkey)))
        {
            // pref is cached
            return $pref;
        }
        else
        {
            // lookup in db 

            $pref = array();

            $sql = "SELECT * FROM pxdb_prefs WHERE pref_name = ".$this->db->qstr($this->_normalize($pname)).($pkey !== null ? " AND pref_key = ".$this->db->qstr($this->_normalize($pkey)) : "");

            $rs = &$this->db->Execute($sql);

            while (!$rs->EOF)
            {
                $key = $rs->fields['pref_key'];
                $value = $rs->fields['pref_value'];

                if ($key)
                {
                    $element = &$pref[$key];
                }
                else
                {
                    $element = &$pref;
                }

                // if we've already got one value for this key
                if (isset($element))
                {
                    if (is_array($element))
                    {
                        // if it's already an array than just append the new value
                        array_push($element, $value);
                    }
                    else
                    {
                        // make $this->$key an array of current value and new value
                        $arr = array($element, $value);
                        $element = $arr;
                    }
                }
                else
                {
                    // load into scalar class namespace
                    $element = $value;
                }

                $rs->MoveNext();

            } // while

            // if array is empty, then return an empty string (?)
            if (sizeof($pref) == 0)
            {
                $pref = '';
            }
            elseif (sizeof($pref) == 1)
            {
                // if there's only one element, then return it -- instead of an array.
                 $pref = array_pop($pref);
            }

            // Save results to speed up lookups for same pref later

            $this->_cache($pname, $pkey, $pref);

            return $pref;

        } // if pref cached

    }

    /**
    * Save pref looked up in db -- to speed up searches later.
    * @param string $pname Name of pref.
    * @param string $pkey Key of pref.
    * @param mixed $pval Value of pref (may be an array).
    * @access private
    */
    function _cache($pname, $pkey, $pval)
    {
        if (!$this->disable_cache)
        {
            $signature = $pname .'_'. $pkey;
            $this->cache["$pname"]["$pkey"] = $pval;
        }
    }

    /**
    * Get a matching cached pref (if any).
    * @param string $pname Name of pref.
    * @param string $pkey Key of pref.
    * @return mixed preferece found or false if there is no cached pref.
    * @access private
    */
    function _cached($pname, $pkey)
    {
        if (!$this->disable_cache)
        {
            $signature = $pname .'_'. $pkey;
            if (isset($this->cache["$pname"]["$pkey"]))
            {
                return $this->cache["$pname"]["$pkey"];
            }
            else
            {
                return false;
            }
        }
        else
        {
            return false;
        }
    }

    /**
    * Clears the preferences cached in memory.
    * Useful if you want to update & retrieve [updated] prefs in the same page.
    * In that case you may also just want to disable caching -- depending on how many
    * times you are going to access the same preferences.
    * @access public
    */
    function clear_cache()
    {
        $this->cache = array();
    }

    /**
    * Always returns an array of preferences desired.
    * get_pref() method will only return an array if there is more than one matching
    * preference.  This method will always return an array -- even if there is only
    * one matching pref.
    * @param string $pname The name of the preference to retrieve
    * @param string $pkey The [optional] key of the preference to retrieve
    * @return array The matching preference(s)
    * @access public
    */
    function get_array($pname, $pkey = null)
    {
        
        $pref = $this->get_pref($pname, $pkey);
        if (!is_array($pref))
        {
            $pref = array($pref);
        }
        return $pref;
    }

    /**
    * Returns a hash/associative array for a pref_name, and/or key
    * Get_array returns only the values in an array.
    * 
    * preference.  This method will always return an array -- even if there is only
    * one matching pref.
    * @param string $pname The name of the preference to retrieve
    * @param string $pkey The [optional] key of the preference to retrieve
    * @return associative array The matching preference(s)
    * @access public
    */
    function get_hash($pname, $pkey = null)
    {
        $sql = "SELECT pref_key, pref_value  FROM pxdb_prefs WHERE pref_name = ".$this->db->qstr($this->_normalize($pname)).($pkey !== null ? " AND pref_key = ".$this->db->qstr($this->_normalize($pkey)) : "");

        $rs = &$this->db->execute($sql);

        while( $val = $rs->FetchRow() )
        {
            $hash[ $val['pref_key'] ] = $val['pref_value'];
        }
        
        return $hash;
    }

    /**
    * Returns a list of values for a given preference.
    * @param string $pname The name of preference
    * @param string $pkey The preference key.
    * @param string $delim the delimter with which to separate multiple preferences.
    * @return string The $delim-separated string of preferences.
    * @access public
    */
    function get_list($pname, $pkey = null, $delim = ',')
    {

        $pref = $this->get_pref($pname, $pkey);
        if (is_array($pref))
        {
            return implode($delim, $pref);
        }
        else
        {
            return ($pref);
        }
    }

    /**
    * Returns a lowercase value unless $case_sensitive is true.
    * @param string $value The value that should be normalized.
    * @return string
    * @access private
    */
    function _normalize($value)
    {
        if (!$this->case_sensitive)
        {
            return strtolower($value);
        }
        else
        {
            return $value;
        }
    }


    /**
    * Removes a preferences from the database.
    * Matches on normalized pref_name and pref_key, unless pref_key is null in which case it removes all prefs that match pref_name.
    * @param string $pname Name of pref to delete.
    * @param string $pkey Key of pref to delete.
    * @param string $pval Value of pref to delete (in cases where there is more than one name/key match to a preference).
    * @access public
    */
    function del_pref($pname, $pkey = null, $pval = null)
    {
        $sql = "DELETE FROM pxdb_prefs WHERE pref_name = ".$this->db->qstr($this->_normalize($pname));

        if ($pkey !== null)
        {
            $sql .= " AND pref_key = ".$this->db->qstr($this->_normalize($pkey));
        }

        if ($pval !== null)
        {
            $sql .= " AND pref_value = ".$this->db->qstr($pval);
        }

        $this->db->Execute($sql);
    }

    /**
    * Sets (overwrites) a preference in the database.
    * This method calls the del_pref() and add_pref() methods.
    * @param string $pname The name of preference to set.
    * @param mixed $arg[1] Value of pref -- or key if arg[2] exists.
    * @param mixed $arg[2] Value for new pref.
    * @access public
    */
    function set_pref($pname)
    {
        if (func_num_args() == 3)
        {
            $pkey = func_get_arg(1);
            $pval = func_get_arg(2);
        }
        elseif (func_num_args() == 2)
        {
            $pkey = null;
            $pval = func_get_arg(1);
        }

        // 1) delete pref -- if it already exists.

        $this->del_pref($pname, $pkey);

        // 2) add the new preference

        $this->add_pref($pname, $pkey, $pval);
    }


    /**
    * Adds a preference to the database.
    * Will automatically lowercase the pref_name and pref_key unless $case_sensitive is true.
    * @param string $pname Name of preference to add.
    * @param string $arg[1] Value of pref -- or key if arg[2] exists.
    * @param string $arg[2] Value for new pref.
    * @access public
    */
    function add_pref($pname)
    {
        if (func_num_args() == 3)
        {
            $pkey = func_get_arg(1);
            $pval = func_get_arg(2);
        }
        elseif (func_num_args() == 2)
        {
            $pkey = null;
            $pval = func_get_arg(1);
        }
        // the value of the $pval may be an array
        if (is_array($pval))
        {
            // SFS while is faster
            while ( list( , $sub_pval) = each( $pval ) )
            {
                $sql = "DELETE FROM pxdb_prefs WHERE pref_name = ".$this->db->qstr($this->_normalize($pname))." AND pref_key = ".$this->db->qstr($this->_normalize($pkey))." AND pref_value = ".$this->db->qstr($sub_pval);
                $this->db->Execute($sql);

                $sql = "INSERT INTO pxdb_prefs (pref_name, pref_key, pref_value) VALUES (".$this->db->qstr($this->_normalize($pname)).", ".$this->db->qstr($this->_normalize($pkey)).", ".$this->db->qstr($sub_pval).")";
                $this->db->Execute($sql);
            }
        }
        else
        {
            $sql = "DELETE FROM pxdb_prefs WHERE pref_name = ".$this->db->qstr($this->_normalize($pname))." AND pref_key = ".$this->db->qstr($this->_normalize($pkey))." AND pref_value = ".$this->db->qstr($pval);
            $this->db->Execute($sql);

            $sql = "INSERT INTO pxdb_prefs (pref_name, pref_key, pref_value) VALUES (".$this->db->qstr($this->_normalize($pname)).", ".$this->db->qstr($this->_normalize($pkey)).", ".$this->db->qstr($pval).")";
            $this->db->Execute($sql);
        }
    }

    /**
    * This function sets default preferences.
    * It first checks to make sure there isn't a preference already loaded, if not,
    * it writes it to the database.
    * @param string $pname Name of preference to add.
    * @param string $arg[1] Value of pref -- or key if arg[2] exists.
    * @param string $arg[2] Value for new pref.
    * @access public
    */
    function set_default_pref($pname)
    {
        if (func_num_args() == 3)
        {
            $pkey = func_get_arg(1);
            $pval = func_get_arg(2);
        }
        elseif (func_num_args() == 2)
        {
            $pkey = null;
            $pval = func_get_arg(1);
        }

        // basically -- check to see if a matching name/key exists -- if not, then set it.

        if (!$this->get_pref($pname, $pkey))
        {
            // create a new one
            $this->add_pref($pname, $pkey, $pval);

        }
    }
}
?>