<?php
// $Id: pxdb_privs.class.php,v 1.6 2006/02/17 16:58:46 omerida Exp $

/**
* Static class for handling priv-getting methods.
* This class abstracts out the privilege system -- providing a unified API for setting both
* field privileges and record privileges -- i.e. field_priv and priv tables.
* 
* Other classes (e.g. pxdb_commit, pxdb_record, pxdb_typesfields) may have their own grant_priv() and get_*_priv()
* methods, but these simply make pxdb_privs:: requests.
* 
* Other classes may access the priv tables directly -- e.g. the _priv_filter_sql() method of the pxdb_collection
* class.  I'm unsure as to whether there is a compelling reason to have this method included here.
* 
* @version $Revision: 1.6 $
* @author Hans Lellelid <hans@velum.net>
* @since 0.0.4
*/
class pxdb_privs 
{
	/**
	* Gets all privs for a field / user-group ID.
	* @param string $type ('field, 'record')
	* @param integer $id
	* @param mixed $ugid Array or comma-separated list.
	* @return array
	* @access public
	*/
	function get($type, $id = null, $ugid = null)
	{				
		// normalize		
		if(is_array($ugid))
        {
            $ugid = implode(',', $ugid);
        }
		
        if (in_array('0', explode(',' , $ugid)))
        {
            // 0 is in the ugid array, do a LEFT join
            $jointype = 'LEFT';
        }
        else 
        {
            // be more efficient
            $jointype = 'INNER';
        }
        
        $jointype = 'LEFT';
        
		$id_col = pxdb_privs::_id_col($type);
		$table  = pxdb_privs::_table($type);
		
		$dbh = &pxdb::dbh();
				
		$restricts = array();		
		if ($id !== null)
        {
            $restricts[] = "p.$id_col = $id";
        }
        
		if ($ugid !== null) 
        {
            $restricts[] = "p.ugid IN ($ugid)";
        }
		
		// if both $id and $ugid are null we
		// need to build an assoc array with ugid
		// OAM NOTE: changing the left join to an inner join below causes lookups for
        // global privs (ugid=0) to fail
		$sql = "SELECT 	p.ugid,
						p.".pxdb_privs::_priv_col('read')." AS ".PXSQL_L."read".PXSQL_R.",
						p.".pxdb_privs::_priv_col('write')." AS ".PXSQL_L."write".PXSQL_R.",
						r.name as ".PXSQL_L."name".PXSQL_R."
				FROM 	$table p
						$jointype JOIN records r ON r.id = p.ugid
				";
						
		if (sizeof($restricts)) 
        {						
			$sql .=	' WHERE ' . implode(' AND ', $restricts);
		}
        
                

        $query = $dbh->execute($sql);
        $all_privs = array();
        while ($priv = $query->fetchRow())
        {            
            // oam  note - for field_privs $id is the field id
            $key = ('field' == $type) ? $id : $priv['ugid']; 
            $all_privs[$priv['ugid']] = $priv;            
        }
        
        return $all_privs;
        
	}
	
	/**
	* Is a particular field readable by a particular ugid?
	* @param string $type field or record
	* @param integer $id typesfield or reocord id
	* @param mixed $ugid array or commasep list of user/group ids
	* @return boolean
	* @access public
	*/
	function is_readable($type, $id = null, $ugid = null)
	{        
        return pxdb_privs::_has_priv($type, 'read', $id, $ugid); 
	}
	
	/**
	* Is a particular field writeable by a particular ugid?
	* @param string $type
	* @param integer $id
	* @param mixed $ugid
	* @return boolean
	* @access public
	*/
	function is_writeable($type, $id = null, $ugid = null)
	{   
        return pxdb_privs::_has_priv($type, 'write', $id, $ugid);		        
	}
		
    /**
    * Tests if a ugid has the priv_label passed, public api invokes this
    * method via is_readable, is_writeable
	* @param string $type
	* @param string $priv_label 'write', 'read'...
	* @param integer $id
	* @param mixed $ugid
    * @return boolean
    * @access private
    */
	function _has_priv($type, $priv_label, $id = null, $ugid = null) 
    {                    
        // get the priv array, key is the ugid
		$privs = pxdb_privs::get($type, $id, $ugid);
                
        if (false !== strpos($ugid, ','))
        {
            $ugids = explode(',', $ugid);
        } else if (!is_array($ugid))
        {
            $ugids = array($ugid);
        }
        else
        {
            $ugids = $ugid;
        }
        
        while (list(,$currUgid) = each($ugids))
        {         
            if (isset($privs[ $currUgid ][ $priv_label ]) && 1 == $privs[ $currUgid ][ $priv_label ])
            {               
                // found the priv!
                return true;                
            } 
        }
        
        // else if we made it this far, then the priv doesn't exits
        return false;
        
    }    
    
	/**
	* Grant priv(s) to a particular ugid for a particular rec/field id.
	* @param string $type
	* @param integer $id
	* @param mixed $ugid
	* @param string $priv
	* @access public
	*/
	function grant($type, $id, $ugid, $priv = 'read')
	{	
		
		if(!is_array($priv)) { $priv = explode(',',$priv); } //normalize
		// knows which field we are dealing with from $_imported array.				
		
		$id_col = pxdb_privs::_id_col($type);
		$table = pxdb_privs::_table($type);
		
		$dbh = &pxdb::dbh();
		
		$cols = array();
		// SFS while is faster
		while (list(, $p) = each($priv))
		{
			$cols[] = PXSQL_L . pxdb_privs::_priv_col($p) . PXSQL_R; // 'read' => [priv_read] (MSSQL)
		}
		
		if (sizeof($cols)) 
        {		
			$sql_cols = array();
			// SFS while is faster
			while (list(, $col) = each($cols))
			{
				$sql_cols[] = $col.' = 1'; // builds an array of elements like 'priv_read = 1'
			}
			
			$update_sql = implode(', ', $sql_cols);
			
			$sql = "UPDATE $table SET $update_sql WHERE $id_col = $id AND ugid = $ugid";
			$ok = $dbh->Execute($sql);
			if (!($ok && 0 < $dbh->Affected_Rows())) 
            {				
				$select_sql = implode(' AND ', $sql_cols);
				$sql = "SELECT COUNT(*) FROM $table WHERE $id_col = $id AND ugid = $ugid AND $select_sql";
				if (0 == $dbh->GetOne($sql))
                {
					$ins_cols = implode(',', $cols);
					$ins_vals = implode(',', array_pad(array(), sizeof($cols), 1));
					$sql = "INSERT INTO $table ($id_col,ugid,$ins_cols) VALUES ($id,$ugid,$ins_vals)";
					$dbh->Execute($sql);
				} 
			} // if update failed
		
		} // if(sizeof($uptdate_cols)
				
	} // grant()

	/**
	* Revoke privileges for a particular user to a particular 
	* If ugid is null, then all privs for record are revoked.  
	* @param string $type
	* @param integer $id
	* @param mixed $ugid
	* @param string $priv
	* @return boolean Success
	*/
	function revoke($type, $id = null, $ugid = null, $priv = null) 
	{		
		// normalize
		if ($priv!==null && !is_array($priv)) 
        {
            $priv = explode(',', $priv);
        }
				
		$id_col = pxdb_privs::_id_col($type);
		$table  = pxdb_privs::_table($type);

		$dbh = &pxdb::dbh();
		
		if($id === null && $ugid === null) 
        {
			trigger_error("Not allowing you to clear the entire $table table (no id or ugid specified).", E_USER_WARNING);
			return false;
		}
		
		$restricts = array();
		if (null !== $id)
        {
            $restricts[] = "$id_col = $id";
        }
        
		if (null !== $ugid) 
        {
            $restricts[] = "ugid IN ($ugid)";
        }
        
		if ($priv) 
        {
		    // SFS while is faster
			while (list(, $p) = each($priv))
			{
				if($p) $restricts[] = pxdb_privs::_priv_col($p) . ' = 1';				
			}
		}
		
		$sql = "DELETE FROM $table WHERE ".implode(' AND ', $restricts); 
		$ok = $dbh->Execute($sql);
		
		return $ok;		
	}	
	
	/**
	* Get ID column for type
	* @param string $type
	* @return string 'fid' for a field type or 'recid' otherwise
	* @access private
	*/
	function _id_col($type)
	{
		if ('field' == $type)
        {
            return 'fid';
        }
        else
        {
            return 'recid';
        }
	}
	
	/**
	* Get table for type
	* @param string $type
	* @return string 'field_priv' for field type or 'priv' otherwise
	* @access private
	*/
	function _table($type)
	{
		if ('field' == $type)
        {
            return 'field_priv';
        }
        else
        {
            return 'priv';
        }
	}
	
	/**
	* Privileges are stored in columns -- we can assume a systematic relationship
	* between the type of priv (e.g. 'read','write','add',etc.) and the column name.
	* HOWEVER, this function exists so that that systematic relationship is not forced.
	* @param string $priv
	* @return string col name
	* @access private
	*/
	function _priv_col($priv)
	{
		return 'priv_'.$priv;
	}
	
} // class

?>