<?php
// $Id: record_privs.class.php,v 1.3 2005/10/17 05:40:29 sandys1 Exp $
require_once(dirname(__FILE__).'/pxdb_privs.class.php');

/**
* This  class provides a cleaner API for working 
* with record related privileges by encapsulating 
* calls to the pxdb_privs class and adding more
* convenient methods.  Methods should be called statically
*
* @author Oscar Merida
*/

class record_privs
{

    /**
    * Returns an array of groups and their privileges to a record.  Includes
    * ugid 0, the global group.
    *
    * @param pxdb_record object
    * @return array
    * @access public
    */    
    function getAllGroups(&$record)    
    {
        if ($priv = $record->get_global_privs())
        {
            $merged[0] = $priv;
        }
        
        $group = $record->get_group_privs();
        
        // SFS is on a war for while
        if (true === is_array($group))
        {
            while (list($key, $privs) = each($group))
            {    
                $merged[$privs['ugid']] = $privs;   
            }
        }
        return $merged; 
    }
    
    /**
    * Grant privileges to a user or group for a given record
    *
    * @access public
    * @param int $record_id The ID of the record to which the user should get privs
    * @param int $ugid The ID of the user or group record that gets privs to the record
    * @param string $priv 'read' or 'write'
    */
    function grant($record_id, $ugid, $priv)
    {        
        if (record_privs::hasPriv($record_id, $ugid))
        {
            // true sets the priv to 1
            record_privs::update($record_id, $ugid, $priv, TRUE);
        }
        else
        {
            pxdb_privs::grant('record', $record_id, $ugid, $priv);
        }
    }
    
    /**
    * Remove privileges to a user or group for a given record
    *
    * @access public
    * @param int $record_id The ID of the record to which the user should be denied privs
    * @param int $ugid The ID of the user or group record that loses privs to the record
    * @param string $priv 'read' or 'write'
    */
    function revoke($record_id, $ugid, $priv)
    {        
        if (record_privs::hasPriv($record_id, $ugid))
        {
            // false sets the priv to 0
            record_privs::update($record_id, $ugid, $priv, FALSE); 
        }
        else
        {
            pxdb_privs::revoke('record', $record_id, $ugid, $priv);
        }
    }
    
    /**
    * Tests to see if a ugid already has an entry in privs for a given record
    *
    * @param integer $record_id
    * @param mixed $ugid Single ugid or array of ugids
    * @param string $priv 'read' or 'write'
    * @return boolean
    * @access public
    */
    function hasPriv($record_id, $ugid, $priv)
    {
        // sanity check
        if (true === empty($record_id) || false === is_numeric($record_id))
        {
            trigger_error('You must provide record_privs::hasPrivs() a valid record ID', E_USER_WARNING);
            return false;
        }
        if (true === empty($ugid) && 0 !== $ugid)
        {
            trigger_error('You must provide record_privs::hasPrivs() a valid user or group ID', E_USER_WARNING);
            return false;
        }
        if ('write' != $priv && 'read' != $priv)
        {
            trigger_error('You must provide record_privs::hasPrivs() a valid privilege of either "read" or "write"', E_USER_WARNING);
            return false;
        }
        
        // logic
        if (true === is_array($ugid))
        {
            $ugids = join(', ', $ugid);
            $sql = "
SELECT * FROM priv
WHERE
    recid='$record_id'
    AND ugid IN ($ugids)
    AND priv_$priv = 1
            ";
        }
        else
        {
             $sql = "
SELECT * FROM priv
WHERE
    recid='$record_id'
    AND ugid = '$ugid'
    AND priv_$priv = 1
            ";
        }
        
        $dbh = &pxdb::dbh();
        $query = $dbh->execute($sql);
                
        if (0 < $query->rowcount())
        {
            return true;
        }
        else
        {
            return false;   
        }
    }
    
    /**
    * Updates a priv for a given record and user/group id
    *
    * @param integer $record_id
    * @param int $ugid Single ugid
    * @param string $priv 'read' or 'write'
    * @param int $setting either 0 or 1 (shoulda used a boolean)
    * @return boolean
    * @access public
    */
    function update($record_id, $ugid, $priv, $setting)
    {
        // since privs are either on or off
        $setting = ($setting==0) ? 0 : 1;
        
        $field = 'priv_'.$priv;
        
        $sql = "UPDATE priv SET $field='$setting'"
               ." WHERE (recid='$record_id' and ugid='$ugid')"
        ;
        
        $dbh = &pxdb::dbh();
        $query = $dbh->execute($sql);
    }
} // end class
?>