<?php
/**
 * General class for handling Syntax CMS-level events related to sections.
 * Initial use is for creating v. 1.3+ creation of section paths.
 *
 * @author Sandy Smith
 * @date 12/14/04
 */
class syntax_section_handler extends pxdb_event_listener
{
    /**
    * @var object ADOdb database handler
    * @access private
    */
    var $dbh;
    
    /**
     * sets triggers to listen for
     *
     * @access public
     *
     */
    function init()
    {
        $this->dbh =& pxdb::dbh();
        $this->addTrigger(SYNTAX_EVENT_INSERT);
        $this->addTrigger(SYNTAX_EVENT_UPDATE);
        $this->addTrigger(SYNTAX_EVENT_PUBLISH);
    }
    
    /**
     * Dispatches incoming trigger to appropriate handler
     *
     * @access public
     * @return boolean false if failure, true otherwise
     * @param string $trigger Trigger (defined by constants)
     * @param array $parameters Associative array of data from event
     *
     */
    function execute($trigger, $parameters)
    {
        // can't assume that we'll always have either id or datatype parameter
        if (empty($parameters['id']))
        {
            trigger_error('No Record ID passed for synchronization', E_USER_WARNING);
            return false;
        }
        
        if (empty($parameters['datatype']))
        {            
            // lookup from db
            $sql = 'select datatype from records where id = '.$parameters['id'];
            
            $rows = $this->dbh->GetAll($sql);
            
            $row = array_pop($rows);
            
            if (!empty($row['datatype']))
            {
                $parameters['datatype'] = $row['datatype'];
            } else {
                return false;
            }            
        }
        
        if (DT_NODE == $parameters['datatype'])
        {
            switch ($trigger)
            {
                case SYNTAX_EVENT_INSERT:
                    $this->updateSectionPath($parameters);                             
                break;
                
                case SYNTAX_EVENT_UPDATE:
                    $this->updateSectionPath($parameters);
                break;
                
                case SYNTAX_EVENT_PUBLISH:
                    $this->updateSectionPath($parameters);
                break;
                
                default:
                    trigger_error('Unrecognized event trigger: '.$trigger, E_USER_WARNING);
                    return false;
                break;
            }            
        }
    }
    
    /**
    * Recurses as needed to fix duplicate sibling and ensure uniqueness
    *
    * @access private
    * @return string of url_id once fixed (or original if OK)
    * @param string $url_id The candidate url_id
    * @param array $siblings
    */
    function _correctDuplicateSibling($url_id, $siblings)
    {
        if (true === in_array($url_id, $siblings))
        {
            // there's a duplicate, so intelligently update the end of it to
            // increment any numbers
            if (preg_match("/\d+$/", $url_id, $matches))
            {
                $num = (int) $matches[1];
                $num++;
            }
            else
            {
                $num = 1;
            }
            $url_id = preg_replace("/(\d+)?$/", $num, $url_id);
            return $this->_correctDuplicateSibling($url_id, $siblings);
        }
        else
        {
            return $url_id;
        }
    }
    
    /**
    * Checks to see if a section is orphan (no parent_
    *
    * @access public
    * @return boolean
    * @param int $id
    */
    function isOrphanSection($id)
    {
    	$sql = "
SELECT parent
FROM noderel
WHERE
    noderelid = " . NODEREL_SECTION_TO_SECTION . "
    AND child = $id
        ";
        $result = $this->dbh->GetOne($sql);
        if ($result)
        {
            return false;
        }
        else
        {
            return true;
        }
    }
    
    /**
    * Gets the siblings for a node
    *
    * @access public
    * @return array Array of url_ids
    * @param int $id
    */
    function getSiblings($id)
    {
    	if (true === $this->isOrphanSection($id))
    	{
            $sql = "
SELECT r.idnum, n.parent
FROM records r
LEFT JOIN noderel n
    ON n.child = r.id
WHERE
    n.parent IS NULL
    AND r.datatype = " . DT_NODE . "
    AND r.id != $id
            ";
            $siblings = $this->dbh->GetCol($sql);
        }
        else
        {
            $sql = "
SELECT r.idnum
FROM records r, noderel n, noderel nsibs
WHERE
    n.child = $id
    AND n.noderelid = " . NODEREL_SECTION_TO_SECTION . "
    AND nsibs.parent = n.parent
    AND nsibs.child != $id
    AND nsibs.noderelid = " . NODEREL_SECTION_TO_SECTION . "
    AND r.id = nsibs.child
            ";
            $siblings = $this->dbh->GetCol($sql);
        }
    	return $siblings;
    }
    
    /**
     * Updates the section path and checks for duplicate siblings and modifies the url_id appropriately
     *
     * @access public
     * @return boolean success or failure
     * @param array $parameters
     *
     */
    function updateSectionPath($parameters)
    {
        // First, get the current url_id
        $sql = "SELECT idnum FROM records WHERE id = $parameters[id]";
        $current_url_id = $this->dbh->GetOne($sql);
        
        // ... and the current url_path
        $sql = "SELECT desc3 FROM records WHERE id = $parameters[id]";
        $current_url_path = $this->dbh->GetOne($sql);
        
        // Second, check for duplicate siblings and modify current url_id if needed
        $siblings = $this->getSiblings($parameters['id']);
        $current_url_id = $this->_correctDuplicateSibling($current_url_id, $siblings);
        
        // copy so it can be replaced in the SQL on subsequent iterations
        $id = $parameters['id'];
        $url_id = $current_url_id;
        $url_ids = array();
        
        // we'll need this counter to build the path
        $i = 0;
        while ($url_id)
        {
            $url_ids[$i] = $url_id;
            $sql = "
SELECT
    r.id, r.idnum
FROM
    records r, noderel n
WHERE
    n.child = $id
    AND n.noderelid = " . NODEREL_SECTION_TO_SECTION . "
    AND r.id = n.parent
            ";
            $parent_info = $this->dbh->GetRow($sql);
            if ($parent_info)
            {
                $id = $parent_info['id'];
                $url_id  = $parent_info['idnum'];
                $i++;
            }
            else
            {
                $url_id = false;
            }
        }

        // let's build it from the top
        $url_ids = array_reverse($url_ids);
        
        // now the path
        $path = implode('/', $url_ids);
        
        // now update processed $url_id
        $sql = "
UPDATE records
SET idnum = '$current_url_id'
WHERE id = $parameters[id]
        ";
        if (! $this->dbh->Execute($sql))
        {
            trigger_error('Error writing new url_id', E_USER_WARNING);
            return false;
        }
        
        if ('' !== $current_url_path && null !== $current_url_path)
        {
            // now update any children's paths every time it occurs
            $sql = "
UPDATE records
SET desc3 = REPLACE(desc3, '$current_url_path/', '$path/')
WHERE
    desc3 LIKE '$current_url_path%'
    AND datatype = " . DT_NODE;
            if (! $this->dbh->Execute($sql))
            {
                trigger_error('Error updating url_path for children', E_USER_WARNING);
                return false;
            }
            // now update the record itself
            $sql = "
UPDATE records
SET desc3 = '$path'
WHERE id = $parameters[id]
            ";
            if (! $this->dbh->Execute($sql))
            {
                trigger_error('Error updating url_path for current record only after updating children', E_USER_WARNING);
                return false;
            }
        }
        else
        {
            $sql = "
UPDATE records
SET desc3 = '$path'
WHERE id = $parameters[id]
            ";
            if (! $this->dbh->Execute($sql))
            {
                trigger_error('Error updating url_path for current record only', E_USER_WARNING);
                return false;
            }
        }
        return true;
    }
    
}
?>