<?php
// $Id: ListWidget.class.php,v 1.10 2005/08/10 04:14:09 sandys1 Exp $

/**
* A class to facilitate the display of form elements containing dynamic, list-type information.
*
* This class is useful for any project that uses db-driven forms to gather user input.  In particular 
* if these forms use any sort of list-type data (collections of radio buttons, checkboxes, or select lists).
*
* This class was built to help enable better tiered development (MVC or whatever), by separating out the 
* actual display of [usually db-derived] list data from the logic that is needed to build the lists.
*
* The basic approach to using this class is:
*
*        1 - build an array of values that need to be displayed
*        2 - create a new instance of this class, specifying the array name, form element name, 
*        and (optionally) the name of the variable that holds the currently-selected value(s)
*        3 - choose from a few different ways of displaying the data (selectlist, radio buttons, 
*        checkboxes)
*
* ------------------------------------------------------------------------------------------------------
* A quick example of how to use this class to build form element(s) (select list, checkboxes, radio 
* buttons) from database objects:
*
* // 1) build an array with needed values from a db (or wherever)
* $Q = mysql_query("SELECT id, name FROM records WHERE datatype=5");
* for($i=0; $R=mysql_fetch_array($Q); $i++) {
*    $docs[$i][value] = $R[id];
*    $docs[$i][label] = $R[name];
*    // note: we could do $docs[] = $R[name] (1-dim array) if we wanted both label & value to be identical.
* }
*
* // 2) create an instance of this class
* $docList = new listWidget($docs, "doc"); // default for currently-selected items here is $doc
* 
* // 3) choose a way to display the data
* 
* <%= $docList->selectList() %> <!-- as a pull-down select list -->
* <!-- or -->
* <%= $docList->selectList(5) %> <!-- multi-select list -->
* <!-- or -->
* <%= $docList->checkBoxesTable(2) %> <!-- a 2-column table of checkboxes -->
* <!-- or -->
* <%= $docList->radioButtonsTable(2) %> <!-- a 2-column table of radio buttons (single select) -->
* <!-- etc. -->
* 
* ------------------------------------------------------------------------------------------------------
* There are a number of ways that this class can be used, some much simpler (and less customizable) than 
* others. Using the $docList example above, here are some more ways that a table of checkboxes can 
* be constructed:
*
* // 1) here's a more complex way of building the table
*    
* $docList->buildTable($docList->checkBoxes("onClick='validateCheckBox()'"), 2);
*
* // 2) here's an *even more* complex way of building the table -- more designer friendly.
*    
* <% $docList->listToTable(2); // creates a 2-col data table %>
* <table border="0" cellspacing="0" cellpadding="0">
* <% while($row = $docList->getRow()) { %>
*    <tr>
*    <% foreach($row as $col) { %>
*        <td class=generic><%=$col%></td>
*    <% } %>
*    </tr>
* <% } %>
* </table>
*
* ------------------------------------------------------------------------------------------------------
* It's good to be aware that there are related options out there for those who would like to have more
* in the way of form building.  This class *is not* a form building class; conceivably it could be used
* in conjunction w/ other tools to build forms.  If you are looking instead for a form-building class,
* you may want to consider:
*
* http://www.zend.com/codex.php?id=320&single=1
* http://delfs.dk/QuickForm.phps
* http://freshmeat.net/projects/multipageforms/
* http://freshmeat.net/projects/htmlformsgenerationandvalidation/
*
* I've looked at several of these packages in more detail -- some neat stuff.  The important difference
* between these tools and this class is that this class is an "list widget" creator -- with an emphasis
* on *list*.  This class is intended to make interactive form elements from array data -- in some
* cases that means this class simply creates form elements (e.g. select lists) but in others it is more
* involved -- e.g. creating tables of checkboxes with the same element name, etc. New methods to this
* class would involve new ways of creating interactive elements from lists of data.
* 
* Note:  this class can be very tastily combined with the f1db_filters class.
* ------------------------------------------------------------------------------------------------------
* Changelog:
* 1.1.2 - SFS 4/23/03 - Fixed XHTML syntax in checkBoxes() and radioButtons() methods
*   (missing a space, and legacy syntax, which broke in hyper-compliant browsers when a
*   checkbox was selected by default)
* 1.1.1 - Fixed duplicate labels in radioButtonsTable() method. ;-)
* 1.1.0 - Added support for using keys besides "value" and "label" in multi-dimensional arrays: added
*            setArrayKeys() method, added default values for $label_key and $value_key and replaced all
*            hard-coded references to 'value' / 'label'
* 1.0.4 - Added Greg Schnippel's fix: is_selected checks to see if element is not empty
* 1.0.3 - Added support for 'value' attribute being key of array (only for non-numeric keys)
* 1.0.2 - Fixed duplicate labels in radioButtonsTable() method.
* 1.0.1 - Added method setUseBrackets() to indicate whether '[]' should be added to array element names
*
* ------------------------------------------------------------------------------------------------------
* @version    1.1.2
* @author    Hans Lellelid
* @author   Sandy Smith
*/

class ListWidget
{
    /**
    * @access private
    */
    var $data;

    /**
    * @access private
    */
    var $selected;

    /**
    * @access private
    */
    var $debug;

    /**
    * @access private
    */
    var $css_type;

    /**
    * @access private
    */
    var $css_style;

    /**
    * @access private
    */
    var $element_name;
    
    /**
    * @access private
    */
    var $raw_data;
    
    /**
    * @access private
    */
    var $table_data;

    /**
    * @access private
    */
    var $default_element;    
    
    /**
    * @access private
    */
    var $use_brackets;    

    /** 
    * @param Array $mainList The array containing the data to display.
    * @param string $elementName The name of the form element -- whether select list, checkboxes, radio buttons, etc.
    * @param mixed $currSelected The currently selected values of the array we passed.
    */
    function ListWidget($mainList, $elementName, $currSelected = false) 
    {

        // defaults
        $this->debug = false;
        $this->use_brackets = true;
        
        $this->value_key = 'value'; // these can be overriden by calling setArrayKeys() method.
        $this->label_key = 'label';
        
        $this->data = $mainList;
        $this->element_name = $elementName;
        
        if ($currSelected === false) 
        {
            /*
            OAM removed reliance on register globals, check in $_REQUEST array instead            
            */
            if ( !empty( $_REQUEST[ $elementName ] ) )
            {
                $this->selected = $_REQUEST[ $elementName ];
            }
            
        }
        else
        {
            $this->selected = $currSelected;
        }
                        
    }

    /**
    * Sets whether array element form names use brackets (e.g. in HTML name="myarr[]").
    *
    * This is set to true by default.
    *
    * @param bool $bit The bit to set use_brackets to
    */
    function setUseBrackets($bit = true) 
    {
        $this->use_brackets = $bit;
    }

    /**
    * Sets the keys to use in the given array.
    *
    * If the array passed to listWidget does not correspond to the
    * 'value' => ? 'label' => ? default, you can set the keys that it should
    * use here. e.g by passing an array like
    * array('value' => 'id', 'label' => 'name')
    *
    * @param array $keys a 2-dimensional array with new keys to use for value & label.
    */
    function setArrayKeys($keys) 
    {

        $this->value_key = $keys['value'];
        $this->label_key = $keys['label'];                
    
    }
    
    /**
    * Returns the number of elements that compose the data for this list.
    *
    * @return int Number of elements in data array.
    */
    function getSize() 
    {        
        return(count($this->data));        
    }
    
    /**
    * Sets the CSS style text (or class) that will be used by the elements
    * this class creates.
    * 
    * @param string $css The style or class that will be specified.
    */
    function setCSS($css) 
    {
        if (preg_match('/[\s]|[:]/', $css)) {
            $this->css_type = 'style';
        } else {        
            $this->css_type = 'class';
        }        
        $this->css_style = $css;
    }
    
    /**
    * This function allows a default (first) element ot be added to the array --
    * for select list in particular.
    *
    * @param $element The default (first) element that is being added.
    */
    function setDefaultElement($element='') 
    {
        $this->default_element = $element;
    }
    
    /**
    * This function determins whether or not a passed value is selected.
    *
    * This function is able to correctly determine whether being selected means
    * being included in an array or whether it means equalling a scalar.
    * 
    * @access private
    * @param mixed $val contains either a scalar or array to be compared against $this->selected
    * @return bool True or False
    */    
    function isSelected($val) 
    {
        
        if ($this->debug)
        {
            echo "<br><font color=green>isSelected: </font>";
            var_dump($val);
            echo " <i>[being compared against]</i> ";
            var_dump($this->selected);            
            echo '<hr>';
        }            
        
        if (is_array($this->selected))
        {
            return in_array($val, $this->selected);
        }
        else
        {
            // return ((!empty($val) && $val !== 0) && $val == $this->selected);
            // SFS I don't see any reason 0 couldn't be a valid value
            // OM: made the final part below === and cast to string so that you can have
            //     a value '' and 0 in the same option list
            return ('' !== $val && null !== $val) && (string) $val === (string) $this->selected;
        }
    }
    
    /**
    * Builds a select list from the class' data array.
    *
    * @param int $size The size (rows) of the select list.
    * @param string $additional_params JavaScript, etc. that should be addded to the <select> element.
    * @param bool $single if set to true forces the select list to be single select
    * @return string The select list HTML form element.
    */
    function selectList($size = 0, $additional_params = '', $single = false) 
    {
            
        if (1 < $size && false === $single)
        {   
            $multiple = true;
        }
        else
        {
            $multiple = false;
        }
        
        $sel = '<select name="' . $this->element_name . ($multiple && $this->use_brackets ? '[]' : '') . '"';
        if ($size)
        {
            $sel .= " size=\"$size\"";
        }
        if (true === $multiple)
        {
            $sel .= ' multiple';
        }
        if ($this->css_style)
        {
            $sel .= ' ' . $this->css_type . '="' . $this->css_style . '"';        
        }
        if ($additional_params)
        {
            $sel .= " $additional_params";
        }
        $sel .= ">\n";
        
        if ($this->default_element)
        {
            if (is_array($this->default_element))
            {
                $fvalue = $this->default_element['value'];
                $flabel = $this->default_element['label'];            
            }
            else
            {
                $fvalue = $this->default_element;
                $flabel = $this->default_element;                
            }
            $sel .= "<option value=\"$fvalue\"" . ($this->isSelected($fvalue) ? ' selected="selected"' : '') . ">$flabel</option>\n";
        }
    
        if ( 0 < count( $this->data ) )
        {
            $data = $this->data;
            while (list($key, $element) = each($data))
            {                        
                if (true === is_array($element))
                {
                    $value = (true === isset($element[$this->value_key]))
                        ? $element[$this->value_key] : null;
                    $label = (true === isset($element[$this->label_key]))
                        ? $element[$this->label_key] : null;
                }
                elseif (false === is_numeric($key))
                {
                    $value = $key;
                    $label = $element;
                }
                else
                {
                    $value = $element;
                    $label = $element;
                }            
                $sel .= "<option value=\"$value\"" . ($this->isSelected($value) ? ' selected="selected"' : '') . ">$label</option>\n";                
            }
        }
        $sel .= "</select>\n";
        
        return ($sel);
    }

    /**
    * This function builds an array of checkbox form elements.
    *
    * This function is designed to be used in conjunction with buildTable() or
    * with the listToTable() & getRow() methods to build usable HTML input
    * elements.
    *
    * @param $additional_params Any parameters that need to be passed to the form element.
    * @param $id_prepend Generates input id based on value; if this is "region" and the value is "3", the input element will contain id="region3"
    * @return array An array of checkboxes and labels
    */
    function checkBoxes($additional_params = '', $id_prepend = null) 
    {
        if (null !== $id_prepend)
        {
            if (0 === preg_match('/^[A-Z]/i', $id_prepend))
            {
                trigger_error('You must begin id attributes with a letter.', E_USER_WARNING);
                return false;
            }
        }
        // an array to store the checkbox elements in -- we probably will want to post-process this with a listToTable function
        $rawlist = array();
        
        $data = $this->data;
        
        if ( 0 < count($data) )
        {
            while (list($key, $element) = each($data))
            {
                if (true === is_array($element))
                {
                    $value = (string) $element[$this->value_key];
                    $label = (string) $element[$this->label_key];
                }
                elseif (false === is_numeric($key))
                {
                    $value = (string) $key;
                    $label = (string) $element;
                }
                else
                {
                    $value = (string) $element;
                    $label = (string) $element;
                }            
                
                $el = '<input type="checkbox"';
                if ('' != $value && null !== $id_prepend)
                {
                    $el .= ' id="' . $id_prepend . $value . '"';
                }
                $el .= 'name="' . $this->element_name . ($this->use_brackets ? '[]' : '') . '" value="' . $value . '"';
                $el .= ('' == $additional_params ? ' ' . $additional_params : '');
                $el .= ($this->css_style ? ' ' . $this->css_type . '="' . $this->css_style . '"' : '');
                $el .= ($this->isSelected($value) ? ' checked="checked"' : '');
                $el .= ' />';
                $el .= ($this->css_style ? '<span ' . $this->css_type . '="' . $this->css_style . '">' : '');
                $el .= $label . ($this->css_style ? '</span>' : '');
                $el .= "\n";
                
                $rawlist[] = $el;
                
            }
        }
        
        // set the class var
        $this->raw_data = $rawlist;
            
        // and return it
        return ($rawlist);
                                
    }
    
    /**
    * Simplifies the process of building a table of checkboxes.
    *
    * @param int $cols The number of columns for the table.
    * @param string $elementparams Any parameters that need to be inserted into the checkbox form element
    * @param string $tableparams Any parameters for the table cells.
    * @return string The actual HTML for the table we built -- complete with form elements, labels, etc.
    */
    function checkBoxesTable($cols, $elementparams = '', $tableparams = '') 
    {
    
        return $this->buildTable($this->checkBoxes($elementparams), $cols, $tableparams);
    
    }
    
    /**
    * This function builds an array of radio button form elements.
    *
    * This function is designed to be used in conjunction with buildTable() or with the listToTable() & getRow() methods to
    * build usable HTML input elements.
    *
    * @param string $additional_params Any additional HTML parameters that need to be inserted in the radio button element (JavaScript, etc.)
    * @return Array An array of radio buttons and labels.
    */    
    function radioButtons($additional_params = '') 
    {
        // an array to store the checkbox elements in -- we probably will want to post-process this with a listToTable function
        $rawlist = array();
        
        $data = $this->data;
        while (list($key, $element) = each($data))
        {
            //print "<br><font color=blue>$key => $element</font>";
            
            if (true === is_array($element))
            {
                $value = $element[$this->value_key];
                $label = $element[$this->label_key];
            }
            elseif (false === is_numeric($key))
            {
                $value = $key;
                $label = $element;
            }
            else
            {
                $value = $element;
                $label = $element;
            }
            
            $el = '<input type="radio" name="' . $this->element_name . "\" value=\"$value\" $additional_params";
            $el .= ($this->css_style ? ' ' . $this->css_type . '="' . $this->css_style . '"' : '');
            $el .= ($this->isSelected($value) ? ' checked="checked"' : '');
            $el .= " />";
            $el .= ($this->css_style ? '<span ' . $this->css_type . '="' . $this->css_style . '">' : '');
            $el .= $label;
            $el .= ($this->css_style ? '</span>' : '');
            $el .= "\n";
        
            $rawlist[] = $el;
            
        }
        
        // set the class var
        $this->raw_data = $rawlist;
        
        // and return it
        return ($rawlist);
    
    }
    
    /**
    * Simplifies the process of building a table of radio buttons.
    *
    * @param int $cols The number of columns for the table.
    * @param string $elementparams Any parameters that need to be inserted into the radio button form element
    * @param string $tableparams Any parameters for the table cells.
    * @return string The actual HTML for the table we built -- complete with form elements, labels, etc.
    */
    function radioButtonsTable($cols, $elementparams = '', $tableparams = '') 
    {
        return $this->buildTable($this->radioButtons($elementparams), $cols, $tableparams);
    }
    
    /**
    * This function just provides a quick solution to the more tedious listToTable() & getRow() combination.
    *
    * Note: there are even easier ways to build tables of radio buttons/checkboxes:  radioButtonsTable() and checkBoxesTable() methods.
    *
    * @param Array $data 1-dimensional array that we want to display.
    * @param int $cols The number of columns to build.
    * @param string $tableparams Any additional parameters for the table.
    * @return string HTML table populated with the elements of $data param.
    */
    function buildTable($data, $cols, $tableparams = '') 
    {
        if (!$this->data)
        {
            return null;
        }
        
        $this->listToTable($cols);
        
        $table = "<table border=\"0\" $tableparams>\n";
                
        while ($row = $this->getRow())
        {
            // grab each row from the 2D array. 
            $table .= "<tr>\n";
            while (list(, $col) = each($row))
            {
                $table .= '<td';
                $table .= ((true === isset($this->css_style) && true === isset($this->css_type)) ? ' ' . $this->css_type . '="' . $this->css_style . '"' : '');
                $table .= ">$col</td>\n";
            }
            $table .= "</tr>\n";
        }
        
        $table .= "</table>\n";
    
        return $table;
    
    }
    
    /**
    * This function gets a row of a 2-D array.
    *
    * Optional number of parameters: can be called in any of these ways:
    *         getRow($tabledata, 3)
    *         getRow($tabledata)
    *         getRow(3)
    *         getRow()
    *
    * @param mixed $param1
    * @param mixed $param2
    * @return array Returns the row -- an array of size [number of table columns]
    */    
    function getRow($param1 = '', $param2 = '') 
    {
        if (true === is_numeric($param1))
        {
            $selected_row = (int) $param1;
        }
        elseif (true === is_numeric($param2))
        {
            $selected_row = (int) $param2;
        }
        else
        {
            $selected_row = -1; // default
        }
        
        if ($param1 && (false === is_numeric($param1)))
        {
            $table = $param1;            
        }
        else
        {
            // use the class variable for table data by default
            $table = $this->table_data;
        }
        
        // This static variable lets one do something like while(getRow($table)) { }
        static $current_row = 0;    

        if ($selected_row >= 0)
        {
            $current_row = $selected_row;
        }

        // This code calculates table dimensions
        $maximum_row = 0;
        $maximum_column = 0;

        // table is defined as columns by rows
        while (list($column, $column_data) = each($table))
        {
            while (list($row, $row_data) = each($column_data))
            {
                if ($row > $maximum_row)
                {
                    $maximum_row = $row;
                }
            }
            $maximum_column = $column;
        }
        
        // table size: columns = maximum_column+1, rows = maximum_row+1

        if ($current_row > $maximum_row)
        {
            $current_row = 0; // reset curr_row
            return 0;
        }
        else
        {
            $row_data = array(); // blow out previous value
            for ($column = 0; $column <= $maximum_column; $column++)
            {
                for ($row = 0; $row <= $maximum_row; $row++)
                {
                    if ($row == $current_row)
                    {
                        if ( isset($table[$column][$row]) )
                        {
                            $row_data[] = $table[$column][$row];
                        }
                        else
                        {
                            $row_data[] = null;
                        }
                    }
                }
            }
            // Increment the static variable for next time this function is called.
            $current_row++;
            return $row_data;
        }
    }

    /**
    * This is the function that converts a 1-D array into a 2-D array (table).
    *
    * @access public
    * @param mixed $param1 Either the name of an array of data -- or the number of columns.
    * @param mixed $param2 Possibly num of columns.
    * @return Array A 2-D array of the data represented in $data param.
    */
    function listToTable($param1 = '', $param2 = '') 
    {
        if (true === is_numeric($param1))
        {
            $columns = (int) $param1;
        }
        elseif (true === is_numeric($param2))
        {
            $columns = (int) $param2;
        }
        else
        {
            $columns = 1; // default
        }
        
        if ($param1 && (false === is_numeric($param1)))
        {
            $data = $param1;        
        }
        else
        {
            $data = $this->raw_data;    // use the class variable for table data by default
        }

        // $columns stores the number of columns we want to break this
        //  array into.
        // $rows will be calculated based on length
        // of data and number of columns

        //  Basically what this function does is convert
        //  a single dimensional array into a 2-dimensional array

        $rows = ceil(sizeof($data) / $columns);

        for ($column = 0; $column < $columns; $column++)
        {
            // loop through the columns
            for ($row = 0; $row < $rows; $row++)
            {
                // loop through the rows
                $key = $row + $column * $rows;
                if (isset($data[$key]))
                {
                    $table[$column][$row] = $data[$key];
                }
            }
        }
        $this->table_data = $table;

        return $table;    
    }
}
?>
