<?php
// $Id: path_persist.class.php,v 1.3 2005/06/03 21:14:28 omerida Exp $

/**
* This class manages a user's session within an environment defined by a "path".
*
* The basic idea is to have a class that remembers what you are doing on a given page (path) and
* allows you to return to exactly that page (with all the same data entered, checkboxes clicked, etc.) without
* writing to a database.  The way to do this in PHP is with sessions -- and this is a class that makes that easy.
* It's difficult to provide a synopsis for using this class without showing you what I built this for -- which
* was specifically to handle storing of current page info within a system of tabs.
*
* The instance of this class needs to be registered as a session var in order to make
* the data persist.  In particular, it captures post/get variables and stores them, allowing
* apps to "remember" where someone left off making changes, etc.  A path is essentially any
* "area" or "place" in your web application that you want to treat as distinct.  The "path" describes
* the heirarchical organization of your application.  E.g. Within the a section called "World" there
* might be a section called "North America" -- this would be represted as "World:North America".
*
* This path-approach was designed to handle tabbed-environments.
*
* While this class is certainly stand-alone, it may make sense to combine it with something like
* the tabset class -- and you may want to use a JavaScript system for moving from one tab to another.
*
* Use it like this:
* -----------------
*
* # SETTING IT UP
* # -------------
*
* // restrict this session info to only this domain
* ini_set('session.cookie_domain', $SERVER_NAME);
*
* session_start();
*
* // also, it's a good idea to make your object variable name unique
* // e.g. $f1ow_session in this case -- that eliminates chance of sessions
* // becoming confused between other sites you build that use the same technology.
*
* if (!session_is_registered('f1ow_session'))
{
*
*    session_register('f1ow_session');
*
*    $f1ow_session = new path_persist();
*
*    // Set any default parameters for this object -- e.g. the default path.
*    $f1ow_session->set_default_param('database', 'sync_intra');
*    $f1ow_session->set_default_param('path', 'object:fields');
*
*    // set any rewrite rules for paths (e.g. default paths)
*    $f1ow_session->set_path_rewrite('object', 'object:fields'); // any time 'object' path is referenced, 'object:fields' will be referenced instead.
*
* }
*
* // can't rely on constructor to initialize the object, because it is only called the first
* // time the class is instantiated. -- we could use also use _wakeup() method to call initialize()
*
* $f1ow_session->initialize();
*
*
* # EXAMPLE 1
* # ---------
* # Using a tabset class in conjunction with path_persist -- to determine which tabs are currently loaded and to provide a framework
* # for switching between tabs.
*
*    <script language="JavaScript">
*
*        // this is a global function used by persistant
*        // tabs.  Implement saveSettings() on each page
*        // to ensure that all necessary vars are stored.
*
*        // note:  this function works best in a fusebox type system where every request is going to the
*        // same page.  You may want to set the form.action to point to different pages depending on which
*        // tab was selected.
*
*        function loadTab(tab, depth)
    {
*            saveSettings();
*            document.frmMain["persist[memorize]"].value = '1';
*
*            // locate the element we are changing
*            strPath = document.frmMain["persist[path]"].value;
*            arrPath = strPath.split(":");
*            arrPath[depth] = tab;
*
*            // remove any trailing element
*            numRemaining = arrPath.length - (depth+1);
*            arrPath.splice(depth+1, numRemaining);
*
*            // re-create path string
*            strNewPath = arrPath.join(":");
*
*            // save + submit
*            document.frmMain["persist[path]"].value = strNewPath;
*            document.frmMain.submit();
*        }
*
*        // override this function on your individual pages to save any settings before switching tabs
*        // e.g. I always set 'mode' or 'action' vars to '' so that when I come back to a tab it doesn't try
*        // to re-perform the same action as when I last left the tab.
*
*        function saveSettings()
    {
*            return true;
*        }
*    </script>
*
* <?php
*
*     $tabs = new tabset();
*     $tabs->set_default_link("javascript: loadTab('[tab_name]', [tabset_depth])");
*     $obj_tab = &$tabs->add_tab('object');
*     $tabs->add_tab('dbfields');
*
*     $obj_subtabs = &$obj_tab->add_subtabs();
*     $obj_subtabs->add_tab('fields');
*     $obj_subtabs->add_tab('privs');
*
*     // show the tabs -- using the current path information
*    $tabs->show($f1ow_session->get_path());
*
*
* Obviously you can do all sorts of things with the path_persist() class.  You can use it in a system where there is only
* one path -- and you simply want to have persistant data (e.g. filters of query strings) stored in a session, you can use
* it with tabs, as the underpinnings of a fusebox framework.  I'm sure there are lots of options that I haven't thought of yet.
*
*
* @author Hans Lellelid
* @version 1.0.0
*/

class path_persist
{
    /**
    * @access private
    */
    var $_params;

    /**
    * @access private
    */
    var $_old_params;

    /**
    * @access private
    */
    var $_stored;

    /**
    * @access private
    */
    var $_path_rewrite_rules;

    /**
    * @access public
    */
    var $path_separator = ":";

    /**
    * The constructor is only used to intialize variables.
    * Because we are persisting (storing) this object, instances of this class must be initialized
    * manually -- or we could rely on _wakup, but either way we are not ready to initialize the
    * class from the constructor.
    */
    function path_persist()
    {
        // this  is only called at the beginning of a users's session
        $this->_params = array();
        $this->_path_rewrite_rules = array();
        $this->_stored = array();
    }

    /**
    * Manages the loading and storing of variables.
    * This is the main initialization class that loads in paramters from $persist array and
    * checks to see if a request was made to save "memorize" current data.  Note that any
    * default params and rewrite rules should be set *before* initializing() the object.
    *
    * @access public
    */
    function initialize()
    {
        // read in the object parameters

        global $persist;

        if ($persist)
        {
            $this->_old_params = $this->_params; // make a copy, which we will use when we store current vars

            $this->_params = array(); // reset the array

            foreach ($persist as $var => $val)
            {
                $this->_params[$var] = $val;
            }
        }
        else
        {
            // first time coming (back?) to this page
            if (true === isset($this->_stored[$this->get_path_str()]))
            {
                $this->_reload_vars();
            }
        }


        // if memorize param is set we need to store all data into the "save_path" param

        if (true === isset($this->_params['memorize']))
        {
            if ( !isset($this->_params['save_path']) )
            {
                $this->_params['save_path'] = '';
            }
            $this->_store_vars($this->_rewrite_path($this->_params['save_path']));

            // we're done w/ memorize, so we can clear it.
            // (we do this so that it won't be passed with pass_form_vars() method.)

            $this->_params['memorize'] = '';

            // remove all the post vars
            // because the page that's being loaded doesn't need to know about (and may be confused by)
            // the data from the previous page.

            $this->_clear_request_vars();


            // if the variables for this new path are stored, then reload them

            if (true === isset($this->_stored[$this->get_path_str()]))
            {
                $this->_reload_vars();
            }

        }
        else
        {
            // [default]
            // one way or another, every time this page is loaded the vars are stored.
            // that way when we leave a page (which is, of course, defined by a unique path)
            // it will have been stored.

            $this->_store_vars();
        }

        // once we have finished storing & reloading data,
        // we set the save path to the current path.

        $this->_params['save_path'] = $this->get_path_str();
    }

    /**
    * This function is for specifying path rewrite rules -- generally for default paths.
    * E.g. if the path 'object' should always default to 'object:fields' then this is
    * where that would be defined.
    *
    * @param $path The path that we want to replace.
    * @param $replace_path The path with which to replace $path param.
    * @access public
    */
    function set_path_rewrite($path, $replace_path)
    {
        $this->_path_rewrite_rules[$path] = $replace_path;
    }

    /**
    * Returns the unique path as a string.
    *
    * @access public
    */
    function get_path_str()
    {
        return $this->_rewrite_path($this->_params['path']);
    }

    /**
    * Returns the correct path to use.
    * Rewrites a path based on a value in the $_path_rewrite_rules array.
    *
    * @return string The corrected (if necessary) form of the path.
    * @access private
    */
    function _rewrite_path($path)
    {
        if (true === isset($this->_path_rewrite_rules[$path]))
        {
            return $this->_path_rewrite_rules[$path];
        }
        else
        {
            return $path;
        }
    }

    /**
    * Gets the current path or path component.
    * This function calls get_path_str() method to return the string and
    * then breaks the path into components.
    *
    * @param $idx If present, specifies which element of the path to return.
    * @return mixed An array representing the path or individual string element of the path.
    */
    function get_path($idx = false)
    {
        $path = explode($this->path_separator, $this->get_path_str());
        if ($idx !== false)
        {
            return $path[$idx];
        }
        else
        {
            return $path;
        }
    }

    /**
    * Sets the current path.
    * You probably don't need to use this function.  Paths probably shouldn't need to be reset after
    * page load, and if they do you could probably use set_path_rewrite() method to do that in a more
    * controlled fashion.
    *
    * @param $path Mixed Could be an array or a string represneting the path we want to set.
    * @param $idx If set, this indicates which component of $path to set.
    */
    function set_path($path, $idx = false)
    {
        if ($idx !== false)
        {
            // 1) break out current path into components
            $pathA = explode($this->path_separator, $this->get_path_str());
            // 2) set specified component to new value
            $pathA[$idx] = $path;
            // 3) re-implode and save
            $this->_params['path'] = implode($this->path_separator, $pathA);
        }
        else
        {
            if (is_array($path))
            {
                // implode the array that was passed
                $this->_params['path'] = implode($this->path_separator, $path);
            }
            else
            {
                // set the path to the value passed
                $this->_params['path'] = $path;
            }
        }

        // set the save_path to newly created path
        $this->_params['save_path'] = $this->get_path_str();
    } // function

    /**
    * This accessor method is the way that all (non-path) data should be accessed.
    * The path data is an exception, however, because it is always processed by
    * _rewrite_path() method before being returned.  Always use get_path_str() or
    * get_path() methods when accessing path info.
    * @return mixed The value of _params[$key]
    * @access public
    */
    function get_param($key)
    {
        return (true === isset($this->_params[$key])) ? $this->_params[$key] : null;
    }

    /**
    * Function to manually set the value of a paramter.
    * Note that this does not affect the information stored in session, *except* that
    * it will change the value printed by ->pass() method. -- so chances are it will set
    * your param to desired value on form submit.
    * @param string $key The key of param to set.
    * @param string $value The new value.
    * @access public
    */
    function set_param($key, $value)
    {
        $this->_params[$key] = $value;
    }

    /**
    * Saves all request vars into a class array.
    * Currently this is only working for HTTP_POST_VARS.  It would
    * be easy to extend it.
    *
    * @access private
    */
    function _store_vars($path=false)
    {
        //global $HTTP_POST_VARS;

        $path = ($path !== false ? $path : $this->get_path_str());

        // reset the stored var
        $this->_stored[$path] = array();

        //print "<br>Savings vars to $path";

        foreach ($_POST as $varname => $value)
        {
            $this->_stored[$path][$varname] = $value;
        } // foreach


        // overwrite the persist array (parameters for this class) with the backup we created
        // in the constructor. i.e. make sure we have are storing most current params

        if ($this->_old_params)
        {
            $this->_stored[$path]['persist'] = $this->_old_params;
        }

        if (true === isset($this->debug) && true == $this->debug)
        {
            print "<br><font color=red>Stored data: <pre>";
            print_r($this->_stored[$path]);
            print "</pre></font>";
        }

        unset($this->_old_params); // so it doesn't get serialized.
    }

    /**
    * Reloads all stored vars into the global scope.
    *
    * Note:  we only re-import parameters (i.e. $_params[]) when
    * the current $_params array is empty -- although, the $persis[] array is always
    * being loaded into
    * the global scope. The reason for this is that the $_params[] array should never
    * revert to a saved copy, but rather should always have the most up-to-date
    * information about the current session.
    *
    * You should never use $persis[varname] to access a path_persist variable.
    * Always use ->get_param() method, which will return the current (non-cached)
    * value for a paramter.  The exception is that you should use get_path() and
    * get_path_str() for accessing the path variable.  The reason for this is that
    * even though the path is a paramter, the actual value of the path that this
    * class uses to store & reload variables may not match the parameter -- i.e. the
    * path may need to be 're-written' (if a a rewrite rule was established for the
    * path in question).
    *
    * @param $path The path to reload -- defaults to the current path.
    * @access private
    */
    function _reload_vars($path = false)
    {
        $path = ($path !== false ? $path : $this->get_path_str());

        if (true === isset($this->debug) && true == $this->debug)
        {
            print "<p><font color=blue>Extracting vars from $path";
        }

        foreach ($this->_stored[$path] as $varname => $value)
        {
            // perform extract()-like function into GLOBAL namespace
            global ${$varname};
            ${$varname} = $value;

            if (true === isset($this->debug) && true == $this->debug)
            {
                print "<br>&nbsp;&nbsp;$varname <= ";
                print_r($value);
            }
        }

        // only import if current array is empty
        if (!$this->_params) $this->_params = $this->_stored[$path]['persist'];

        if (true === isset($this->debug) && true == $this->debug)
        {
            print "</font>";
        }

        // note:  we do *not* re-import parameters after re-loading
        // the variables.  although, the $persis[] array is being reloaded
        // to the global scope.

        // you should never use $persis[varname] to access a path_persist variable.
        // always use ->get_param() method, which will return the current (non-cached)
        // value for a paramter.
    }

    /**
    * This function clears variables from the global namespace and the $HTTP_POS_VARS array.
    * It is used to avoid passing form vars between path spaces (sections defined by a unique path)
    * when using forms.
    *
    * @access private
    */
    function _clear_request_vars()
    {
        foreach ($_POST as $varname => $value)
        {
            global ${$varname};
            unset(${$varname});
            unset($GLOBALS[$varname]);
            unset($_POST[$varname]);
        } // foreach
    }

    /**
    * Sets the default value for a parameter.
    * You can use this to set the default value for any parameter -- including
    * a path.  Of course the path will be interpreted by the _rewrite_path() method,
    * so a default path will be modified by any calls to set_path_rewrite().
    *
    * @access public
    */
    function set_default_param($key, $value)
    {
        if (!isset($this->_params[$key]))
        {
            $this->_params[$key] = $value;
        }
    }

    /**
    * Passes the parameters for this objects as form elements or query string components.
    * @return string of form elements (type=hidden) or query string containing session information.
    * @access public
    */
    function pass($method='form')
    {
        // we need to make sure this always shows up in the form elements
        // since we may have javascript setting the value.

        if (!isset($this->_params['memorize']))
        {
            $this->_params['memorize'] = 0;
        }
        $formstr = '';
        if ( 'form' == $method )
        {
            foreach ($this->_params as $varname => $value)
            {
                $formstr .= "<input type=\"hidden\" name=\"persist[$varname]\" value=\"$value\">";
            }
        }
        else
        {
            $i = 0;
            foreach ($this->_params as $varname => $value)
            {
                $formstr .= (($i++ ? "&" : "") . "persist[$varname]=" . urlencode($value));
            }
        }

        return $formstr;
    }

    /**
    * Returns the array (hash) of paramters
    * @return array Parameters
    * @access public
    */
    function get_params()
    {
        return $this->_params;
    }
}

?>