<?php
// need timeLibrary to deal with timeouts appropriately
require_once(dirname(__FILE__) . '/timeLibrary.php');

/**
* This class replicates the functionality of the procedural DocManagment.php (by OM & NC) in an object-oriented way.  Currentlty, it will return filetype, filesize, and required file viewers for any document on local servers or available by HTTP or FTP (FTP untested).  For the list of known filetypes, see filetypes.xml.
* 
* Note that the constructor is intended to be called OUTSIDE of a while loop
* displaying list of documents, while many of the methods are for within.
*
* The method set_file() is key -- you call it each time you begin work with a new file 
* (rather than creating a new instance of the class.
*
* Note about execution time:  A couple of the methods employed in this class can potentially be
* very time consuming.  These are the method used for fetching a remote file head
* (packaged as a separate class) and the method for determining a filesize by downloading
* it piece by piece.  If you need things to run fast, setting a low timeout value may mean that
* you won't be able to get filesize/filetype for remote files.  There is a bug in the way
* PHP does remote socket connections, so even if you set a timeout of 1 second, each file processed
* could potentially take longer.  If it MUST be fast, using extension for filetype and skipping
* filesize (for remote files) altogether gives the best performance because you don't need to talk
* to a remote server.
*
* If you don't want to have to deal with whether each file is local or remote, use the methods
* to set the local filepath, local file prefix, and site url.  The class can then determine
* whether local or remote automatically and adjust accordingly.
*
* @version 1.2.0 (suitable for production use),  March 10, 2002
* @copyright Forum One Communications 2001-2002
* @author Jason Roberts [Nyk Cowham, Oscar Merida earlier versions]
*/
class documentInfo
{
####################  VARIABLES #####################################
/**
* The location of all the files needed for functionality in this class.
* @access private
*/
var $class_path;

/**
* Multi-dimentional array that holds what we know about each filetype.  Read further for details.
* Details: there are currently 5 elements per filetype entry.
* e.g for $filetypes_array['HTML']
*   ['label'] is a textual label, (e.g. 'Web Page')
*   ['extensions'] is an array, with alternate extensions (html, htm)
*   ['mimetypes'] is an array, with alternate mimetypes (text/htm, etc.)
*   ['viewer_url'] is the url of a free fileviewer, if one exists (http://www.opera.com/)
*   ['viewer_text'] is a short description of the viewer
*      (e.g. To view Web Pages, you'll need a web browser.  You can get one from opera's web site)
* @access private
*/
var $filetypes_array;

/**
* Array with extensions as the keys and filetypes as the values.  We use either this or the mimetypes array to match filetype.
* @access private
*/
var $filetypes_by_extension;

/**
* Array with mimetypes as the keys and filetypes as the values.  We use either this or the extensions array to match filetype.
* @access private
*/
var $filetypes_by_mimetype;

/**
* cumulative array of all the filetypes encountered in the existance of this object. 
* @access private
*/
var $encountered_filetypes;

/**
* array of filetypes that we don't want to display a viewer for (e.g. HTML, which is self-explanatory)
* @access private
*/
var $suppressed_viewers;

/**
* The target window for file viewer downloads.  This is used in the get_viewer_link() function.
* default is nothing (open in same window).
* @access private
*/
var $viewer_download_target;

/**
* The text to display for link to file viewer (if one exists).  This is used in the get_viewer_link() function.  Default is "download here".
* default is "download here".
* @access private
*/
var $viewer_link_text;


/**
* The file that we are getting info for.  this must be either a valid system path starting at the root, or a http-accessible file
* @access private
*/
var $file;

/**
* Anything that precedes the true local filename that would need to be stripped off.  Example is /files.fcgi?.
* @access private
*/
var $local_file_prefix;

/**
* The location of the true local filestore, which can be used for operations requiring the true local path, such as filesize();
* @access private
*/
var $local_filestore_path;

/**
* Whether the file is 'local' or 'remote'.
* @access    private
*/
var $location;

/**
 * The HTTP HEAD of the file we're looking at.
 * @access private
 */
var $file_head;

/**
* the extension that is stripped off of the passed file using regex, used for further processing.  created in the constructor.  I assume that a 'DOT' followed by any number of alphanumeric characters is an extension.
* @access private
*/
var $extension;

/**
 * At present, this is anything that comes after the extension but that PHP's parse_url() still considers to be part of the path.  For example, in ".../test.asp/extra", "/extra" will wind up there.
 * Basically, PHP's parse_url() function isn't smart enough to recognize all possible characters that could start a query string.
 * @access    private
 */
var $post_extension;

/**
 * The mimetype of the file.
 * @access private
 */
var $mimetype;

/**
* the filtype (e.g. DOC, PDF) that we determine based on the extension of the document.  this maps directly to the filetype array.
* @access private
*/
var $filetype;

/**
* Array containing switches telling how we should get filetype.
* @access private
*/
var $filetype_methods;

/**
* Array containing the values allowed for the above.
* @access private
*/
var $allowed_filetype_methods;

/**
 * How we got the filetype -- mimetype or extension?  (or something else?)
 * @access private
 */
var $filetype_methods_used;

/**
* filesize in bytes of the file we're analyzing.
* @access private
*/
var $filesize;

/**
* Array containing switches telling how we should get filesize.
* @access private
*/
var $filesize_methods;

/**
* Array containing the values allowed for the above.
* @access private
*/
var $allowed_filesize_methods;

/**
 * Method by which we deterimine the filesize (fiesystem, head, download -- primarily debugging use.
 * @access private
 */
var $filesize_methods_used;

/**
* Allowed time for processing per file, expressed in seconds.
* @access private
*/
var $timeout;

/**
* Amount of time used thus far, counting only the major filetype/size methods.
* @access private
*/
var $time_used;

/**
 * When reading http files, how many bytes to get at a time.  Larger is faster, but allows more error in the result (overshoot).
 * @access private
 */
var $remote_read_chunk;

/**
* Location of the script containing the functions we use to parse the XML filetypes document.
* @access private
*/
var $xml_filetypes_path;

/**
* Location of the script containing the functions we use to parse the XML filetypes document.
* @access private
*/
var $xml2array_functions_path;

####################  /VARIABLES #####################################

/**
* Constructor: set default values, create instance of the class.
* @param void
* @return void
* @access public
*/
function documentInfo()
{
    // "Constants"
    #$this->class_path = '/local/www/php/classes/documentInfo/'; 
    $this->class_path = dirname(__FILE__) . '/';
    $this->xml2array_functions_path = $this->class_path . 'xml2array.php';
    $this->xml_filetypes_path = $this->class_path . 'filetypes.xml';
    // keys used in the XML document
    $this->LABEL_KEY = 'label';
    $this->EXTENSIONS_KEY = 'extensions';
    $this->VIEWER_URL_KEY = 'viewer_url';
    $this->VIEWER_TEXT_KEY = 'viewer_text';
    $this->MIMETYPES_KEY = 'mimetypes';
    $this->FILETYPE_UNKNOWN = 'UNKNOWN';
    $this->allowed_filetype_methods = array('remote' => array('head','extension'),
        'local' => array('extension'));
    $this->allowed_filesize_methods = array('remote' => array('head','download'),
        'local' => array('filesystem'));
    
    // Default values
    $this->set_viewer_download_target(''); // open in same window
    $this->set_viewer_link_text('[download here]');
    $this->set_remote_read_chunk(1024);
    $this->set_filetype_methods('local','extension');
    $this->set_filetype_methods('remote','head,extension');
    $this->set_filesize_methods('local','filesystem');
    $this->set_filesize_methods('remote','head,download');
    $this->set_timeout(5);
    $this->suppressed_viewers = array();
} // constructor method

/**
* Set the maximum amount of time to process one file.  Only records major methods for filetype/size.  Default is set in constructor.
* Includes:
*     Time to download a file for filesize if 'download' option is enabled
*     Time to fetch remote head of a document if 'head' option is enabled
* @param float $seconds number of seconds to wait
* @return void
* @access public
*/
function set_timeout($seconds)
{
    $this->timeout = $seconds;
}

/**
* Sets the target window to open the viewer download.  Default is nothing (open in the same window.)  Use _blank to force each to open in a new window.
* @param string $target the target window name
* @return void
*/
function set_viewer_download_target($target)
{
    $this->viewer_download_target = $target;
}

/**
* Sets the text to use for links to fileviewer download site.  Default is '[download here]'.
* @param string $text the link text that you want to use
* @return void
* @access public
*/
function set_viewer_link_text($text)
{
    $this->viewer_link_text = $text;
}

/**
* Sets the true local path where files are stored, such as /local/private/mysite/filestore/.  Setting this and local_file_prefix allow the object to use local methods for filesize, which are faster and more accurate.  Note that using these functions presents a possible security risk -- be careful not to reveal the local path!
* @param string $path The path to the local filestore, as PHP will see it
* @return void
* @access public
*/
function set_local_filestore_path($path)
{
    $this->local_filestore_path = $path;
}

/**
* Sets any prefix on local files, such as /files.fcgi?.  Setting this and local_filestore_path allows the object to use local methods for filesize, which are faster and more accurate.  Note that using these functions presents a possible security risk -- be careful not to reveal the local path!
* @param string $prefix The complete prefix preceeding filename (e.g. /files.fcgi?)
* @return void
* @access public
*/
function set_local_file_prefix($prefix)
{
    $this->local_file_prefix = $prefix;
}

/**
* Set the absolute URL that the outside world uses to access files on the site.  For local files, this will be combined with the value set using set_file() (subtracting any file checker url) to create a fully-qualified URL.
* For example, for a normal Forum One site you might use http://www.example.com/files.cgi
* @param string $url URL, with or without the trailing /, depending on how
* your local URLs are stored in the DB.
* @return void
* @access public
*/
function set_public_filestore_uri($url)
{
    $this->public_filestore_uri = $url;
}

/**
* Add a filetype to the array of filetypes that we do not display viewer information for.  This affects the output of get_viewer_text(), get_viewer_url(), get_viewer_link(), and get_required_viewers().  Call this method multiple times to set a number of suppressed filetypes.
* Details:  There is no "ALL" option, because the methods that display viewer info are
* optional.  If you don't want to display anything, don't call any of them!
* Note that the filetype you are adding to this array MUST be one that is in the filetypes_array.
* This array has one entry per filetype, not per exension.  .htm and .html are both the HTML
* file type.
* @param string $filetype the filetype to add (must match a type in the array)
* @return void
* @since 1.1.0 | December 13, 2001
*/
function suppress_viewer($filetype)
{
    $filetype = strtoupper($filetype);
    $this->suppressed_viewers[] = $filetype;
}

/**
* Almost a 2nd constructor -- call for each new file that you're working with.  Requires $file, which is a fully-qualified URI (http only), a complete local filepath, or a relative URI on the local system.  If  relative URI is used, also use the methods to set local file prefix if any (e.g. file.cgi), local filestore path, and local filestore URI.  This should save the need to do that stuff outide of the class.
* Clears out all data pertaining to the last file using private method clear().
* @param string $file valid path to a file on local sytsem, or file available by http
* @return boolean returns nothing if all goes well, FALSE if it can't open the file for reading
* @access public
*/
function set_file($file)
{
    // unset all variables specific to the last file, to get ready for this new file
    $this->clear();
    if ($this->get_location($file) == 'local')
    {
        $this->file = $this->fetch_true_local_path($file);
    }
    else
    {
        //remote file
        // file path for processing is as-entered
        $this->file = $file;
    }
} //set_file()

/**
* For local files, convert the path entered to the true local path (e.g. '/files.cgi/filename.ext' -> '/var/www/vhosts/sitename.www/files/filename.ext') so that local filesystem functions can work.  NEVER use this for public display -- we don't want to reveal system file structure.
* if the filepath entered does not contain the file prefix (or if file prefix is not set), assumes that the path passed is the actual path and doesn't touch it
* @param string $path the path to be processed
* @return string $path the processed path
* @access private
*/
function fetch_true_local_path($path)
{
    if (!empty($this->local_filestore_path))
    {
        // local filesystem path has been configured
        if (!empty($this->local_file_prefix))
        {
            // some prefix (probably a filechecker) being used, so remove it in
            // preparation for adding the real path on.  
            // we'll assume that we're taking '/files.cgi/filename.ext' 
            // and turning it into 'filename.ext'
            if (strpos($path, $this->local_file_prefix) === 0)
            {
                // the $path string begins with the file prefix (found at character 0)
                // strip off the file prefix (e.g. /files.php/filename.ext' -> 'filename.ext'
                $path = str_replace($this->local_file_prefix, '', $path);
                // pre-pend the local filestore path onto the path to form the true local path
                // such as '/var/www/vhosts/sitename.www/files/' + 'filename.ext';
                $path = $this->local_filestore_path.$path;
            }
            else
            {
                // no need to do anything.  assume that the filepath entered by the user is the actual absolute system path
            }
        }
    }
    else
    {
        // no need to do anything.  assume that the filepath entered by the user is the actual absolute system path
    }
    return $path;
} // end method

/**
* Return the extension of the file, converted to all lowercase.  if the extension can not be determined for some reason (like it's not there), will return 'NONE'.
* At present, for offsite stuff we're using PHP's parse_url() function.  Any alphanumeric portion of the 'path' fragment following a 'DOT' is considered the extension.
* @param void
* @return string/boolean extension (lower case)
* @access public
*/
function get_extension()
{
    if (!isset($this->location))
        $this->get_location();
    switch ($this->location)
    {
        case 'remote':
            // use PHP native parse_url, then some regex
            $url_parts = parse_url($this->file);
            $path = $url_parts['path'];
            preg_match("/\.([A-Za-z0-9]*)(.*)/i", $path, $matches);
            $this->extension = strtolower($matches[1]);
            $this->post_extension = $matches[2]; // anything whacky that followed extension
        break;
        case 'local':
            // use PHP native pathinfo to split it up
            $path_parts = pathinfo($this->file);
            $this->extension = strtolower($path_parts["extension"]);
        break;
    }
    if ($this->extension == '')// if we didn't set it
        $this->extension = 'NONE';
    return $this->extension;
}

/**
* Determine if the file is on the local filesystem or remote, based on path/url provided. Doesn't try to see if it actually exists, just looks at whether on the path/url provided contains a ://.
* @param void
* @return string returns 'local' or 'remote'
* @access private
*/
function get_location($url = NULL)
{
    if (!isset($url)) $url = $this->file;
    $position = stristr($url, "://"); // contains '://' to indicate protocol
    if (false === $position)
    {
        $location = 'local';
    }
    else
    {
        $location = 'remote';
    }
    $this->location = $location;
    return $this->location;
} // end function

/**
* Get the HTTP head of a document, which may then be used for Content-Length and Content-Type.
* Calls an external class to do the actual work.
* @param void
* @return string/boolean the HEAD of the document we're interested in, or FALSE.
* @access public
*/
function fetch_remote_head()
{
    include_once ($this->class_path.'fetchRemoteHead.php');
    $fetchRemoteHead = new fetchRemoteHead();
    $fetchRemoteHead->set_timeout($this->get_time_remaining());
    $file_head = $fetchRemoteHead->get_remote_head($this->file);
    $this->file_head = $file_head;
    $this->add_time_used($fetchRemoteHead->get_elapsed_time());
    return $file_head;
}

/**
* Attempt to determine mimetype based on HEAD of document.
*
* @param void void
* @return string/boolean mimetype or FALSE if it can't do it.
* @access private
*/
function get_mimetype_head()
{
    // get the file head if we don't already have it
    if (isset($this->file_head))
        $file_head = $this->file_head;
    else
        $file_head = $this->fetch_remote_head();
    if (isset($file_head)) // at this point, don't bother if we don't have the head
    {
        // pull 'Content-Type' from the HEAD
        preg_match("/Content\-Type: ([A-Za-z0-9\/\.\-]*)/i",$file_head, $matches);
        if ($matches[1] != '') // if it's set and isn't '', return it
        {
            $this->mimetype = $matches[1];
            return $this->mimetype;
        }
        else return false;
    }
    else return false;
} // end function

/**
* Control what method(s) we use to get filetype.  This might be based on speed/accuracy issues.
* Options for the mode (first parameter) are 'local' and 'remote'.  Options for method
* (2nd parameter) are:
* mode = 'local': 'extension'
* mode = 'remote': 'extension' (parse extension from url)
* mode = 'remote': 'head' (parse content-type [mimetype] from remote head)
* The order these are specified in determines
* the order in which they will be used.  For example, 'head' alone specifies that ONLY
* mimetype will be used.  'head,extension' (NO space) means that mimetype will be used
* preferentially, but extension will be used if mimetype fails.
* Defaults:
*    local: extension
*    remote: head, extension
* @param string $mode local or remote
* @param string $methods head or extension
* @return void
* @access public
*/
function set_filetype_methods($mode, $methods)
{
    if (array_key_exists($mode,$this->allowed_filetype_methods)) // make sure it's a valid mode to configure
    {
        $methods_array = explode(',', $methods);
        if (count($methods_array) > 0) // an argument was entered (good!)
        {
            $legal_values = true; // assume they've entered valid modes
            foreach ($methods_array as $method)
            {
                if (!in_array($method, $this->allowed_filetype_methods[$mode]))
                    $legal_values = false; // if any value is out of scope, ditch
            }
        }
        else return false;
        if ($legal_values)
        {
            unset ($this->filetype_methods[$mode]);
            foreach ($methods_array as $method)
            {
                if ($method != '')
                    $this->filetype_methods[$mode][] = $method;
            }
            return true;
        }
        else return false;
    }
    else return false;
} // end function

/**
* Private Method. Get the filetype based on mimetype (content-type) from a remote document head.  Return FALSE if it doesn't work (mimetype unavailable, or no match in filetypes XML).  Called by get_filetype() if appropriate -- don't call directly.
* @param void
* @return string/boolean the filetype code, or FALSE
* @access private
*/
function fetch_filetype_head()
{
    if (!isset($this->mimetype))
        $this->get_mimetype_head();
    if ($this->mimetype) // don't bother if it didn't work
    {
        if(array_key_exists($this->mimetype, $this->filetypes_by_mimetype))
        {
            $filetype = $this->filetypes_by_mimetype[$this->mimetype];
            return $filetype;
        }
        else return false;
    } // end if mimetype
    else return false;
} // end function

/**
* Private Method. Get the filetype based on extension.  Return FALSE if it doesn't work (extension unavailable, or no match in filetypes XML).  Called by get_filetype() if appropriate -- don't call directly.
* @param void
* @return string/boolean the filetype code, or FALSE
* @access private
*/
function fetch_filetype_extension()
{
    if (!isset($this->extension))
        $this->get_extension();
    if ($this->extension) // don't bother if we didn't get it
    {
        if(array_key_exists($this->extension, $this->filetypes_by_extension))
        {
            $filetype = $this->filetypes_by_extension[$this->extension];
            return $filetype;
        }
        else return false;
    }
    else return false;
}

/**
* Load up our 3 arrays of filetype information based on the filetypes.xml document, using an external function library to do the actual work.
* @param void
* @access private
* @return void
*/
function set_filetype_arrays()
{
    # mime-types: http://www.utoronto.ca/webdocs/HTMLdocs/Book/Book-3ed/appb/mimetype.html
    require_once ($this->xml2array_functions_path);
    $filetypes_arrays = XML_filetypes_from_XML($this->xml_filetypes_path);
    $this->filetypes_array = $filetypes_arrays['complete'];
    $this->filetypes_by_mimetype = $filetypes_arrays['by_mimetype'];
    $this->filetypes_by_extension = $filetypes_arrays['by_extension'];
} // end method

/**
* Private Method. Get the filetype of the file, either based on either mimetype or extension.
* To determine which to use, we look at the $filetype_methods array, which is configurable using the set_filetype_methods() function.
* @param void
* @return string either the filetype (from official list) or 'UNKNOWN'
* @access private
* @see set_filetype_methods
*/
function fetch_filetype()
{
    // load info from XML, if not already done
    if (!isset($this->filetypes_array))
    {
        $this->set_filetype_arrays();
    }

    if (!isset($this->location))
    {
        $this->get_location();
    }
    
    $i = 0;
    $filetype = FALSE; // init for loop
    while (!$filetype && isset($this->filetype_methods[$this->location][$i]))
    {
        // loop through the list of methods available, using the functions of the same name
        $filetype = $this->{'fetch_filetype_'.$this->filetype_methods[$this->location][$i]}();
        $filetype_methods_used[] = $this->filetype_methods[$this->location][$i];
        $i++;
    } // end while

    // if we didn't get it, then set it to the UNKNOWN filetype
    if (!$filetype)
    {
        $filetype = $this->FILETYPE_UNKNOWN;
        // record that we failed in the array of used methods
        $filetype_methods_used[] = 'FAILED';
    }
    $this->filetype = $filetype;
    $this->filetype_methods_used = $filetype_methods_used;
    // record that we've found this file in the array of encountered filetypes
    $this->encountered_filetypes[] = $filetype;
    return $filetype;
} // end function

/**
* Return the filetype of the url or path specified in set_file().  This is an accessor only -- fetch_filetype and related methods do the actual work, and are called if needed.  You can feel free to use this method multiple times without incurring a performance hit.
* @param void
* @return string filetype either the filetype code (PDF, DOC, et.) or UNKNOWN.  FALSE not possible.
* @access public
*/
function get_filetype()
{
    if (isset($this->filetype))
    {
        $filetype = $this->filetype;
    }
    else
    {
        $filetype = $this->fetch_filetype();
    }
    return $filetype;
} // get_filetype

/**
* Returns the textual label of the filetype (e.g. "Microsoft Word").  Should never return FALSE, but rather "Unknown File Type" or whatever the label for UNKNOWN filetype is. 
* @param $filetype (optional, if not specified will use the current value of the class variable)
* @return string textual label for this filetype (e.g. "Microsoft Word").
* @access public
*/
function get_label($filetype = NULL)
{
    // if we haven't set the filetype as a paramenter, then use the class variable
    if (!isset($filetype))
    {
        $filetype = $this->get_filetype();
    }
    if (false === empty($this->filetypes_array[$filetype][$this->LABEL_KEY]))
    {
        $label = $this->filetypes_array[$filetype][$this->LABEL_KEY];
    }
    else
    {
        $label = false;
    }
    return $label;
}

/**
* Returns the URL where a viewer for this filetype can be obtained, which may be FALSE if there is no viewer defined in the master filetype array.
* @param $filetype (optional, if not specified will use the current value of the class variable)
* @return string/boolean url where you can get this fileviewer, or FALSE if there is none available.
* @access public
*/
function get_viewer_url($filetype)
{
    // if we haven't set the filetype as a paramenter, then use the class variable
    if (!isset($filetype))
    {
        $filetype = $this->get_filetype();
    }
    if (false === empty($this->filetypes_array[$filetype][$this->VIEWER_URL_KEY]))
    {
        $viewer_url = $this->filetypes_array[$filetype][$this->VIEWER_URL_KEY];
    }
    else
    {
        $viewer_url = false;
    }

    return $viewer_url;
}

/**
* Returns a textual description (1 sentence?) of the file viewer available for the specified filetype, or FALSE if none is provided.
* @param $filetype (optional, if not specified will use the current value of the class variable)
* @return string/boolean textual description (1 sentence?) of the file viewer available for the specified filetype, or FALSE if none is available.
* @access public
*/
function get_viewer_text($filetype)
{
    // if we haven't set the filetype as a paramenter, then use the class variable
    if (!isset($filetype))
    {
        $filetype = $this->get_filetype();
    }
    if (false === empty($this->filetypes_array[$filetype][$this->VIEWER_TEXT_KEY]))
    {
        $viewer_text = $this->filetypes_array[$filetype][$this->VIEWER_TEXT_KEY];
    }
    else
    {
        $viewer_text = false;
    }
    return $viewer_text;
}

/**
* Return a link to the fileviewer, if there is one available.  FALSE if not.
* You can change the window that the link leads to using set_viewer_dowload_target()
* Calls get_viewer_url() to actually get the url, then just slaps a link on it.
* @param $filetype (optional, if not specified will use the current value of the class variable)
* @return string/boolean link to the fileviewer, or FALSE
* @access public
* @see set_viewer_download_target
*/
function get_viewer_link($filetype)
{
    // if we haven't set the filetype as a paramenter, then use the class variable
    if (!isset($filetype))
    {
        $filetype = $this->get_filetype();
    }
    $url = $this->get_viewer_url($filetype);
    if (true === is_string($url))
    {
        $viewer_link = ' < a href = "'.$url.'" target = "'.$this->viewer_download_target.'" > '.$this->viewer_link_text.' < /a > ';
        return $viewer_link;
    }
    else return false;
}

/**
* Gets an associtive array of label =>  viwer info(including link) for the filetypes encountered since this class was created.  If optional $types parameter is passed, it will instead use those type(s).  $type may be a single value ('PDF'), or an array ('PDF','DOC').
* If you don't want to show viewer information for some filetypes (e.g. HTML), use
* suppress_viewer() method.  This would generally be if they're too commonplace to bother
* with.
* @param array/string $types the filetypes we want viewers for.
* @return array/boolean [label] => text/link, sorted by label.  FALSE if there are no viewers to display.
* @access public
* @see suppress_viewer()
*/
function get_required_viewers($types = NULL)
{
    if (isset($types))
    {
        if (is_array($types))
        { // make certin it's an array
            $filetypes = $types; // if is array, just set our variable == to it
        }
        else
        {
            $filetypes[] = $types; // if not array, make an array from the string
        }
    }
    else
    {
        $filetypes = $this->encountered_filetypes;
    }
    array_unique($filetypes);
    while (list(, $filetype) = each($filetypes))
    {
        if (!in_array($filetype, $this->suppressed_viewers))
        {
            // create a couple of local scope variables, just for simplicity
            $label = $this->get_label($filetype);
            $viewer_text = $this->get_viewer_text($filetype);
            $viewer_link = $this->get_viewer_link($filetype);
            $required_viewers[$label]= $viewer_text.' '.$viewer_link;
        }
    }
    reset($filetypes);
    if (count($required_viewers) > 0)
    {
        ksort($required_viewers); // alphebetize
        return $required_viewers;
    }
    else
    {
        return false;
    }
}

/**
* Number of bytes to fetch at a time when determining the filesize of a remote file by downloading it in chunks and counting the chunks.  Larger numbers are more efficient (fewer requests), but smaller are more accurate (finer resolution).
* @param integer bytes
* @access public
* @return void
*/
function set_remote_read_chunk($bytes)
{
    $this->remote_read_chunk = $bytes;
}

/**
* Control what method(s) we use to get filesize.  This might be based on speed/accuracy issues.
* Options for the mode (first parameter) are 'local' and 'remote'.  Options for method
* (2nd parameter) are:
* mode = 'local': 'filesystem' (use functions on local filesystem)
* mode = 'remote': 'head' (use content-length from remote head)
* mode = 'remote': 'download' (download the file in chunks to see how many bytes come)
* The order specified indicates the order in which they will be used.
* For example, 'head' alone specifies that ONLY
* head will be used.  'head,download' (NO space) means that head will be used
* preferentially, but download will be used if head fails.
* @param string $mode local or remote
* @param string $methods filesystem (local), head or download (remote)
* @return void
*/
function set_filesize_methods($mode, $methods)
{
    if (array_key_exists($mode,$this->allowed_filesize_methods)) // make sure it's a valid mode to configure
    {
        $methods_array = explode(',', $methods);
        if (count($methods_array) > 0) // an argument was entered (good!)
        {
            $legal_values = true; // assume they've entered valid modes
            while (list(, $method) = each($methods_array))
            {
                if (!in_array($method, $this->allowed_filesize_methods[$mode]))
                {
                    $legal_values = false; // if any value is out of scope, ditch
                }
            }
            reset($methods_array);
        }
        else
        {
            return false;
        }
        if ($legal_values)
        {
            unset ($this->filesize_methods[$mode]);
            while (list(, $method) = each($methods_array))
            {
                if ('' != $method)
                {
                    $this->filesize_methods[$mode][] = $method;
                }
            }
            reset($methods_array);
            return true;
        }
        else
        {
            return false;
        }
    }
    else
    {
        return false;
    }
} // end function

/**
* Private Method.  Attempt to determine filesize based on local filesystem. DON'T CALL DIRECTLY -- USE GET_FILESIZE().
* @param void
* @return int/boolean return filesize in bytes or FALSE if it can't do it.
* @access private
*/
function fetch_filesize_filesystem()
{
    if ($filesize = filesize($this->file))
    {
        return $filesize;
    }
    else
    {
        return false;
    }
} // end function

/**
* Private Method.  Attempt to determine filesize based on HEAD of document. DON'T CALL DIRECTLY -- USE GET_FILESIZE().
* @param void
* @return int/boolean return filesize in bytes or FALSE if it can't do it.
* @access private
*/
function fetch_filesize_head()
{
    // read the file head if we don't already have it
    if (!isset($this->file_head))
    {
        $this->fetch_remote_head();
    }
    if ($this->file_head) // don't bother if we can't get it
    {
        // pull 'Content-Length' from the HEAd
        preg_match("/Content\-Length: ([0-9]*)/i",$this->file_head, $matches);

        // if it's set and isn't 0, return it, else FALSE
        if ($matches[1] != '' && $matches[1] > 0)
        {
            return $matches[1];
        }
        else
        {
            return false;
        }
    }
    else
    {
        return false;
    }

} // end function

/**
* Private Method. Determine the filesize of the document by reading it remotely, if possible.  Returns filesize in bytes, or FALSE. DON'T CALL DIRECTLY -- USE GET_FILESIZE().  This will add a trailing / to a url like http://www.example.com, making it http://www.example.com/ before it tries to fetch it -- otherwise it would fail.
* @param void
* @return string/boolean returns filesize in bytes or FALSE (will return FALSE on zero bytes, too).
* @access private
*/
function fetch_filesize_download($url = NULL, $timeout = NULL)
{
    if (!isset($url))
    {
        $url = $this->file;
    }
    if (!isset($timeout))
    {
        $timeout = $this->get_time_remaining();
    }
    // fopen() requires the trailing / on a url -- add it if it's not there?
    $parts = parse_url($url);
    if ($parts['path'] == '' && $parts['query'] == '')
    {
        $url .= '/';
    }
    // not tested on FTP
    $file = @fopen($url, 'r');  // try to open file for reading
    if ($file)
    {
        // Not tested with ftp connections.
        $filesize = 0;
        $complete = false;
        $start_time = timeLibrary::get_microtime();
        $elapsed_time = 0;
        while (!$complete && !$overtime)
        {
                fread ($file, $this->remote_read_chunk);
                $filesize += $this->remote_read_chunk;
                // record time
                $elapsed_time = timeLibrary::get_interval($start_time);
                if ($elapsed_time > $timeout)
                    $overtime = true;
                // record progress
                if (feof ($file))
                    $complete = true;
        } // end while
        fclose ($file);
        $this->add_time_used($elapsed_time);
        // return result -- return FALSE if filesize is 0 or incomplete
        if ($complete && $filesize > 0) return $filesize;
        else return false;
    }
    else return false;
}

/**
* Private Method. Determine the filesize of the document, if possible.  Returns filesize in appropriate units.  Default settings are currently: <1 Kb = x.xx Kb, 1-1024 Kb = x Kb, > 1024 Kb = x.xx Mb.  Why?  That seems to be the amount of info the user would be interested in.
* Details:
* requires $this->file : a valid local system path to a file or a valid URL (http or ftp).  this is set in constructor.  Uses settings set in set_filesize_methods() to control how it gets filesize.
* @param void
* @return string/boolean returns filesize, with label (100 Kb) or FALSE if filesize can't be determined.
* @access private
*/
function fetch_filesize()
{
    // see where it is
    if (!isset($this->location))
    {
        $this->get_location();
    }
    $i = 0;
    $filesize = false; // init for loop
    while (!$filesize && isset($this->filesize_methods[$this->location][$i]))
    {
        $my_function = 'fetch_filesize_' . $this->filesize_methods[$this->location][$i];
        $filesize = $this->$my_function();
        $this->filesize_methods_used[] = $this->filesize_methods[$this->location][$i];
        $i++;
    } // end while

    $this->filesize = $filesize; // even if false
    if ($filesize)
    {
        return $filesize;
    }
    else
    {
        $this->filesize_methods_used[] = 'FAILED';
        return false;
    }
}// end fetch_filesize

/**
* Return the filesize of the specified file, or false if can't be determined (or is zero).  This is just an accessor - fetch_filesize and related methods do the real work (and are called if needed).  You can call this method multiple times without a performance hit.
* @param void
* @access public
* @return string/boolean - filesize with appropriate label, or FALSE.
*/
function get_filesize()
{
    if (isset($this->filesize))
    {
        $filesize = $this->filesize;
    }
    else
    {
        $filesize = $this->fetch_filesize();
    }
    if ($filesize) // not FALSE or zero
    {
        if ($filesize > 1024)
        {
            // round so that Kilobytes always comes out with no decimal places
            $filesize = (round($filesize/1024,0))*1024;
        }
        // get the appropriate units
        $j = 0;
        $ext = array("B","KB","MB","GB","TB");
        while ($filesize >= pow(1024,$j))
        {
            ++$j;
        }
        if ($j == 1)
        {
            $j = 2; // never use bytes, always KB or greater
        }
        $filesize = round($filesize / pow(1024,$j-1) , 2);
        $filesize_units = $ext[$j-1];
        // return a string such as 0.54 KB, 101 KB, or 2.15 MB
        return $filesize.' '.$filesize_units;
    } // if $filesize
    else
    {
        return false;
    }
}


/**
* Alternate method for accessing the filesize, to get the return as an integer, bytes only. Will call the get_filesize function, if it has not already been called.
* @param void
* @return nteger/boolean filesize, in bytes, or FALSE if unable to get filesize.
* @access public
* @see get_filesize
*/
function get_filesize_bytes()
{
    if (!isset($this->filesize))
    {
        $this->fetch_filesize();
    }
    return $this->filesize;
}

/**
* Record time used by time-consuming methods.  Adds number of seconds (float) passed as a parameter to the cumulative total.
* @param float $seconds time in seconds
* @return void
* @access public
*/
function add_time_used($time)
{
    $this->time_used += $time;
}

/**
* Return the cumulative time used thus far by time-consuming methods.
* @param void
* @return float $seconds time in seconds
* @access public
*/
function get_time_used()
{
    return $this->time_used;
}

/**
* Return the amount of time remaining before timeout, determined by subtracting cumulative time used from the timeout value set
* @param void
* @return float $seconds time in seconds
* @access private
*/
function get_time_remaining()
{
    $time_remaining = $this->timeout - $this->get_time_used();
    // complicates things if it's negative
    if ($time_remaining < 0)
    {
        $time_remaining = 0;
    }
    return $time_remaining;
}
/**
* Clear out all defined variables that are specific to one file, so that we are ready to process the next file.
* Detail: We do this rather than creating a new instance of the class so that we don't have the overhead of creating/destroying classes, and also so that we can retain information such as the encountered_filetypes array
* @param none
* @return none
* @access private
*/
function clear()
{
    unset($this->file);
    unset($this->extension);
    unset($this->post_extension);
    unset($this->filetype);
    unset($this->filetype_methods_used);
    unset($this->filesize);
    unset($this->filesize_methods_used);
    unset($this->location);
    unset($this->file_head);
    unset($this->mimetype);
    unset($this->time_used);
} // clear()

/**
* Wrapper to get a document info object in one call for Syntax instances.
*
* @param string file path, from db - partial and relative to web root
* @return document info object
*/
function stxFactory( $file )
{
    $prefs = &pxdb_prefs::singleton();
    $fullpath = PXDB_FILESTORE_PATH . str_replace( $prefs->get_pref('datastore','url'),'',$file );
    
    if ( !file_exists( $fullpath ) )
    {
        trigger_error( "stxFactory: $file does not exist", E_USER_WARNING );
        return false;
    }
    
    $docInfo =& new documentInfo();		
    $docInfo->set_file( $fullpath );
    return $docInfo;            
}

######################### DEPRECATED METHODS #################################

/**
* Deprecated -- behavior of get_filesize improved to make this unecessary.
* @deprecated
*/
function get_filesize_kb()
{
    $filesize = get_filesize();
    return $filesize;
}

/**
* Deprecated -- use the appropriate function to get the specific info you want.
* @deprecated
*/
function get_filetype_info($filetype)
{
    if (!isset($this->filetype))
    {
        $this->get_filetype();
    }
    return $this->filetype;
}

} // class
?>