<?php
/**
* Class for getting content from modules or sections to the browser.
* Many methods of this class include template files wrapped in either caching
* logic or in output buffering logic.
*
* @author hans lellelid, hans@forumone.com
* @access public
* @package SyntaxCMS
*/
class F1CMS
{
    /**
    * Load content from a content "module".
    * Modules usually correspond to datatypes -- e.g. 'document' might
    * be a module -- but modules can also be applications (like a calendar)
    * that encompass more than one datatype. In order to be invoked by this
    * method, the module must have been declared using the Modules::addModule()
    * method.
    *
    * IMPORTANT CACHING NOTE:  Because of the inability for output buffers (at this
    * PHP version) to layer nicely, embedded modules may not be cached correctly.
    *
    * So, for the time being, avoid invoking this method in a module that was invoked via this method.
    * (or disable caching.)
    *
    * @param array $params A hash of paramters to the module.  Must include
    *                     at minimum, the 'module' and 'action' params.
    * @return string result from module call (usu HTML)
    * @access public
    */
    function module($params)
    {
        // this method creates a new ModuleRequest -- so make sure class has been loaded
        include_once(SITE_LIB_PATH . '/syntaxcms/Request/ModuleRequest.class.php');

        // The GLOBAL Request
        $GlobalReq =& Site::getRequest();

        // Create new customized $Request object that will be used
        // by the action page.
        $Request =& new ModuleRequest();

        // Current [calling] section which will not be part of the local-scope $Request object
        if ( 'sectionrequest' == strtolower(get_class($GlobalReq)) )
        {
            $Request->setVar('section', $GlobalReq->getSectionID());
        }

        // set up the request with new information from function params
        $Request->addVars($params); // or do we want to clear out $_REQUEST...?

        // method can be used 2 ways
        if ( isset($params['request']) )
        {
            $Request->setRequest($params['request']);
        }
        else
        {            
            if (Modules::isModule($params['module']))
            {            
                $Request->setModule($params['module']);
                $Request->setAction($params['action']);
            }
            else
            {
                trigger_error( $params['module'] . ' is not a recognized module', E_USER_WARNING );
                return false;
            }
        }

        // specify that this is an "included" request

        
        $Request->isIncluded(true);

        // Cache
        // -----
        // Check caching settings for this module.
        // OAM: assigning by reference below causes errors when you have module
        // calls embedded within other module calls.
        $Cache = Site::getCache();
        
        if (Modules::getAttribute($Request->getModule(), 'cache') == false)
        {
            $Cache->caching = false;
        }

        // Check to see if content has been updated
        // OAM: allow setting the cache key to check in user.conf.php
        $cache_key = Modules::getAttribute($Request->getModule(), 'cache_key');
        if (empty($cache_key))
        {
            $cache_key = 'content';
        }

        F1CMS::checkCache($cache_key); // flushes the cache if content has been updated.

        $cache_handle = $Cache->generateID($Request);

        // LOAD CONTENT
        // ------------
        if ($content = $Cache->start($cache_handle , $cache_key))
        {
            if (DEBUG)
            {
                Log::entry(F1_DEBUG_LOG, 'F1CMS::module()', __FILE__, __LINE__, 'Cached content found for ' . $Request->getModule() . '/' . $Request->getAction() . " [$cache_handle]");
            }
            return trim($content);
        }
        else
        {
            // now include the action script

            if (false !== $Request->getScript())
            {
                include($Request->getScript());
            }

            $content = $Cache->end(CACHE_LIFETIME);

            if (DEBUG && !empty($content)) Log::entry(F1_DEBUG_LOG, "F1CMS::module()",__FILE__,__LINE__,"Cache content created for ".$Request->getModule()."/".$Request->getAction()." [$cache_handle]");

            // make sure to set it back (because we are modifying global instance)
            $Cache->caching = CACHING_ENABLED;

            // return content
            return trim($content);
        } // if cached
        
    }

    /**
    * Load content for a content "section"
    * This is similar to the module() method, except that it allows you to
    * load a section page within the current page ... Might currently produce unwanted
    * results, but should also allow for future flexibility.
    *
    * @param array $params A hash of paramters to the section.  Must include
    *                        at minimum the 'request' (path) param.
    * @access public
    * @return string
    */
    function section($params)
    {
        ob_start();
        ob_implicit_flush(false);

        // The GLOBAL Request
        $GlobalReq =& Site::getRequest();

        include_once(SITE_LIB_PATH . '/syntaxcms/Request/SectionRequest.class.php');

        // Create new customized $Request object that will be used
        // by the action page.
        $Request =& new SectionRequest();

        // set up the request with new information from function params
        $Request->addVars($params); // or do we want to clear out $_REQUEST...?
        $Request->setPath($params['request']);

        // specify that this is an included request
        $Request->isIncluded(true);

        // now include the action script
        include($Request->getScript());

        $content = ob_get_contents();

        $content = trim($content);

        ob_end_clean();

        return $content;
    }

    /**
    * Builds the breadcrumb trail.
    * uses the breadcrumb.tpl template to handle the actual display of the
    * breadcrumb trail.  -- basic display logic is encapsulated in breadcrumb.tpl
    * @return string The breadcrumb
    * @access public
    */
    function breadcrumb()
    {
        ob_start();
        ob_implicit_flush(false);

        $Request =& Site::getRequest();
        $trail = $Request->getTrail();
        include(SITE_TPL_PATH . '/breadcrumb.tpl');

        $content = ob_get_contents();
        ob_end_clean();
        return $content;
    }

    /**
    * Build the navigation list for "functions" or "applications".
    * "Functions" are modules that have been tagged as "navigable".
    * This is the driving reason behind the Modules::findModules()
    * method.
    * @return string navigation
    * @access public
    */
    function functionNav()
    {
        ob_start();
        ob_implicit_flush(false);

        $Request =& Site::getRequest();

        // Find all modules that are navigable.  Order
        // defined will be the order they appear here.
        $modules = Modules::findModules('navigation', true);

        include(SITE_TPL_PATH . '/nav/functions.tpl');

        $content = ob_get_contents();
        ob_end_clean();
        return $content;
    }

    /**
    * Build the navigation for "browsing" site content by topic / issue picklists.
    * @return string navigation
    * @access public
    */
    function browseNav()
    {
        ob_start();
        ob_implicit_flush(false);

        $dbh =& pxdb::dbh();

        $Request =& Site::getRequest();

        // 1) find topics
        $sql = "SELECT name as value, name as label FROM topics ORDER BY weight";
        $topics = (array) $dbh->GetAll($sql);

        // 2) find issues
        $sql = "SELECT name as value, name as label FROM issues ORDER BY weight";
        $issues = (array) $dbh->GetAll($sql);

        include(SITE_TPL_PATH . '/nav/browse.tpl');

        $content = ob_get_contents();
        ob_end_clean();
        return $content;
    }

    /**
    * Build the navigation options for "browsing" site content by any number of picklists.
    *
    * A sample browseBy specification is below. The key of the browseBy array is the
    * pick table to use to get options. Note that you can ommit all the parameters and
    * the function defaults to using the name field for name, value and weight for sorting,
    * and the table name for the listName and first option in the list.
    * $browseby = array(
    *    'topics' => array (
    *        'nameField' => 'name',
    *        'labelField'=> 'name',
    *        'order'     => 'weight',
    *        'listName'  => 'Topics',
    *   ),
    *    'issues' => array (),
    *   ),
    *);
    *
    * @parameter array, array of nested arrays specifying what lists to use and how.
    * @return string navigation
    * @access public
    */
    function buildBrowseArrays($browseBy)
    {
        // if no order field is specified, default to the weight field
        // SFS change to while
        while (list($table, $params) = each($browseBy))
        {
            if (!isset($params['listName']) || '' == $params['listName'])
            {
                $params['listName'] = $table;
            }

             $browseArrays[$table] = F1CMS::buildBrowseArray($table, $params);
        } // while
        return $browseArrays;
    }

    /**
    * This method actually builds the array of option/values for a browseBy picklist
    *
    * @param string, table name
    * @param array, brose by options (see above)
    * @see buildBrowseArrays
    * @return array
    * @access public
    */
    function buildBrowseArray($table, $params)
    {
        $dbh =& pxdb::dbh();

        // if no value field is specified, default to the name field
        if (!isset($params['valueField']) || '' == $params['valueField'])
        {
            $params['valueField'] = 'name';
        }

        // if no label field is specified, default to the name field
        if (!isset($params['labelField']) || '' == $params['labelField'])
        {
            $params['labelField'] = 'name';
        }

        // if no order field is specified, default to the weight field
        if (!isset($params['order']) || '' == $params['order'])
        {
            $params['order'] = 'weight';
        }

        $sql = 'SELECT ' . $params['valueField'] . ' as value, ' .
            $params['labelField'] . ' as label' . ' FROM ' . $table .
            ' ORDER BY ' . $params['order'];

        // push the options into the browseArrays stacks
        $r = (array) $dbh->GetAll($sql);
        return $r;
    } // end func

    /**
    * Display pagination.
    * @param object pxdb_collection &$Records pxdb_collection class or subclass
    * @param integer $rows_per_page
    * @param integer $current_page
    * @param array $params Hash of any additional params that should be passed in querystring.
    * @return string
    * @access public
    */
    function pager(&$Records, $rows_per_page, $current_page, $params)
    {
        ob_start();
        ob_implicit_flush(false);

        include(SITE_TPL_PATH . '/paginate.tpl');

        $content = ob_get_contents();
        $content = trim($content);
        ob_end_clean();
        return $content;
    }

    /**
    * Gets the content for the [right] sidebar.
    * @return string sidebar contents
    * @access public
    */
    function sidebar()
    {
        ob_start();
        ob_implicit_flush(false);

        $Request =& Site::getRequest();
        if ($Request->getSidebarScript())
        {
            include($Request->getSidebarScript());
        }

        $content = ob_get_contents();
        $content = trim($content);
        ob_end_clean();
        return $content;
    }

    /**
    * Returns the timestamp stored under content_updated for a group
    *
    * @param string $group The cache group to check.
    * @return unix timestam
    * @access public
    */
    function getContentUpdated($group = null)
    {
        $cache_group = ($group !== null ? $group : CACHE_DEF_GROUP);
        $prefs =& pxdb_prefs::singleton();
        return $prefs->get_pref('content_updated', $group);
    }

    /**
    * Examines pxdb_prefs table to see if any new content has been added.  It checks this based on group -- which
    * may correspond to datatype.
    * @param string $group The cache group to check.
    * @access public
    */
    function checkCache($group=null)
    {
        $cache_group = ($group !== null ? $group : CACHE_DEF_GROUP);
        $Site =& Site::singleton();
        if ($Site->Cache->caching)
        {
            $prefs =& pxdb_prefs::singleton();
            $updated = $prefs->get_pref('content_updated', $cache_group);
            if ($updated)
            { // type agnostic
                if (DEBUG) Log::entry(F1_DEBUG_LOG, "F1CMS::checkCache()",__FILE__,__LINE__,"Flushing cache for $cache_group.");
                // flush the cache.
                if (false === $Site->isCacheFlushed($cache_group))
                {
                    // SFS: Suppress useless unhandled warning in PEAR class
                    @$Site->Cache->flush($cache_group);
                    $Site->setCacheFlushed(true);
                }
                
                // remove pref
                $prefs->del_pref('content_updated', $cache_group);
            }

        } // if caching
    }

    /**
    * Alternative signature method for calling f1cms::module(). The function is a little
    * more explicit, since every call to a module must include the module name and
    * action name.
    *
    * @access public
    * @param string $mdl The module you're calling
    * @param string $action The action you want to call for the module
    * @param array $params Associative array of arguments to pass to the module
    * @return string buffered output from the module
    */
    function callModule($module, $action, $params = null)
    {
        $params['module'] = $module;
        $params['action'] = $action;

        if ( FALSE !== ($result = f1cms::module($params)))
        {
            return $result;
        }
        else
        {
            trigger_error( 'callModule failed', E_USER_WARNING );
            return null; // on a production machine we want no content returned
        }            
    }

    /**
    * Alternative pagination. Not customizeable. Whether it's 'smarter'
    * is subjective and open to debate. Deprecated.
    *
    * @deprecated
    * @param int $perPage Number of results to show per page
    * @param int $totalItems The total number of results before pagination
    * @param string $noun A descriptor for the items (e.g. documents, toothbrushes, etc.)
    * @return string Buffered output
    */
    function smarterPager($perPage, $totalItems, $noun)
    {
        $total_pages = ceil($totalItems / $perPage);
        $orig_page = $_REQUEST['page'];

        $showing = ($perPage < $totalItems) ? $perPage : $totalItems;
        ob_start();
?>
<p>Currently showing <?= $showing ?> of <?= $totalItems ?> <?= $noun ?>.</p>
<p>Page:
<?php
            for ($p = 1; $p <= $total_pages; $p++)
            {
                if (!isset($_GET['page']))
                {
                    $_GET['page'] = 1;
                } // end if

                $hide_gets = array('navtype', 'request');
                $qs = array();
                foreach ($_GET as $g => $val)
                {
                  if (!in_array($g, $hide_gets))
                  {
                      if ($g == 'page')
                      {
                          $qs[] = "$g=$p";
                      }
                      else
                      {
                          $qs[] = "$g=$val";
                      }
                  }
                }

                $qs = implode('&', $qs);
                echo ($p==$_REQUEST['page']) ? " $p " : ' <a href="?'.$qs.'">'.$p.'</a> ';
            }
?>
</p>
<?php
        $c = ob_get_contents();
        ob_end_clean();

        return $c;
    }
    
    /**
    * Returns the url for the detail page of a record
    * @param object Record object
    * @access public
    */
    function getDetailUrl( $Record )
    {        
        $datatype= $Record->get_datatype();
        if ( !isset( $detail_modules[$datatype] ) )
        {
            $detail_module = Modules::getCapabilityProvider('detail', $Record->get_datatype() );
        
            if ('general' == $detail_module)
            {
                // try to map detail through a datatype module if available
                if ($modules = Modules::findModules('datatype', $Record->get_datatype(), $strict=false))
                {
                    $detail_module = $modules[0];
                }
            }
            $detail_modules[ $datatype ] = $detail_module;
        }
        
        return '/content/' . $detail_modules[$datatype] . '/detail/' . $Record->get_id() . '/'; 
        
    }
}
?>