<?php
/**
* Class to organize the modules/components that make up the site.
* 
* Only one copy of this class should exist at one time.  No reason why 
* it would need to be instantiated more than once, since modules are scopeless.
*
* In fact, no need to instantiate this class ever, as it will be automatically 
* instantiated if no instnace exists when you make the first static call to a 
* class method.
*
* E.g.
*	Modules::addModule('document', 'Documents');
*   Modules::addCapability('document', 'list');
*   Modules::addCapability('document', 'detail');
* etc.
*
*
* Modules are NOT concerned with filesystem paths -- or even requested url paths.
* This is handled completely by the Request class.  It may make sense later
* to build in path support into modules -- e.g. to allow a module to specify
* which scripts handle which actions.  For now, we will rely on convention for
* this and simply build request paths in the Request class.  The Request class
* similarly handles any rewriting (a la Apache RewriteRule) of requests -- again 
* this is not handled on the module level because we probably need more flexilibity 
* in rewriting requests (e.g. a request for a section could also be rewritten).
*
* @author Hans Lellelid <hans@forumone.com>
* @package SyntaxCMS
*/
class Modules
{
	/**
	* The name of a module to use as a generic module. (if requested
	* module does not exist)
	* @var string
	*/
	var $genericModule = null;
	
	/**
	* The registered modules.
	* @var array
	*/
	var $modules;
	
	/**
	* Modules that can provide functionality for other modules.
	* @var array
	*/
	var $fallbackProviders;
	
	/**
	* Initializes class variables.
	* @access private
	*/
	function Modules() 
	{
		$this->modules=array();
		$this->fallbackProviders = array();				
	}
	
	/**
	* Returns a reference to a single instance of this object
	*
	* @access public
	* @return object Modules reference
	*/
	function &singleton()
	{
		static $instance;
		
		if (!isset($instance)) 
        {
			$instance = new Modules();
		}
		
		return $instance;		
	}
	
	/**
	* Checks to see if specified module is in directory.
	* @param string $module
	* @return boolean
	* @access public
	*/
	function isModule($module)
	{
		$Modules =& Modules::singleton();
        // OAM - change to see if its a configured module , better than hitting the filesystem
        if (isset($Modules->modules[$module]))
        {
            return true;
        }
        else
        {
            return false;
        }
	}
	
	/**
	* Add a module to the current configuration.
	* Pushing a module w/ same name as a module already on the stack
	* will overwrite the first one.
	* @param string $module The name of the module we are adding.
	* @param string $label Display name for the module
	* @access publicDATATYPE_RC_ARTICLE
	*/
	function addModule($module, $label='')
	{
		$Modules =& Modules::singleton();
	
		if (empty($label)) 
        {
            $label = $module; 
        }
		
		// Setup skeleton for full module array			
        $arrModuleAttribs = array('name' => $module, 'label' => $label,	'provides' => array());
		// Add module to array
        $Modules->modules[$module] = $arrModuleAttribs;
		
		// Call the modules registration script.			
        include_once(SITE_MOD_PATH . "/$module/init/register.php");
	}

	/**
	* Removes a module from the stack.
	* @param string $module The name of the module to remove.
	*/
	function removeModule($module)
	{
		$Modules =& Modules::singleton();
		unset($Modules->modules[$module]);
	}	


	/**
	* Set the capabilities (or "what module provides") for specified module.
	* Omitting the second param, or passing a null param will result in emptying
	* the array.
	* @param string $module The module name.
	* @param array $provides The array of capabilities -- e.g. 'list', 'detail', etc. that module provides.
	* @access public
	*/
	function setCapabilities($module, $provides=null)
	{	
		$Modules =& Modules::singleton();
		
		if (null == $provides) 
        { 
            $provides = array(); 
        }
		
		$Modules->modules[$module]['provides'] = $provides;
	}

	/**
	* Specify that one module depends on another.
	* This method doesn't really do anything (i.e. doesn't return an error).
	* @param string $module The module that depends.
	* @param string $dep_module The module that is depended on.
	* @access public
	*/
	function addDependency($module, $dep_module)
	{
		$Modules =& Modules::singleton();
		if (!isset($Modules->modules[$module]['depends'])) 
        {
            $Modules->modules[$module]['depends']=array(); 
        }
		$Modules->modules[$module]['depends'] = array_merge($Modules->modules[$module]['depends'], (array) $dep_module);
	}
	
	/**
	* Add / set an attribute for a module.
	* This is a catchall for handling useful attributes of modules
	* that we don't want to have separate methods for -- e.g. 'visible'
	* or 'application' etc.
	* @param string $module The module name for which we are setting an attrib.
	* @param string $attr_name The attribute name that we are setting.
	* @param mixed $attr_value The value for the attribute.
	* @access public
	*/
	function setAttribute($module, $attr_name, $attr_value)
	{
		$Modules =& Modules::singleton();
		$Modules->modules[$module][$attr_name] = $attr_value;
	}
	
	/**
	* Get an attribute of a given module.
	* Used in conjunction / as the counterpart to setAttribute.
	* @param string $module The module name for which we are getting an attrib.
	* @param string $attr_name The attribute name that we are getting.
	* @return mixed The value of the $attr_name attribute or null.
	* @access public
	*/
	function getAttribute($module, $attr_name)
	{
		$Modules =& Modules::singleton();
		if (isset($Modules->modules[$module][$attr_name])) 
        {
			return $Modules->modules[$module][$attr_name];
		}
        else 
        {
			return null;
		}
	}
	
	/**
	* Returns a list of all specified attribute for all modules.
	* @param string $attr_name The attribute name.
	* @param array $modules Can specify subset of modules within which to aggregate specified attribute.
	* @return array attributes
	* @access public
	*/
	function aggregateAttribute($attr_name, $modules=null)
	{
		$Modules =& Modules::singleton();
		if (null === $modules) 
        {
            $modules = array_keys($Modules->modules); 
        }
		
		$results = array();
        while (list(,$module) = each($modules))		
        {		
            if ( isset( $Modules->modules[$module][$attr_name] ) )
            {
                $add = (array) $Modules->modules[$module][$attr_name];
            }
            
            if ( !empty( $add ) )
            {
                $results = array_merge($results, $add);
            }
		}
		return $results;
	}
	
	/**
	* Returns a list of modules that match certain criteria.
	* This will start out being very simple -- pass it an attribute name
	* and value, and it will return all matching Module keys (which are by definition
	* unique).
	* @param string $attrib_name The attribute to search for.
	* @param mixed $attrib_value The value of the attribute to match.
	* @param boolean $strict Whether to also match type.
	* @return array The module keys (module names) that match the search.
	* @access public
	*/
	function findModules($attrib_name, $attrib_value, $strict=false)
	{
		$Modules =& Modules::singleton();		        
		$matches = array();
		$module_keys = array_keys($Modules->modules);
        while (list( ,$key ) = each( $module_keys ))
        {
			if (true == $strict) 
            { // match type?
				if ( isset($Modules->modules[$key][$attrib_name]) 
                    && ($Modules->modules[$key][$attrib_name] === $attrib_value)
					|| (is_array($Modules->modules[$key][$attrib_name]) 
                    && in_array($attrib_value,$Modules->modules[$key][$attrib_name],true))) 
                {
					$matches[] = $key;
				}
			} 
            else 
            {
                
				if (isset($Modules->modules[$key][$attrib_name]) 
                     && ( 
                        $Modules->modules[$key][$attrib_name] == $attrib_value
                        || ( 
                            is_array($Modules->modules[$key][$attrib_name]) 
                            && in_array($attrib_value,$Modules->modules[$key][$attrib_name])
                           )
                        )
                    )
                {
					$matches[] = $key;
				}
			}			
		}
		return $matches;		
	}
	
	/**
	* Add a capability.  This corresponds to the "Action" that can be performed
	* on a given module, and is expected to translate into a filesystem file (PHP script)
	* @param string $module The module name.
	* @param string $action The capability -- e.g. 'list', 'detail', etc. that module provides.
    * @param string  $label A label for the module
	* @access public
	*/
	function addCapability($module, $action, $label=null)
	{	
		$Modules =& Modules::singleton();
		if (null == $label)
        {
            $label = $action; 
        }
		
		$arrActionAttribs = array('name'=>$action, 'label'=>$label);									
		$Modules->modules[$module]['provides'][$action] = $arrActionAttribs;
	}
	
	/**
	* Remove a capability.
	* @param string $module The module name.
	* @param string $action The capability -- e.g. 'list', 'detail', etc. that module provides.
	* @access public
	*/
	function removeCapability($module, $action)
	{	
		$Modules =& Modules::singleton();
		
		$currProvides = $Modules->modules[$module]['provides'];
		
		// build new array that doesn't have $action we are removing		
		$newProvides = array_diff($currProvides, array($action));		
		// set values
		$Modules->setCapabilities($module, $newProvides);
	}

	/**
	* Checks whether a module provides a specified capability.
	* (whether module supports specified action)
	* @param string $module Module name.
	* @param string $action The action to look for in module's 'provides' array.
	* @return boolean
	* @access public
	*/
	function hasCapability($module, $action)
	{
		$Modules =& Modules::singleton();				
		return isset($Modules->modules[$module]['provides'][$action]);
	}
	
	/**
	* Registers a module to use in case a request is made for a non-existant
	* module, or there is an attempt to perform a non-existant action 
	* @param string $module The module to use for requests that do not exist.
	* @access public
	*/
	function regGenericModule($module)
	{		
		$Modules =& Modules::singleton();		
		$Modules->genericModule = $module;
	}
	
	/**
	* Gets the generic module if one has been established.
	* @return string The name of the generic module.
	* @access public
	*/
	function getGenericModule()
	{
		$Modules =& Modules::singleton();
		return $Modules->genericModule;		
	}
	
	/**
	* Registers a module to use as a functionality provider for the
	* specified module, in the even that the module is called to perform
	* an unsupported action/capability.
	*
	* This is still very basic, and could be made more complex by allowing
	* modules to use functionality from certain modules for certain actions.
	* Also -- there is the issue of module templates....
	*
	* @param string $requestedModule The module that was requested (and will be incomplete)
	* @param string $providingModule The module that can provide that same functionality.
	* @param string $requestedAction If specified will specify that fallback only applies for specified action.
	*/
	function regFallbackProvider($requestedModule,$providingModule,$requestedAction = null)
	{
		$Modules =& Modules::singleton();				
		if (null !== $requestedAction)
		{
			$Modules->fallbackProviders[$requestedModule][$requestedAction] = $providingModule;
		}
		else
		{
			$Modules->fallbackProviders[$requestedModule]['_default'] = $providingModule;
		}
	}
	
	/**
    * Returns the name fo a fallback provider for the requested action, if one exist.
    *
	* @param string $requestedModule
	* @param string $requestedAction
	* @return string Fallback provider -- or NULL if none found.
	*/
	function getFallbackProvider($requestedModule, $requestedAction = null)
	{
		$Modules =& Modules::singleton();
		if (null !== $requestedAction)
		{
			if (isset($Modules->fallbackProviders[$requestedModule][$requestedAction]))
			{
				return $Modules->fallbackProviders[$requestedModule][$requestedAction];
			}
			elseif (Modules::hasCapability(Modules::getGenericModule(), $requestedAction))
			{
				return Modules::getGenericModule();
			}
			else
			{
				return null;
			}
		}
		else
		{
			if (isset($Modules->fallbackProviders[$requestedModule]['_default']))
			{
				return $Modules->fallbackProviders[$requestedModule]['_default'];
			}
			elseif (Modules::getGenericModule())
			{
				return Modules::getGenericModule();
			}
			else
			{
				return null;
			}
		}
	}
	
    /**
    * Returns the Module that provides a capability for a datatype.
    *
    * @param string, Capability name
    * @param integer, datatype
    * @return string
    */
    function getCapabilityProvider($capability, $datatype)
    {
        // grab the last one on the stack
        $myMod = array_pop(Modules::findModules('datatype', $datatype));        
        if (Modules::hasCapability($myMod, $capability))
        { 
            return $myMod;            
        }
        else
        {            
            return Modules::getFallbackProvider($myMod, $capability);
        }
    }
}
?>