<?php
/**
* Manages user controlled redirects used in error script.  This class provides a
* simple collection interface for the admin module.
*
* Example:
*   $redirects = new Stx_Redirects(NAV_ROOT_EL);
*   $redirects->getPagedCollection( $page, 20);
*   while ( $r = $redirects->fetchNext() )
*   {
*       // do stuff
*   }
*
* @author Oscar Merida <oscar@forumone.com>
* @date Dec 2, 2005
* @package SyntaxCMS
*/
class Stx_Redirects 
{
    /**
    * The root section string (usually the NAV_ROOT_EL for the site) allows
    * the same redirect to point to different targets depending on what the 
    * root section is set to.  This is useful when you have a single syntax 
    * database content store feeding multiple site instances.
    */
    var $root_section;
    
    /**
    * ADOdb recordset
    */
    var $query = null;
    
    /**
    * Constructor
    *
    * @param string
    */    
    function Stx_Redirects($root)
    {
        $this->set_root_section($root);
    }
    
    /**
    * Sets the root section to restrict redirects
    *
    * @param string
    */
    function set_root_section($root)
    {
        $this->root_section = $root;
    }
    
    /**
    * Returns an AdoDB recordset that is paged
    * @param integer
    * @param integer
    * @return object
    */
    function get_paged_collection($page, $rows_per_page)
    {
        if ( !empty($this->root_section))
        {            
            $db = pxdb::dbh();
            $sql = 'SELECT * FROM stx_redirects'
                  .' WHERE root_section="' . $this->root_section . '"'
                  .' ORDER BY redirect';
            $this->query = $db->PageExecute($sql, $rows_per_page, $page);
        }
        else
        {
            trigger_error( 'Root section not specified', E_USER_WARNING );
            return false;
        }
    }
    
    /**
    * Returns the next object in the result set as an array
    * @return array, false when done
    */
    function fetch_next()
    {
        if ( empty($this->query) )
        {
            trigger_error( 'Query is empty', E_USER_WARNING );
        }
        else
        {
            return $this->query->FetchRow();
        }
    }
    
    /**
    * Returns the total number of rows found
    * @return integer
    */
    function count()
    {
        return $this->query->_maxRecordCount;
    }
    
    /**
    * Returns the last page number in a paged query
    * @return integer
    */
    function get_last_page()
    {
        return $this->query->_lastPageNo;
    }
}

/**
* Methods for working with a single redirect.
* Since the same match string can be with multiple root nodes (if you use one
* SyntaxCMS content store on multiple sites.  You must set the root section
* to work with a redirect.  If it is empty, most of the functions will return false
* and display an error.
*
* USAGE
* $redirect = new Stx_Redirect();
* $redirect->set_root_section( '_home' );
* $redirect->set_match('event_calendar');
*
* if ( $redirect->exists() )
* {
*     $url = $redirect->get_target_url();
* }
*/
class Stx_Redirect 
{
    /**
    * Database id
    * @access private
    */
    var $id;
    
    /**
    * The root section string (usually the NAV_ROOT_EL for the site) allows
    * the same redirect to point to different targets depending on what the 
    * root section is set to.  This is useful when you have a single syntax 
    * database content store feeding multiple site instances.
    */
    
    var $root_section;
    /**
    * String to match for redirect
    * @access private
    */
    var $match;
    
    /**
    * Target url (absolute or fully qualified w/http)
    * @access private
    */
    var $target_url;
    
    /**
    * Sets the id
    */
    function set_id($id)
    {
        $this->id = $id;
    }
    
    /**
    * Sets the string used to match in the request uri, the string
    * is converted to lowercase and matched regardless of case
    * @access public
    */
    function set_match($match)
    {
        $match = trim($match, '/');
        $match = str_replace('\\', '/', $match );
        $match = preg_replace("/[^\w\.]+/", "_", $match); // remove special chars
        $match = strtolower($match);
        $this->match = $match;
    }

    /**
    * Sets the string used to match in the request uri
    * @access public
    */
    function get_match()
    {
        return $this->match;
    }
    
    /**
    * Sets the target URL
    * @access public
    */
    function set_target_url($url)
    {
        if ( preg_match('/^(http|https):\/\//', $url) )
        {
            $url = trim($url);
        }
        else
        {
            //$url = trim($url, '/');
            $url = str_replace('\\', '/', $url );
            $url = preg_replace("/[\s\.]+/", "_", $url); // remove special chars
        }
        $this->target_url = $url;
    }
    
    /**
    * Gets the target URL
    * @access public
    */
    function get_target_url()
    {
        return $this->target_url;
    }
    
    /**
    * Sets the root section to restrict redirects
    *
    * @param string
    */
    function set_root_section($root)
    {
        $this->root_section = $root;
    }    
    
    /**
    * Tests if a redirect under a root section already exists in the db
    * @return boolean
    * @access public
    */
    function exists()
    {
        if ( !empty($this->match) && !empty($this->root_section) )
        {
            $dbh = pxdb::dbh();
            $sql = 'SELECT * FROM stx_redirects '
                  .' WHERE redirect="' . $this->match . '" and root_section="' . $this->root_section. '"';

            $query = $dbh->execute($sql);
            if ($row = $query->FetchRow())
            {
                $this->load($row['id']);
                return true;
            }
            else
            {
                return false;
            }
        }
        else
        {
            trigger_error( 'Cannot test, match string and/or root section are not set', E_USER_WARNING );
            return null;
        }
    }
    
    /**
    * Attempts to load a redirect given its id
    *
    * @param integer id
    * @access public
    * @return boolean
    */
    function load($id)
    {
        $dbh = pxdb::dbh();
        $sql = 'SELECT * FROM stx_redirects where id="' . $id . '" AND root_section="' . $this->root_section. '"';
        if ( $row = $dbh->GetRow($sql) )
        {
            $this->id = $row['id'];
            $this->match = $row['redirect'];
            $this->target_url =  $row['target'];
            return true;
        }
        else
        {
            return false;
        }
    }
    
    /**
    * Saves redirect changes to the database
    * @access public
    * @return boolean, true on success
    */
    function update()
    {
        if ( !empty($this->root_section) && !empty($this->id ) )
        {
            $sql = 'UPDATE stx_redirects SET redirect="' . $this->match 
                         . '", target="' . $this->target_url .'"'
                         . ' WHERE id="' . $this->id .'" and root_section="' . $this->root_section. '"';
                     
            $dbh = pxdb::dbh();
            if ( $dbh->execute($sql))
            {
                return true;
            }
            else
            {
                trigger_error( 'Error saving redirect to database', E_USER_ERROR );
                return false;
            }
        }
        else
        {
            trigger_error( 'Could not save redirect to database, id and or root section are emtpy', E_USER_ERROR );
            return false;
        }
    }
    
    /**
    * Saves a new redirect info to database.
    * @access public
    * @return boolean, true on success
    */
    function save()
    {
        if ( !empty($this->match) && !empty( $this->target_url) && !empty($this->root_section) )
        {
            $sql = 'INSERT INTO stx_redirects VALUES ("", "' . $this->match 
                 . '", "' . $this->target_url .'", "' . $this->root_section . '")';
                 
            $dbh = pxdb::dbh();
            if ( $dbh->execute($sql))
            {
                return true;
            }
            else
            {
                trigger_error( 'Could not save redirect to database', E_USER_ERROR );
                return false;
            }
        }
        else
        {
            trigger_error( 'Could not save redirect to database: redirect info incomplete', E_USER_ERROR );
            return false;
        }
    }
    
    
    /**
    * Deletes the redirect from the database.
    * @access public
    * @return boolean, true on success
    */
    function delete()
    {
        if ( !empty($this->id) && !empty($this->root_section))
        {
            $sql = 'DELETE FROM stx_redirects '
                  ." WHERE id='" . $this->id . "' AND root_section='" . $this->root_section. "'";
            
            $dbh = pxdb::dbh();
            if ( $dbh->execute($sql))
            {
                return true;
            }
            else
            {
                trigger_error( 'Could not delete redirect from database', E_USER_ERROR );
                return false;
            }
        }
    }
}
?>
