<?php

include_once(SITE_LIB_PATH . '/syntaxcms/Request/Request.class.php');

/**
* Class for handling requests of type 'module'.
* @author hans lellelid, hans@forumone.com
* @package SyntaxCMS.Request
*/
class ModuleRequest extends Request 
{
    function ModuleRequest()
    {     
        parent::Request();
    }
    
    /**
    * 
    * @return string Module portion of current request.
    */
    function getModule()
    {
        if (Modules::isModule($this->getPathElement(0)))
        {
            return $this->getPathElement(0);
        }
        else
        {
            return Modules::getGenericModule();
        }
    }
    
    /**
    * Sets the module (first part of path)
    * @param string $module
    */
    function setModule($module)
    {        
        $this->setPathElement(0, $module);
    }
    
    /**
    * Returns the action of current path.
    * @return string
    */
    function getAction()
    {    
        // checking is too complex to know whether this action is valid (because
        // of fallback providers, etc.        
        return $this->getPathElement(1);        
    }

    /**
    * Sets the module (second part of path)
    * @param string $action
    */
    function setAction($action)
    {
        $this->setPathElement(1, $action);
    }

    /**
    * Overrides parent. Set the request path for the module to use
    * (processes to account for default_path)
    * @param string $path
    * @return string the Path that was set.  (may be different if default_path was set for that module)
    */
    function setPath($path)
    {        
        $path = trim($path, '/ '); // removing trailing or leading '/' and spaces
                
        if (!empty($path)) 
        {
            $path_parts = explode('/', $path);    
        } 
        else
        {
            $path_parts = array();        
        }                
        // check for path default
        // oam, if the module has this capability, set it
        // since this is called before getScript it was causing valid
        // actions to be replaced by the  default_action
        if(1 < count($path_parts) && Modules::hasCapability($path_parts[0], $path_parts[1]))
        {
            $path_parts = $path_parts;
        }
        // before getting to the default action,  we have to check if there's also 
        // a fallthrough handler
        else if (!empty($path_parts[1]) && $module = Modules::getFallbackprovider($path_parts[0], $path_parts[1]))
        {
            if (1 < count($path_parts))
            {
                $keep_parts = array_slice($path_parts, 2);             
            }
            
            $start_parts = array($module, $path_parts[1]);
            $path_parts = array_merge($start_parts, $keep_parts);            
        }
        else if ($action = Modules::getAttribute($path_parts[0], 'default_action'))
        {
            
            // save the original values
            $orig = $path_parts;
            $path_parts = array($path_parts[0], $action);
            
            // persist the rest of the path after action
            array_shift($orig);
            if (0 < count($orig))
            {
                while(list(,$val) = each($orig))
                {
                    array_push($path_parts, $val);
                }
            }
        }
        elseif (1 == count($path_parts))
        {
            $path_parts = array($path_parts[0], 'list');
        }
                
        $this->arrPath = $path_parts;                
        return $path_parts;
    }

    /**
    * Gets the script that is supposed to handle the current request.
    * @return string Path to script that should be executed.
    */
    function getScript()
    {
            
        if (Modules::isModule($this->getModule()))
        {
            if (Modules::hasCapability($this->getModule(), $this->getAction()))
            {
                // this should usually be the case                            
                $module = $this->getModule();
            }
            else
            {
                if ($action_map = Modules::getAttribute($this->getModule(), 'action_alias'))
                {
                    $module = $this->getModule();
                    $this->setAction($action_map[$this->getAction()]);
                }
                else
                {
                    if (DEBUG)
                    {
                        Log::entry(F1_DEBUG_LOG, "ModuleRequest::getScript()", __FILE__, __LINE__, "Module '" . $this->getModule() . "' does not provide support for action '" . $this->getAction() . "'; trying fallback.");
                    }
                    
                    // pass request to fallbackProvider, if defined, or to generic module if not.                
                    $module = Modules::getFallbackProvider($this->getModule(), $this->getAction());                    
                }
            }
        }
        else
        {
            if (DEBUG)
            {
                Log::entry(F1_DEBUG_LOG, "ModuleRequest::getScript()", __FILE__, __LINE__, "Module '" . $this->getModule() . "' does not exist; trying generic.");
            }
            
            // pass request to generic module            
            $module = Modules::getGenericModule();
        }
            
        if (empty($module) || !Modules::hasCapability($module, $this->getAction())) 
        {
            // SFS improve handling
            trigger_error('Could not find a module to handle this request: '.$this->getPath(), E_USER_WARNING);
            header('HTTP/1.0 404 Not Found');
            include('http://'.$_SERVER['HTTP_HOST'].'/error.php');
            exit();
        }
                
        return SITE_MOD_PATH . "/$module/" . $this->getAction() . '.php';
        
    }
    
    /**
    * Get breadcrumb trail for this module.
    * -- may need to override this to add additional nodes.
    * @return array
    */
    function getBaseTrail()
    {
        // Get the top-level element:  that is the "Home" section.
        $arrTrail = $this->getTrailStart();

        // OAM: added option of specifying a section as the parent of this module request
        // so that instead of linking the module to /content/module/
        // the breadcrumb will link it to /section/url_path/
        if ($parent_urlpath = Modules::getAttribute($this->getModule(), 'parent_url_path'))
        {
            if ($parentSection = Section::getByUrlPath($parent_urlpath))
            {
                $arrTrail[] = array(
                    'url'=>$parentSection->getNavPath().'/',
                    'crumb'=>$parentSection->getCrumb()
               );
            }
            else
            {
                trigger_error('parent_url_path does not exist: '.$parent_urlpath, E_USER_WARNING);
            }
        }
        elseif ('general' != $this->getModule())
        {
            // Add the current module to the breadcrumb
            $arrTrail[] = array(
                'url' => '/content/' . $this->getModule() . '/',
                'crumb' => Modules::getAttribute($this->getModule(), 'label')
           );
        }
        return $arrTrail;
    }
    
    /**
    * Gets the breadcrumb trail.
    * @return array
    */
    function getTrail()
    {            
        if (!sizeof($this->arrTrail)) 
        {        
            $this->arrTrail = $this->getBaseTrail();
        } 
        
        return $this->arrTrail;
    }
    
    /**
    * Returns the current page title.
    * @return string The page title.
    */
    function getPageTitle()
    {    
        if (!empty($this->strPageTitle)) 
        {
            return $this->strPageTitle;
        } 
        else
        {
            return Modules::getAttribute($this->getModule(), 'label');
        }
    }
}// class
?>