<?php

/**
* A class that represents a URL request to a web application.
*
* The Request class must be instantiated (i.e. cannot be used statically)
* because a request has a scope and is instantiated within methods of the
* static Content class.
*
* This is a base class that is extended and overridden by the ModuleRequest
* and SectionRequest subclasses.
*
* @author Hans Lellelid <hans@forumone.com>
* @version $Revision: 1.10 $
* @package SyntaxCMS.Request
*/
class Request
{
    /**
    * Sub-requests
    * @var array
    */
    var $children = array();

    /**
    * Parent request
    * @var object Request
    */
    var $parent = null;

    /**
    * @var array
    */
    var $arrPath = array();

    /**
    * @var array
    */
    var $arrPathDefaults = array();

    /**
    * @var array
    */
    var $arrRequestVars = array();

    /**
    * blnIncluded
    * @var boolean
    */
    var $blnIncluded = false;

    /**
    * Page Title
    * @var string
    */
    var $strPageTitle = '';

    /**
    * META Keywords
    * @var string
    */
    var $strMetaKeywords = '';

    /**
    * META Description
    * @var string
    */
    var $strMetaDescription = '';

    /**
    * Hard-coded breadcrumb trail.
    * @var array
    */
    var $arrTrail = array();

    /**
    * Name of script to use for right side bar.
    * @var string
    */
    var $sidebarScript = null;

    /**
    * Initialize the class. Sets the $arrRequestVars attribute.
    */
    function Request()
    {
        // Set contents of arrRequest to superglobal $_REQUEST array
        $this->setVars(array_merge($_POST, $_GET));
    }

    /**
    * Whether or not the current request is being included within another
    * page.
    * @param boolean $new_value The new value for $_included.
    * @access public
    */
    function isIncluded($new_value = null)
    {
        if (null !== $new_value)
        {
            $this->blnIncluded = $new_value;
        }
        return $this->blnIncluded;
    }

    /**
    * Set the request path to use (processes to account for default_path)
    * and now defaults to "list" if no action is specified.
    * @param string $path
    * @return string the Path that was set.  (may be different if default_path was set for that module)
    */
    function setPath($path)
    {        
        $path = trim($path, '/ '); // removing trailing or leading '/' and spaces

        if (!empty($path) && ($default_path = $this->getPathDefault($path)))
        {
            // yes,assignment
            $path = $default_path;
        }

        if (!empty($path))
        {
            $arrPath = explode('/', $path);
        }
        else
        {
            $arrPath = array();
        }

        $this->arrPath = $arrPath;

        return $arrPath;
    }


    /**
    * Gets a string representation of current path.
    * @return string Path built from $arrPath
    * @access public
    */
    function getPath()
    {
        return implode('/', $this->arrPath);
    }

    /**
    * Gets a specified element from the path.  (starting at 0).
    * @param integer $num The index to retrieve
    * @return string element of $arrPath
    * @access public
    */
    function getPathElement($num)
    {
        if (isset($this->arrPath[$num]))
        {
            return $this->arrPath[$num];
        }
        else
        {
            return null;
        }
    }

    /**
    * Sets a specified element of the path.
    * @param integer $num The index to set.
    * @param string $value The value to set it to.
    * @access public
    */
    function setPathElement($num, $value)
    {
        $this->arrPath[$num] = $value;
    }

    /**
    * Returns a count of the number of elements in the path
    *
    * @return integer
    * @access public
    */
    function countPathElements()
    {
        return count($this->arrPath);
    }
    
    /**
    * Sets a default path to use for a module. Use
    * Modules::setAttribute('module_name', 'default_action', 'action_name') now.
    * @param string $path_to_redirect
    * @param string $redirected_path
    * @access private
    * @deprecated
    */
    function setPathDefault($path_to_redirect, $redirected_path)
    {
        $path_to_redirect = trim($path_to_redirect, '/ '); // removing trailing or leading '/'
        $redirected_path = trim($redirected_path, '/ ');

        $this->arrPathDefaults[$path_to_redirect] = $redirected_path;
    }

    /**
    * Gets a default rewrite path (if set). Use
    * Modules::setAttribute('module_name', 'default_action', 'action_name') now.
    * @return mixed Value at $arrPathDefaults or NULL if not set.
    * @param string $pathToRedirect
    * @deprecated
    */
    function getPathDefault($pathToRedirect)
    {
        // removing trailing or leading '/'
        $pathToRedirect = trim($pathToRedirect, '/ ');
        if (isset($this->arrPathDefaults[$pathToRedirect]))
        {
            return $this->arrPathDefaults[$pathToRedirect];
        }
        else
        {
            return null;
        }
    }

    /**
    * Gets a value from request (by default $_REQUEST).
    * @param string $varname
    * @return mixed Value of $arrRequestVars[$vname] or NULL if not set.
    */
    function getVar($varname)
    {
        if (isset($this->arrRequestVars[$varname]))
        {
            return $this->arrRequestVars[$varname];
        }
        else
        {
            return null;
        }
    }

    /**
    * Gets all values from request (by default $_REQUEST).
    * @return array All vars.
    */
    function getVars()
    {
        return $this->arrRequestVars;
    }


    /**
    * Sets a specific var in $arrRequestVars array.
    * @param string $varname
    * @param mixed $value
    * @access public
    */
    function setVar($varname, $value)
    {
        $this->arrRequestVars[$varname] = $value;
    }

    /**
    * Unsets a variable.
    * @param string $varname The variable name
    */
    function delVar($varname)
    {
        unset($this->arrRequestVars[$varname]);
    }

    /**
    * Set the $arrRequestVars variable.
    * If no param, or null param, then it will reset the $arrRequestVars to an
    * empty array.
    * @param array $vars
    */
    function setVars($vars = null)
    {
        if ($vars == null)
        {
            $this->arrRequestVars = array();
        }
        else
        {
            $this->arrRequestVars = $vars;
        }
    }

    /**
    * Adds an array of vars to the $arrRequestVars variable.
    * Like setVars() but does not erase current contents.
    * @param array $vars
    */
    function addVars($vars)
    {
        $this->arrRequestVars = array_merge($this->arrRequestVars, $vars);
    }


    /**
    * Gets the top-level section/node.
    * This is the HOME section.  The main navigation tier happens to be a section
    * for this site framework.  You could of course change this, but more than likely
    * a "homepage" is going to be a site section (with customized template) rather than
    * a content module / application.
    * @return array The Top-level navigation component.
    * @access public
    */
    function getTrailStart()
    {

        // Get the top-level element:  that is the "Home" section.
        include_once(SITE_LIB_PATH . '/syntaxcms/Request/SectionRequest.class.php');
        $SectionRequest =& new SectionRequest();
        $SectionRequest->setPath('/');

        $arrTrail = $SectionRequest->getTrail();

        return $arrTrail;

    }

    /**
    * Set the breadcrumb trail that will be appended to top-level node.
    * @param array $trail The multi-dimentional array that is the breadcrumb trail.
    * @access public
    */
    function setTrail($trail)
    {
        // if only a hash was passed instead of an array of hashes
        if (!is_array($trail[0]))
        {
            $trail = array($trail);
        }
        // add a dimension
        $this->arrTrail = $trail;
    }

    /**
    * Appends a trail "point" (e.g. node) to the breadcrumb trail.
    * @param array $point Hash of trail data [crumb, url].
    * @access public
    */
    function addTrailPoint($point)
    {
        if (!sizeof($this->arrTrail))
        {
            $this->arrTrail = $this->getBaseTrail();
        }
        $this->arrTrail[] = $point;
    }

    /**
    * Manually sets a trail "point" (e.g. node) in the breadcrumb trail.
    * @param array $point Hash of trail data [crumb, url].
    * @access public
    */
    function setTrailPoint($index, $point)
    {
        if (!sizeof($this->arrTrail))
        {
            $this->arrTrail = $this->getBaseTrail();
        }
        $this->arrTrail[$index] = $point;
    }

    /**
    * An abstract method that you should override in your subclasses.
    * This class will return contents of $arrTrail array, appended to
    * the top-level section 'HOME').  There is currently no proper way of
    * overriding this auto-prepend behavior.
    * @return array
    * @access public
    */
    function getTrail()
    {
        return array_merge($this->getTrailStart(), $this->arrTrail);
    }


    /**
    * An abstract method that you must override in your subclasses.
    * The assumption is that if you are using Request instead of a subclass,
    * then you are probably already in the script you want to load.  (e.g. you're
    * not using load.php)
    * @return object PEAR::Error because this is an abstract method & should not be invoked.
    */
    function getScript()
    {
        trigger_error("No overridden getScript() method found.",E_USER_WARNING);
    }

    /**
    * Returns the script to use for the sidebar.
    * @return string
    * @access public
    */
    function getSidebarScript()
    {
        return $this->sidebarScript;
    }

    /**
    * Sets the script to use for sidebar.
    * @param string $script
    * @access public
    */
    function setSidebarScript($script)
    {
        $this->sidebarScript = $script;
    }

    /**
    * An abstract method that you may override in your subclasses.
    * By default just returns the value in $strPageTitle
    * @return string
    * @access public
    */
    function getPageTitle()
    {
        return $this->strPageTitle;
    }

    /**
    * A method for getting the window title.
    * This is based on the breadcrumb trail.
    * @param string $delim This is the text to separate elements of title.
    * @return string
    * @access public
    */
    function getWindowTitle($delim = ': ')
    {
        if (!sizeof($this->arrTrail))
        {
            $this->arrTrail = $this->getTrail();
        }

        // replace first element with a custom element
        // containg SITE_NAME constant instead of home.
        $path = $this->arrTrail; // make a copy
        $path[0]['name'] = SITE_NAME;

        $arrTitle = array();
        $p_keys = array_keys($path);

        //oam while each is faster
        while (list(,$k) = each($p_keys))
        {
            if (!empty($path[$k]['crumb']) && '_' != $path[$k]['crumb'][0])
            {
                if (isset($path[$k]['name']) && !empty($path[$k]['name']))
                {
                    $arrTitle[] = strip_tags($path[$k]['name']);
                }
                else
                {
                    $arrTitle[] = strip_tags($path[$k]['crumb']);
                }
            }
        }

        return implode($delim, $arrTitle);
    }

    /**
    * A method to manually set PageTitle, for cases when it cannot be dynamically
    * built.  (e.g. modules don't have a rule for page title, while sections do).
    * @param string $page_title
    * @access public
    */
    function setPageTitle($page_title)
    {
        $this->strPageTitle = $page_title;
    }

    /**
    * An abstract method that you must override in your subclasses.
    * By default just returns the value in $strMetaKeywords
    * @return string The Keywords to use in META tag.
    * @access public
    */
    function getMetaKeywords()
    {
        return $this->strMetaKeywords;
    }

    /**
    * A method to manually set MetaKeywords, for cases when it cannot be dynamically
    * built.  (e.g. modules don't have a rule for keywords, while sections do).
    * @param string $keywords
    * @access public
    */
    function setMetaKeywords($keywords)
    {
        $this->strMetaKeywords = $keywords;
    }

    /**
    * An abstract method that you must override in your subclasses.
    * By default just returns the value in $strMetaDescription.
    * @return string The Description to use in META tag.
    * @access public
    */
    function getMetaDescription()
    {
        return $this->strMetaDescription;
    }

    /**
    * A method to manually set MetaDescription, for cases when it cannot be dynamically
    * built.  (e.g. modules don't have a rule for description, while sections do).
    * @param string $desc
    * @access public
    */
    function setMetaDescription($desc)
    {
        $this->strMetaDescription = $desc;
    }
    
    /**
    * Returns a query string for all the variablies in the Request
    *
    * Useful for persisting of values via href links.
    *
    * @param array option set of variables to exclude
    * @return string
    * @access public
    */
    function getVarsAsQueryString($ignore  = null)
    {
        
        if (null != $ignore)
        {
            if (!is_array($ignore))
            {
                $ignore = (array) $ignore;
            }
            
            $keep = array_diff(array_keys($this->arrRequestVars), $ignore);            
        }
        else
        {
            $keep = array_keys($this->arrRequestVars);            
        }
        
        $keep = array_diff($keep, array('navtype', 'request'));        
                
        $qs = array();
        while (list(,$var) = each($keep))
        {            
            // currently this doesn't support arrays like $pxdb
            if (!empty($this->arrRequestVars[$var]) && !is_array($this->arrRequestVars[$var]))
            {
                $qs[] = "$var=".urlencode($this->arrRequestVars[$var]);
            }
        }        
        return implode("&", $qs);        
    } 
    
    /**
    * Returns a query string for all the variablies in the Request
    *
    * Useful for persisting of values via href links.
    *
    * @param array option set of variables to exclude
    * @return string
    * @access public
    */
    function getVarsAsHiddenInputs($ignore  = null)
    {
        
        if (null != $ignore)
        {
            if (!is_array($ignore))
            {
                $ignore = (array) $ignore;
            }
            
            $keep = array_diff(array_keys($this->arrRequestVars), $ignore);            
        }
        else
        {
            $keep = array_keys($this->arrRequestVars);            
        }
        
        $keep = array_diff($keep, array('navtype', 'request'));        
                
        $qs = array();
        while (list(,$var) = each($keep))
        {            
            if (!empty($this->arrRequestVars[$var]))
            {
                $qs[] = '<input type="hidden" name="' . $var .'" value="' . urlencode($this->arrRequestVars[$var]) . '" />';
            }
        }        
        return implode("\n", $qs);        
    }     
} // class
?>