<?php

include_once(SITE_LIB_PATH . '/syntaxcms/Request/Request.class.php');

/**
* This class handles requests made for navigating by Section.
* @access public
* @author Hans Lellelid
* @package SyntaxCMS.Request
*/
class SectionRequest extends Request {

    /**
    * Path of information for each Section.
    * Used by breadcrumb trail builder.
    * @var array
    */
    var $arrSections=array();


    /**
    * The path that includes hidden NAV_ROOT_EL section.
    * @var array
    */
    var $arrFullPath;

    /**
    * Stores the current Section object.
    * Mostly a sort of cache so that recurring calls to getSection()
    * don't re-instantiate.
    * @var object Section
    */
    var $Section=null;

    /**
    * The top section element.
    * OR error if section can't be found.
    * @var object or array
    */
    var $currSection=null;


    /**
    * Whether Section has been parsed.
    * @var boolean
    */
    var $blnParsed=false;

    /**
    * Init & call parent's constructor.
    */
    function SectionRequest()
    {
        parent::Request();
    }

    /**
    * Pops & returns the hash for current session.
    * This is used by the getSection() and getSectionID() methods.
    * Only really exists for efficiency -- so that if curr section id is only
    * needed, no need to instantiate Section object.
    * @access private
    */
    function _popSection()
    {
        // parse the path to determine which section we are currently
        // at.  Do this with several iterative queries and store the
        // results in a class var -- to speed up lookups in the future.

        if (!$this->blnParsed)
        {
            $this->parsePath();
        }

        $arrSections = $this->arrSections; // copy it


        $sec = array_pop($arrSections); // array pop _does_ modify the array

        return $sec;
    }

    /**
    * Shortcut for getting curr section id.
    * @return integer
    */
    function getSectionID()
    {
        $sec = $this->_popSection();
        if ($sec)
        {
            return $sec['id'];
        }
        else
        {
            return false;
        }
    }

    /**
    * Get current section as a Section object.
    * @return object Section
    */
    function &getSection()
    {
        if ($this->Section === null)
        {
            $sec = $this->_popSection();
            if ($sec)
            {
                $this->Section =& new Section($sec['id']);
            }
            else
            {
                return false;
            }
        }
        return $this->Section;
    }

    /**
    * Returns the depth of the current section.
    * A useful function for building breadcrumbs, etc.
    * @return integer The current section's depth.
    */
    function getSectionDepth()
    {
        return sizeof($this->arrSections) - 1;
    }

    /**
    * Whether or not passed Section ID is in the path of sections.
    * @return boolean
    * @param integer $section_id
    */
    function isParentSection($section_id)
    {
        $section_ids = pxdb_tools::array_thread('id', $this->arrSections);
        array_pop($section_ids); // because current section is last element -- and
                                // to be a ParentSection, it shouldn't be the current section.
        return in_array($section_id, $section_ids);
    }

    /**
    * This parsePath implementation avoids the more expensive crazy left join that hans implemented.
    *
    * @author oscar merida
    */
    function parsePath()
    {
        $dbh =& pxdb::dbh();

        // 0) set parsed pref now -- even if there is an error,
        // we need to know that parsePath() has run.

        $this->blnParsed = true;

        // 1) need to add NAV_ROOT_EL to beginning of path
        //    oam: if it is defined
        if (defined('NAV_ROOT_EL'))
        {
            // SFS Account for when NAV_ROOT_EL is not a top-level node:
            $nav_root_elements = explode('/', NAV_ROOT_EL);
            $this->arrFullPath = array_merge($nav_root_elements, $this->arrPath);
        }
        else
        {
            $this->arrFullPath = $this->arrPath;
        }
        // SFS I don't think we need this any more, since URL IDs can be duplicative 
        //$this->arrFullPath = array_unique($this->arrFullPath);
        
        for ($i = 0, $max = count($this->arrFullPath); $i < $max; $i++)
        {
            $url_path_parts = array();
            for ($j = 0; $j <= $i; $j++)
            {
                $url_path_parts[] = $this->arrFullPath[$j];
            }
            $url_paths[] = "'" . implode('/', $url_path_parts) . "'";
        }

        $sql = "
SELECT id, idnum, title, name
FROM records
WHERE desc3 in (" . join(',' , $url_paths) . ")
        ";
        
        $results = $dbh->execute($sql);

        if (0 < $results->RowCount())
        {
            while ($row = $results->fetchRow())
            {
                $sec[$row['idnum']] = array(   
                    'id' => $row['id'],
                    'urlid' => $row['idnum'],
                    'crumb' => $row['title'],
                    'name'  => $row['name']
                );
            } // end while

            // now to sort hem in arrFullPath order
            
            for ($i = 0, $max = count($this->arrFullPath); $i < $max; $i++)
            {
                if (isset($sec[$this->arrFullPath[$i]]))
                {
                    $this->arrSections[] = $sec[$this->arrFullPath[$i]];
                }
                else 
                {
                    $this->arrSections[] = null;
                }
            }
        }
        else
        {
            trigger_error ('Could not find current section: '.$this->getPath(), E_USER_WARNING);
            return false;
        }
        return true;
    }

    /**
    * Build breadcrumb from NAV_ROOT_EL to current section, including all ancestors.
    * @return array
    */
    function getBaseTrail()
    {
        $arrTrail = array();

        if (!$this->blnParsed)
        {
            if (! $res = $this->parsePath())
            {
                return false;
            }
        }

        // loop through the sections.
        for ($i = 0, $z = sizeof($this->arrSections); $i < $z; $i++)
        {
            $arrPoint = array();

            // build the trail, with URLs to previous sections.
            $arrPoint['id']     = $this->arrSections[$i]['id'];
            $arrPoint['crumb']  = $this->arrSections[$i]['crumb'];
            $arrPoint['url']    = '';

            // The URL will look like /section/about/contact
            if (NAV_ROOT_EL != $this->arrSections[$i]['urlid'] )
            {
                $url = '/section/';
                // start at 1 to account for NAV_ROOT_EL
                $url .= implode('/', array_slice($this->arrFullPath, 1, $i));
            }
            else
            {
                $url = '/';
            }

            $arrPoint['url'] = $url;
            $arrTrail[] = $arrPoint;
        }

        return $arrTrail;
    }

    /**
    * Get array of current breadcrumb trail.
    * @return array
    */
    function getTrail()
    {
        if (!sizeof($this->arrTrail))
        {
            $this->arrTrail = $this->getBaseTrail();
        }

        return $this->arrTrail;
    }

    /**
    * Returns the script to use for displaying the section.
    * Note: unlike the module-based navigation we always use
    * the same script here.
    * @return string
    */
    function getScript()
    {
        return  SITE_PATH . '/section.php';
    }

    /**
    * Returns the META-tag keywords.
    * @return string The Keywords to use in META tag.
    */
    function getMetaKeywords()
    {
        $Section =& $this->getSection();
        if ($Section)
        {
            return $Section->getMetaKeywords();
        }
        else
        {
            return '';
        }
    }

    /**
    * Returns the META-tag description.
    * @return string The Description to use in META tag.
    */
    function getMetaDescription()
    {
        $Section =& $this->getSection();
        if ($Section)
        {
            return $Section->getMetaDescription();
        }
        else
        {
            return '';
        }
    }

    /**
    * Returns the current page title.
    * @return string The page title.
    */
    function getPageTitle()
    {
        $Section =& $this->getSection();
        if (!PEAR::isError($Section))
        {
            return $Section->getName(); // Full name
        }
        else
        {
            return '';
        }
    }


} // class

?>