<?php

/**
* Provides a wrapper for the logic to use pxdb_collection classes to navigate the tree
* of Sections in a Syntax CMS site. It provides exposure to methods to get the home section,
* get children of a current section, or iterate over the entire tree (with controls you
* can set).
*
* E.G. to get the home node and its children:
*
* $section_navigator =& SectionNavigator::singleton();
* $section =& $section_navigator->getHomeNode();
* $section_navigator->setSection($section);
* echo('<a href="'.$section_navigator->getNavPath().'">'.$section->get_field('name').'</a><br>');
* $section_navigator->setDescend(false); // don't get children
* while ($section = $section_navigator->getNextSection())
* {
*   echo('<a href="'.$section_navigator->getNavPath().'">'.$section->get_field('name').'</a><br>');
* }
*
* Hey, kids! Here's a handy recipie tip: use in conjunction with the PathInfo class
* for that smooth, velvety logic you count on from Kraft!(tm)
*
* @author Sandy Smith <ssmith@forumone.com>
* @date 4/21/03
* @package SyntaxCMS
*/
class SectionNavigator
{

    /**
    * @var object pxdb_record $section - the Section currently being navigated
    * @access private
    */
    var $section;
    
    
    /**
    * @var boolean $hasChildren
    * @access private
    */
    var $hasChildren;
    
    /**
    * @var object pxdb_collection $children - the pxdb_collection object containing the children
    * @access private
    */
    var $children;
    
    /**
    * @var object pxdb_record $homeSection - the home section
    * @access private
    */
    var $homeSection;
    
    /**
    * @var array A Stack of sections, from root node to current descendant
    * @access private
    */
    var $sectionStack = array();
    
    /**
    * @var array A Stack of pxdb_collection objects containing children of that level's section
    * @access private
    */
    var $childrenStack = array();
    
    /**
    * @var boolean Whether to descend into children
    * @access private
    */
    var $descend = true;
    
    /**
    * @var integer $depth The depth from the starting node
    * @access private
    */
    var $depth = 0;
    
    /**
    * @var boolean controls if we return hidden sections or not
    * @access private
    */
    var $_showHiddenSections = TRUE;
    /**
    * Constructor. Can optionally take an initializing section object
    *
    * @access public
    * @return boolean success
    * @param object pxdb_record $section - optional
    */
    function SectionNavigator($section = null)
    {
        if (null != $section)
        {
            if (false === $this->setSection($section))
            {
                trigger_error("You passed a section but it failed.", E_USER_WARNING);
                return false;
            }
            else
            {
                return true;
            }
        }
        else
        {
            return true;
        }
    }
    
    /**
    * Instantiate one copy of this object.
    * @access public
    * @return object SectionNavigator
    * @param object pxdb_record
    */
    function &singleton($section = null)
    {
        static $instance;
        if (!isset($instance))
        {
            $instance = new SectionNavigator($section);
        }
        return $instance;
    }        

    
    /**
    * Set the section to be used, resets internal vars
    *
    * @access public
    * @return mixed object pxdb_record if URL Path was passed or boolean success or failure
    * @param mixed $section Either URL PATH* string or object pxdb_record *this is changed
    */
    function setSection($section)
    {
        if (true === empty($section) || (false === is_string($section) && false === is_object($section)))
        {
            trigger_error('setSection requires a URL Path string or a valid section object', E_USER_WARNING);
            return false;
        }
        if (true === is_object($section))
        {
           $this->section =& $section;
        }
        else
        {
           $this->section =& $this->_getSectionByUrlPath($section);
        }
        unset($this->children);
        unset($this->hasChildren);
        return (true === is_object($section)) ? true : $this->section;
    }
    
    /**
    * Sets the pxdb_collection of children of the current section
    *
    * @access private
    * @return boolean false if children isn't a pxdb_collection
    * @param object pxdb_collection &$children The collection of children of the current section
    */
    function _setChildren(&$children)
    {
        if (false === is_object($children))
        {
            trigger_error('_setChildren requires a valid section object', E_USER_WARNING);
            return false;
        }
        $this->children =& $children;
        return true;        
    }
    
    
    /**
    * Returns a boolean, tests whether the current Section has children.
    * Can be used as a static method
    *
    * @access public
    * @return boolean
    * @param object pxdb_record $section (optional) The Section to be tested
    */
    function hasChildren($section = null)
    {
        if (null == $section)
        {
            $section =& $this->section;
        }
        if (true === empty($section))
        {
            trigger_error("hasChildren needs a section to be set or passed.", E_USER_WARNING);
            return false;
        }
        
        if (true === empty($this->hasChildren))
        {
            $children =& $this->getChildren($section);
            $this->hasChildren = (0 < $children->num_records()) ? true : false;
        }
        return $this->hasChildren;
    }
    
    /**
    * Returns the collection object with the children of the current node
    *
    * @access public
    * @return object pxdb_collection Collection of children
    * @param object pxdb_record section to be tested (optional)
    */
    function &getChildren($section = null)
    {
        if (null == $section)
        {
            $section = $this->section;
        }
        if (true === empty($section))
        {
            trigger_error("getChildren needs a section to be set or passed.", E_USER_WARNING);
            return false;
        }
        if (true === empty($this->children))
        {
            $children =& $this->_getChildrenCollection($section->get_field('url_path'));
            $this->_setChildren($children); // update so we don't have to do this again.
            
            return $children;
        }
        else
        {
            return $this->children;
        }
    }
    
    /**
     * Generates the pxdb_collection for the getChildren method,
     * used to abstract it for overloading.
     *
     * @access private
     * @return object Reference to the pxdb_collection class containing the children
     * @param string $url_path The URL PATH* of the section *Change
     *
     */
    function &_getChildrenCollection($url_path = null)
    {
        if (null === $url_path)
        {
            trigger_error('_getChildrenCollection needs a url_path to be passed.', E_USER_WARNING);
            return false;
        }
        $section_search = &pxdb_search::filter(DATATYPE_SECTION);
        $section_search->add_value('url_path', $url_path);
        
        $children_search = &pxdb_search::filter(DATATYPE_SECTION);
        $children_search->add_parent($section_search);
        
        $children = &new pxdb_collection(DATATYPE_SECTION);
        $children->set_order('weight');
        $children->find($children_search);
        return $children;
    }
    
    
    /**
    * Gets the home node section and returns it
    *
    * TODO: oam, make this callable statically?
    
    * @access public
    * @return object pxdb_record the home section
    */
    function &getHomeSection()
    {
        if (empty($this->homeSection))
        {
            $this->homeSection =& $this->_getHomeSection();
        }        
        return $this->homeSection;
    }
    
    /**
     * Abstraction method for implementaion of getHomeSection method, although
     * it's far more efficient to override the _getSectionByUrlPath method.
     *
     * @access private
     * @return object pxdb_record Reference to a pxdb_record object
     *
     */
    function &_getHomeSection()
    {
        $section = &$this->_getSectionByUrlPath(NAV_ROOT_EL);
        return $section;
    }
    
    /**
    * Abstraction method. Gets a Section by its URL Path
    *
    * @access private
    * @return object pxdb_record Reference to a pxdb_record object or false on failure
    * @param string $url_path The URL ID of the section you're trying to get
    */
    function &_getSectionByUrlPath($url_path)
    {
        if (true === empty($url_path) || false === is_string($url_path))
        {
           trigger_error(E_USER_WARNING, 'A URL ID string must be passed to _getSectionbyUrlId');
           return false;
        }
        
        
        $section_search = &pxdb_search::filter(DATATYPE_SECTION);
        $section_search->add_value('url_path', $url_path);
        
        $sections = &new pxdb_collection(DATATYPE_SECTION);
        $sections->find($section_search);
        $section =& $sections->fetch_record();
        
        return $section;
    }
    
    /**
    * Returns boolean whether the current (or passed) section is the home section
    *
    * @access public
    * @return boolean
    * @param object pxdb_record
    */
    function isHomeSection($section = null)
    {
        if (null == $section)
        {
            $section =& $this->section;
        }
        if (true === empty($section))
        {
            trigger_error("isHomeSection needs a section to be set or passed.", E_USER_WARNING);
            return false;
        }
        // replaced _home with NAV_ROOT_EL, usually defined in static.conf.php 
        return (NAV_ROOT_EL == $section->get_field('url_path')) ? true : false;        
    }
    
    /**
    * Generates the URL to access the section.
    *
    * @access public
    * @return string    
    * @param object pxdb_record
    * @param string url root to prepend to section path (defaults to section)
    */
    function getNavPath($section = null, $url_root = 'section')
    {
        if (null == $section)
        {
            $section =& $this->section;
        }
        
        if (true === empty($section))
        {
            trigger_error("getNavPath needs a section to be set or passed.", E_USER_WARNING);
            return false;
        }

        // it's simple if it's the home section
        if (true === $this->isHomeSection($section))
        {
            return '/';
        }
		// it's a section link
		if (0 === strpos($section->get_field('url_id'), '*64_')) 
		{
			return pxdb_tools::decode_section_link($section->get_field('url_id'));
		}

		// it's a section page
        $return = preg_replace('/^' . str_replace('/', '\/', NAV_ROOT_EL) . '\/?/', '', $section->get_field('url_path'));
        
        return '/'.$url_root.'/' . $return;
    }
    
    /**
    * Returns the parent to the current or passed section
    *
    * @access public
    * @return object pxdb_record of type section or bool false
    * @param object pxdb_record of type section
    */
    function &getParent($section = null)
    {
        if (null == $section)
        {
            $section =& $this->section;
        }
        if (true === empty($section))
        {
            trigger_error("getParent needs a section to be set or passed.", E_USER_WARNING);
            return false;
        }
        $parents =& $this->_getParentCollection($section->get_field('url_path'));
        return $parents->fetch_record();
    }
    
    /**
     * Abstraction method for implementation of getting the collection with a Parent in it.
     *
     * @access private
     * @return object pxdb_collection Reference to the PxDB collection containing the parent
     * @param string $url_path The URL PATH* of the section that is a child of the parent you want. *Changed
     *
     */
    function &_getParentCollection($url_path = null)
    {
        if (null === $url_path)
        {
            trigger_error('_getParentCollection requires URL Path to be passed', E_USER_WARNING);
            return false;
        }
        $section_search =& pxdb_search::filter(DATATYPE_SECTION);
        $section_search->add_value('url_path', $url_path);
        
        $parent_search =& pxdb_search::filter(DATATYPE_SECTION);
        $parent_search->add_child($section_search);
        
        $parents =& new pxdb_collection(DATATYPE_SECTION);
        $parents->find($parent_search);
        
        return $parents;
    }
    
    
    /**
    * Sets whether or not to descend into children of the current section.
    *
    * @access public
    * @param boolean $descend Whether or not to descend.
    * @return boolean success
    */
    function setDescend($descend)
    {
        if (! is_bool($descend))
        {
            trigger_error('setDescend requires a boolean', E_USER_WARNING);
            return false;
        }
        $this->descend = $descend;
        return true;
    }
    
    /**
    * returns the value of descend, whether to descend into children or not
    *
    * @access private
    * @return boolean Whether to descend into children of current section
    */
    function _descend()
    {
        return $this->descend;
    }
        
    /**
    * Add a pxdb_record of section type and pxdb_collection of children
    * to the section and children stacks, respectively
    *
    * @access private
    * @param object pxdb_record $section A pxdb_record of type section
    * @param object pxdb_collection $children A collection of children sections
    * @return boolean success
    */
    function _addToStacks($section, $children)
    {
        if (! $section || ! $children)
        {
            trigger_error("_addToStacks: both a section and children must be passed", E_USER_WARNING);
            return false;
        }
        // note: important not to pass to this function by reference,
        // in case you change the value it was referring to
        $this->sectionStack[] =& $section; // ok to reference a copy
        $this->childrenStack[] =& $children;
        return true;
    }
    
    /**
    * Remove a pxdb_record object of type section and a pxdb_collection from the stacks.
    * Should be simultaneous, so both in one function. Returns the removed objects.
    *
    * @access private
    * @return array An array of two objects, first being the pxdb_record section and the second the pxdb_collection of children
    */
    function _removeFromStacks()
    {
        return array(array_pop($this->sectionStack), array_pop($this->childrenStack)); 
    }
    
    /**
    * Get a pxdb_record object of type section and a pxdb_collection from the stacks.
    * Should be simultaneous, so both in one function.
    *
    * @access private
    * @return array An array of two objects, first being the pxdb_record section and the second the pxdb_collection of children
    */
    function _getFromStacks()
    {
        $section =& $this->sectionStack[count($this->sectionStack) - 1];
        $children =& $this->childrenStack[count($this->childrenStack) - 1];
        if (is_object($section) && is_object($children))
        {
            return array($section, $children);
        }
        else
        {
            return false;
        }
    }
    
    /**
    * Iterator for moving through the nodes. Use getDepth() to determine the current
    * depth in the tree (with home and its children being OR the section you pass in
    * being depth 0). Use setDescend(bool) to tell the class whether or not to descend
    * into children of a given section. If left to its own devices, it will get all
    * sections, hidden or not, in the current request path or not. Returns false when
    * there are no more sections to get.
    *
    * @access public
    * @param object pxdb_record of type section from which to start tree walk
    * @return object pxdb_record of type section or false
    */
    function &getNextSection($section = null)
    {
        if (null == $section)
        {
            $section =& $this->section;
        }
        elseif (true !== empty($section))
        {
            $this->depth = 0; // make sure we come back to the passed section
            $this->setSection($section); // do I want to do this here?
        }
        else
        {
            trigger_error("getNextSection needs a section to be set or passed.", E_USER_WARNING);
            return false;
        }
        
        // if it has children and descending is permitted, add to stack and descend
        if (true === $this->hasChildren() && true === $this->_descend() && true === $this->is_authorized())
        {
            if (false === $this->isHomeSection())
            {
                $this->depth++;
            }
            $children =& $this->getChildren();
            $this->_addToStacks($section, $children);
            $section =& $children->fetch_record();
            $this->setSection($section);
            return $section;
        }
        else
        {
            // get the next one in line
            if ((list ($parent_section, $children) = $this->_getFromStacks()) && $section =& $children->fetch_record())
            {
                $this->setSection($section);
                return $section;
            }
            else
            {
                // we're at the end of this level, go back up and get the children until it works
                while (0 <= --$this->depth)
                {
                     // go up a level if not at home depth already
                    $remove = $this->_removeFromStacks();
                    // we're not back to the top level
                    if(list ($parent_section, $children) = $this->_getFromStacks())
                    {
                        // grab the next one from the parent list
                        if ($section =& $children->fetch_record())
                        {
                            $this->setSection($section);
                            return $section;
                        }
                    }
                }
                // we're at the top level, clean up and return false
                $this->depth = 0;
                unset($this->childrenStack);
                unset($this->sectionStack);
                $var = FALSE;
                return $var;
            } // end if next one in line
        } // end if descending
    }
    
    /**
    * Gets the current value of depth when using the getNextSection() method
    *
    * @access public
    * @return integer
    */
    function getDepth()
    {
        return $this->depth;
    }
    
    /**
    * Returns the number of children at the current depth.
    *
    * @access public
    * @return integer
    */
    function countChildren()
    {
        $collection =& $this->getChildren();
        return $collection->num_records();
    }
    
    /**
    * Controls wether we return hidden children sections
    *
    * @param bool True=return hidden sections (default)
    * @access public
    */
    function setShowHiddenSections($show = true)
    {
        $this->_showHiddenSections = $show;
    }
    
    /**
    * Returns a pxdb_collection containing all sections that are ancestors of the current section.
    *
    * Obeys showHiddenSections
    * @param object Section -- optional, section to start at
    * @access public
    * @return pxdb_collection, false if no ancestors.
    */
    function &getAncestorCollection($section = null)
    {
        if (null == $section)
        {
            $section =& $this->section;
        }
        elseif (true !== empty($section))
        {
            $this->depth = 0; // make sure we come back to the passed section
            $this->setSection($section); // do I want to do this here?
        }
        else
        {
            trigger_error("getNextSection needs a section to be set or passed.", E_USER_WARNING);
            return false;
        }
        
        $nav_path = explode('/', $section->getNavPath());
        array_pop($nav_path); // the last element in the stack is the sect we are looking at
        
        if (0 < count($nav_path))
        {            
            $z = count($nav_path);
            $ancestors = array();
            while (0 < $z)
            {                
                // check if it should be hidden                
                $current = $nav_path[$z-1]; 
                if (!preg_match('/^_/', $current) || $this->_showHiddenSections)
                {                    
                    $base_path = join('/', $nav_path);
                    $base_path = preg_replace('/^\/section/', NAV_ROOT_EL , $base_path);
                    
                    if (!empty($base_path))
                    {
                        $ancestors[ count($nav_path) ] = "'" . $base_path . "'";
                    }
                }
                
                array_pop($nav_path);
                $z = count($nav_path);
            }
            
            if (0 < count($ancestors))
            {
                $filter =& pxdb_search::filter($section->get_datatype());
                $filter->add_value('url_path', join(',', $ancestors), 'IN', null, $quote=false);
                
                $collection =& new pxdb_collection($section->get_datatype());
                $collection->set_order('desc3');
                $collection->find($filter);
                $collection->execute();
                
                return $collection;
            }
            else
            {
                $var = false;
                return $var;
            }
        }
        else
        {
            $var = false;
            return $var;
        }
    }
    
    /**
    * Returns an array of all children ids from the current section
    *
    * Note, this method moves the children pointer so if you are using it inside
    * a loop you should use a new instance of SectionNavigator and not reuse an
    * existing instance.
    *
    * @return array
    * @access public
    */
    function getDescendantIds()
    {
        $previous_descend = $this->_descend();

        $this->setDescend(true);
        $descendants = array();

        while ($section =& $this->getNextSection())
        {
            $descendants[] = $section->get_field('id'); 
        }

        $this->setDescend($previous_descend);
        return $descendants;
    }
    
    /**
    * Tests if the current section is readable
    *
    * @return bool
    */
    function is_authorized()
    {
        if (!isset($this->section) || empty($this->section))
        {
            trigger_error('$this->section is empty or not set', E_USER_WARNING);
        }
        else
        {
            return $this->section->is_authorized();
        }
        
    }
}
?>