<?php

/**
* A class that holds top-level methods and attributes about this site.
* This class provides methods that get around needing to use GLOBAL variables
* in any other methods.  E.g. instead of doing global $Request to get the top-level
* request, we do $Request =& Site::getRequest();  This preserves namespace at
* a small performance cost.
*
* @author hans lellelid, hans@forumone.com
* @package SyntaxCMS
*/
class Site
{
    /**
    * Top-level Request object.
    * @var object Request
    */
    var $Request = null;

    /**
    * Global Cache_Output object.
    * @var object Cache_Output
    */
    var $Cache = null;

    /**
    * The pxdb_user object
    * @var object pxdb_user
    */
    var $User = null;
    
    /**
    * Holds page-request wide status of cache flushing
    * @var boolean
    * @access private
    */
    var $cacheFlushed = array();
    

    /**
    * Constructor
    *
    * @access public
    */
    function Site()
    {
        // Include Defaults
        // include_once(CONFIG_PATH . '/object_init/'.get_class($this).'.conf.php');
    }

    /**
    * Instantiate one copy of this object.
    * @access public
    * @return object Site
    */
    function &singleton()
    {
        static $instance;
        if (!isset($instance))
        {
            $instance = new Site();
        }
        return $instance;
    }

    /**
    * Gets the top-level (page-request-level) Request object.
    * @access public
    * @return object Request
    */
    function &getRequest()
    {
        $Site =& Site::singleton();
        return $Site->Request;
    }

    /**
    * Sets the top-level Request object.
    * This is the Request that actually fetched current page.   Within a
    * given page $Request may be re-instantiated to perform custom work.
    * @access public
    * @param object Request &$new
    */
    function setRequest(&$new)
    {
        $Site =& Site::singleton();
        $Site->Request =& $new;
    }

    /**
    * This is the error-handler function registered w/ PHP.
    * This function is bound to PEAR::Error AND to PHP's standard error, which
    * means that it can take either 1 hash param (PEAR) or many params (PHP).
    * @param object PEAR::Error
    * @access public
    */
    function errorHandler($err)
    {
        $errno = Err::getCode($err);

        // 1) Log Error

        if ($errno & LOG_ERRORS)
        {
            Err::logError($err);
        }

        // 2) Is it a die error?
        if ($errno & DIE_ERRORS)
        {
            if (DEBUG)
            {
                $action = 'display';
            }
            else
            {
                $action = 'redirect';
            }
        }

        // 3) override and always display if DEBUG is set
        switch ($action)
        {
            case 'display':
                if (strstr($_SERVER['SCRIPT_URI'], 'error.php'))
                {
                    $error = $error;
                }
                include(SITE_PATH . '/error.php');
            break;

            case 'redirect':
                if (strstr($_SERVER['SCRIPT_URI'], 'error.php')) 
                {
                    $arg = '?error=error';
                }
                header("Location: http://".$_SERVER['HTTP_HOST']."/error.php$arg");
            break;
        }

    }

    /**
    * This is the shutdown function registered w/ PHP.
    * The most important thing (currently) that this function does is log
    * any errors to php_errors.log in SITE_LOG_PATH.
    * @return boolean Success
    */
    function shutdown()
    {
        // Perform Logging
        // ---------------
        if ($log_entries = Log::getLog())
        { // assignment
            $timestr = strftime("%c");
            // SFS changed to while
            while (list(, $entry) = each($log_entries))
            {
                $error_log_file = SITE_LOG_PATH . DIRECTORY_SEPARATOR . $entry['target'];
                $line = "$timestr\t[{$entry[source]}]\t{$entry[level]}\t{$entry[message]}\t{$entry[file]} [{$entry[line]}] ".$_SERVER['SCRIPT_URL']."\n";
                error_log($line, 3, $error_log_file);
            }
        } // if there are log entries

        return true;

    }

    /**
    * Gets the PEAR::Cache instance.
    * @return object Site
    * @access public
    */
    function &getCache()
    {
        $Site =& Site::singleton();       
        return $Site->Cache;
    }

    /**
    * Set the PEAR::Cache instance to use
    * @param object PEAR::Cache::Output &$new class
    * @access public
    */
    function setCache($new)
    {        
        $Site = &Site::singleton();
        $Site->Cache = $new;
    }

    /**
    * Set whether cache has been flushed in this request or not
    * @param boolean $is_flushed
    * @param string $group The group to set (optional but strongly recommended).
    * @return boolean success (true) or error (false)
    * @access public
    */
    function setCacheFlushed($is_flushed = false, $group = null)
    {
        if (false === is_bool($is_flushed))
        {
            trigger_error('You must pass a boolean of whether the cache has been flushed to setCacheFlushed', E_USER_WARNING);
            return false;
        }
        
        if (null !== $group && false === is_string($group))
        {
            trigger_error('You must pass a string for the group to set in setCacheFlushed', E_USER_WARNING);
            return false;
        }
        
        $site =& Site::singleton();
        if (null === $group)
        {
            $site->cacheFlushed[] = $is_flushed;
        }
        else
        {
            $site->cacheFlushed[$group] = $is_flushed;
        }
        return true;
    }

    /**
    * Retuns whether the cache has been flushed of not. Defaults to false.
    * If no group is provided, it checks to see if any group has been flushed.
    *
    * @param string $group The group to check.
    * @access public
    * @return boolean
    */
    function isCacheFlushed($group = null)
    {
        $site =& Site::singleton();
        if (true === is_string($group) && '' !== $group)
        {
            if (!isset($site->cacheFlushed[$group]) || false === is_bool($site->cacheFlushed[$group]))
            {
                return false;
            }
            return $site->cacheFlushed[$group];
        }
        else
        {
            $flushed_statuses = array_values($site->cacheFlushed);
            if (true === in_array(true, $flushed_statuses))
            {
                return true;
            }
        }
        // in all other cases, return false
        return false;
    }
    
    /**
    * Gets the currently logged-in user (pxdb_user) if set.
    * This is just for convenience -- the actual value
    * is not stored as part of the site object, but rather
    * is derived from pxdb_user::existant().
    * @return object reference pxdb_user
    * @access public
    */
    function &getUser()
    {
        return pxdb_user::existant();
    }

}
?>