<?php
/**
* Statically-called utilities, formerly in functions.lib.php
*
* @author Sandy Smith <ssmith@forumone.com>
* @access public
* @date 3/11/05
* @package SyntaxCMS.Utilities
*/
class StxUtilities
{
    
    /**
    * Truncates a block of text to the nearest end-of-sentence character after
    * the character count (currently period, question mark, and exclamation point),
    * or the whole string if it is too short.
    * Ex: truncate_to_sentence('Hello World! How are you?', 8) would return 'Hello World!'
    * Ex: truncate_to_sentence('Hello World! How', 80) would return 'Hello World! How'
    *
    * @author AVC
    * @access public
    * @param string text
    * @param int length
    * @return string
    */
    function truncateToSentence($text, $length)
    {
        return preg_replace('/(^.{'.$length.'}.*?[.?!]).+/is', '$1', $text);
    }
    
    /**
    * Creates an a href tag given a url and its label
    *
    * @author OAM
    * @param string url
    * @param string link text
    * @param string link title (optional)
    * @access public
    */
    function createLink($url, $text, $title = null)
    {
        if (null == $title)
        {
            $title = $text;
        }
        
        return '<a href="' . $url . '" title="' . htmlentities($title) . '">' . $text . '</a>';
    }
    
    /**
    * Formats text for display.
    * This is for descriptions & longer text bodies.  Is smart
    * about whether to use nl2br() or to simply display raw.
    *
    * @param string $text String to be formatted
    * @return string Formatted $text
    * @access public
    */
    function formatText($text)
    {
        if (!preg_match("/<p[^>]*>|<br[^>]*>|<li[^>]*>|<table[^>]*>/i", $text))
        {
            $text = nl2br($text);
        }
        $text = str_replace('', '&rsquo;', $text);
        return $text;
    }
    
    /**
    * Formats a timestamp into a date.
    * @param integer $time_stamp Unix Timestamp
    * @param string $format Time formatting string
    * @return string formatted date
    * @access public
    */
    function formatTimestamp($time_stamp, $format = LOCALE_DATE_STD)
    {
        if ($time_stamp)
        {
             return trim(strftime($format, $time_stamp));
        }
        else
        {
            return null;
        }    
    }
    
    /**
    * Formats time string (such as a SQL formatted date) into a standard date format.
    * @param integer $time SQL Timestamp
    * @param string $format Time formatting string
    * @return string formatted date
    * @access public
    */
    function formatDateString($time, $format = LOCALE_DATE_STD)
    {
        return StxUtilities::formatTimestamp(strtotime($time), $format);
    }
    
    /**
    * A sort of factory method for the Request object, that looks
    * at a path and loads the correct Request module.  Very site-specicific
    * (because of path conventions) -- so don't want this to be in Request
    * classes [yet].
    * @param string $path
    * @return object reference to new Request instance
    * @access public
    */
    function &requestFactory($path)
    {
        if ($path == '/' || preg_match("/\/?(section|content)(\/[^\$\?]+)/", $path, $matches))
        {        
            
            if ($matches[1] == 'section' || $path == '/')
            {
                $class_name = 'SectionRequest';
            }
            elseif ($matches[1] == 'content')
            {
                $class_name = 'ModuleRequest';
            }
            
            include_once(SITE_LIB_PATH . "/syntaxcms/Request/$class_name.class.php");
            $Request = &new $class_name();
                    
            if ($path == '/')
            {
                $Request->setPath($path);
            }
            else
            {
                $Request->setPath($matches[2]);        
            }
        }
        else
        {
            return false;
        }
    
        return $Request;
    }
    
    /**
    * Takes a Section object and looks for its template. Put here
    * so the Section object can be regenerated if its properties change.
    * Useful for inheriting templates from parents.
    *
    * @access public
    * @return string 
    * @param object Section
    */
    function getSectionTemplate(&$section, $url_root='section')
    {            
        $section_navigator = &SectionNavigatorOptimized::singleton();
        $section_navigator->setSection($section);
        
        // enable template inheritance
        // OAM: inheritance is only needed if the flag is set, a little more efficient to 
        //      move that test to the first since if it fails, PHP won't execute
        //      the section navigator call.
        while (1 == $section->get_field('inherit_template') 
               && false === $section_navigator->isHomeSection() 
             )
        {
            $section = &$section_navigator->getParent();
            $section_navigator->setSection($section);
        }
        
        // OM: some reason this needs to be called before the IF statement
        //     so that its properly looked up.
        $template_field = $section->get_field('template', 'filename');        
        // home section is handled specially as usual        
        if (!empty($template_field))
        {
            $section_specific_template = '/' . $url_root. '/' . $template_field;
        }
        else if (false === $section_navigator->isHomeSection() || NAV_ROOT_EL != $section->get_field('url_path'))
        {
            
            $section_specific_template =  preg_replace("/\/$/", '', $section_navigator->getNavPath(null, $url_root)) . '.tpl';
            
        } 
        else 
        {     
            $section_specific_template = '/' . $url_root . '/' . NAV_ROOT_EL . '.tpl';
        }
        
        if (file_exists(SITE_TPL_PATH . $section_specific_template))
        {            
            $template = $section_specific_template;
            
            if (DEBUG)
            {
                Log::entry(F1_DEBUG_LOG, 'section.php', __FILE__, __LINE__, "Found section-specific template: $template");
            }
        }
        else
        {
            if (isset($section_specific_template) && !empty($template_field))
            {
                trigger_error('Section specific template but file does not exist: ' . SITE_TPL_PATH . $section_specific_template , E_USER_WARNING);
            }
            
            if ('section' != $url_root)
            {                
                $template = '/'. $url_root . '/section.tpl';
            }
            else
            {
                $template = '/section.tpl';
            }
            
            if (DEBUG) 
            {
                Log::entry(F1_DEBUG_LOG, 'section.php', __FILE__, __LINE__, "Using generic template: $template");
            }
        }
        return $template;
    }
    
    /**
    * Finds the last date of a numerically given month and year
    *
    * @access public
    * @return int last date of given month
    * @param int $month to have last date found for
    * @param int $year to make sure leap year is calculated correctly
    * @author Sandy Smith <ssmith@forumone.com>
    */
    function findLastDate($month, $year)
    {
        if (! is_integer($month) || ! is_integer($year))
        {
            trigger_error('You must pass integers for month and year.', E_USER_WARNING);
            return false;
        }
        if (12 < $month || 1 > $month)
        {
            trigger_error('Your month is out of range.', E_USER_WARNING);
            return false;
        }
        $string_month = (string) $month;
        if (4 <> strlen($string_month))
        {
            trigger_error('You must pass a 4-digit month', E_USER_WARNING);
            return false;
        }
        $month = (int) $month;
        $last_dates = array(
            1 => 31,
            2 => (0 == $year % 4 && (0 != $year % 100 || 0 == $year % 400)) ? 29 : 28,
            3 => 31,
            4 => 30,
            5 => 31,
            6 => 30,
            7 => 31,
            8 => 31,
            9 => 30,
            10 => 31,
            11 => 30,
            12 => 31
      );
        
        return $last_dates[$month];
    }
    
    /**
    * Encodes ASCII text using numeric entities.
    * Useful for obfuscating e-mail addresses to
    * prevent them from being harvested by spambots.
    * Note--this will not work well with entities or non-ASCII characters.
    *
    * @access public
    * @return string obfuscated text
    * @param string $text Plain ASCII text to be obfuscated
    */
    function obfuscateAscii($text)
    {
        $text = (string) $text; // make sure it's a string
        $obfuscated_ascii = '';
        for ($i = 0; $i < strlen($text); $i++)
        {
            $decimal = ord($text{$i});
            $obfuscated_ascii .= "&#$decimal;";
        }
        return $obfuscated_ascii;
    }
    
    /** ***********************************************************************
    *
    * is_valid_email(): an e-mail validation utility routine
    * Version 1.1.1 -- September 10, 2000
    *
    * Written by Michael A. Alderete
    * Please send bug reports and improvements to: <michael@aldosoft.com>
    *
    * This function matches a proposed e-mail address against a validating
    * regular expression. It's intended for use in web registration systems
    * and other places where the user is inputting their e-mail address and
    * you want to check that it's OK.
    *
    * It does NOT attempt to prove that the address exists, only that it
    * COULD exist. Validators which look up the host, try to validate an MX
    * record, etc. can significantly slow the response time of your
    * application; network connections are significantly more expensive than
    * regular expressions, especially on days when the Internet is slow.
    *
    * It also doesn't attempt to ensure that the TLD is a known valid TLD.
    * Doing a lookup against a table isn't too expensive, performance-wise,
    * but given the explosion in Internet usage, keeping a lookup table
    * up-to-date would be problematic.
    *
    * The validating regular expression is significantly more accurate than
    * many others I've seen in example code. It still won't prevent users
    * from entering bogus entries like "a@b.com", but then, if the user
    * really doesn't want to give your their e-mail address, there's nothing
    * you can do about it, other than make a confirmation e-mail a
    * requirement to registering.
    *
    * You should clean up the input e-mail address in your code before you
    * validate it, with trim() and strtolower(), if you're planning to save
    * it into a database or otherwise preserve it beyond the one-time use.
    * The validator doesn't depend on it, but you'll want to save clean data!
    *
    * Example of usage: (assumes the user entry form variable is $f_email)
    *
    *    $f_email = strtolower(trim($f_email));
    *    if (is_valid_email($f_email)) {
    *          // Do database save here
    *          // Or send them a confirmation e-mail
    *    }
    *
    * Implementation notes:
    *
    * The routine uses the Perl Compatible Regular Expressions (pcre) routine
    * preg_match(), which my benchmarking indicates is about twice as fast
    * as the standard ereg() routines in this usage (10 seconds vs. 21
    * seconds to do 50,000 validations). If your installation of PHP doesn't
    * have the pcre package compiled in, you can switch to using the
    * is_valid_email_eregi() version, also provided.
    *
    * Notes about the expression:
    *   - The user name needs to be pretty tolerant; you'd be amazed at
    *     what can be in this portion of a valid address
    *   - The expression allows an unlimited number of sub-domains, e.g.:
    *     user@my.really.deep.sub.domain.com is allowed
    *   - The only four-character TLD that's valid is "arpa", but the
    *     expression doesn't know that, and allows any four-character TLD.
    *
    * ***********************************************************************
    *
    * @access public
    * @author Michael A. Alderete <michael@aldosoft.com>
    * @param string $address E-mail address
    * @return boolean
    */
    function isValidEmail($address)
    {
    
        return (preg_match(
            '/^[-!#$%&\'*+\\.\/0-9=?A-Z^_`{|}~]+'.   // the user name
            '@'.                                    // the ubiquitous at-sign
            '([-0-9A-Z]+\.)+' .                     // host, sub-, and domain names
            '([0-9A-Z]){2,4}$/i',                   // top-level domain (TLD)
            trim($address)));
    }
    
    /**
    * takes an image, finds the width and height, allows you to set a maximum width or height
    * and returns an HTML img tag. Only works on local images.
    *
    * @access public
    * @param string $url The local URL (everything after the domain)
    * @param array $restrict Associative array. Valid keys are 'by' (string 'width' or 'height') and 'max' (int)
    * @param string $class CSS class
    * @param string $alt Alt text
    * @return string The img tag.
    */
    function getImageLink($url, $restrict = array('by' => 'width', 'max' => '200'), $class = null, $alt = null)
    {
        $fullpath = SITE_PATH.$url;
        $info = getimagesize($fullpath);
    
        $dim_check = ($restrict['by'] == 'width') ? 0 : 1;
        $dim_label = ($restrict['by'] == 'width') ? 'width' : 'height';
    
        if ($info[$dim_check] > $restrict['max']) 
        {
            $img = '<img src="' . $url . '" ' . $dim_label . '="' . $restrict['max'] . '"';
            
        }
        else
        {
            $img = '<img src="' . $url . '" ' . $info[3];
        } // end if
        
        if ($class)
        {
            $img .=' class="' . $class . '" ';   
        }
        
        if (null !== $alt)
        {
            $img .= ' alt="' . htmlentities($alt) . '" ';
        }
        return $img.'>';
    }
    
    /**
    * single function call for getting a listWidget object based on 
    * a Syntax pick table object.
    *
    * @param string, name of list widget
    * @param string, name of pick table
    * @param array, optional array of values to prepend to options array
    * @return object ListWidget
    * @access public
    */    
    function &listFromPicktable($name, $picktable, $prepend = null, $value_field = 'id')
    {
        pxdb_import('content.output.pxdb_picktable');
        
        $pickstable = new pxdb_picktable($picktable);
        
        $picks = $pickstable->get_listwidget_array(null, $value_field);
        
        if (null !== $prepend)
        {
            array_unshift($picks, $prepend);
        }
        
        return new ListWidget($picks, $name);
    }
    
    /**
    * single function call for getting a listWidget object based on 
    * a Syntax pick table object.
    *
    * @param string, name of list widget
    * @param string, name of pick table
    * @param array, optional array of values to prepend to options array
    * @return object ListWidget
    * @access public
    */    
    function &listFromPicktableTree($name, $picktable, $prepend = null, $depth_label = null)
    {
        pxdb_import('content.output.pxdb_picktable_tree');
        
        $pickstable = new pxdb_picktable_tree($picktable);        
        $picks = $pickstable->get_listwidget_array(null, $depth_label);
        
        if (null !== $prepend)
        {
            array_unshift($picks, $prepend);
        }
        
        return new ListWidget($picks, $name);
    }
    
    /**
    * Calculates a sort direction for a field.
    *
    * @param string $field FIeld to sort by
    * @param object $Request the request object
    * @return string The SQL direction for sorting
    * @access public
    */
    function getSortDirection($field, &$Request)
    {
        if ($Request->getVar('orderby_field') == $field)
        {        
            if ($Request->getVar('orderby_direction') == 'ASC')
            {
                return 'DESC';
            }
            else
            {
                return 'ASC';
            }
        }
        else
        {
            return 'ASC';
        }
    }
    
    /**
    * Tells whether a link is external to the current site
    *
    * @param string $url the url to test
    * @return bool true if external, false if on this site
    * @access public
    */
    function isUrlExternal($url)
    {
        return (false !== strpos($url, '://')) && (false === strpos($url, $_SERVER['HTTP_HOST']));
    }
    
    function makeUrlsLinks($text)
    {
        return preg_replace("/\b((http(s?):\/\/)|(www\.))([\w\.\-]+)([\/\w+\.]+)(\/*)\b/i", "<a href=\"http$3://$4$5$6$7\" target=\"_blank\">$2$4$5$6$7</a>", $text);    
    }
    
}
?>