<?php
/**
* Defines generic interface for database searches.  You can customize behavior
* by sub-classing and changing the get_search_sql and displayLinks methods. 
*
* TODO: integrate with Syntax and Collections object better instead of treating
*   get_search_sql as a wrapper function to a collection object.
*   For now, this means you can customize search behavior per object by either
*   writing straight up sql in get_search_sql or creating a collection object
*   within that method and return the collection's get_sql result.
*
* @author Oscar Merida
*/
class DBSearch {

    /**
    * search term
    * @var string
    * @access private
    */
    var $term;
    
    /**
    * Return only results user has privs to or not
    * @var boolean
    * @access private
    */
    var $enforcePrivs;

    /**
    * name of subclass to use
    * @var string
    * @access private
    */
    var $name;
    
    /**
    * Fields to search through, also fulltext index
    * @var array
    * @access private
    */
    var $dbfields;
    
    /**
    * db fields to return in select statement
    * @var array
    * @access private
    */
    var $getFields;
    
    /**
    * SQL for ordering results
    * @var string
    * @access private
    */
    var $orderby;
    
    /**
    * Content Type ID
    * @var int
    * @access private
    */
    var $datatype;
    
    /**
    * Constructor
    *
    * @access public
    * @param int $typeid Content Type ID
    * @param string $classname The name of the subclass to search for
    */
    function DBSearch($typeid, $classname)
    {
        $this->datatype = $typeid;
        $this->name = ucwords($classname);
        $this->dbfields = array('name', 'salutation', 'title', 'fname', 'lname', 'city', 'desc1', 'desc2', 'desc3', 'text1', 'text2', 'text3', 'text4', 'text5');
        $this->getFields = array('id', 'name', 'title', 'created', 'modified');
        $this->orderby = 'created DESC';
        $this->enforcePrivs = true;
    } // end func
    
    /**
    * Generate subclasses depending on typeid and content type name
    *
    * @access public
    * @param int $typeid Content Type ID
    * @param string $type The name of the content type to search for
    * @return object subclass of DBSearch
    */
    function factory($typeid, $type)
    {
        // strip out spaces from $typelabel
        $classname = "DBSearch_$type";
        if (class_exists($classname))
        {
            $object = new $classname($typeid, $classname);
        }
        else
        {
            $object = new DBSearch($typeid, $classname);
        } // end if
        return $object;
    }

    /**
    * Generate SQL for single search term when using LIKE
    *
    * @access public
    * @param string $term The search term
    * @return string SQL for WHERE clause
    */
    function expand_term($term)
    {
        reset($this->dbfields);
        while(list(, $field) = each($this->dbfields))
        {
            $sql[] = "$field LIKE '%{$term}%'";
        }
        reset($this->dbfields);
        $sql = implode(' OR ', $sql);
        return $sql;
    }
    
    /**
    * Generate SQL for each search term when using LIKE
    *
    * @access public
    * @return string SQL for WHERE clause
    */
    function expand_all_terms()
    {
        // creates the 'like' expressions for an SQL query given the search terms and the fields to search
        $terms = explode(' ', $this->term);
        foreach($terms as $term)
        {
            $cond[] = '(' . $this->expand_term($term) . ')';
        }
        $sql = implode(' OR ', $cond);
        return $sql;
    }

    /**
    * Set a search term
    *
    * @access public
    * @param string $term The search term
    */
    function setTerm($term)
    {
        $this->term = trim($term);
    } // end func

    /**
    * Get the search term
    *
    * @access public
    * @return mixed string Search term or boolean false if not present
    */
    function getTerm()
    {
        return (!empty($this->term)) ? $this->term : false;
    }

    /**
    * Set a limit on number of results
    *
    * @access public
    * @param string $limit The limit SQL
    */
    function setLimit($limit)
    {
        $this->limit = $limit;
    } // end func

    /**
    * Get the search term
    *
    * @access public
    * @return mixed string Search term or boolean false if not present
    */
    function getLimit()
    {
        return (!empty($this->limit)) ? $this->limit : false;
    }

    /**
    * Populate $this->query with resultset
    *
    * @access public
    */
    function getResults()
    {
        $sql = $this->getSearchSql();

        // oam hack: we use pxdb_filters to enforce privs
        $list_filter = &pxdb_search::filter($this->datatype);
        $collection = new pxdb_collection($this->datatype);
        // filter by sunrise/sunset
        if (DT_NODE != $this->datatype)
        {
            $list_filter->add_value('sunset', DB_TIME_STAMP, '>=', null, false);
            $list_filter->add_value('sunrise', DB_TIME_STAMP, '<=', null, false);
        }
        else
        {
            // restrict sections to children (or self) of NAV_ROOT_EL
            $list_filter->add_value('url_path', NAV_ROOT_EL . '%', 'LIKE', null, true);
        }
        
        $sql = $list_filter->filter_sql($sql);
        
        if (TRUE == $this->enforcePrivs)
        {
            $sql = $collection->priv_filter_sql($sql);
        }
                
        $dbh = &pxdb::dbh();
        if ($this->getLimit()) 
        {
            $sql.=' LIMIT '.$this->getLimit();
        } // end if
        $this->query = $dbh->execute($sql);
    } // end func

    /**
    * Returns number of results
    *
    * @access public
    * @return int number of rows in result set
    */
    function countResults()
    {
        return $this->query->RowCount();
    }
    
    /**
    * Output results in ordered list format in HTML
    *
    * @access public
    * @param int $max Maximum number of results to return--can be supported in subclasses, not supported in parent
    * @return mixed string Search term or boolean false if not present
    */
    function results($max = null)
    {
        if (empty($this->results)) 
        {
            $this->getResults();
        } // end if

        if (0 < $this->countResults()) 
        {
            ob_start();
?>            
            <?= $this->displayLinks($max) ?>
<?php
            $contents = ob_get_contents();
            ob_end_clean();
            return $contents;
        }
        else
        {
            return null;
        } // end if
    } // end func


    /*
    * Override these methods to customize search results per datatype
    */
    
    /**
    * Generates the SQL for the search. Override for other content type if desired
    *
    * @access public
    * @return string SQL
    */
    function getSearchSql() 
    {
        $sql = '
SELECT
    '.implode(",\n    ", $this->getFields) . ",
    MATCH(`name`,`salutation`,`title`,`fname`,`lname`,`city`,`desc1`,`desc2`,`desc3`,`text1`,`text2`,`text3`,`text4`,`text5`) AGAINST ('" . addslashes($this->getTerm()) . "') AS score
FROM records
WHERE 
    (MATCH(`name`,`salutation`,`title`,`fname`,`lname`,`city`,`desc1`,`desc2`,`desc3`,`text1`,`text2`,`text3`,`text4`,`text5`) AGAINST ('" . addslashes($this->getTerm()) . "')) > 0
    AND datatype = " . $this->datatype . '
    AND approved = 1
ORDER BY score, ' . $this->orderby;
        return $sql;
    } // end func
    
    /**
    * Display results links. Override for custom output per content type.
    *
    * @access public
    * @return string HTML of results
    */
    function displayLinks( $max = null )
    {
        ob_start();
        $detailModule = Modules::getCapabilityProvider('detail', $this->datatype);
        
        if ('general' == $detailModule)
        {
            // try to map detail through a datatype module if available
            $modules =  Modules::findModules('datatype', $this->datatype, $strict=false);
            if (!empty($modules))
            {
                $detailModule = $modules[0];
            }
        }
?>
<ol>
<?php
        $i = 0;
        while ($row = $this->query->fetchRow() ) 
        {
            $id_link = '/' . $row['id'] . '/';
?>
    <li>
        <a href="/content/<?= $detailModule ?>/detail<?= $id_link ?>"><?= stripslashes($row['name']) ?></a>
    </li>   
<?php
            $i++;
            if ( !empty($max) && $i >= $max )
            {
                break;
            }
        } // end while
?>
</ol>
<?php
        $content = ob_get_contents();
        ob_end_clean();
        return $content;
    } // end func
} // end class
?>