<?php
/**
* Implements logic for ordering arbitrary db sets.  Defines and automates
* the majority of the functional logic for reordering any set of records. You
* should only need to define 2 methods in a child class that extends this one
* getWeightsRaw() and saveWeight().
*
* @author Oscar Merida <oscar@oscarm.org>
*/
class Orderer {
    
    /**
    * Re initializes a sequence, assuring all weights are unique
    */
    function initSequence( $objs )
    {
        $c = count( $objs );
        $i = 1;
        
        foreach ($objs as $id=>$val) 
        {
            $new[ $id ] = ($i*5);
            $i++;
        }
        return $new;
        
    }
    
    function _saveWeight( $id, $wt, $table )
    {
        $d[ 'id' ] = $id;
        $d[ 'weight'] = $wt;
        $where[ 'id' ] = $id;
        
        $dbh =& pxdb::dbh();        
        return $dbh->replace( $table, $d, $where );
    }

    function saveWeights( $objs )
    {
        foreach ( $objs as $id=>$wt )
        {
            $this->saveWeight( $id, $wt );   
        }
        
    }
    
    /**
    * @private
    * @return array
    */
    function getWeights()
    {
        $objs = $this->getWeightsRaw();
        // check if there are is one weight per id
        $id = count( array_keys( $objs ) );
        $wt = count( array_unique( $objs ) );
        
        if ( $wt < $id )
        {
            $new = $this->initSequence( $objs );
            $this->saveWeights( $new );
            $objs = $this->getWeightsRaw();
        }
        return $objs;        
    }
    
    /**
    * Tests if an object is the first one in the set
    */
    function isFirst( $id, $weights )
    {
        $first = array_shift( array_keys( $weights ) );
        return ( $first == $id ) ? true : false;
    }
    
    /**
    * Tests if an object is the last one in the set
    */
    function isLast ( $id, $weights )
    {
        $last = array_pop ( array_keys( $weights ) );
        return ( $last == $id ) ? true : false;        
    }
    
    /**
    * Moves an object up in the stack
    */
    function moveUp ( $id )
    {

        $weights = $this->getWeights();
        if ( $this->isFirst( $id, $weights ) )
        {
            return;            
        } else {
            //figure out which one we have to swap with
            $keys = array_keys( $weights );
            $swapIndex = array_search( $id, $keys ) - 1;
           
            return $this->swap( $id, $keys[ $swapIndex ], $weights );
        }
        
    }
    
    /**
    * Moves an object down in the stack
    */
    function moveDown( $id )
    {
        $weights = $this->getWeights();
        
        if ( $this->isLast( $id, $weights ) )
        {
            return;            
        } else {
            //figure out which one we have to swap with
            $keys = array_keys( $weights );
            $swapIndex = array_search( $id, $keys ) + 1;
           
            return $this->swap( $id, $keys[ $swapIndex ], $weights );
        }
    }
    
    /**
    * Swaps object 1 and object 2 in the stack
    */
    function swap( $id1, $id2, $weights )
    {
        
        $temp = $weights[ $id1 ];
        $weights[ $id1 ] = $weights[ $id2 ];
        $weights[ $id2 ] = $temp;
        
        return $this->saveWeights( $weights );   
    }
    /**
    * @public
    * @param integer object id
    * @param string  direction, up or down
    */
    function moveObject( $id, $dir )
    {
        
        switch ( $dir )
        {
            case 'up':
                $this->moveUp( $id );
            break;
            
            case 'down':
                $this->moveDown( $id );
            break;            
        }
        
    }

    /*
    * ABSTRACT METHODS
    */
    function getWeightsRaw() {
        trigger_error( 'getWeightsRaw not implemented', E_USER_ERROR );   
    }
    
    function saveWeight() {
        trigger_error( 'saveWeight not implemented', E_USER_ERROR );   
    }
    
}

